/*
 * callbacks.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>

#include <gtk/gtk.h>
#include <glade/glade.h>
#include "callbacks.h"
#include "pdb.h"
#include "common.h"
#include "methyl.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "error_analysis.h"
#include "pcsfit_metal.h"
#include "select_display.h"
#include "tensor_list.h"
#include "molmol_iso.h"
#include "pymol_iso.h"
#include "notebook.h"
#include "rdc.h"
#include "help.h"
#include "save_tensor.h"
#include "about.h"

#if OS == WIN32_OS
	#include <windows.h>
#endif


G_MODULE_EXPORT void 
on_window_about_close (GtkDialog *dialog, gpointer user_data);


GtkWidget *_window_about;

extern gboolean _is_running;

G_MODULE_EXPORT void
on_window_destroy (GtkObject * window, gpointer user_data)
{
  gtk_main_quit ();
}

G_MODULE_EXPORT void
on_Load_PDB_clicked (GtkButton * button, gpointer user_data)
{

  load_PDB ();
}

G_MODULE_EXPORT void
on_Clear_PDB_clicked (GtkButton * button, gpointer user_data)
{
  free_pdb_structure (_pdb_str);
}

/***********************MENU DEALING BEGIN*****************************/
G_MODULE_EXPORT void
on_imagemenuitem12_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	about_dialog_compatible_start((GtkWindow *) _window);
}

G_MODULE_EXPORT void
on_imagemenuitem_home_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  g_spawn_command_line_async
    ("xdg-open http://compbio.chemistry.uq.edu.au/bmmg/christophe/numbat.html",
     NULL);
  
#if OS == WIN32_OS
	ShellExecute(NULL, "open", "http://compbio.chemistry.uq.edu.au/bmmg/christophe/numbat.html", NULL, NULL, 1);
#else
	g_spawn_command_line_async
    ("xdg-open http://compbio.chemistry.uq.edu.au/bmmg/christophe/numbat.html",
     NULL);
#endif 
}

G_MODULE_EXPORT void
on_imagemenuitem3_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  gtk_main_quit ();
}

G_MODULE_EXPORT void
on_menu_bubble_toggled (GtkCheckMenuItem * checkmenuitem, gpointer user_data)
{
  swap_bubble (checkmenuitem);
}

G_MODULE_EXPORT void
on_window_about_close (GtkDialog * dialog, gpointer user_data)
{
  gtk_widget_destroy (_window_about);
}

G_MODULE_EXPORT void
on_imagemenuitem15_activate(GtkMenuItem * menuitem, gpointer user_data)
{
		fprintf(stderr, "help general clicked\n");
		help_general();
}

/***********************MENU DEALING END*****************************/

/***********************HELP DEALING BEGIN****************************/
G_MODULE_EXPORT void
on_button3_clicked (GtkButton * button, gpointer user_data)

{
}

G_MODULE_EXPORT void 
on_help_tensor_fit_clicked (GtkButton * button, gpointer user_data)
{
	help_tensor_fit();
}

G_MODULE_EXPORT void 
on_help_input_data_clicked (GtkButton * button, gpointer user_data)
{
	help_input_data();
}

G_MODULE_EXPORT gboolean
on_window_help_tensor_fit_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		end_help_tensor_fit();
		return(TRUE);
}

G_MODULE_EXPORT gboolean
on_window_help_input_data_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		end_help_input_data();
		return(TRUE);
}

G_MODULE_EXPORT gboolean
on_window_help_general_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		end_help_general();
		return(TRUE);
}


/***********************HELP DEALING END****************************/

/******************CH3 DEALING BEGIN*******************************/
G_MODULE_EXPORT void
on_Define_CH3_COMMON_clicked (GtkButton * button, gpointer user_data)
{
  start_CH3 ();
		
}

G_MODULE_EXPORT void
on_Cancel_CH3_clicked (GtkButton * button, gpointer user_data)
{
  cancel_CH3 ();
  //gtk_widget_hide(_window_CH3);
}

G_MODULE_EXPORT void
on_OK_CH3_clicked (GtkButton * button, gpointer user_data)
{
  save_CH3 ();
}

G_MODULE_EXPORT void
on_Add_CH3_clicked (GtkButton * button, gpointer user_data)
{
  add_item_CH3 ();
}

G_MODULE_EXPORT void
on_Remove_CH3_clicked (GtkButton * button, gpointer user_data)
{
  remove_item_CH3 ();
}

G_MODULE_EXPORT void
on_button_auto_CH3_clicked (GtkButton * button, gpointer user_data)
{
  auto_CH3 ();
}

G_MODULE_EXPORT gboolean 
on_window_CH3_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		cancel_CH3 ();
		return(TRUE);
}


/******************CH3 DEALING END*******************************/

/****************LOADING OF DATA****BEGIN**********************/
G_MODULE_EXPORT void
on_Load_PCS1_clicked (GtkButton * button, gpointer user_data)
{
  load_PCS (0);
}

G_MODULE_EXPORT void
on_Load_PCS2_clicked (GtkButton * button, gpointer user_data)
{
  load_PCS (1);
}

G_MODULE_EXPORT void
on_Load_PCS3_clicked (GtkButton * button, gpointer user_data)
{
  load_PCS (2);
}

G_MODULE_EXPORT void
on_Load_PCS4_clicked (GtkButton * button, gpointer user_data)
{
  load_PCS (3);
}


/****************LOADING OF DATA**END**************************/



/****************SAVING OF DATA****BEGIN**********************/
G_MODULE_EXPORT void
on_Save_PCS1_clicked (GtkButton * button, gpointer user_data)
{
  save_PCS (button, user_data, 0);
}

G_MODULE_EXPORT void
on_Save_PCS2_clicked (GtkButton * button, gpointer user_data)
{
  save_PCS (button, user_data, 1);
}

G_MODULE_EXPORT void
on_Save_PCS3_clicked (GtkButton * button, gpointer user_data)
{
  save_PCS (button, user_data, 2);
}

G_MODULE_EXPORT void
on_Save_PCS4_clicked (GtkButton * button, gpointer user_data)
{
  save_PCS (button, user_data, 3);
}

/****************SAVING OF DATA**END**************************/



/****************DISPLAYING OF DATA****BEGIN**********************/
G_MODULE_EXPORT void
on_Display_PCS1_clicked (GtkButton * button, gpointer user_data)
{
  run_display_PCS (0);
}

G_MODULE_EXPORT void
on_Display_PCS2_clicked (GtkButton * button, gpointer user_data)
{
  run_display_PCS (1);
}

G_MODULE_EXPORT void
on_Display_PCS3_clicked (GtkButton * button, gpointer user_data)
{
  run_display_PCS (2);
}

G_MODULE_EXPORT void
on_Display_PCS4_clicked (GtkButton * button, gpointer user_data)
{
  run_display_PCS (3);
}

G_MODULE_EXPORT void
on_all_residue_display_clicked (GtkButton * button, gpointer user_data)
{    
    all_residue_display();
}

G_MODULE_EXPORT void
on_none_residue_display_clicked (GtkButton * button, gpointer user_data)
{    
    none_residue_display();
}

G_MODULE_EXPORT void
on_all_atom_display_clicked (GtkButton * button, gpointer user_data)
{    
    all_atom_display();
}

G_MODULE_EXPORT void
on_none_atom_display_clicked (GtkButton * button, gpointer user_data)
{    
    none_atom_display();
}

G_MODULE_EXPORT void
on_cancel_display_clicked (GtkButton * button, gpointer user_data)
{
  cancel_display ();
}

G_MODULE_EXPORT gboolean 
on_window_display_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
    cancel_display ();
    return(TRUE);
}

G_MODULE_EXPORT void
on_apply_display_clicked (GtkButton * button, gpointer user_data)
{
  apply_display ();
}

G_MODULE_EXPORT void
on_copy_from_display_changed (GtkComboBox * combox, gpointer user_data)
{
  copy_display_association (combox);
}

/****************DISPLAYING OF DATA**END**************************/


/****************SELECTING OF DATA****BEGIN**********************/

G_MODULE_EXPORT void
on_all_residue_select_clicked (GtkButton * button, gpointer user_data)
{    
    all_residue_select();
}

G_MODULE_EXPORT void
on_none_residue_select_clicked (GtkButton * button, gpointer user_data)
{    
    none_residue_select();
}

G_MODULE_EXPORT void
on_all_atom_select_clicked (GtkButton * button, gpointer user_data)
{    
    all_atom_select();
}

G_MODULE_EXPORT void
on_none_atom_select_clicked (GtkButton * button, gpointer user_data)
{    
    none_atom_select();
}

G_MODULE_EXPORT void
on_Select_PCS1_clicked (GtkButton * button, gpointer user_data)
{
  run_select_PCS (0);
}

G_MODULE_EXPORT void
on_Select_PCS2_clicked (GtkButton * button, gpointer user_data)
{
  run_select_PCS (1);
}

G_MODULE_EXPORT void
on_Select_PCS3_clicked (GtkButton * button, gpointer user_data)
{
  run_select_PCS (2);
}

G_MODULE_EXPORT void
on_Select_PCS4_clicked (GtkButton * button, gpointer user_data)
{
  run_select_PCS (3);
}

G_MODULE_EXPORT void
on_cancel_select_clicked (GtkButton * button, gpointer user_data)
{
  cancel_select ();
}


G_MODULE_EXPORT gboolean 
on_window_select_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
    cancel_select ();
    return(TRUE);
}


G_MODULE_EXPORT void
on_apply_select_clicked (GtkButton * button, gpointer user_data)
{
  apply_select ();
}


G_MODULE_EXPORT void
on_copy_from_select_changed (GtkComboBox * combox, gpointer user_data)
{
  copy_select_association (combox);
}

/****************SELECTING OF DATA**END**************************/


/****************GENERATING OF DATA****BEGIN**********************/
G_MODULE_EXPORT void
on_Generate_PCS1_clicked (GtkButton * button, gpointer user_data)
{
  start_tl (0);
}

G_MODULE_EXPORT void
on_Generate_PCS2_clicked (GtkButton * button, gpointer user_data)
{
  start_tl (1);
}

G_MODULE_EXPORT void
on_Generate_PCS3_clicked (GtkButton * button, gpointer user_data)
{
  start_tl (2);
}

G_MODULE_EXPORT void
on_Generate_PCS4_clicked (GtkButton * button, gpointer user_data)
{
  start_tl (3);
}

G_MODULE_EXPORT void
on_Generate_RDC1_clicked (GtkButton * button, gpointer user_data)
{
}
G_MODULE_EXPORT void
on_Generate_RDC2_clicked (GtkButton * button, gpointer user_data)
{
}
G_MODULE_EXPORT void
on_Generate_RDC3_clicked (GtkButton * button, gpointer user_data)
{
}
G_MODULE_EXPORT void
on_Generate_RDC4_clicked (GtkButton * button, gpointer user_data)
{
}

/****************GENERATING OF DATA**END**************************/

/****************PDB SELECTOR BEGING******************************/
G_MODULE_EXPORT void
on_all_pdb_selector_PCS1_clicked (GtkButton * button, gpointer user_data)
{
  all_none_clicked_pdb_selector (0, TRUE);
}

G_MODULE_EXPORT void
on_all_pdb_selector_PCS2_clicked (GtkButton * button, gpointer user_data)
{
  all_none_clicked_pdb_selector (1, TRUE);
}

G_MODULE_EXPORT void
on_all_pdb_selector_PCS3_clicked (GtkButton * button, gpointer user_data)
{
  all_none_clicked_pdb_selector (2, TRUE);
}

G_MODULE_EXPORT void
on_all_pdb_selector_PCS4_clicked (GtkButton * button, gpointer user_data)
{
  all_none_clicked_pdb_selector (3, TRUE);
}

G_MODULE_EXPORT void
on_all_pdb_selector_PCS_MULTI_clicked (GtkButton * button, gpointer user_data)
{
  all_none_clicked_pdb_selector (MULTIPLE_PCS_N_BUTTON, TRUE);
}

G_MODULE_EXPORT void
on_none_pdb_selector_PCS1_clicked (GtkButton * button, gpointer user_data)
{
  all_none_clicked_pdb_selector (0, FALSE);
}

G_MODULE_EXPORT void
on_none_pdb_selector_PCS2_clicked (GtkButton * button, gpointer user_data)
{
  all_none_clicked_pdb_selector (1, FALSE);
}

G_MODULE_EXPORT void
on_none_pdb_selector_PCS3_clicked (GtkButton * button, gpointer user_data)
{
  all_none_clicked_pdb_selector (2, FALSE);
}

G_MODULE_EXPORT void
on_none_pdb_selector_PCS4_clicked (GtkButton * button, gpointer user_data)
{
  all_none_clicked_pdb_selector (3, FALSE);
}

G_MODULE_EXPORT void
on_none_pdb_selector_PCS_MULTI_clicked (GtkButton * button,
					gpointer user_data)
{
  all_none_clicked_pdb_selector (MULTIPLE_PCS_N_BUTTON, FALSE);
}

/****************PDB SELECTOR END*********************************/

G_MODULE_EXPORT void
on_Calculate_tensor_PCS1_clicked (GtkButton * button, gpointer user_data)
{
  print_pdb_selector (0);
  calculate_single_tensor (0, NULL, NULL);
}

G_MODULE_EXPORT void
on_Calculate_tensor_PCS2_clicked (GtkButton * button, gpointer user_data)
{
  print_pdb_selector (1);
  calculate_single_tensor (1, NULL, NULL);
}

G_MODULE_EXPORT void
on_Calculate_tensor_PCS3_clicked (GtkButton * button, gpointer user_data)
{
  print_pdb_selector (2);
  calculate_single_tensor (2, NULL, NULL);
}

G_MODULE_EXPORT void
on_Calculate_tensor_PCS4_clicked (GtkButton * button, gpointer user_data)
{
  print_pdb_selector (3);
  calculate_single_tensor (3, NULL, NULL);
}

G_MODULE_EXPORT void
on_calculate_tensor_multiple_PCS_clicked (GtkButton * button,
					  gpointer user_data)
{
  print_pdb_selector (MULTIPLE_PCS_N_BUTTON);
  start_calculate_multiple_tensor (button);
}


/****************tensor listing START*********************************/
G_MODULE_EXPORT void
on_add_tl_clicked (GtkButton * button, gpointer user_data)
{
  add_manual_tl ();
}

G_MODULE_EXPORT void
on_copy_tl_clicked (GtkButton * button, gpointer user_data)
{
  copy_tl ();
}

G_MODULE_EXPORT void
on_Cancel_tensor_listing_clicked (GtkButton * button, gpointer user_data)
{
  cancel_tl ();
}

G_MODULE_EXPORT gboolean 
on_window_tensor_list_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		cancel_tl ();
		return(TRUE);
}

G_MODULE_EXPORT void
on_OK_tensor_listing_clicked (GtkButton * button, gpointer user_data)
{
  ok_tl ();
}

G_MODULE_EXPORT void
on_remove_tl_clicked (GtkButton * button, gpointer user_data)
{
  remove_tl ();
}

/****************tensor listing END*********************************/


/****************VISUALIZE START*********************************/
G_MODULE_EXPORT void
on_add_tl5_clicked (GtkButton * button, gpointer user_data)
{
  add_manual_vm ();
}

G_MODULE_EXPORT void
on_copy_tl5_clicked (GtkButton * button, gpointer user_data)
{
  copy_vm ();
}

G_MODULE_EXPORT void
on_Cancel_tensor_listing5_clicked (GtkButton * button, gpointer user_data)
{
  cancel_vm ();
}

G_MODULE_EXPORT gboolean 
on_window_iso_molmol_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		cancel_vm ();
		return(TRUE);
}

G_MODULE_EXPORT void
on_OK_tensor_listing5_clicked (GtkButton * button, gpointer user_data)
{
  ok_vm (button);
}

G_MODULE_EXPORT void
on_save_vm_clicked (GtkButton * button, gpointer user_data)
{
  save_vm (button);
}

G_MODULE_EXPORT void
on_remove_tl5_clicked (GtkButton * button, gpointer user_data)
{
  remove_vm ();
}

G_MODULE_EXPORT void
on_spin_x_start_value_changed (GtkSpinButton * spin, gpointer user_data)
{
  update_vm_gride_end (spin);
}

G_MODULE_EXPORT void
on_spin_y_start_value_changed (GtkSpinButton * spin, gpointer user_data)
{
  update_vm_gride_end (spin);
}

G_MODULE_EXPORT void
on_spin_z_start_value_changed (GtkSpinButton * spin, gpointer user_data)
{
  update_vm_gride_end (spin);
}

G_MODULE_EXPORT void
on_spin_n_step_value_changed (GtkSpinButton * spin, gpointer user_data)
{
  update_vm_gride_end (spin);
}

G_MODULE_EXPORT void
on_spin_step_size_value_changed (GtkSpinButton * spin, gpointer user_data)
{
  update_vm_gride_end (spin);
}


G_MODULE_EXPORT void
on_Visualize_MolMol_clicked (GtkButton * button, gpointer user_data)
{
  start_vm ();
}

G_MODULE_EXPORT void
on_button_auto_set_up_clicked (GtkButton * button, gpointer user_data)
{
  update_vm_gride (button);
}

/****************VISUALIZE END*********************************/



/***************MORE******************************************/
G_MODULE_EXPORT void
on_more_PCS1_clicked (GtkButton * button, gpointer user_data)
{
  more_tensor (0, button);
}

G_MODULE_EXPORT void
on_more_PCS2_clicked (GtkButton * button, gpointer user_data)
{
  more_tensor (1, button);
}

G_MODULE_EXPORT void
on_more_PCS3_clicked (GtkButton * button, gpointer user_data)
{
  more_tensor (2, button);
}

G_MODULE_EXPORT void
on_more_PCS4_clicked (GtkButton * button, gpointer user_data)
{
  more_tensor (3, button);
}

G_MODULE_EXPORT void
on_more_PCSMULTI_clicked (GtkButton * button, gpointer user_data)
{
  more_tensor (-1, button);
}

G_MODULE_EXPORT void
on_OK_tensor_listing4_clicked (GtkButton * button, gpointer user_data)
{
  ok_more_tensor ();
}


G_MODULE_EXPORT void
on_button_force_UTR_clicked (GtkButton * button, gpointer user_data)
{
  force_UTR();
}


G_MODULE_EXPORT gboolean 
on_window_tensor_work_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		cancel_more_tensor ();
		return(TRUE);
}

G_MODULE_EXPORT void
on_add_tl3_clicked (GtkButton * button, gpointer user_data)
{
  add_manual_more_tensor ();
}

G_MODULE_EXPORT void
on_copy_tl3_clicked (GtkButton * button, gpointer user_data)
{
  copy_more_tensor ();
}


G_MODULE_EXPORT void
on_remove_tl3_clicked (GtkButton * button, gpointer user_data)
{
  remove_more_tensor ();
}

G_MODULE_EXPORT void
on_box_more_rotation_order_changed(GtkComboBox * combo, gpointer user_data){
		update_tensor_work();
}
G_MODULE_EXPORT void
on_box_more_magnitude_convention_changed(GtkComboBox * combo, gpointer user_data){
		update_tensor_work();
}
G_MODULE_EXPORT void
on_box_more_angle_orientation_changed(GtkComboBox * combo, gpointer user_data){
		update_tensor_work();
}
G_MODULE_EXPORT void
on_box_more_angle_unit_changed(GtkComboBox * combo, gpointer user_data){
		update_tensor_work();
}



/***************MORE END**************************************/


/****************VISUALIZE PYMOL START*********************************/
G_MODULE_EXPORT void
on_add_tl2_clicked (GtkButton * button, gpointer user_data)
{
  add_manual_vp ();
}

G_MODULE_EXPORT void
on_copy_tl2_clicked (GtkButton * button, gpointer user_data)
{
  copy_vp ();
}

G_MODULE_EXPORT void
on_Cancel_tensor_listing2_clicked (GtkButton * button, gpointer user_data)
{
  cancel_vp ();
}

G_MODULE_EXPORT gboolean 
on_window_iso_pymol_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		cancel_vp ();
		return(TRUE);
}


G_MODULE_EXPORT void
on_OK_tensor_listing2_clicked (GtkButton * button, gpointer user_data)
{
  ok_vp (button);
}

G_MODULE_EXPORT void
on_save_vp_clicked (GtkButton * button, gpointer user_data)
{
  save_vp (button);
}

G_MODULE_EXPORT void
on_remove_tl2_clicked (GtkButton * button, gpointer user_data)
{
  remove_vp ();
}

G_MODULE_EXPORT void
on_spin_x_start2_value_changed (GtkSpinButton * spin, gpointer user_data)
{
  update_vp_gride_end (spin);
}

G_MODULE_EXPORT void
on_spin_y_start2_value_changed (GtkSpinButton * spin, gpointer user_data)
{
  update_vp_gride_end (spin);
}

G_MODULE_EXPORT void
on_spin_z_start2_value_changed (GtkSpinButton * spin, gpointer user_data)
{
  update_vp_gride_end (spin);
}

G_MODULE_EXPORT void
on_spin_n_step2_value_changed (GtkSpinButton * spin, gpointer user_data)
{
  update_vp_gride_end (spin);
}

G_MODULE_EXPORT void
on_spin_step_size2_value_changed (GtkSpinButton * spin, gpointer user_data)
{
  update_vp_gride_end (spin);
}

G_MODULE_EXPORT void
on_Visualize_Pymol_clicked (GtkButton * button, gpointer user_data)
{
  start_vp ();
}

G_MODULE_EXPORT void
on_button_auto_set_up2_clicked (GtkButton * button, gpointer user_data)
{
  update_vp_gride (button);
}

/****************VISUALIZE PYMOL END*********************************/

/***************RUNNING DIALOG**************************************/
G_MODULE_EXPORT void
on_button_stop_clicked (GtkButton * button, gpointer user_data)
{
  progressbar_reset ();
  _is_running = FALSE;
}

G_MODULE_EXPORT gboolean 
on_window_running_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		progressbar_reset ();
  	_is_running = FALSE;
		return(TRUE);
}

G_MODULE_EXPORT void
on_button_running_ok_clicked (GtkButton * button, gpointer user_data)
{
  progressbar_reset ();
  _is_running = FALSE;
}


/***************RUNNING DIALOG END**************************************/


/***************NOTEBOOK START**************************************/

G_MODULE_EXPORT void
on_notebook_exp_switch_page (GtkNotebook * notebook, GtkNotebookPage * page,
			     guint page_num, gpointer user_data)
{
  if (page_num < N_PCS_ARRAY)
    {
      notebook_EXP_changed (page_num);
    }
}

G_MODULE_EXPORT void
on_notebook_ten_switch_page (GtkNotebook * notebook, GtkNotebookPage * page,
			     guint page_num, gpointer user_data)
{
  if (page_num < N_PCS_ARRAY)
    {
      notebook_TEN_changed (page_num);
    }
}

/***************NOTEBOOK END**************************************/

/***************RDC START**************************************/

G_MODULE_EXPORT void
on_Save_RDC_clicked (GtkButton * button, gpointer user_data)
{
  start_rdc ();
}

G_MODULE_EXPORT void
on_combobox_RDC_ATOM1_changed (GtkComboBox * combox, gpointer user_data)
{

}
G_MODULE_EXPORT void
on_combobox_RDC_ATOM2_changed (GtkComboBox * combox, gpointer user_data)
{

}
G_MODULE_EXPORT void
on_combobox_RDC_RES_changed (GtkComboBox * combox, gpointer user_data)
{

}

G_MODULE_EXPORT void
on_CANCEL_RDC_button_clicked (GtkButton * button, gpointer user_data)
{
  cancel_rdc ();
}

G_MODULE_EXPORT gboolean 
on_window_rdc_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		cancel_rdc ();
		return(TRUE);
}

G_MODULE_EXPORT void
on_SAVE_RDC_button_clicked (GtkButton * button, gpointer user_data)
{
  save_rdc (button);
}

G_MODULE_EXPORT void
on_add_tl4_clicked (GtkButton * button, gpointer user_data)
{
  add_manual_rdc ();
}

G_MODULE_EXPORT void
on_copy_tl4_clicked (GtkButton * button, gpointer user_data)
{
  copy_rdc ();
}

G_MODULE_EXPORT void
on_remove_tl4_clicked (GtkButton * button, gpointer user_data)
{
  remove_rdc ();
}

/***************RDC END**************************************/

/***************ERROR START**************************************/

G_MODULE_EXPORT void
on_Error_analasys_clicked (GtkButton * button, gpointer user_data)
{
  start_error_analysis (button);
}

G_MODULE_EXPORT void
on_CANCEL_error_analysis_clicked (GtkButton * button, gpointer user_data)
{
  cancel_error_analysis ();
}

G_MODULE_EXPORT gboolean 
on_window_error_analysis_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		cancel_error_analysis ();
		return(TRUE);
}

G_MODULE_EXPORT void
on_OK_error_analysis_clicked (GtkButton * button, gpointer user_data)
{
  ok_error_analysis (button);
}

G_MODULE_EXPORT void
on_add_tl7_clicked (GtkButton * button, gpointer user_data)
{
  add_manual_error_analysis ();
}

G_MODULE_EXPORT void
on_copy_tl7_clicked (GtkButton * button, gpointer user_data)
{
  copy_error_analysis ();
}

G_MODULE_EXPORT void
on_remove_tl7_clicked (GtkButton * button, gpointer user_data)
{
  remove_error_analysis ();
}

G_MODULE_EXPORT void
on_Cancel_sanson_clicked (GtkButton * button, gpointer user_data)
{
  cancel_sanson ();
}

G_MODULE_EXPORT gboolean 
on_window_sanson_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		cancel_sanson ();
		return(TRUE);
}

G_MODULE_EXPORT void
on_save_sanson_clicked (GtkButton * button, gpointer user_data)
{
  save_sanson (button);
}

G_MODULE_EXPORT void
on_view_sanson_clicked (GtkButton * button, gpointer user_data)
{
  view_sanson ();
}


/***************ERROR END**************************************/


/***************RACS START**************************************/
G_MODULE_EXPORT void
on_Define_RACS_clicked (GtkButton * button, gpointer user_data)
{
  start_racs ();
}

G_MODULE_EXPORT void
on_checkbutton_RACS_PCS1_toggled (GtkToggleButton * togglebutton,
				  gpointer user_data)
{
  toggle_racs (togglebutton);
}

G_MODULE_EXPORT void
on_checkbutton_RACS_PCS2_toggled (GtkToggleButton * togglebutton,
				  gpointer user_data)
{
  toggle_racs (togglebutton);
}

G_MODULE_EXPORT void
on_checkbutton_RACS_PCS3_toggled (GtkToggleButton * togglebutton,
				  gpointer user_data)
{
  toggle_racs (togglebutton);
}

G_MODULE_EXPORT void
on_checkbutton_RACS_PCS4_toggled (GtkToggleButton * togglebutton,
				  gpointer user_data)
{
  toggle_racs (togglebutton);
}


G_MODULE_EXPORT void
on_checkbutton_RACS_PCSGEN_toggled (GtkToggleButton * togglebutton,
				    gpointer user_data)
{
  toggle_racs (togglebutton);
}

G_MODULE_EXPORT void
on_button_ok_RACS_clicked (GtkButton * button, gpointer user_data)
{
  ok_racs ();
}

G_MODULE_EXPORT void
on_button_cancel_RACS_clicked (GtkButton * button, gpointer user_data)
{
  cancel_racs ();
}

G_MODULE_EXPORT gboolean 
on_window_RACS_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		cancel_racs ();
		return(TRUE);
}

G_MODULE_EXPORT void
on_button_auto_RACS_clicked (GtkButton * button, gpointer user_data)
{
  auto_name ();
}

/***************RACS END**************************************/


/**************SAVE_TENSOR START**************************************/

G_MODULE_EXPORT void
on_Save_tensor_clicked (GtkButton * button, gpointer user_data)
{
  start_save_tensor ();
}

G_MODULE_EXPORT void
on_add_tl8_clicked (GtkButton * button, gpointer user_data)
{
  add_manual_save_tensor ();
}

G_MODULE_EXPORT void
on_copy_tl8_clicked (GtkButton * button, gpointer user_data)
{
  copy_save_tensor ();
}

G_MODULE_EXPORT void
on_remove_tl8_clicked (GtkButton * button, gpointer user_data)
{
  remove_save_tensor ();
}

G_MODULE_EXPORT void
on_Cancel_save_tensor_clicked (GtkButton * button, gpointer user_data)
{
  cancel_save_tensor ();
}

G_MODULE_EXPORT gboolean 
on_window_save_tensor_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
		cancel_save_tensor ();
		return(TRUE);
}

G_MODULE_EXPORT void
on_save_for_pdb_clicked (GtkButton * button, gpointer user_data)
{
  save_for_pdb();
}

G_MODULE_EXPORT void
on_save_for_sanson_clicked (GtkButton * button, gpointer user_data)
{
  save_for_sanson ();
}

G_MODULE_EXPORT void
on_save_for_txt_clicked (GtkButton * button, gpointer user_data)
{
  save_for_txt();
}

/**************SAVE TENSOR END**************************************/




