/*
 * common.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#ifdef HAVE_CONFIG_H
    #include <config.h>
#endif
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <math.h>
#include <glib/gstdio.h>

#include "pdb.h"
#include "racs.h"
#include "pcs.h"
#include "common.h"
extern GtkWidget *_window;
extern GtkWidget *_window_tensor_work;
extern GtkWidget * _window_running;
gboolean _is_running;

GtkProgressBar * pbar;

//~ box_error display a popup box error that display
//~ the message given by the 2 strings.
void
box_error (char *text1, char *text2)
{
  GtkWidget *dialog;
  //gboolean answer;	
  fprintf(stderr, text1, text2);
  dialog = gtk_message_dialog_new (NULL,
				   GTK_DIALOG_DESTROY_WITH_PARENT,
				   GTK_MESSAGE_ERROR,
				   GTK_BUTTONS_CLOSE,
				   text1, text2, g_strerror (1));

	gtk_window_set_icon_from_file ((GtkWindow *)dialog,
                                  NUMBAT_ICON_FILE,
                                  NULL);
	gtk_window_set_title  ((GtkWindow *)dialog, "Error message");
	gtk_window_set_position ((GtkWindow *)dialog, GTK_WIN_POS_MOUSE);
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
}


//~ box_message display a popup box message that display
//~ the message given by the string.
void
box_message (char *text)
{
  GtkWidget *dialog;
  fprintf (stderr, text);
  dialog = gtk_message_dialog_new (NULL,
				   GTK_DIALOG_DESTROY_WITH_PARENT,
				   GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, text);
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
}

//~ to set the title of the main window (with the pdb loaded)
void set_title_window(char* title){
  //char ti[MAX_FILE_NAME+strlen(PROGRAM_NAME)+2+10];
  char ti[2000];   
  //strcpy(ti, PROGRAM_NAME);
  if(EULER_CONVENTION == ZYZ_CONVENTION){
      sprintf(ti, "%s  %s", PROGRAM_NAME, title);
    //strcat(ti, "\t");
  }
    else{
        //strcat(ti, "\t");
        sprintf(ti, "%s (/!\\ ZXZ)  %s", PROGRAM_NAME, title);
    }
  //strcat(ti, title);
  gtk_window_set_title((GtkWindow *)_window, ti);
}

void set_name_label_pdb(char* name){
    set_title_window(name);    
}

void set_name_label_pcs(int n_button, char * name){
  GladeXML *gxml;	
  GtkLabel * label;
  char lab[100];
	
  gxml = glade_get_widget_tree(_window);
  sprintf(lab, "label_name_PCS%d", n_button + 1);	 
  label = (GtkLabel *) glade_xml_get_widget (gxml, lab);
  gtk_label_set_text (label, name);
  sprintf(lab, "label_name_T_PCS%d", n_button + 1);	 
  label = (GtkLabel *) glade_xml_get_widget (gxml, lab);
  gtk_label_set_text (label, name);
}
	
int get_format(char * name){
    int l;
    int m;
    l = strlen(name);
    
    m = l-strlen(FORMAT_NUMBAT_EXT);
    if(m >= 0){
        if(strcmp(name + m, FORMAT_NUMBAT_EXT) == 0){
            return(FORMAT_NUMBAT);
        }
    }
    m = l-strlen(FORMAT_PALES_EXT);
    if(m >= 0){
        if(strcmp(name + m, FORMAT_PALES_EXT) == 0){
            return(FORMAT_PALES);
        }
    }
    m = l-strlen(FORMAT_TBL_EXT);
    if(m >= 0){
        if(strcmp(name + m, FORMAT_TBL_EXT) == 0){
            return(FORMAT_XPLOR);
        }
    }
	m = l-strlen(FORMAT_LST_EXT);
    if(m >= 0){
        if(strcmp(name + m, FORMAT_LST_EXT) == 0){
            return(FORMAT_LST);
        }
    }
    return(FORMAT_UNKNOWN);
}


void set_multiple_selection(GtkTreeView * tv){
	
	GtkTreeSelection*   ts;
	ts = gtk_tree_view_get_selection(tv);
	gtk_tree_selection_set_mode (ts, GTK_SELECTION_MULTIPLE);
}

double my_fmax(double a, double b){
	
	if (a > b){
		return a;
	}
	return(b);
}

gboolean is_line_empty(char * line){
	int n1, n2;
	n1 = strlen(line);
	n2 = strspn(line, " \t\n");
	if(n1 == n2){
		return(TRUE);
	}
	return(FALSE);
}




int progressbar_init(GtkTextBuffer** buffer)
{
  /* pbar is declared global */
  GladeXML *gxml;
  GtkTextView * txt;
  GtkWidget * button;
  GtkTextIter iterstart;
  GtkTextIter iterend;
  	
  gxml = glade_get_widget_tree(_window_running);
  gtk_widget_show(_window_running);
  pbar = (GtkProgressBar *) glade_xml_get_widget (gxml, "progressbar1");
  txt = (GtkTextView *) glade_xml_get_widget (gxml, "textview2");
	
  button = glade_xml_get_widget (gxml, "button_running_ok");
  gtk_widget_hide(button);
  button = glade_xml_get_widget (gxml, "button_stop");
  gtk_widget_show(button);
	
  *buffer = gtk_text_view_get_buffer(txt);
  gtk_text_buffer_get_iter_at_offset(*buffer,&iterstart,0);
  gtk_text_buffer_get_iter_at_offset(*buffer,&iterend,-1);
  gtk_text_buffer_delete(*buffer, &iterstart, &iterend);
  if (pbar == NULL){
	  return(-1);
  }

  _is_running = TRUE;	
  gtk_progress_bar_set_pulse_step(pbar,0.01);
  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pbar), 0.0);
  while (gtk_events_pending())
    gtk_main_iteration();

  return 1;
}

gboolean progressbar_update(gdouble percent)
{
  gtk_progress_bar_pulse (GTK_PROGRESS_BAR(pbar));
  while (gtk_events_pending()){
    gtk_main_iteration();
  }
  if( _is_running ){	
	return TRUE;
  }
	else{		
		return FALSE;
	}
}

void progressbar_finish(void){
  GladeXML *gxml;
  GtkWidget * button;
  gxml = glade_get_widget_tree(_window_running);
  button = glade_xml_get_widget (gxml, "button_running_ok");
  gtk_widget_show(button);
  button = glade_xml_get_widget (gxml, "button_stop");
  gtk_widget_hide(button);
  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pbar), 0.0);
  while (gtk_events_pending()){
    gtk_main_iteration();
  }
}

void progressbar_reset(void)
{
  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pbar), 0.0);
  while (gtk_events_pending()){
    gtk_main_iteration();
  }
	
  gtk_widget_hide(_window_running);
}

void B0_from_Hz_2_T(double * B0){
    *B0 = (*B0) * pow(10, 6) / GAMMA_H_FOR_B0 * 2 * M_PI;
    fprintf(stderr, "B0 in Tesla: %lf\n", *B0);
}

gboolean focus_on_name( GtkComboBox *combo_box, char * name, int COLUMN_ID){
    gboolean result;
    GtkTreeModel* model;
    GtkTreeIter iter;
    char *test;
    test = NULL;
    model = gtk_combo_box_get_model(combo_box);
    if(model == NULL){
        return(FALSE);
    }
    result = gtk_tree_model_get_iter_first  (model, &iter);

    while(result == TRUE){
        gtk_tree_model_get(model, &iter, COLUMN_ID, &test, -1);
        if(strcmp(test, name) == 0){
            gtk_combo_box_set_active_iter(combo_box, &iter);
            g_free(test);
            return(TRUE);
        }
        result = gtk_tree_model_iter_next(model, &iter);
    }
    if(test != NULL){
        g_free(test);
        return(FALSE);
    }
    return(FALSE);
}

//path_ is allocated and contains the
//PATH to the home directory of numbat
//path_ == NULL in case of problem
gchar * get_numbat_directory(){
    char *value;
    char *numbat_rel_path;
    char * path_;
    int ret;
    path_ = NULL;
#if OS == WIN32_OS    
	//fprintf(stderr, "HOMEPATH (win): %s\n", value);
    value = getenv ("HOMEPATH");
    numbat_rel_path = g_strdup_printf("%s", "\\numbat_temp");
#endif
#if OS != WIN32_OS
    value = getenv ("HOME");
	//fprintf(stderr, "HOMEPATH (linux): %s\n", value);
    numbat_rel_path = g_strdup_printf("%s", "/.numbat");
#endif
    path_ = g_strdup_printf ("%s%s", value, numbat_rel_path);
    g_free(numbat_rel_path);
    ret = g_access (path_, F_OK);
    if(ret != 0){
        //fprintf(stderr, "'%s' directory missing, trying to create it...\n", path_);
        ret = g_mkdir (path_, S_IRWXU);
        if(ret != 0){
            /*box_error( "Error, impossible to create the directory\n'%s'\n\
check your right in your home directory\nexiting", path_);
*/
            exit(-1);
        }
        else{
            //fprintf(stderr, "'%s' directory successfully created\n", path_);
        }
    }
    return(path_);
}
