/*
 * common.h
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glade/glade.h>

#define PROGRAM_NAME "Numbat"
#define VERSION "0.999"
#define TENSOR_LENGTH 1

#define LINUX 0
#define MAC 1
#define WIN32_OS 2

#define OS LINUX
//#define DEVELOPMENT
//Uncomment the mode DEVELOPMENT when you want to prepare a tarball for distribution

#ifdef DEVELOPMENT
	#define GLADE_FILE "numbat.glade"
	#define NUMBAT_ICON_FILE "numbat_icon_24.gif"
	#define NUMBAT_LOGO_FILE "numbat.gif"
	#define NUMBAT_HELP_DIR "help_txt/"
#elif OS == LINUX
  #define GLADE_FILE PACKAGE_DATA_DIR"/numbat/glade/numbat.glade"
  #define NUMBAT_ICON_FILE PACKAGE_DATA_DIR"/numbat/glade/numbat_icon_24.gif"
  #define NUMBAT_LOGO_FILE PACKAGE_DATA_DIR"/numbat/glade/numbat.gif"
  #define NUMBAT_HELP_DIR  PACKAGE_DOC_DIR"/doc/numbat/"
#elif OS == MAC
  #define GLADE_FILE PACKAGE_DATA_DIR"/numbat/glade/numbat.glade"
  #define NUMBAT_ICON_FILE PACKAGE_DATA_DIR"/numbat/glade/numbat_icon_24.gif"
  #define NUMBAT_LOGO_FILE PACKAGE_DATA_DIR"/numbat/glade/numbat.gif"
  #define NUMBAT_HELP_DIR  PACKAGE_DOC_DIR"/doc/numbat/"
#elif OS == WIN32_OS
	#define GLADE_FILE "glade/numbat.glade"
	#define NUMBAT_ICON_FILE "glade/numbat_icon_24.gif"
	#define NUMBAT_LOGO_FILE "glade/numbat.gif"
	#define NUMBAT_HELP_DIR "help_txt/"
#endif




#define USE_GREEK_LETTER
//If the fonts are not correctly installed, the greck letter won't display
//correctly. In that case, just comment the line '#define USE_GREEK_LETTER'
//to use alternative notation

#ifdef USE_GREEK_LETTER
    #define STRING_DELTA_CHI_AX "\316\224\317\207<sub>ax</sub>"
    #define STRING_DELTA_CHI_AX_UNIT "\316\224\317\207<sub>ax</sub> (10<sup>-32</sup> m<sup>3</sup>)"
    #define STRING_DELTA_CHI_RH "\316\224\317\207<sub>rh</sub>"
    #define STRING_DELTA_CHI_RH_UNIT "\316\224\317\207<sub>rh</sub> (10<sup>-32</sup> m<sup>3</sup>)"
    #define STRING_DELTA_CHI_AX_ALTERNATIF "\316\224\317\207ax"
    #define STRING_DELTA_CHI_RH_ALTERNATIF "\316\224\317\207rh"
    #define STRING_DELTA_CHI_X "\316\224\317\207<sub>x</sub>"
    #define STRING_DELTA_CHI_Y "\316\224\317\207<sub>y</sub>"
    #define STRING_DELTA_CHI_Z "\316\224\317\207<sub>z</sub>"
    #define STRING_ALPHA "\316\261"
    #define STRING_BETA "\316\262"
    #define STRING_GAMMA "\316\263"
    #define STRING_SIGMA "\317\203"
    #define STRING_GAMMA1_RDC "\316\263<sub>1</sub> (10<sup>6</sup> rad s<sup>-1</sup> T<sup>-1</sup>) :"
    #define STRING_GAMMA2_RDC "\316\263<sub>2</sub> (10<sup>6</sup> rad s<sup>-1</sup> T<sup>-1</sup>) :"
    #define STRING_TEMPERATURE_KELVIN "T (K):"
    #define STRING_CSA_SIGMA11 "\317\203<sub>11</sub> (ppm)"
    #define STRING_CSA_SIGMA22 "\317\203<sub>22</sub> (ppm)"
    #define STRING_CSA_SIGMA33 "\317\203<sub>33</sub> (ppm)"
    #define STRING_CSA_BETA "\316\262 (\302\260)"
    #define STRING_DELTA_CHI "\316\224\317\207"
    #define STRING_MAGNITUDE_CONVENTION_dXz_dXy_dXx "|\316\224\317\207z|>|\316\224\317\207y|>|\316\224\317\207x|"
#else
    #define STRING_DELTA_CHI "X"
    #define STRING_DELTA_CHI_AX "X<sub>ax</sub>"
    #define STRING_DELTA_CHI_RH "X<sub>rh</sub>"
    #define STRING_DELTA_CHI_X "X<sub>x</sub>"
    #define STRING_DELTA_CHI_Y "X<sub>y</sub>"
    #define STRING_DELTA_CHI_Z "X<sub>z</sub>"
    #define STRING_DELTA_CHI_AX_ALTERNATIF "X<sub>ax</sub>"
    #define STRING_DELTA_CHI_RH_ALTERNATIF "X<sub>rh</sub>"
    #define STRING_DELTA_CHI_AX_UNIT "X<sub>ax</sub> (10<sup>-32</sup> m<sup>3</sup>)"
    #define STRING_DELTA_CHI_RH_UNIT "X<sub>rh</sub> (10<sup>-32</sup> m<sup>3</sup>)"
    #define STRING_ALPHA "alpha"
    #define STRING_BETA "beta"
    #define STRING_GAMMA "gamma"
    #defien STRING_SIGMA "sigma"
    #define STRING_GAMMA1_RDC "gamma<sub>1</sub> (10<sup>6</sup> rad s<sup>-1</sup> T<sup>-1</sup>) :"
    #define STRING_GAMMA2_RDC "gamma<sub>2</sub> (10<sup>6</sup> rad s<sup>-1</sup> T<sup>-1</sup>) :"
    #define STRING_TEMPERATURE_KELVIN "T (Kelvin):"
    #define STRING_CSA_SIGMA11 "Sigma<sub>11</sub> (ppm)"
    #define STRING_CSA_SIGMA22 "Sigma<sub>22</sub> (ppm)"
    #define STRING_CSA_SIGMA33 "Sigma<sub>33</sub> (ppm)"
    #define STRING_CSA_BETA "Beta (degree)"
    #define STRING_MAGNITUDE_CONVENTION_dXz_dXy_dXx "|dXz|>|dXy|>|dXx|"
#endif

#define FORMAT_NUMBAT_EXT ".npc"
#define FORMAT_TBL_EXT ".tbl"
#define FORMAT_PALES_EXT ".pales"
#define FORMAT_LST_EXT ".lst"

#define FACT_USI (10000/12/M_PI)
#define GAMMA_H_FOR_B0 (267.522*pow(10,6))
#define CONSTANTE_MU0 (1.256637061*pow(10,-6))
#define CONSTANTE_k_BOLTZ (1.3806504 * pow(10, -23))
#define CONSTANTE_h_dirac (1.05457168 * pow(10, -34))

#define TICK_SIZE 13

//THE fact_usi take into account (i) the 1/(12PI) (ii)
//the fact that length are in Angstrom and Xax in 10-32m3
//and (iii) that PCS are in ppm (10^6)
 
enum{
	ZXZ_CONVENTION,
	ZYZ_CONVENTION
};


#define EULER_CONVENTION ZYZ_CONVENTION
//#define EULER_CONVENTION ZXZ_CONVENTION

//enum the different format suported
enum
{
FORMAT_XPLOR,
FORMAT_PALES,
FORMAT_NUMBAT,  
FORMAT_LST,
FORMAT_UNKNOWN
};

enum
{
    USE_RADIAN_FOR_TW,
    USE_DEGREE_FOR_TW
};

extern GtkWidget *_window_about;
extern GtkWidget *_window_CH3;
extern GtkWidget *_window;

GdkColor _color_methyl;
GdkColor _color_dumby;
char line[MAX_CAR];	//buffer line to read the file

void
set_title_window(char* title);

void 
box_error( char * text1, char * text2);

void 
box_message( char * text);

int get_format(char * name);
void set_name_label_pcs(int n_button, char * name);
void set_name_label_pdb(char * name);
void set_multiple_selection(GtkTreeView * tv);
double my_fmax(double a, double b);
gboolean is_line_empty(char * line);


int progressbar_init(GtkTextBuffer** buffer);
gboolean progressbar_update(gdouble percent);
void progressbar_reset(void);
void progressbar_finish(void);
void B0_from_Hz_2_T(double * B0);
gboolean focus_on_name( GtkComboBox *combo_box, char * name, int COLUMN_ID);
gchar * get_numbat_directory();
