/*
 * error_analysis.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <math.h>
#include <glib/gstdio.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>
#include <gsl/gsl_statistics.h>
#include <unistd.h>

#include "pdb.h"
#include "methyl.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "tensor_list.h"
#include "common.h"
#include "error_analysis.h"
#include "pcsfit_metal.h"
#include "molmol_iso.h"
#include "unify_tensor.h"
#include "rdc.h"


extern GtkWidget * _window_error_analysis;
extern GtkWidget * _window_sanson;
extern GtkTreeModel * _model_tl;

GtkFrame * _frame_1;
GtkFrame * _frame_2;
GtkFrame * _frame_3;
GtkFrame * _frame_4;

double ** _sanson[3];

int _number_of_tensor;
int _array_size;
/**************************START INTERFACE*******************************/
void start_error_analysis(GtkButton * button){
    if(_pdb_str == NULL){
        box_error( "You should first load a pdb file.\n", NULL);
        return;
    }
	uncheck_all((GtkTreeModel *) _model_tl, COLUMN_USED_TL);
	
    gtk_widget_show(_window_error_analysis);
}


int read_multi_tensor_only_from_tl(GtkTreeModel * model, 
								   tensor my_tensor_multi[N_PCS_ARRAY]){
	GtkTreeIter iter;
    gboolean is_toggle;
	gboolean my_bool;
	int serial;
	int n_tensor;
	
	n_tensor = 0;
	
	is_toggle = FALSE;
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (model, &iter, COLUMN_USED_TL, &is_toggle, -1); 
        if (is_toggle == TRUE){
			n_tensor++;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model, &iter);
    }
    
    if(n_tensor == 0){
        box_error("error, at least one tensor should be selected for calculation\n", NULL);
        return (-1);
    }
	
	
	my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (model, &iter, COLUMN_USED_TL, &is_toggle, -1); 
        if (is_toggle == TRUE){
			int my_n_button;
			char * exp;
			gtk_tree_model_get(model, &iter, COLUMN_EXP_TL, &exp, -1);
			my_n_button = which_pcs_experiment(exp);
            //fprintf(stderr, "WHICH_PCS_EXPERIMENT %d\n", my_n_button);
			tensor * my_tensor;
			my_tensor = &(my_tensor_multi[my_n_button]);
			gtk_tree_model_get(model, &iter, COLUMN_X_TL, &(my_tensor->x), -1);
  		  	gtk_tree_model_get(model, &iter, COLUMN_Y_TL, &(my_tensor->y), -1);
    		gtk_tree_model_get(model, &iter, COLUMN_Z_TL, &(my_tensor->z), -1);
    		gtk_tree_model_get(model, &iter, COLUMN_AX_TL, &(my_tensor->ax), -1);
    		gtk_tree_model_get(model, &iter, COLUMN_RH_TL, &(my_tensor->rh), -1);
    		gtk_tree_model_get(model, &iter, COLUMN_ALPHA_TL, &(my_tensor->a), -1);
    		gtk_tree_model_get(model, &iter, COLUMN_BETA_TL, &(my_tensor->b), -1);
    		gtk_tree_model_get(model, &iter, COLUMN_GAMMA_TL, &(my_tensor->g), -1);
			gtk_tree_model_get(model, &iter, COLUMN_SERIAL_TL, &serial, -1);
			my_tensor->a = my_tensor->a/180*M_PI;
			my_tensor->b = my_tensor->b/180*M_PI;
			my_tensor->g = my_tensor->g/180*M_PI;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model, &iter);
    }
	
	return(n_tensor);
}

void init_tensor_to_exeption(tensor * my_tensor){
	my_tensor->x = EXEPTION_VALUE;
	my_tensor->y = EXEPTION_VALUE;
	my_tensor->z = EXEPTION_VALUE;
	my_tensor->a = EXEPTION_VALUE;
	my_tensor->b = EXEPTION_VALUE;
	my_tensor->g = EXEPTION_VALUE;
	my_tensor->ax = EXEPTION_VALUE;
	my_tensor->rh = EXEPTION_VALUE;
}

gboolean is_tensor_exeption(tensor * my_tensor){
	if((my_tensor->x == EXEPTION_VALUE) &&
	(my_tensor->y == EXEPTION_VALUE) &&
	(my_tensor->z == EXEPTION_VALUE) &&
	(my_tensor->a == EXEPTION_VALUE) &&
	(my_tensor->b == EXEPTION_VALUE) &&
	(my_tensor->g == EXEPTION_VALUE) &&
	(my_tensor->ax == EXEPTION_VALUE) &&
	(my_tensor->rh == EXEPTION_VALUE)){
		return(TRUE);
	}
	return(FALSE);
}

void read_error_control_data(error_control_data* err_ctrl, GtkButton * button){
	GladeXML *gxml;
    gxml = glade_get_widget_tree((GtkWidget *)button);
	
	GtkSpinButton * spin_noise_PDB;
	GtkSpinButton * spin_noise_PCS1;
	GtkSpinButton * spin_noise_PCS2;
	GtkSpinButton * spin_noise_PCS3;
	GtkSpinButton * spin_noise_PCS4;
	GtkSpinButton * spin_subset;
	GtkSpinButton * spin_iter_error;
	GtkSpinButton * spin_seed_error;
	
    spin_noise_PDB = (GtkSpinButton *) glade_xml_get_widget (gxml, "spin_noise_PDB");
	spin_noise_PCS1 = (GtkSpinButton *) glade_xml_get_widget (gxml, "spin_noise_PCS1");
	spin_noise_PCS2 = (GtkSpinButton *) glade_xml_get_widget (gxml, "spin_noise_PCS2");
	spin_noise_PCS3 = (GtkSpinButton *) glade_xml_get_widget (gxml, "spin_noise_PCS3");
	spin_noise_PCS4 = (GtkSpinButton *) glade_xml_get_widget (gxml, "spin_noise_PCS4");
	spin_subset = (GtkSpinButton *) glade_xml_get_widget (gxml, "spin_subset");
	spin_iter_error = (GtkSpinButton *) glade_xml_get_widget (gxml, "spin_iter_error");
	spin_seed_error = (GtkSpinButton *) glade_xml_get_widget (gxml, "spin_seed_error");
	
	err_ctrl->noise_structure = gtk_spin_button_get_value (spin_noise_PDB);
	err_ctrl->noise_data_PCS[0] = gtk_spin_button_get_value (spin_noise_PCS1);
	err_ctrl->noise_data_PCS[1] = gtk_spin_button_get_value (spin_noise_PCS2);
	err_ctrl->noise_data_PCS[2] = gtk_spin_button_get_value (spin_noise_PCS3);
	err_ctrl->noise_data_PCS[3] = gtk_spin_button_get_value (spin_noise_PCS4);
	err_ctrl->subset_percentage = gtk_spin_button_get_value (spin_subset);
	err_ctrl->n_sample = gtk_spin_button_get_value (spin_iter_error);
	err_ctrl->seed = gtk_spin_button_get_value (spin_seed_error);
}

void launch_error_analysis(error_control_data * err_ctrl
						   , tensor my_tensor_multi[N_PCS_ARRAY])
{
	int my_n_button;
	int my_n_button_array[N_PCS_ARRAY];
	int i;
	my_n_button = -1;

	int compt;
	compt = 0;
	//BE CARFULL WHEN ASKING FOR MULTIPLE, my_n_button[4] MUST be coherent
	for(i = 0; i < N_PCS_ARRAY; i++){
		my_n_button_array[i] = FALSE;
		if((err_ctrl->use_data_PCS[i]) == TRUE){
			compt ++;
			my_n_button = i;
			my_n_button_array[i] = TRUE;
		}
	}
	
	
	switch(compt){
		case 0:{
			fprintf(stderr, "problem shouldn't occur\n");
			return;
		}
		break; // SINGLE
		case 1: {
			calculate_single_tensor(my_n_button, err_ctrl, my_tensor_multi);
		}
		break; //MULTI
		default: {
			calculate_multiple_tensor(my_n_button_array, err_ctrl, my_tensor_multi);
		}
		break;
	}
}


void ok_error_analysis(GtkButton * button){
	tensor my_tensor[N_PCS_ARRAY];
	error_control_data err_ctrl;
	int resu2;
	int i;
	for(i = 0; i < N_PCS_ARRAY; i++){
		init_tensor_to_exeption(&(my_tensor[i]));
	}
	
	resu2 = read_multi_tensor_only_from_tl(_model_tl, my_tensor);
	if(resu2 <= 0){
		return;
	}
	for(i = 0; i < N_PCS_ARRAY; i++){
		if(is_tensor_exeption(&(my_tensor[i])) == FALSE){
			err_ctrl.use_data_PCS[i] = TRUE;
		}
        else err_ctrl.use_data_PCS[i] = FALSE;
	}
    
    for(i = 0; i < N_PCS_ARRAY; i++){
        //fprintf(stderr, "%d  %d\n", i, err_ctrl.use_data_PCS[i]);
    }
    
	read_error_control_data(&err_ctrl, button);
	{
		double noise_active;
		int j;
		noise_active = 0;
		for(j = 0; j < N_PCS_ARRAY; j++){
			if(err_ctrl.use_data_PCS[j] == TRUE){
				noise_active += err_ctrl.noise_data_PCS[j];
			}
		}
		noise_active += err_ctrl.noise_structure;
		noise_active += (100.0 - err_ctrl.subset_percentage);
		if(noise_active == 0){
			box_error("You should add some structural noise\nand / or experimental noise\nand / or use a percentage of data set.\n", NULL);
			return;
		}
		if(err_ctrl.subset_percentage == 0){
			box_error("You should use a percentage of data larger than 0.\n", NULL);
		}
	}
	
	launch_error_analysis(&err_ctrl, my_tensor);
	
	gtk_widget_hide(_window_error_analysis);
}

void cancel_error_analysis(void){
	gtk_widget_hide(_window_error_analysis);
}

void init_error_analysis(void){
	int i;
    GladeXML *gxml;
	int tab[N_PCS_ARRAY];
    gxml = glade_get_widget_tree(_window_error_analysis);
    GtkTreeView * treeview;
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview13");
    add_columns_tl(treeview, TENSOR_ONLY_VALID);
	_frame_1 = (GtkFrame *) glade_xml_get_widget (gxml, "frame_noise_PCS1");
	_frame_2 = (GtkFrame *) glade_xml_get_widget (gxml, "frame_noise_PCS2");
	_frame_3 = (GtkFrame *) glade_xml_get_widget (gxml, "frame_noise_PCS3");
	_frame_4 = (GtkFrame *) glade_xml_get_widget (gxml, "frame_noise_PCS4");
    gtk_tree_view_set_model ((GtkTreeView *)treeview, GTK_TREE_MODEL (_model_tl));
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
	set_multiple_selection(GTK_TREE_VIEW (treeview));

	for(i = 0; i < N_PCS_ARRAY; i++){
		_tensor_valid[i] = NULL;
		tab[i] = -1;
	}
	_multi_tensor_valid = NULL;
	change_view_frame(tab);
    _sanson[0] = NULL;
    _sanson[1] = NULL;
    _sanson[2] = NULL;
    _number_of_tensor = -1;
	_array_size = -1;
}


void add_manual_error_analysis(void){
    GladeXML *gxml;
    double x, y, z, Ax, Rh, a, b, g;
    
    gxml = glade_get_widget_tree(_window_error_analysis);

    x = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_x7"));
    y = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_y7"));
    z = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_z7"));
    a = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_alpha7"));
    b = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_beta7"));
    g = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_gamma7"));
    Ax = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Ax7"));
    Rh = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Rh7"));
    add_to_tensor_listing(Ax, Rh, x, y, z, a, b, g, MANUAL_TENSOR, -1,
						   0, 0, 0, 0, 0, 0, 0, 0);
}


void copy_error_analysis(void){
  GtkTreeView *treeview;
  GladeXML *gxml;  
	
  gxml = glade_get_widget_tree(_window_error_analysis);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview13"); 
  copy_tensor_from_list(gxml, treeview, 7);
}

void remove_error_analysis(void){
  GtkTreeView *treeview;
  GladeXML *gxml;  
    
  gxml = glade_get_widget_tree(_window_error_analysis);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview13");
  remove_tensor_from_list(gxml, treeview, 7);
}

/**************************END INTERFACE*******************************/



// We add new calculated tensor each time
void append_tensor_valid(int n_button, int serial_tensor){
	GSList* temp;
	temp = g_slist_append (_tensor_valid[n_button], GINT_TO_POINTER(serial_tensor));
	_tensor_valid[n_button] = temp;
}

void append_multi_tensor_valid( int n_button, int serial_tensor, int is_first){
	GSList* temp;
	int * tab;
	int i;
	
	fprintf(stderr, "start append_multi\n");
	if (is_first){
		fprintf(stderr, "is_first YES (n_buton: %d serial:%d)\n", n_button, serial_tensor);
		tab = (int *)calloc( N_PCS_ARRAY, sizeof(int));
		for(i = 0; i < N_PCS_ARRAY; i++){
			tab[i] = -1;
		}
		temp = g_slist_append (_multi_tensor_valid, tab);
		_multi_tensor_valid = temp;
	}
	
	fprintf(stderr, "adding (n_buton: %d serial:%d)\n", n_button, serial_tensor);
	temp = g_slist_last(_multi_tensor_valid);
	tab = (int *) g_slist_nth_data(temp, 0);
	tab[n_button] = serial_tensor;
	
	//temp = g_slist_append (_multi_tensor_valid, GINT_TO_POINTER(serial_tensor));
}

//Something in the PCS data has changed, 
//being values, PCS used, methyl association. We clear
void clear_tensor_valid(int n_button){
	int i;
	fprintf(stderr , "clearing list %d\n", n_button);
	g_slist_free(_tensor_valid[n_button]);
	_tensor_valid[n_button] = NULL;
	
	for(i = 0; i < g_slist_length( _multi_tensor_valid); i++){
		int * tab;
		tab = (int *) g_slist_nth_data(_multi_tensor_valid, i);
		if(tab[n_button] != -1){
			fprintf(stderr , "removing from MULTI list element %d (n_button %d)\n", i, n_button);
			_multi_tensor_valid = g_slist_delete_link(_multi_tensor_valid
													  , g_slist_nth(_multi_tensor_valid, i));
			i--; //we just removed an lement from the list!
		}
	}
	if(g_slist_length( _multi_tensor_valid)){
		_multi_tensor_valid = NULL;
	}
}

/*
void clear_multi_tensor_valid(void){
	fprintf(stderr , "clearing list MULTI\n");
	g_slist_free(_multi_tensor_valid);
	_multi_tensor_valid = NULL;
}
*/

//for debug purpose
void print_tensor_valid(void){
	int i, j;
	int serial;
	//fprintf(stderr, "starting to print\n");
	for(i = 0; i < N_PCS_ARRAY; i++){
		for(j = 0; j < g_slist_length(_tensor_valid[i]); j++){
			serial = GPOINTER_TO_INT(g_slist_nth_data(_tensor_valid[i], j));
	 		fprintf(stderr, "PCS exp %d: %d\n", i+1, serial);
		}
	}
		

	for(j = 0; j < g_slist_length(_multi_tensor_valid); j++){
		int * tab;
		tab = (int *) g_slist_nth_data(_multi_tensor_valid, j);
		for(i = 0; i < N_PCS_ARRAY; i++){
			fprintf(stderr, "element of list %d, n_button %d, serial %d.\n",
					j, i, tab[i]);
		}
	}
}

//can the tensor be used for error analysis?
//only if nothing has changed for PCS data...
gboolean is_tensor_single_valid(int serial){
	GSList* l;
	int i;
	for(i = 0; i < N_PCS_ARRAY; i++){
		l = g_slist_find(_tensor_valid[i], GINT_TO_POINTER(serial));
		if(l != NULL){
			return(TRUE);
		}
	}
	return(FALSE);
}

//WARNING, the tab CAN't be stock (a long time) in calling function, because
//the pointer migth change when PCS data are changed for example
gboolean is_tensor_multi_valid(int serial, int ** tab_ret){
	int i, j;
	for(i = 0; i < g_slist_length(_multi_tensor_valid); i++){
		int * tab;
		tab = (int *) g_slist_nth_data(_multi_tensor_valid, i);
		for(j = 0; j < N_PCS_ARRAY; j++){
			if(tab[j] == serial){
				*tab_ret = tab; //WARNING maybe incorrect
				return(TRUE);
			}
		}
	}
	*tab_ret = NULL;
	return(FALSE);
}

gboolean is_tensor_valid(int serial, int ** tab_ret){
	if(is_tensor_single_valid(serial)){
		return(TRUE);
	}
	if( is_tensor_multi_valid(serial, tab_ret)){
		return(TRUE);
	}
	return(FALSE);
}

void change_view_frame(int *tab){
	if(tab[0] != -1){
		gtk_widget_show((GtkWidget *)_frame_1);
	}
	else{
		gtk_widget_hide((GtkWidget *)_frame_1);
	}
	
	if(tab[1] != -1){
		gtk_widget_show((GtkWidget *)_frame_2);
	}
	else{
		gtk_widget_hide((GtkWidget *)_frame_2);
	}
	
	if(tab[2] != -1){
		gtk_widget_show((GtkWidget *)_frame_3);
	}
	else{
		gtk_widget_hide((GtkWidget *)_frame_3);
	}
	
	if(tab[3] != -1){
		gtk_widget_show((GtkWidget *)_frame_4);
	}
	else{
		gtk_widget_hide((GtkWidget *)_frame_4);
	}
}



tensor_analysis * new_tensor_analysis(int number_of_tensor, int array_size){
	int i;
	int array_total;
	tensor_analysis * t_a;
	
	t_a = (tensor_analysis * ) malloc(sizeof(tensor_analysis));
	t_a->number_of_tensor = number_of_tensor;
	t_a->array_size = array_size;
	array_total = number_of_tensor * 9;
	
	//LABEL can't be longer than 5 (or change in error_analysis.h")
	sprintf(t_a->label[0], "Ax");
	sprintf(t_a->label[1], "Rh");
	sprintf(t_a->label[2], "x ");
	sprintf(t_a->label[3], "y ");
	sprintf(t_a->label[4], "z ");
	sprintf(t_a->label[5], "a ");
	sprintf(t_a->label[6], "b ");
	sprintf(t_a->label[7], "g ");
    sprintf(t_a->label[8], "c ");
	
	for(i = 0; i < 9; i++){
		//t_a->mean[i] = (double *) calloc(array_total, sizeof(double));
	}
	t_a->parameters = (double **) calloc(array_total, sizeof(double *));
	for(i = 0; i < array_total; i++){
		t_a->parameters[i] = (double * )calloc(array_size, sizeof(double));
	}
	return(t_a);
}

void free_tensor_analysis(tensor_analysis * t_a){
	int i;
	int array_total;
	int array_size;
	int number_of_tensor;
	
	array_size = t_a->array_size;
	number_of_tensor = t_a->number_of_tensor;
	array_total = number_of_tensor * 9;
	
	for(i = 0; i < 9; i++){
		//free(t_a->mean[i]);
		//t_a->mean[i] = NULL;
	}
	
	for(i = 0; i < array_total; i++){
		free(t_a->parameters[i]);
		t_a->parameters[i] = NULL;
	}
	free(t_a->parameters);
	t_a->parameters = NULL;
	
	free(t_a);
}

void store_tensor_analysis(tensor_analysis *t_a, int tensor_num, int position, 
						   tensor * my_tensor_loc, double c)
{

	int number_of_tensor;
	int array_size;
	molmol_data mm_data;
	number_of_tensor = t_a->number_of_tensor;
	array_size = t_a->array_size;
	
	if(number_of_tensor < tensor_num){
		fprintf(stderr, "error with tensor_analyis, number_of_tensor < tensor_num\n");
		return;
	}
	if(array_size < position){
		fprintf(stderr, "error with tensor_analyis, array_size < position\n");
		return;
	}
	
	/////////////CONVERTION UTR/////////////////
	mm_data.Ax = my_tensor_loc->ax;
	mm_data.Rh = my_tensor_loc->rh;
	mm_data.X = my_tensor_loc->x;
	mm_data.Y = my_tensor_loc->y;
	mm_data.Z = my_tensor_loc->z;
	mm_data.a = my_tensor_loc->a;
	mm_data.b = my_tensor_loc->b;
	mm_data.g = my_tensor_loc->g;
	unique_tensor(&mm_data);
	my_tensor_loc->ax=   mm_data.Ax ;
	my_tensor_loc->rh=   mm_data.Rh ;
	my_tensor_loc->x =    mm_data.X ; 
	my_tensor_loc->y =    mm_data.Y ; 
	my_tensor_loc->z =    mm_data.Z ; 
	my_tensor_loc->a =    mm_data.a ; 
	my_tensor_loc->b =    mm_data.b ; 
	my_tensor_loc->g =    mm_data.g ; 
	
	/////////////END CONVERTION UTR/////////////////
	
	
	t_a->parameters[(tensor_num -1)*9 + 0 ][position] = my_tensor_loc->ax;
	t_a->parameters[(tensor_num -1)*9 + 1 ][position] = my_tensor_loc->rh;
	t_a->parameters[(tensor_num -1)*9 + 2 ][position] = my_tensor_loc->x;
	t_a->parameters[(tensor_num -1)*9 + 3 ][position] = my_tensor_loc->y;
	t_a->parameters[(tensor_num -1)*9 + 4 ][position] = my_tensor_loc->z;
	t_a->parameters[(tensor_num -1)*9 + 5 ][position] = my_tensor_loc->a;
	t_a->parameters[(tensor_num -1)*9 + 6 ][position] = my_tensor_loc->b;
	t_a->parameters[(tensor_num -1)*9 + 7 ][position] = my_tensor_loc->g;
  t_a->parameters[(tensor_num -1)*9 + 8 ][position] = c;
	
}

void statistic_tensor_analysis(tensor_analysis * t_a){
	int i, t;
	int number_of_tensor;
	number_of_tensor = t_a->number_of_tensor;
	
	for(t = 0; t < number_of_tensor; t++){
		fprintf(stderr, "************tensor***********\n");
		for(i = 0; i < 9; i++){
			fprintf(stderr, "%s mean : %lf\n", t_a->label[i], gsl_stats_mean(t_a->parameters[t*9+i], 1, t_a->array_size));
		}
		fprintf(stderr, "\n");
		for(i = 0; i < 9; i++){
			fprintf(stderr, "%s standard deviation : %lf\n", t_a->label[i], gsl_stats_sd(t_a->parameters[t*9+i], 1, t_a->array_size));
		}
		fprintf(stderr, "\n");
		for(i = 0; i < 9; i++){
			fprintf(stderr, "%s absolute deviation : %lf\n", t_a->label[i], gsl_stats_absdev(t_a->parameters[t*9+i], 1, t_a->array_size));
		}
		fprintf(stderr, "\n");
	}
}

void free_sanson(void){
    int i, j;
    for(i = 0; i < 3; i++){
        if(_sanson[i] != NULL){
            for(j = 0; j < _number_of_tensor; j++){
                if(_sanson[i][j]!= NULL){
                    free(_sanson[i][j]);
                    _sanson[i][j] = NULL;
                }
            }
            free(_sanson[i]);
            _sanson[i] = NULL;
        }
    }
    _number_of_tensor = -1;
	_array_size = -1;
}

void start_sanson(void){
	gtk_widget_show(_window_sanson);
}

void prepare_sanson(tensor_analysis *t_a){
    
    free_sanson();
	_number_of_tensor = t_a->number_of_tensor;
	_array_size = t_a->array_size;
    int i, j, k;

    for(i = 0; i < 3; i++){
        _sanson[i] = calloc(_number_of_tensor, sizeof(double*));
        for(j = 0; j< _number_of_tensor; j++){
            _sanson[i][j] = calloc(_array_size, sizeof(double));
            for(k =0; k < _array_size; k++){
                _sanson[i][j][k] = t_a->parameters[j*9 + 5+i ][k];
            }
        }
    }
}

void cancel_sanson(void){
	gtk_widget_hide(_window_sanson);
    free_sanson();
}

//Print the Sanson Map world into a file
void write_sanson_world(FILE * F){
    int i, j;

    for(i = 0; i <= 180; i+=20){
        for(j = -90; j <= 90; j+=10){
            fprintf(F, "% 4d\t% 4d\n", i, j);
        }
        fprintf(F, "\n");
    }
    //fprintf(F, "\n");
    for(i = -20; i >= -180; i-=20){
        for(j = -90; j <= 90; j+=10){
            fprintf(F, "% 4d\t% 4d\n", i, j);
        }
        fprintf(F, "\n");
    }
    //fprintf(F, "\n");
    for(i = 0; i <= 80; i+=10){
        for(j = -180; j <= 180; j+=20){
            fprintf(F, "% 4d\t% 4d\n", j, i);
        }
        fprintf(F, "\n");
    }
    //fprintf(F, "\n");
    fprintf(F, "% 4d\t% 4d\n", 0, 90);
    fprintf(F, "% 4d\t% 4d\n", 0, 90);
    fprintf(F, "\n");
    for(i = -10; i >= -80; i-=10){
        for(j = -180; j <= 180; j+=20){
            fprintf(F, "% 4d\t% 4d\n", j, i);
        }
        fprintf(F, "\n");
    }
    fprintf(F, "\n");
    fprintf(F, "% 4d\t% 4d\n", 0, -90);
    fprintf(F, "% 4d\t% 4d\n", 0, -90);
}

double heavy (double x){
    if(x <= 0.0){
        return(0.0);
    }
    else{
        return(1.0);
    }
}

double sing( double x){
    if(x < 0){
        return(-1.0);
    }
    else{
        return(1.0);
    }
}

void lat_and_lon_from_coor(double x, double y, double z, double * lat, double * lon){
    double la, lo;
    
    la = M_PI/2.0 -acos(z);
    lo = atan2(y, x);
    la = la / M_PI * 180.0;
    lo = lo / M_PI * 180.0;
    
    
    *lat = la;
    *lon = lo;
}

void euler_2_sanson(double a, double b, double g, double coordinate[6], int type){
    int i, j;
    double R[3][3];
    double V[3][3];
    double C[3];
    double lat, lon;
    
    filZX_or_YZ(R, a, b, g);
    for(i = 0; i < 3; i++){
        for(j = 0; j < 3; j++){
            V[i][j]=0;
        }
    }
    for(i = 0; i < 3; i++){
        V[i][i] = 1.0 * type;
    }
    
    for(i = 0; i < 3; i++){
        //C[0] = R[0][0]*V[i][0] + R[0][1]*V[i][1] + R[0][2]*V[i][2];
        //C[1] = R[1][0]*V[i][0] + R[1][1]*V[i][1] + R[1][2]*V[i][2];
        //C[2] = R[2][0]*V[i][0] + R[2][1]*V[i][1] + R[2][2]*V[i][2];
        //I use the transpose because I want the new coordinate of
        //the tensor in the same reference.
        C[0] = R[0][0]*V[i][0] + R[1][0]*V[i][1] + R[2][0]*V[i][2];
        C[1] = R[0][1]*V[i][0] + R[1][1]*V[i][1] + R[2][1]*V[i][2];
        C[2] = R[0][2]*V[i][0] + R[1][2]*V[i][1] + R[2][2]*V[i][2];
        lat_and_lon_from_coor(C[0], C[1], C[2], &lat, &lon);
        coordinate[2*i+0] = lon;
        coordinate[2*i+1] = lat;
    }
    
}

void write_sanson_coord(FILE * F){
    
    int i, j, k;
    double coordinate[6];
    
    for(i = 0; i < 3; i++){
        if(_sanson[i] == NULL){
            return;
        }
    }
    if(_number_of_tensor == -1){
        return;
    }
    if(_array_size == -1){
        return;
    }
    
    //fprintf(F, "\n");
    
    for(k =0; k < _array_size; k++){
        for(j = 0; j< _number_of_tensor; j++){
            for(i = 0; i < 3; i++){
                if(_sanson[i][j] == NULL){
                    continue;
                }
            }
            euler_2_sanson(_sanson[0][j][k], _sanson[1][j][k], _sanson[2][j][k],
                           coordinate, 1);
            fprintf(F, "% 9.3lf % 9.3lf % 9.3lf % 9.3lf % 9.3lf % 9.3lf ",
                    coordinate[0], coordinate[1], coordinate[2],
                    coordinate[3], coordinate[4], coordinate[5]);
        }
        fprintf(F, "\n");
        for(j = 0; j< _number_of_tensor; j++){
            for(i = 0; i < 3; i++){
                if(_sanson[i][j] == NULL){
                    continue;
                }
            }
            euler_2_sanson(_sanson[0][j][k], _sanson[1][j][k], _sanson[2][j][k],
                           coordinate, -1);
            fprintf(F, "% 9.3lf % 9.3lf % 9.3lf % 9.3lf % 9.3lf % 9.3lf ",
                    coordinate[0], coordinate[1], coordinate[2],
                    coordinate[3], coordinate[4], coordinate[5]);
        }
        fprintf(F, "\n");
    }
}

void write_sanson_mac(FILE * F, char * name_plot, char * name_world, int number_of_tensor){
    int i;
    
    if(number_of_tensor == -1){
        return;
    }

    fprintf(F, "reset\n");
    fprintf(F, "set noxtics\n");
    fprintf(F, "set noytics\n");
    fprintf(F, "set angles degrees\n");
    fprintf(F, "set nogrid\n");
    fprintf(F, "set noborder\n");
    //fprintf(F, "width=40\n");
    //fprintf(F, "height=width*(3./4.)\n");
    //fprintf(F, "set xrange [-width : width]\n");
    //fprintf(F, "set yrange [-height : height]\n");
    //fprintf(F, "drad=pi/180.\n");
    fprintf(F, "set title \"Sanson-Flamsteed Equivalent Projection\"\n");
    //fprintf(F, "ap=12\n");
    fprintf(F, "long0=0\n");
    //fprintf(F, "XC(lat,lon)=ap*lat*drad\n");
    //fprintf(F, "YC(lat,lon)=(lon)*ap*cos(lat)*drad\n");
    fprintf(F, "XC(lat,lon)=lat\n");
    fprintf(F, "YC(lat,lon)=(lon-long0)*cos(lat)\n");
    fprintf(F, "plot '%s'  using (YC($2,$1)):(XC($2,$1)) t '' with lines lw 0.1 lt -1", name_world);
    for(i = 0; i < number_of_tensor; i++){
        fprintf(F, ", '%s' using (YC($%d,$%d)):(XC($%d,$%d)) t '' with points %d", name_plot,
                6*i+2, 6*i+1, 6*i+2, 6*i+1, 3*i+1);
        fprintf(F, ", '%s' using (YC($%d,$%d)):(XC($%d,$%d)) t '' with points %d", name_plot,
                6*i+4, 6*i+3, 6*i+4, 6*i+3, 3*i+2);
        fprintf(F, ", '%s' using (YC($%d,$%d)):(XC($%d,$%d)) t '' with points %d", name_plot,
                6*i+6, 6*i+5, 6*i+6, 6*i+5, 3*i+3);
    }
    fprintf(F, "\n");
    //fprintf(F, "pause 2\n");
    fprintf(F, "pause -1\n");
    //fprintf(F, "quit\n");
}

void view_sanson(void){
    FILE * F_sanson_plot;
    FILE * F_sanson_mac;
    FILE * F_world_plot;
    static gint  gnu_stdin = -1;
    gint  gnu_stdout;
    gint  gnu_stderr;
    GtkEntry * entry;
	GladeXML *gxml; 
    gboolean result_bool;
    GPid pid;
    gchar *argv[2];
    GError * error = NULL;
    
    char prog[1500];
    
    char * name_F_sanson_plot = "./sanson_plot.txt";
    char * name_F_sanson_mac = "./sanson_plot.gnuplot";
    char * name_F_world_plot = "./world.txt";
    
    if (!(F_sanson_plot = g_fopen(name_F_sanson_plot , "w"))) {
        fprintf(stderr, "can't open %s\n", name_F_sanson_plot);
        return;
    }
    
    if (!(F_sanson_mac = g_fopen(name_F_sanson_mac , "w"))) {
        fprintf(stderr, "can't open %s\n", name_F_sanson_mac);
        return;
    }
    
    if (!(F_world_plot = g_fopen(name_F_world_plot , "w"))) {
        fprintf(stderr, "can't open %s\n", name_F_world_plot);
        return;
    }
    
    gtk_widget_hide(_window_sanson);
    
    write_sanson_coord(F_sanson_plot);
    write_sanson_mac(F_sanson_mac, name_F_sanson_plot, name_F_world_plot, _number_of_tensor);
    write_sanson_world(F_world_plot);
    
    fclose(F_sanson_plot);
    fclose(F_sanson_mac);
    fclose(F_world_plot);
    
    fprintf(stderr, "trying to launch gnuplot with the command:\n");
	
	gxml = glade_get_widget_tree(_window_sanson);
  	entry = (GtkEntry *) glade_xml_get_widget (gxml, "entry_gnuplot"); 

    sprintf(prog, gtk_entry_get_text(entry));
/* //NOT COMPATIBLE WITH WINDOWS
    if ( !(fpipe = (FILE*)popen(com,"r")) ) {  // If fpipe is NULL
        fprintf(stderr, "Problems with pipe\n");
        exit(1);
    }
*/
    //////////
    argv[0] = prog;
    argv[1] = name_F_sanson_mac;
    argv[2] = NULL;
    fprintf(stderr, "%s %s", argv[0], argv[1]);
    
    if(gnu_stdin != -1){
        close(gnu_stdin);
    }
    result_bool =  g_spawn_async_with_pipes(
                        NULL,
                        argv,
                        NULL,
                        G_SPAWN_SEARCH_PATH,
                        NULL,
                        NULL,
                        &pid,
                        &gnu_stdin,
                        &gnu_stdout,
                        &gnu_stderr,
                        &error);

    g_spawn_close_pid(pid);      
    if(result_bool == TRUE){
        close(gnu_stderr);
        close(gnu_stdout);
        //close(gnu_stdin);
    }
    
    if (error != NULL){
        fprintf (stderr, "Error message: %s\n", error->message);
        g_error_free (error);
    } 
    else{
        
    }
    return;
    
    free_sanson();
}

void save_sanson(GtkButton * button){
    
    GtkWidget *dialog;
  	GtkFileFilter *filter1;
    GtkFileFilter *filter4;
  	gint result;

  dialog =
    gtk_file_chooser_dialog_new
    ("In which file do you want to save your gnuplot macro?", NULL,
     GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
		
	gtk_window_set_icon_from_file ((GtkWindow *)dialog,
                                  NUMBAT_ICON_FILE,
                                  NULL);
  gtk_window_set_transient_for((GtkWindow*) dialog, (GtkWindow*)_window_sanson);

  filter1 = gtk_file_filter_new ();
  filter4 = gtk_file_filter_new ();
  gtk_file_filter_add_pattern (filter1, "*.gnuplot");
  gtk_file_filter_set_name (filter1, "*.macro for gnuplot");
  gtk_file_filter_add_pattern (filter4, "*.*");
  gtk_file_filter_set_name (filter4, "*.* ALL");
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter1);
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter4);

  result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
    {
    case GTK_RESPONSE_ACCEPT:
      {
        char * filename;
        FILE * F_sanson_plot;
        FILE * F_sanson_mac;
        FILE * F_world_plot;
        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
          
        char name_F_sanson_plot[1000];
        char name_F_sanson_mac[1000];
        char name_F_world_plot[1000];
    
        sprintf(name_F_sanson_mac, filename);
        sprintf(name_F_sanson_plot, "%s%s", filename, ".txt");
        sprintf(name_F_world_plot,  "%s%s", filename, ".world.txt");
          
        if (!(F_sanson_plot = g_fopen(name_F_sanson_plot , "w"))) {
            fprintf(stderr, "can't open %s\n", name_F_sanson_plot);
            return;
        }
    
        if (!(F_sanson_mac = g_fopen(name_F_sanson_mac , "w"))) {
            fprintf(stderr, "can't open %s\n", name_F_sanson_mac);
            return;
        }
    
        if (!(F_world_plot = g_fopen(name_F_world_plot , "w"))) {
            fprintf(stderr, "can't open %s\n", name_F_world_plot);
            return;
        }
    
        gtk_widget_hide(_window_sanson);
    
        write_sanson_coord(F_sanson_plot);
        write_sanson_mac(F_sanson_mac, name_F_sanson_plot, name_F_world_plot, _number_of_tensor);
        write_sanson_world(F_world_plot);
        fclose(F_sanson_plot);
        fclose(F_sanson_mac);
        fclose(F_world_plot);
        g_free (filename);
	  }
      break;
    default:
      break;
    }
  gtk_object_destroy((GtkObject *)filter1);
  gtk_object_destroy((GtkObject *)filter4);
  gtk_widget_destroy (dialog); 
    
  gtk_widget_hide(_window_sanson);
  free_sanson();
}


