
/*
 * error_analysis.h
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

typedef struct tensor_analysis {
	int number_of_tensor; // max N_PCS_ARRAY
	int array_size;
	char label[9][5]; // = {"Ax", "Rh", "x", "y", "z", "a", "b", "g", "c"};
	double ** parameters; //order ax, rh, x, y, z, a, b, g, c
	//double * mean[9]; //order: ax, rh, x, y, z, a, b, g, c
} tensor_analysis;

typedef struct error_control_data {
    double noise_structure;
    double noise_data_PCS[N_PCS_ARRAY];
	double subset_percentage;
    gboolean use_data_PCS[N_PCS_ARRAY];
	int n_sample;
	int seed;
} error_control_data;

GSList* _tensor_valid[N_PCS_ARRAY];
GSList * _multi_tensor_valid;

void print_tensor_valid(void);
void append_tensor_valid(int n_button, int serial_tensor);
void clear_tensor_valid(int n_button);
void start_error_analysis(GtkButton * button);
void ok_error_analysis(GtkButton * button);
void cancel_error_analysis(void);
void init_error_analysis(void);
void add_manual_error_analysis(void);
void copy_error_analysis(void);
void remove_error_analysis(void);
gboolean is_tensor_valid(int serial, int ** tab_ret);
gboolean is_tensor_single_valid(int serial);
void append_multi_tensor_valid( int n_button, int serial_tensor, int is_first);
gboolean is_tensor_multi_valid(int serial, int ** tab_ret);
void change_view_frame(int *tab);

tensor_analysis * new_tensor_analysis(int number_of_tensor, int array_size);
void free_tensor_analysis(tensor_analysis * t_a);
void store_tensor_analysis(tensor_analysis *t_a, int tensor_num, int position, 
						   tensor * my_tensor_loc, double c);
void statistic_tensor_analysis(tensor_analysis * t_a);


void euler_2_sanson(double a, double b, double g, double coordinate[6], int type);
void cancel_sanson(void);
void view_sanson(void);
void save_sanson(GtkButton * button);
void prepare_sanson(tensor_analysis *t_a);
void start_sanson(void);
void write_sanson_world(FILE * F);
void write_sanson_mac(FILE * F, char * name_plot, char * name_world, int number_of_tensor);
