/*
 * help.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <math.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>
#include <glib/gstdio.h>

#include "pdb.h"
#include "methyl.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "tensor_list.h"
#include "common.h"
#include "error_analysis.h"
#include "pcsfit_metal.h"
#include "molmol_iso.h"
#include "unify_tensor.h"
#include "rdc.h"
#include "help.h"
#include "help_tip.h"

#define N_HELP_FILE 15
static char *help_file_widget[N_HELP_FILE * 2] = {
		"textview_help_display", "display_help.txt",
		"textview_help_select", "select_help.txt",
		"textview_help_generate", "generate_help.txt",
		"textview_help_CH3", "methyl_group_help.txt",
		"textview_help_RACS", "racs_help.txt",
		"textview_help_RDC", "save_rdc_help.txt",
		"textview_help_molmol", "iso_molmol_help.txt",
		"textview_help_pymol", "iso_pymol_help.txt",
		"textview_help_error_analysis", "error_analysis_help.txt",
		"textview_help_save_tensor", "save_tensor_help.txt",
		"textview_help_general", "general_help.txt",
		"textview_help_input_data", "input_data_help.txt",
		"textview_help_tensor_fit", "tensor_fit_help.txt",
		"textview_help_error_result", "error_result_help.txt",
		"textview_help_more", "more_help.txt"
};

#define N_OF_TOOL_TIP  123 //MUST BE equal to number of element diveded by 2 in next list
static char *tool_tip_widget[N_OF_TOOL_TIP * 2] = {
	"Load_PDB", pdb_tip,
	"Load_PCS1", pcs_tip,
	"Load_PCS2", pcs_tip,
	"Load_PCS3", pcs_tip,
	"Load_PCS4", pcs_tip,
	"Generate_PCS1", generate_tip,
	"Generate_PCS2", generate_tip,
	"Generate_PCS3", generate_tip,
	"Generate_PCS4", generate_tip,
	"Define_CH3_COMMON", define_ch3_tip,
	"Select_PCS1", select_tip,
	"Select_PCS2", select_tip,
	"Select_PCS3", select_tip,
	"Select_PCS4", select_tip,
	"Display_PCS1", display_tip,
	"Display_PCS2", display_tip,
	"Display_PCS3", display_tip,
	"Display_PCS4", display_tip,
	"Save_PCS1", save_pcs_tip,
	"Save_PCS2", save_pcs_tip,
	"Save_PCS3", save_pcs_tip,
	"Save_PCS4", save_pcs_tip,
	"Save_RDC", save_rdc_tip,
	"Visualize_MolMol", view_molmol_tip,
	"Visualize_Pymol", view_pymol_tip,
	"Error_analasys", error_analysis_tip,
	"Calculate_tensor_PCS1", calculate_tensor_tip,
	"Calculate_tensor_PCS2", calculate_tensor_tip,
	"Calculate_tensor_PCS3", calculate_tensor_tip,
	"Calculate_tensor_PCS4", calculate_tensor_tip,
    "calculate_tensor_multiple_PCS", calculate_tensor_multi_tip,
	"Save_tensor", save_tensor_tip,
	"all_pdb_selector_PCS1", all_pdb_selector_tip,
	"all_pdb_selector_PCS2", all_pdb_selector_tip,
	"all_pdb_selector_PCS3", all_pdb_selector_tip,
	"all_pdb_selector_PCS4", all_pdb_selector_tip,
	"all_pdb_selector_PCS_MULTI", all_pdb_selector_tip,
	"none_pdb_selector_PCS1", none_pdb_selector_tip,
	"none_pdb_selector_PCS2", none_pdb_selector_tip,
	"none_pdb_selector_PCS3", none_pdb_selector_tip,
	"none_pdb_selector_PCS4", none_pdb_selector_tip,
	"none_pdb_selector_PCS_MULTI", none_pdb_selector_tip,
	"more_PCS1", more_tip,
	"more_PCS2", more_tip,
	"more_PCS3", more_tip,
	"more_PCS4", more_tip,
  "more_multi_PCS", more_tip,
  "event_PCS_M", pcs_multi_tab_tip,
  "Define_RACS", define_RACS_tip,
  "button_auto_CH3",auto_name_CH3_tip,
  "button_auto_RACS", auto_name_RACS_tip,
  "all_residue_select", select_all_residue_tip,
  "none_residue_select", select_none_residue_tip,
  "all_atom_select", select_all_atom_tip,
  "none_atom_select", select_none_atom_tip,
  "all_residue_display", select_all_residue_tip,
  "none_residue_display", select_none_residue_tip,
  "all_atom_display", select_all_atom_tip,
  "none_atom_display", select_none_atom_tip,
  "Add_CH3", add_CH3_tip,
  "Remove_CH3", remove_CH3_tip,
  "OK_CH3",apply_CH3_tip,
  "Cancel_CH3",cancel_CH3_tip,
	"button_ok_RACS", apply_RACS_tip,
	"button_cancel_RACS", cancel_RACS_tip,
	"add_tl" ,add_tensor_list_tip,
	"add_tl2",	add_tensor_list_tip,
	"add_tl3",	add_tensor_list_tip,
	"add_tl4",	add_tensor_list_tip,
	"add_tl5",	add_tensor_list_tip,
	"add_tl7",	add_tensor_list_tip,
	"add_tl8",	add_tensor_list_tip,
	"remove_tl", remove_tensor_list_tip,
	"remove_tl2", remove_tensor_list_tip,
	"remove_tl3", remove_tensor_list_tip,
	"remove_tl4", remove_tensor_list_tip,
	"remove_tl5", remove_tensor_list_tip,
	"remove_tl7", remove_tensor_list_tip,
	"remove_tl8", remove_tensor_list_tip,
	"copy_tl", copy_tensor_list_tip,
	"copy_tl2", copy_tensor_list_tip,
	"copy_tl3", copy_tensor_list_tip,
	"copy_tl4", copy_tensor_list_tip,
	"copy_tl5", copy_tensor_list_tip,
	"copy_tl7", copy_tensor_list_tip,
	"copy_tl8", copy_tensor_list_tip,
	"OK_tensor_listing", ok_generate_tensor_tip,
	"Cancel_tensor_listing", cancel_generate_tensor_tip,
	"button_auto_set_up", molmol_auto_set_up_tip,
	"button_auto_set_up2", pymol_auto_set_up_tip,
	"save_vm", save_macro_molmol_tip,
 	"OK_tensor_listing5", call_molmol_tip,
	"Cancel_tensor_listing5", cancel_molmol_tip,
	"save_vp", save_macro_pymol_tip,
	"OK_tensor_listing2", call_pymol_tip, 
	"Cancel_tensor_listing2", cancel_pymol_tip,
	"help_input_data", help_input_data_tip,
	"help_tensor_fit", help_tensor_fit_tip,
	"SAVE_RDC_button", ok_save_rdc_tip,
	"CANCEL_RDC_button" ,cancel_save_rdc_tip,
	"apply_display", apply_display_tip,
	"cancel_display", cancel_display_tip,
	"apply_select", apply_select_tip,
	"cancel_select", cancel_select_tip,
	"OK_error_analysis",	ok_error_analysis_tip,
	"CANCEL_error_analysis",	cancel_error_analysis_tip,
	"save_for_sanson",	save_tensor_sanson_tip,
	"save_for_txt",	save_tensor_txt_tip,
	"save_for_pdb",	save_tensor_pdb_tip,
	"Cancel_save_tensor",	cancel_save_tensor_tip,
	"save_sanson", error_analysis_save_sanson_tip,
	"view_sanson", error_analysis_view_sanson_tip,
	"Cancel_sanson", error_analysis_cancel_tip,
	"button_running_ok", running_ok_tip,
	"button_stop", running_stop_tip,
	"eventbox_input_PCS1", input_PCS1_tip,
	"eventbox_input_PCS2", input_PCS2_tip,
	"eventbox_input_PCS3", input_PCS3_tip,
	"eventbox_input_PCS4", input_PCS4_tip,
	"eventbox_tensor_PCS1", tensor_PCS1_tip,
	"eventbox_tensor_PCS2", tensor_PCS2_tip,
	"eventbox_tensor_PCS3", tensor_PCS3_tip,
	"eventbox_tensor_PCS4", tensor_PCS4_tip
};

extern GtkWidget * _window;
extern GtkWidget * _window_help_tensor_fit;
extern GtkWidget * _window_help_input_data;
extern GtkWidget * _window_help_general;
GtkTooltips* _all_tool;


void init_menu_help(GladeXML *gxml){
	GtkWidget* menu_check;
	GtkWidget* menu_help;

	menu_check = gtk_check_menu_item_new_with_label("Show bubble help");
	menu_help = glade_xml_get_widget (gxml, "menu5");
	gtk_check_menu_item_set_active((GtkCheckMenuItem *)menu_check, TRUE);
	gtk_menu_shell_append((GtkMenuShell *) menu_help, (GtkWidget *) menu_check);
	gtk_widget_show(menu_check);
	g_signal_connect (menu_check, "toggled", G_CALLBACK (swap_bubble), NULL);
	
	_all_tool = gtk_tooltips_new();
	set_bubble_tool_tip(gxml);
	gtk_tooltips_enable(_all_tool);
}


void set_bubble_tool_tip(GladeXML *gxml){
	int i;

	GtkWidget* widge;

	for(i = 0; i < N_OF_TOOL_TIP; i++){
		widge = glade_xml_get_widget (gxml, tool_tip_widget[2*i]);
		gtk_tooltips_set_tip (_all_tool, widge, tool_tip_widget[i*2+1]  ,NULL);
	}
}

void swap_bubble (GtkCheckMenuItem *checkmenuitem){
	gboolean is_checked;
	is_checked = gtk_check_menu_item_get_active(checkmenuitem);
	fprintf(stderr, "swap_bublle funciont\n");
	if(is_checked){
        fprintf(stderr, "trying to enable bubble help\n");
		gtk_tooltips_enable(_all_tool);
	}
	else{
        fprintf(stderr, "trying to disable bubble help\n");
		gtk_tooltips_disable(_all_tool);
	}
}

int replace_tag(GtkTextBuffer* buffer, char * oldtext, char * newtext){
		gboolean found;
		GtkTextIter iter;
		GtkTextIter match_start;
		GtkTextIter match_end;
		GtkTextIter limit;
		GtkTextSearchFlags flags;
		
		flags = GTK_TEXT_SEARCH_TEXT_ONLY;
		gtk_text_buffer_get_iter_at_offset(buffer,&iter,0);
  	gtk_text_buffer_get_iter_at_offset(buffer,&limit,-1);
		
		found = gtk_text_iter_forward_search (&iter, oldtext, flags, &match_start, &match_end, &limit);
		while(found == TRUE){

				gtk_text_buffer_delete(buffer, &match_start, &match_end);

				gtk_text_buffer_insert (buffer, &match_start, newtext, -1);

				gtk_text_buffer_get_iter_at_offset(buffer,&limit,-1);
				iter = match_start;
				found = gtk_text_iter_forward_search (&iter, oldtext, flags, &match_start, &match_end, &limit);
		}
		return(0);
}

int replace_tag_by_pict(GtkTextBuffer* buffer, char * filename)
{
		gboolean found, found2;
		GtkTextIter iter;
		GtkTextIter match_start1;
		GtkTextIter match_end1;
		GtkTextIter match_start2;
		GtkTextIter match_end2;
		GtkTextIter limit;
		GtkTextSearchFlags flags;
		GdkPixbuf*  pixbuf;
		pixbuf = NULL;
		
		flags = GTK_TEXT_SEARCH_TEXT_ONLY;
		gtk_text_buffer_get_iter_at_offset(buffer,&iter,0);
  	gtk_text_buffer_get_iter_at_offset(buffer,&limit,-1);
		found = gtk_text_iter_forward_search (&iter, "<pic>", flags, &match_start1, &match_end1, &limit);
		while(found == TRUE){
				found2 = gtk_text_iter_forward_search (&match_end1, "</pic>", flags, &match_start2, &match_end2, &limit);
				if (found2 == FALSE){
						fprintf(stderr, "Parsing error (<pic>) in the help file %s\n", filename);
						return(-1);
				}
				gchar *  str = gtk_text_buffer_get_text(buffer,&match_end1, &match_start2, FALSE);
				gchar *  filename = g_strconcat (NUMBAT_HELP_DIR, str, NULL);
				pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
				gtk_text_buffer_delete(buffer, &match_start1, &match_end2);
				if (pixbuf != NULL){
						gtk_text_buffer_insert_pixbuf (buffer, &match_start1, pixbuf);
				}
				else {
						gtk_text_buffer_insert (buffer, &match_start1, "<missing picture: \0", -1);
						gtk_text_buffer_insert (buffer, &match_start1, filename, -1);
						gtk_text_buffer_insert (buffer, &match_start1, ">\0", -1);
				}
				gtk_text_buffer_get_iter_at_offset(buffer,&limit,-1);
				
				if(filename != NULL){
					g_free(filename);
					filename = NULL;
				}
				if(str != NULL){
					g_free(str);
					str = NULL;
				}
				if(pixbuf != NULL){
					pixbuf = NULL;
				}
				iter = match_start1;
				found = gtk_text_iter_forward_search (&iter, "<pic>", flags, &match_start1, &match_end1, &limit);
		}
		return(0);
}

void init_help(GladeXML *gxml){
		int i;
		char * filename;
		FILE * file;
		GtkTextView * txt;
  	GtkTextIter iterstart;
		GtkTextIter iterend;
		GtkTextBuffer* buffer;
		char line[MAX_CAR];
		
    
		filename = NULL;
		file = NULL;
		int flag;
		flag = 0;
		
		for(i = 0; i < N_HELP_FILE; i++){
				txt = (GtkTextView *) glade_xml_get_widget (gxml, help_file_widget[2*i]);
				filename = g_strconcat (NUMBAT_HELP_DIR, help_file_widget[2*i+1], NULL);
				file = g_fopen (filename, "r");
				if(file == NULL){
						if(flag == 0){
							box_error("Couldn't open '%s'\nHelp might not be fully available\n\n", filename);
							flag = 1;
							continue;
						}
						else{
								fprintf(stderr, "Couldn't open '%s'\nHelp might not be fully available\n\n", filename);
								continue;
						}
				}
	
  			buffer = gtk_text_view_get_buffer(txt);
				
  			gtk_text_buffer_get_iter_at_offset(buffer,&iterstart,0);
  			gtk_text_buffer_get_iter_at_offset(buffer,&iterend,-1);
  			gtk_text_buffer_delete(buffer, &iterstart, &iterend);
				
				while(fgets (line, MAX_CAR, file) != NULL){
						gtk_text_buffer_insert(buffer, &iterstart, line, -1);
				}
				
				replace_tag_by_pict(buffer, filename);
				replace_tag(buffer, "<dX>", "\316\224\317\207\0");
				replace_tag(buffer, "<pi>", "\317\200\0");
				replace_tag(buffer, "<sigma>", "\317\203\0");
				replace_tag(buffer, "<alpha>", "\316\261\0");
				replace_tag(buffer, "<beta>", "\316\262\0");
				replace_tag(buffer, "<gamma>", "\316\263\0");
				replace_tag(buffer, "<sum>", "\316\243\0");
				
				if(file != NULL){
					fclose(file);
					file = NULL;
				}
				if(filename != NULL){
					g_free(filename);
					filename = NULL;
				}
		}
		
		
}

void help_tensor_fit(void)
{
		gtk_widget_show(_window_help_tensor_fit);
}

void help_input_data(void)
{
		gtk_widget_show(_window_help_input_data);
}

void help_general(void)
{
		gtk_widget_show(_window_help_general);
}

void end_help_tensor_fit(void)
{
		gtk_widget_hide(_window_help_tensor_fit);
}

void end_help_input_data(void)
{
		gtk_widget_hide(_window_help_input_data);
}

void end_help_general(void)
{
		gtk_widget_hide(_window_help_general);
}
