/*
 * main.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gtk/gtk.h>
#include <glade/glade.h>
#include "callbacks.h"
#include "pdb.h"
#include "common.h"
#include "methyl.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "select_display.h"
#include "tensor_list.h"
#include "molmol_iso.h"
#include "pymol_iso.h"
#include "notebook.h"
#include "tensor_found.h"
#include "rdc.h"
#include "error_analysis.h"
#include "help.h"
#include "save_tensor.h"


GtkWidget *_window_CH3;
GtkWidget *_window;
GtkWidget *_window_tensor_list;
GtkWidget *_window_iso_molmol;
GtkWidget *_window_iso_pymol;
GtkWidget *_window_tensor_work;
GtkWidget *_window_running;
GtkWidget *_window_rdc;
GtkWidget *_window_error_analysis;
GtkWidget *_window_sanson;
GtkWidget *_window_RACS;
GtkWidget *_window_save_tensor;
GtkWidget *_window_select;
GtkWidget *_window_display;
GtkWidget *_window_help_tensor_fit;
GtkWidget *_window_help_input_data;
GtkWidget *_window_help_general;

GladeXML *_gxml;

void init(GtkWidget *);

GtkWidget*
create_window (void)
{
	GtkWidget *window;
	GladeXML *gxml;

	gxml = NULL;
		
	gxml = glade_xml_new (GLADE_FILE, NULL, NULL);
		
	if(gxml == NULL){
			box_error("\nUnable to locate the file 'numbat.glade'.\n\nYou should try to copy the file 'numbat.glade' \
in its destination:\n\n\t%s\n\n\nTo controle the destination location of this file during installation,\n\
look at the option of the configure script\n(type './configure --help')\n\
and look for the option '--datadir'\nwhich control the installation path of the file numbat.glade\n",
								GLADE_FILE);
							
			exit (-1);
	}
	
	glade_xml_signal_autoconnect (gxml);
	window = glade_xml_get_widget (gxml, "window");
	_window = window;
	_gxml = gxml;
	return window;
}



int
main (int argc, char *argv[])
{
    if(OS == WIN32_OS){
        putenv("PATH=%PATH%;.\\dll"); 
    }
 	GtkWidget *window;

	gtk_set_locale ();
	gtk_init (&argc, &argv);
	if(OS == WIN32_OS){
        gtk_rc_parse (".\\share\\themes\\Clearlooks\\gtk-2.0\\gtkrc");
        putenv("GTK_PATH="); 
        fprintf(stderr, "OS : WIN32\n");
	}
#if OS != WIN32_OS
        fprintf(stderr, "NUMBAT_HELP_DIR: %s\n", NUMBAT_HELP_DIR);
        fprintf(stderr, "PACKAGE_DOC_DIR: %s\n", PACKAGE_DOC_DIR);
        fprintf(stderr, "PACKAGE_DATA_DIR: %s\n", PACKAGE_DATA_DIR);
#endif
        /*
    char * path_numbat;
    path_numbat = get_numbat_directory();
    if(path_numbat != NULL){
        fprintf(stderr, "PATH_NUMBAT: %s\n", path_numbat);
        g_free(path_numbat);
    }
*/
	window = create_window ();
	gtk_widget_show (window);
	
	init(window);

	gtk_main ();
	return 0;
}



void init_greck_letter_label(GladeXML *gxml){
    GtkLabel * label;
    char label_name[50];
    char txt[100];
    int i;
    for(i=1; i<8; i++){
        sprintf(label_name, "label_tensor_ax_%d", i);
        label = (GtkLabel *) glade_xml_get_widget (gxml, label_name);
        gtk_label_set_markup(label, STRING_DELTA_CHI_AX);
        
        sprintf(label_name, "label_tensor_rh_%d", i);
        label = (GtkLabel *) glade_xml_get_widget (gxml, label_name);
        gtk_label_set_markup(label, STRING_DELTA_CHI_RH);
        
        sprintf(label_name, "label_tensor_alpha_%d", i);
        label = (GtkLabel *) glade_xml_get_widget (gxml, label_name);
        gtk_label_set_markup(label, STRING_ALPHA);
        
        sprintf(label_name, "label_tensor_beta_%d", i);
        label = (GtkLabel *) glade_xml_get_widget (gxml, label_name);
        gtk_label_set_markup(label, STRING_BETA);
        
        sprintf(label_name, "label_tensor_gamma_%d", i);
        label = (GtkLabel *) glade_xml_get_widget (gxml, label_name);
        gtk_label_set_markup(label, STRING_GAMMA);
    }
    label = (GtkLabel *) glade_xml_get_widget (gxml, "label_sigma_tensor_list");
    gtk_label_set_markup(label, STRING_SIGMA);
    
    label = (GtkLabel *) glade_xml_get_widget (gxml, "label_gamma1");
    gtk_label_set_markup(label, STRING_GAMMA1_RDC);
    
    label = (GtkLabel *) glade_xml_get_widget (gxml, "label_gamma2");
    gtk_label_set_markup(label, STRING_GAMMA2_RDC);
    
    label = (GtkLabel *) glade_xml_get_widget (gxml, "label_temperature_RDC");
    gtk_label_set_markup(label, STRING_TEMPERATURE_KELVIN);
    
    label = (GtkLabel *) glade_xml_get_widget (gxml, "label_temperature_RACS");
    gtk_label_set_markup(label, STRING_TEMPERATURE_KELVIN);
		
    
    label = (GtkLabel *) glade_xml_get_widget (gxml, "label_sigma11");
    gtk_label_set_markup(label, STRING_CSA_SIGMA11);
    
    label = (GtkLabel *) glade_xml_get_widget (gxml, "label_sigma22");
    gtk_label_set_markup(label, STRING_CSA_SIGMA22);
    
    label = (GtkLabel *) glade_xml_get_widget (gxml, "label_sigma33");
    gtk_label_set_markup(label, STRING_CSA_SIGMA33);
    
    label = (GtkLabel *) glade_xml_get_widget (gxml, "label_beta_CSA");
    gtk_label_set_markup(label, STRING_CSA_BETA);
		
		label = (GtkLabel *) glade_xml_get_widget (gxml, "label_sigma_noise_str");
		gtk_label_set_markup(label, STRING_SIGMA);
		
		label = (GtkLabel *) glade_xml_get_widget (gxml, "label_sigma_noise_PCS1");
		gtk_label_set_markup(label, STRING_SIGMA);
		
		label = (GtkLabel *) glade_xml_get_widget (gxml, "label_sigma_noise_PCS2");
		gtk_label_set_markup(label, STRING_SIGMA);
		
		label = (GtkLabel *) glade_xml_get_widget (gxml, "label_sigma_noise_PCS3");
		gtk_label_set_markup(label, STRING_SIGMA);
		
		label = (GtkLabel *) glade_xml_get_widget (gxml, "label_sigma_noise_PCS4");
		gtk_label_set_markup(label, STRING_SIGMA);
        
    sprintf(txt, "%s:", STRING_DELTA_CHI_X);
        label = (GtkLabel *) glade_xml_get_widget (gxml, "label_name_Xx_TW1");
        gtk_label_set_markup (label, txt);
        label = (GtkLabel *) glade_xml_get_widget (gxml, "label_name_Xx_TW2");
        gtk_label_set_markup (label, txt);
        
	sprintf(txt, "%s:", STRING_DELTA_CHI_Y);	
        label = (GtkLabel *) glade_xml_get_widget (gxml, "label_name_Xy_TW1");
        gtk_label_set_markup (label, txt);
        label = (GtkLabel *) glade_xml_get_widget (gxml, "label_name_Xy_TW2");
        gtk_label_set_markup (label, txt);
        
    sprintf(txt, "%s:", STRING_DELTA_CHI_Z);
        label = (GtkLabel *) glade_xml_get_widget (gxml, "label_name_Xz_TW1");
        gtk_label_set_markup (label, txt);
        label = (GtkLabel *) glade_xml_get_widget (gxml, "label_name_Xz_TW2");
        gtk_label_set_markup (label, txt);
    
    sprintf(txt, "%s:", STRING_DELTA_CHI_AX);
        label = (GtkLabel *) glade_xml_get_widget (gxml, "label_name_Ax_TW1");
        gtk_label_set_markup (label, txt);
        label = (GtkLabel *) glade_xml_get_widget (gxml, "label_name_Ax_TW2");
        gtk_label_set_markup (label, txt);
    sprintf(txt, "%s:", STRING_DELTA_CHI_RH);
        label = (GtkLabel *) glade_xml_get_widget (gxml, "label_name_Rh_TW1");
        gtk_label_set_markup (label, txt);
        label = (GtkLabel *) glade_xml_get_widget (gxml, "label_name_Rh_TW2");
        gtk_label_set_markup (label, txt);
        
    
}


void
init(GtkWidget * window)
{
	GladeXML *gxml;	
    gxml = _gxml;

	//GtkWidget * window_about;
	_color_methyl.red = 45000;
	_color_methyl.blue = 65000;
	_color_methyl.green = 65000;
	_color_dumby.red = 6500;
	_color_dumby.blue = 6500;
	_color_dumby.green = 4500;
	
  init_pdb_structure (); //just to initialize the pdb_str correctly
    //window_about = glade_xml_get_widget (gxml, "window_about");
    _window_CH3 = glade_xml_get_widget (gxml, "window_CH3");
    _window_select = glade_xml_get_widget (gxml, "window_select");
    _window_display = glade_xml_get_widget (gxml, "window_display");
    _window_tensor_list = glade_xml_get_widget (gxml, "window_tensor_list");
    _window_iso_molmol = glade_xml_get_widget (gxml, "window_iso_molmol");
	_window_iso_pymol = glade_xml_get_widget (gxml, "window_iso_pymol");
	_window_tensor_work = glade_xml_get_widget (gxml, "window_tensor_work");
	_window_running = glade_xml_get_widget (gxml, "window_running");
	_window_rdc = glade_xml_get_widget (gxml, "window_rdc");
	_window_error_analysis = glade_xml_get_widget (gxml, "window_error_analysis");
	_window_sanson = glade_xml_get_widget(gxml, "window_sanson");
	_window_RACS = glade_xml_get_widget(gxml, "window_RACS");
    _window_save_tensor = glade_xml_get_widget(gxml, "window_save_tensor");
	_window_help_tensor_fit = glade_xml_get_widget(gxml, "window_help_tensor_fit");
	_window_help_input_data = glade_xml_get_widget(gxml, "window_help_input_data");
	_window_help_general = glade_xml_get_widget(gxml, "window_help_general");
    
  gtk_window_set_transient_for((GtkWindow*) _window_RACS, (GtkWindow*)_window);
	gtk_window_set_transient_for((GtkWindow*) _window_tensor_list, (GtkWindow*)_window);
	gtk_window_set_transient_for((GtkWindow*) _window_iso_molmol, (GtkWindow*)_window);
	gtk_window_set_transient_for((GtkWindow*) _window_iso_pymol, (GtkWindow*)_window);
	gtk_window_set_transient_for((GtkWindow*) _window_tensor_work, (GtkWindow*)_window);
	gtk_window_set_transient_for((GtkWindow*) _window_running, (GtkWindow*)_window);
	gtk_window_set_transient_for((GtkWindow*) _window_rdc, (GtkWindow*)_window);
	gtk_window_set_transient_for((GtkWindow*) _window_error_analysis, (GtkWindow*)_window);
	gtk_window_set_transient_for((GtkWindow*) _window_sanson, (GtkWindow*)_window);
  gtk_window_set_transient_for((GtkWindow*) _window_sanson, (GtkWindow*)_window_running);
  gtk_window_set_transient_for((GtkWindow*) _window_CH3, (GtkWindow*)_window);
  gtk_window_set_transient_for((GtkWindow*) _window_save_tensor, (GtkWindow*)_window);
  gtk_window_set_transient_for((GtkWindow*) _window_select, (GtkWindow*)_window);
  gtk_window_set_transient_for((GtkWindow*) _window_display, (GtkWindow*)_window);
	//GtkWindowGroup* GWG = gtk_window_group_new();
	//gtk_window_group_add_window(GWG, (GtkWindow *)window);
	//gtk_window_group_add_window(GWG, (GtkWindow *)_window_tensor_list);
	//gtk_window_group_add_window(GWG, (GtkWindow *)window);
		
	//WARNING, MOST OF THE FOLLOWING init call CAN'T BE REORDERED!
	init_tensor_listing();
	init_vm(); // AFTER init_tensor_listing
	init_vp(); // AFTER init_tensor_listing
	init_more_tensor();  // AFTER init_tensor_listing
	//gtk_widget_destroy(window_about); //
	init_array_CH3();
	init_PCS();
	init_select_display();
    set_title_window("");
	init_tensor();
	init_PDB_selector();
	display_boundaries_list();
	init_TF();
	display_matrix_init();
	display_matrix_multi_init();
	init_notebook();
	init_rdc();
	init_error_analysis();
	init_menu_help(gxml);
  init_racs();
  init_CH3();
  init_save_tensor();
    
  if(EULER_CONVENTION == ZXZ_CONVENTION){
  	fprintf(stderr, "ZXZ_CONVENTION in use\n");
  }
  if(EULER_CONVENTION == ZYZ_CONVENTION){   
		fprintf(stderr, "ZYZ_CONVENTION in use\n");
  }  
    
  init_greck_letter_label(gxml);
	init_help(gxml);
		

    
}
