/*
 * methyl.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "pdb.h"
#include "racs.h"
#include "pcs.h"
#include "methyl.h"
#include "common.h"
#include "tensor.h"
#include "error_analysis.h"

extern GtkWidget *_window_CH3;

#define N_CH3_NAME 12
static char *CH3_NAME[N_CH3_NAME * 3] = {
	"HE1", "HE2", "HE3", //MET
    "HB1", "HB2", "HB3", //ALA, 
    "HG11", "HG12", "HG13", //VAL,
    "HG21", "HG22", "HG23", //VAL, ILE, THR
    "HD11", "HD12", "HD13", //LEU, ILE
    "HD21", "HD22", "HD23", //LEU,
    "1HG1", "2HG1", "3HG1",
    "1HG2", "2HG2", "3HG2",
    "1HD1", "2HD1", "3HG1",
    "1HD2", "2HD2", "3HG2",
    "1HE", "2HE", "3HE",
    "1HB", "2HB", "3HB"
};



GArray *_articles_CH3; //*used by the GUI list
GArray *_str_CH3; //Where we save the methyl association

//~ to be called ONCE! by main.c at initialisation
void init_array_CH3(void){
    _articles_CH3 = NULL;
    _str_CH3 = NULL;
    _str_CH3 = g_array_sized_new (FALSE, FALSE, sizeof (structure_CH3), 1);

}

//~ if we clean / change the PDB file...
void clear_CH3(void){
    g_array_free(_str_CH3, TRUE);
    _str_CH3 = g_array_sized_new (FALSE, FALSE, sizeof (structure_CH3), 1);
}


gboolean is_equivalent(structure_CH3 * fooa, structure_CH3 * foob){
    char * a1, *a2, *a3, *a4;
    char * b1, *b2, *b3, *b4;
    int cmp;
    a1 = fooa->atom1;
    a2 = fooa->atom2;
    a3 = fooa->atom3;
    a4 = fooa->atom_pcs;
    b1 = foob->atom1;
    b2 = foob->atom2;
    b3 = foob->atom3;
    b4 = foob->atom_pcs;
    
    cmp = 0;
    
    if((strcmp(a1, b1) == 0)||(strcmp(a1, b2) == 0)||(strcmp(a1, b3) == 0)){
        cmp ++;
    }
    if((strcmp(a2, b1) == 0)||(strcmp(a2, b2) == 0)||(strcmp(a2, b3) == 0)){
        cmp ++;
    }
    if((strcmp(a3, b1) == 0)||(strcmp(a3, b2) == 0)||(strcmp(a3, b3) == 0)){
        cmp ++;
    }
    
    if((strcmp(a4, b4) == 0)&&(cmp == 3)){
        return (TRUE);
    }
    else{
        return (FALSE);
    }
}

//~ transffer information from GUI _articles_CH3 to data structure_CH3
//~ called by callback when user click on apply.
void save_CH3(void){
    int i, j;
    char * a1, *a2, *a3, *a4;
    GArray * my_str_CH3;
    gboolean my_b;
    structure_CH3 foo;
	
    
    if(check_validity_methyl_association() == FALSE){
        box_error("error\nThe methyl association you try\n\
to define is not valid\n", NULL);
        return;
    }
    
    my_str_CH3 = g_array_sized_new (FALSE, FALSE, sizeof (structure_CH3), 1);
    
    for(i = 0; i < _articles_CH3->len; i++){
        a1 = g_array_index (_articles_CH3, Item_CH3 , i).atom1;
        a2 = g_array_index (_articles_CH3, Item_CH3 , i).atom2;
        a3 = g_array_index (_articles_CH3, Item_CH3 , i).atom3;
        a4 = g_array_index (_articles_CH3, Item_CH3 , i).atom_pcs;
        if((strcmp(a1, "NONE") == 0) || (strcmp(a2, "NONE") == 0) ||
           (strcmp(a3, "NONE") == 0) || (strcmp(a4, "NONE") == 0)){
               continue;
        }
        strcpy(foo.atom1, a1);
        strcpy(foo.atom2, a2);
        strcpy(foo.atom3, a3);
        strcpy(foo.atom_pcs, a4);
        foo.intact = FALSE;

        g_array_append_vals (my_str_CH3, &foo, 1);
    }
    
    // We compare the two different structures to be able
    // to populate the 'intact' field
    // The intact field is used to know how to react with
    // redefinition of methyl association.
    for(i = 0; i < _str_CH3-> len; i++){
        for(j = 0; j < my_str_CH3->len; j++){
            my_b = is_equivalent(&g_array_index (my_str_CH3, structure_CH3 , j)
                                 ,&g_array_index (_str_CH3, structure_CH3 , i));
            if(my_b == TRUE){
                g_array_index (my_str_CH3, structure_CH3 , j).intact = my_b;
                continue;
            }
        }
    }
    
    //quick fix for a behaviour bug
    if(_str_CH3->len == 0){
        for(j = 0; j < my_str_CH3->len; j++){
            g_array_index (my_str_CH3, structure_CH3 , j).intact = TRUE;
        }
    }

    clear_CH3();
    _str_CH3 = my_str_CH3;
    
    gtk_widget_hide(_window_CH3);
    
	for(i = 0; i < N_PCS_ARRAY; i++){
        clear_tensor_valid(i);
        update_pcs_list(i, FALSE); //we don't want to overwrite
        display_pcs_list(i, -1);
    }
}

//~ read form data (structure_CH3) to store in GUI (_articles_CH3)
void
add_items_CH3 (void)
{
  Item_CH3 foo;
  int i;
  g_return_if_fail (_articles_CH3 != NULL);
  if (_str_CH3->len == 0){
    foo.atom1 = g_strdup ("NONE");
    foo.atom2 = g_strdup ("NONE");
    foo.atom3 = g_strdup ("NONE");
    foo.atom_pcs = g_strdup ("NONE");
    g_array_append_vals (_articles_CH3, &foo, 1);
  }
  else {
      for(i = 0; i < _str_CH3->len; i++){
        foo.atom1 = g_strdup( g_array_index (_str_CH3, structure_CH3, i).atom1);
        foo.atom2 = g_strdup( g_array_index (_str_CH3, structure_CH3, i).atom2);
        foo.atom3 = g_strdup( g_array_index (_str_CH3, structure_CH3, i).atom3);
        foo.atom_pcs = g_strdup( g_array_index (_str_CH3, structure_CH3, i).atom_pcs);
        g_array_append_vals (_articles_CH3, &foo, 1);
      }
  }
}

//model used to display the atom names
GtkTreeModel *
create_items_model_CH3 (void)
{
  gint i = 0;
  GtkListStore *model;
  GtkTreeIter iter;

  /* create array */
  _articles_CH3 = g_array_sized_new (FALSE, FALSE, sizeof (Item_CH3), 1);

  add_items_CH3 ();

  /* create list store */
  model = gtk_list_store_new (NUM_ITEM_COLUMNS_CH3, G_TYPE_STRING, G_TYPE_STRING,
                              G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);

  /* add items */
  for (i = 0; i < _articles_CH3->len; i++)
    {
      gtk_list_store_append (model, &iter);

      gtk_list_store_set (model, &iter,
                          COLUMN_ATOM1,
                          g_array_index (_articles_CH3, Item_CH3, i).atom1,
                          COLUMN_ATOM2,
                          g_array_index (_articles_CH3, Item_CH3, i).atom2,
                          COLUMN_ATOM3,
                          g_array_index (_articles_CH3, Item_CH3, i).atom3,
                          COLUMN_ATOM_PCS,
                          g_array_index (_articles_CH3, Item_CH3, i).atom_pcs,
                          -1);
    }

  return GTK_TREE_MODEL (model);
}

gboolean is_H_inside(char *name){
    int i;
    for(i = 0; i< strlen(name); i++){
        if (name[i] == 'H'){
            return(TRUE);
        }
    }   
    return(FALSE);
}

// Comparison function to sort according to alphabet
gint
compare_methyl_func (GtkTreeModel *model,
                          GtkTreeIter  *a,
                          GtkTreeIter  *b,
                          gpointer      userdata)
{
    gchar *name1, *name2;
    gboolean is_1_H, is_2_H;
    gtk_tree_model_get(model, a, 0, &name1, -1);
    gtk_tree_model_get(model, b, 0, &name2, -1);
    if(strcmp(name1, name2) == 0){
        return(0);
    }
    if(strcmp(name1, "NONE") == 0){
        return(-1);
    }
    if(strcmp(name2, "NONE") == 0){
        return(1);
    }
    is_1_H = is_H_inside(name1);
    is_2_H = is_H_inside(name2);
    if((is_1_H == TRUE)&&(is_2_H == TRUE)){ 
        return (strcmp(name1, name2));
    }
    if((is_1_H == FALSE)&&(is_2_H == FALSE)){ 
        return (strcmp(name1, name2));
    }
    if(is_1_H == TRUE){
        return(-1);
    }
    return(1);
}


void sort_methyl(GtkTreeSortable *sortable){

    gtk_tree_sortable_set_sort_func(sortable, 0, compare_methyl_func,
                                    GINT_TO_POINTER(0), NULL);
    gtk_tree_sortable_set_sort_column_id(sortable, 0, GTK_SORT_ASCENDING);
}


//model to display atom names choice
GtkTreeModel *
create_atom_list_model (void)
{

  gint i = 0;
  GtkListStore *model;
  GtkTreeIter iter;

  /* create list store */
  //model = gtk_list_store_new (NUM_ATOM_COLUMNS, G_TYPE_STRING, G_TYPE_INT);
  model = gtk_list_store_new (NUM_ATOM_COLUMNS, G_TYPE_STRING);

  gtk_list_store_append (model, &iter);
  gtk_list_store_set (model, &iter,
                        COLUMN_ATOM_TEXT, "NONE",
                        -1);
    for (i = 0; i < _pdb_str->n_atom_type; i++)
    {
      char str[MAX_ATOM_NAME_LENGTH+1];
      sprintf(str, "%s", _pdb_str->tab_atom_type[i]);
      if(strcmp(str, DUMMY_NAME) == 0){
          continue;
      }
      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter,
                          COLUMN_ATOM_TEXT, str,
                          -1);
    }

  sort_methyl((GtkTreeSortable*)model);
  return GTK_TREE_MODEL (model);
}

//model to display the possible import PCS

GtkTreeModel *
create_copy_from_model (int n_button)
{
  gint i = 0;
  GtkListStore *model;
  GtkTreeIter iter;

  model = gtk_list_store_new (NUM_ATOM_COLUMNS, G_TYPE_STRING);
  gtk_list_store_append (model, &iter);
  gtk_list_store_set (model, &iter,
                        COLUMN_ATOM_TEXT, "NONE",
                        -1);
    for (i = 0; i < N_PCS_ARRAY; i++)
    {
      char str_from[6];
      if(i == n_button){
          //continue;
      }
      sprintf(str_from, "PCS%d", i + 1);
      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter,
                          COLUMN_ATOM_TEXT, str_from,
                          -1);
    }
  return GTK_TREE_MODEL (model);
}


gboolean is_name_present_in_liststore(char * name, GtkListStore *model,
                             GtkTreeIter * iter)
{

    //GtkTreeIter iter;
    gboolean my_bool;
    char * name2;
    
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *) model, iter);
    while (my_bool){
        gtk_tree_model_get ((GtkTreeModel *)model, iter, 
                          COLUMN_ATOM_TEXT, &name2,
                          -1);
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *) model, iter);
        if(strcmp(name, name2) == 0){
            return(TRUE);
        }
        g_free(name2);
    }
    iter = NULL;
    return(FALSE);
}

//model to display PCS names choice
GtkTreeModel *
create_PCS_list_model (void)//int n_button)
{
    gint i = 0;
    int j;
    GtkListStore *model;
    GtkTreeIter iter; 
    GtkTreeIter iter_pos;
    model = gtk_list_store_new (NUM_ATOM_COLUMNS, G_TYPE_STRING);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter,
                        COLUMN_ATOM_TEXT, "NONE",
                        -1);
    for(j = 0; j < N_PCS_ARRAY; j++){
        if(_str_pcs[j] != NULL){
            for (i = 0; i < _str_pcs[j]->len; i++)
            {
                char str[MAX_METHYL_STR+1];
                sprintf(str, "%s", g_array_index (_str_pcs[j], experiment_pcs , i).atom_name);
                if(is_name_present_in_liststore(str, model, &iter_pos)){
                    continue;
                }
                gtk_list_store_append (model, &iter);
                gtk_list_store_set (model, &iter,
                            COLUMN_ATOM_TEXT, str,
                            -1);
            }
        }
    }
    sort_methyl((GtkTreeSortable*)model);
    return GTK_TREE_MODEL (model);
}

//~ when one click one add button
void
add_item_CH3 (void)
{
  Item_CH3 foo;
  GtkTreeIter iter;
  GtkTreeView * treeview;
  GladeXML *gxml;
  GtkTreeModel *model;
  gxml = glade_get_widget_tree(_window_CH3);
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview2");
  model = gtk_tree_view_get_model (treeview);
    
  g_return_if_fail (_articles_CH3 != NULL);

  foo.atom1 = g_strdup ("NONE");
  foo.atom2 = g_strdup ("NONE");
  foo.atom3 = g_strdup ("NONE");
  foo.atom_pcs = g_strdup ("NONE");
  g_array_append_vals (_articles_CH3, &foo, 1);

  gtk_list_store_append (GTK_LIST_STORE (model), &iter);
  gtk_list_store_set (GTK_LIST_STORE (model), &iter,
                      COLUMN_ATOM1, foo.atom1,
                      COLUMN_ATOM2, foo.atom2,
                      COLUMN_ATOM3, foo.atom3,
                      COLUMN_ATOM_PCS, foo.atom_pcs,
                      -1);
}

//~ when one click on remove button
void
remove_item_CH3 (void)
{
  GtkTreeIter iter;
  GtkTreeView *treeview;
  GtkTreeModel *model;
  GtkTreeSelection *selection; 
  GladeXML *gxml;  
    
  gxml = glade_get_widget_tree(_window_CH3);
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview2");    
  selection = gtk_tree_view_get_selection (treeview);
  model = gtk_tree_view_get_model (treeview);
  if (gtk_tree_selection_get_selected (selection, NULL, &iter))
    {
      gint i;
      GtkTreePath *path;

      path = gtk_tree_model_get_path (model, &iter);
      i = gtk_tree_path_get_indices (path)[0];
      gtk_list_store_remove (GTK_LIST_STORE (model), &iter);

      g_array_remove_index (_articles_CH3, i);

      gtk_tree_path_free (path);
    }
}

//~ when the cells are edited, update the GUI _articles_CH3
void
cell_edited_CH3 (GtkCellRendererText *cell,
             const gchar         *path_string,
             const gchar         *new_text,
             gpointer             data)
{
  GtkTreeModel *model = (GtkTreeModel *)data;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
  GtkTreeIter iter;


  gint column = GPOINTER_TO_INT ( g_object_get_data (G_OBJECT (cell), "column"));

	
  gtk_tree_model_get_iter (model, &iter, path);

  switch (column)
    {
    case COLUMN_ATOM1:
      {
        gint i;
        gchar *old_text;

        gtk_tree_model_get (model, &iter, column, &old_text, -1);
        g_free (old_text);

        i = gtk_tree_path_get_indices (path)[0];
        g_free (g_array_index (_articles_CH3, Item_CH3, i).atom1);
        g_array_index (_articles_CH3, Item_CH3, i).atom1 = g_strdup (new_text);

        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            g_array_index (_articles_CH3, Item_CH3, i).atom1, -1);
      }
      break;

    case COLUMN_ATOM2:
      {
        gint i;
        gchar *old_text;

        gtk_tree_model_get (model, &iter, column, &old_text, -1);
        g_free (old_text);

        i = gtk_tree_path_get_indices (path)[0];
        g_free (g_array_index (_articles_CH3, Item_CH3, i).atom2);
        g_array_index (_articles_CH3, Item_CH3, i).atom2 = g_strdup (new_text);

        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            g_array_index (_articles_CH3, Item_CH3, i).atom2, -1);
      }
      break;
        
    case COLUMN_ATOM3:
      {
        gint i;
        gchar *old_text;

        gtk_tree_model_get (model, &iter, column, &old_text, -1);
        g_free (old_text);

        i = gtk_tree_path_get_indices (path)[0];
        g_free (g_array_index (_articles_CH3, Item_CH3, i).atom3);
        g_array_index (_articles_CH3, Item_CH3, i).atom3 = g_strdup (new_text);

        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            g_array_index (_articles_CH3, Item_CH3, i).atom3, -1);
      }
      break;
    case COLUMN_ATOM_PCS:
      {
        gint i;
        gchar *old_text;
        gchar * text_CH3;
        char trunc_new[MAX_METHYL_STR];
        strncpy(trunc_new, new_text, MAX_METHYL_STR -1);
        trunc_new[MAX_METHYL_STR -1] = '\0';  
        gtk_tree_model_get (model, &iter, column, &old_text, -1);
        g_free (old_text);

        i = gtk_tree_path_get_indices (path)[0];
        g_free (g_array_index (_articles_CH3, Item_CH3, i).atom_pcs);
        g_array_index (_articles_CH3, Item_CH3, i).atom_pcs = g_strdup (trunc_new);
        //HACK for auto_CH3
        gtk_tree_model_get (model, &iter, COLUMN_ATOM1, &text_CH3, -1);  
        g_array_index (_articles_CH3, Item_CH3, i).atom1 = g_strdup (text_CH3);
        gtk_tree_model_get (model, &iter, COLUMN_ATOM2, &text_CH3, -1);   
        g_array_index (_articles_CH3, Item_CH3, i).atom2 = g_strdup (text_CH3);
        gtk_tree_model_get (model, &iter, COLUMN_ATOM3, &text_CH3, -1);   
        g_array_index (_articles_CH3, Item_CH3, i).atom3 = g_strdup (text_CH3);
        g_free (text_CH3);
          //
        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            g_array_index (_articles_CH3, Item_CH3, i).atom_pcs, -1);
      }
      break;
    }

  gtk_tree_path_free (path);
}

// add columns to the treeview.
void
add_columns (GtkTreeView  *treeview, 
             GtkTreeModel *items_model,
             GtkTreeModel *numbers_model, GtkTreeModel *pcs_list_model)
{
  GtkCellRenderer *renderer;

  /* ATOM1 column */
  renderer = gtk_cell_renderer_combo_new ();
  g_object_set (renderer,
                "model", numbers_model,
                "text-column", COLUMN_ATOM_TEXT,
                "has-entry", FALSE,
                "editable", TRUE,
                NULL);
  g_signal_connect (renderer, "edited",
                    G_CALLBACK (cell_edited_CH3), items_model);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_ATOM1));

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "Atom 1", renderer,
                                               "text", COLUMN_ATOM1,
                                               NULL);

  /* ATOM2 column */
  renderer = gtk_cell_renderer_combo_new ();
  g_object_set (renderer,
                "model", numbers_model,
                "text-column", COLUMN_ATOM_TEXT,
                "has-entry", FALSE,
                "editable", TRUE,
                NULL);
  g_signal_connect (renderer, "edited",
                    G_CALLBACK (cell_edited_CH3), items_model);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_ATOM2));

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "Atom 2", renderer,
                                               "text", COLUMN_ATOM2,
                                               NULL);

  /* ATOM3 column */
  renderer = gtk_cell_renderer_combo_new ();
  g_object_set (renderer,
                "model", numbers_model,
                "text-column", COLUMN_ATOM_TEXT,
                "has-entry", FALSE,
                "editable", TRUE,
                NULL);
  g_signal_connect (renderer, "edited",
                    G_CALLBACK (cell_edited_CH3), items_model);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_ATOM3));

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "Atom 3", renderer,
                                               "text", COLUMN_ATOM3,
                                               NULL);
  /* ATOM_PCS column */
  renderer = gtk_cell_renderer_combo_new ();
  g_object_set (renderer,
                "model", pcs_list_model,
                "text-column", COLUMN_ATOM_TEXT,
                "has-entry", TRUE,
                "editable", TRUE,
                NULL);
  g_signal_connect (renderer, "edited",
                    G_CALLBACK (cell_edited_CH3), items_model);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_ATOM_PCS));

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "Atom PCS", renderer,
                                               "text", COLUMN_ATOM_PCS,
                                               NULL);
}

void cancel_CH3(void){
    gtk_widget_hide(_window_CH3);
}

void init_CH3(void){
    
    GtkWidget *treeview;
    GladeXML *gxml;
    
    gxml = glade_get_widget_tree(_window_CH3);
		treeview = glade_xml_get_widget (gxml, "treeview2");
    
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
    gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)),
                                GTK_SELECTION_SINGLE);
    
    
}

//called u callback
void
start_CH3 (void)
{


    if(_pdb_str == NULL){
        box_error( "You should first load a pdb file.\n", NULL);
        return;
    }
	
	gtk_widget_show (_window_CH3);

}

void update_CH3_tree_after_new_pdb(void){
    GtkTreeView *treeview;
    GtkTreeModel *items_model;
    GtkTreeModel *atom_list_model;
    GtkTreeModel *pcs_list_model;

    GladeXML *gxml; 
    
    //1. empty the treeview and remove column
    gxml = glade_get_widget_tree(_window_CH3);
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview2");
    empty_treeview(treeview, TRUE);
    
    //2. reconstruct the treeview
    atom_list_model = create_atom_list_model ();
    pcs_list_model  = create_PCS_list_model ();
    items_model = create_items_model_CH3 ();
    add_columns (GTK_TREE_VIEW (treeview), items_model, atom_list_model, pcs_list_model);
    gtk_tree_view_set_model ((GtkTreeView *)treeview, GTK_TREE_MODEL (items_model));
    g_object_unref (atom_list_model);
    g_object_unref (pcs_list_model);
    g_object_unref (items_model);
    
}

void update_CH3_tree_after_new_pcs(void){
    
    GtkTreeModel *pcs_list_model;
    GtkTreeModel *new_pcs_list_model;
    GtkTreeView * treeview;

    GtkTreeViewColumn *tree_column;
    GladeXML *gxml; 
    GList* list_renderer;
    GtkCellRenderer *renderer;
    guint i;


    fprintf(stderr, "UPDATING CH3 AFTER NEW PCS\n");
    gxml = glade_get_widget_tree(_window_CH3);
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview2");
    tree_column =  gtk_tree_view_get_column (treeview, COLUMN_ATOM_PCS);
    fprintf(stderr, "Title column: %s\n", gtk_tree_view_column_get_title(tree_column));
    list_renderer = gtk_tree_view_column_get_cell_renderers(tree_column);
    if(list_renderer == NULL){
        return;
    }
    
    new_pcs_list_model = create_PCS_list_model();
    
    for(i = 0; i < g_list_length(list_renderer); i++){
        renderer = (GtkCellRenderer *) g_list_nth_data (list_renderer, i);
        if(renderer == NULL){
            continue;
        }
        
        
        g_object_get (renderer,
                "model", &pcs_list_model,
                NULL);
        
        g_object_set (renderer,
                "model", new_pcs_list_model,
                NULL);
    }
    g_list_free(list_renderer);
}

void empty_treeview(GtkTreeView *treeview, gboolean FLAG_CH3){
    
    empty_treeview_row(treeview, FLAG_CH3);
    empty_treeview_column(treeview);
}

void empty_treeview_column(GtkTreeView *treeview){
    GList* column_list;
    GtkTreeViewColumn *column;
    guint i;
    column_list = gtk_tree_view_get_columns (treeview);
    for(i = 0; i < g_list_length(column_list); i++){
        column = (GtkTreeViewColumn *) g_list_nth_data (column_list, i);
        gtk_tree_view_remove_column (treeview, column);
    }
    g_list_free(column_list);
}

void empty_tree_model(GtkTreeModel * model){
    GtkTreeIter iter;
    GtkTreePath *path;
    gboolean test_bool;
    gint i;
    
    if(model == NULL){
        return;
    }
    test_bool = gtk_tree_model_get_iter_first(model, &iter);

    while(test_bool!= FALSE){
        path = gtk_tree_model_get_path (model, &iter);
        i = gtk_tree_path_get_indices (path)[0];
        gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
        g_array_remove_index (_articles_CH3, i);
        gtk_tree_path_free (path);
        test_bool =  gtk_tree_model_get_iter_first(model, &iter);
    }
}

void empty_treeview_row(GtkTreeView *treeview, gboolean FLAG_CH3){
    GtkTreeIter iter;
    GtkTreePath *path;
    GtkTreeModel *model;
    gboolean test_bool;
    gint i;
    
    model = gtk_tree_view_get_model (treeview);
    
    if(model == NULL){
        return;
    }
    test_bool = gtk_tree_model_get_iter_first(model, &iter);

    while(test_bool!= FALSE){
        path = gtk_tree_model_get_path (model, &iter);
        i = gtk_tree_path_get_indices (path)[0];
        gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
        if(FLAG_CH3 == TRUE){
            g_array_remove_index (_articles_CH3, i);
        }
        gtk_tree_path_free (path);
        test_bool =  gtk_tree_model_get_iter_first(model, &iter);
    }
}

void delete_columns_CH3(GtkTreeView * treeview){
    GList * list;
    GtkTreeViewColumn * column;
    int i;
    list = gtk_tree_view_get_columns(treeview);
    for(i = 0; i < g_list_length (list); i++){
        column = g_list_nth_data (list, i);
        gtk_tree_view_remove_column (treeview, column);
    }
    g_list_free (list);
}


//~ do_presence look if the name is already present in the tab
//~ if yes, return TRUE
//~ if no, return FALSE
//~ it adds the name to the tab
//~ index being the size of the tab.
gboolean do_presence(char ** tab, int index, char * name){
    gboolean my_bool;
    int i;
    my_bool = FALSE;
    for (i = 0; i < index; i++){
        if (strcmp(tab[i], name) == 0){
            my_bool = TRUE;
            break;
        }
    }
    tab[index] = name;
    return(my_bool);
}

gboolean check_validity_methyl_association(void){

    int i, j, k;
    char * a1, * a2, *a3, *a4;
    char ** atom;
    char ** pcs;
    
    j = k = 0;
    if (_articles_CH3 == NULL){
        return(TRUE);
    }
    if (_articles_CH3->len == 0){
        return(TRUE);
    }
    atom = calloc(3*_articles_CH3->len, sizeof(char *));
    if (atom == NULL){
        box_error("error, couldn't allocate memorie\n\
assuming methyl association is well formed\n", NULL);
        return(TRUE);
    }
    pcs = calloc(_articles_CH3->len, sizeof(char *));
    if (pcs == NULL){
        box_error("error, couldn't allocate memorie\n\
assuming methyl association is well formed\n", NULL);
        return(TRUE);
    }
    for(i = 0; i < _articles_CH3->len; i++){
        a1 =  g_array_index (_articles_CH3, Item_CH3, i).atom1;
        a2 =  g_array_index (_articles_CH3, Item_CH3, i).atom2;
        a3 =  g_array_index (_articles_CH3, Item_CH3, i).atom3;
        a4 =  g_array_index (_articles_CH3, Item_CH3, i).atom_pcs;
        if((strcmp(a1, "NONE") == 0) && (strcmp(a2, "NONE") == 0) &&
           (strcmp(a3, "NONE") == 0) && (strcmp(a4, "NONE") == 0)){
               continue;
           }
        if((strcmp(a1, "NONE") == 0) || (strcmp(a2, "NONE") == 0) ||
           (strcmp(a3, "NONE") == 0) /*|| (strcmp(a4, "NONE") == 0)*/){
               free(atom);
               free(pcs);
               return(FALSE);
        }
        if(do_presence(atom, j, a1) == TRUE){
            free(atom);
            free(pcs);
            return(FALSE);
        }
        j++;
        if(do_presence(atom, j, a2) == TRUE){
            free(atom);
            free(pcs);
            return(FALSE);
        }
        j++;
        if(do_presence(atom, j, a3) == TRUE){
            free(atom);
            free(pcs);
            return(FALSE);
        }
        j++;
        if(do_presence(pcs, k, a4) == TRUE){
            free(atom);
            free(pcs);
            return(FALSE);
        }
        k++;
    }
    free(atom);
    free(pcs);
    return(TRUE);
}


void auto_CH3(void){
    GtkTreeView * treeview;
    GladeXML * gxml;
    GtkTreeModel *model;
    GtkTreeIter iter_pos1, iter_pos2, iter_pos3;
    GtkTreeIter iter;
    gboolean test_bool;
    GtkListStore *pdb_list_model;
    gboolean bool1, bool2, bool3;
    int i;
    gboolean first = TRUE;
    
    gxml = glade_get_widget_tree(_window_CH3);
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview2");
    empty_treeview_row(treeview, TRUE);
    clear_CH3();
    model = gtk_tree_view_get_model (treeview);
    if(model == NULL){
        return;
    }
    pdb_list_model = (GtkListStore *) create_atom_list_model ();
    
    for(i = 0; i < N_CH3_NAME; i++){
        bool1 = is_name_present_in_liststore(CH3_NAME[3*i+0], pdb_list_model, &iter_pos1);
        bool2 = is_name_present_in_liststore(CH3_NAME[3*i+1], pdb_list_model, &iter_pos2);
        bool3 = is_name_present_in_liststore(CH3_NAME[3*i+2], pdb_list_model, &iter_pos3);
        //fprintf(stderr, "i %d b1 %d b2 %d b3  %d\n", i, bool1, bool1, bool1);
        if(bool1&&bool2&&bool3){
            add_item_CH3();
            if(first){
                test_bool = gtk_tree_model_get_iter_first(model, &iter);
                first = FALSE;
            }
            else{
                test_bool = gtk_tree_model_iter_next(model, &iter);
            }
                
            if(test_bool != FALSE){
                gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_ATOM1,
                    CH3_NAME[3*i+0], -1);
                gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_ATOM2,
                    CH3_NAME[3*i+1], -1);
                gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_ATOM3,
                    CH3_NAME[3*i+2], -1);
            }
        }
    }
    
    fprintf(stderr, "CHRISTOPHE: YOU SHOULD FREE THE PDB MODEL\n");
}
