/*
 * molmol_iso.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <glib/gstdio.h>

#define epsilon_molmol_r2 0.1

#include "pdb.h"
#include "common.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "tensor_list.h"
#include "molmol_iso.h"
#include "unify_tensor.h"


extern GtkWidget *_window_iso_molmol;
extern GtkTreeModel * _model_tl;




void write_pdb_tensor(FILE * F_pdb, molmol_data * mm_data)
{
	double xa, ya, za, xM, yM, zM;
	int i, j;
  double R[3][3];
    
    
  filZX_or_YZ(R, mm_data->a, mm_data->b , mm_data->g);
    
	xa = TENSOR_LENGTH;
  ya = TENSOR_LENGTH;
  za = TENSOR_LENGTH;
	xM = mm_data->X;
	yM = mm_data->Y;
	zM = mm_data->Z;
    
  fprintf(F_pdb, "ATOM  90000  OO  TSR  9000%12.3f%8.3f%8.3f  1.00  0.00\n", xM, yM, zM);
  fprintf(F_pdb, "ATOM  90001  OX  TSR  9000%12.3f%8.3f%8.3f  1.00  0.00\n", 
					xM+xa*R[0][0], yM+xa*R[0][1], zM+xa*R[0][2]);
  fprintf(F_pdb, "ATOM  90002  OY  TSR  9000%12.3f%8.3f%8.3f  1.00  0.00\n", 
					xM+ya*R[1][0], yM+ya*R[1][1], zM+ya*R[1][2]);
  fprintf(F_pdb, "ATOM  90003  OZ  TSR  9000%12.3f%8.3f%8.3f  1.00  0.00\n", 
					xM+za*R[2][0], yM+za*R[2][1], zM+za*R[2][2]);
	xa =  TENSOR_LENGTH;
	ya = -TENSOR_LENGTH;
	za = -TENSOR_LENGTH;
	fprintf(F_pdb, "ATOM  90010  OO  TSR  9001%12.3f%8.3f%8.3f  1.00  0.00\n", xM, yM, zM);
  fprintf(F_pdb, "ATOM  90011  OX  TSR  9001%12.3f%8.3f%8.3f  1.00  0.00\n", 
					xM+xa*R[0][0], yM+xa*R[0][1], zM+xa*R[0][2]);
  fprintf(F_pdb, "ATOM  90012  OY  TSR  9001%12.3f%8.3f%8.3f  1.00  0.00\n", 
					xM+ya*R[1][0], yM+ya*R[1][1], zM+ya*R[1][2]);
  fprintf(F_pdb, "ATOM  90013  OZ  TSR  9001%12.3f%8.3f%8.3f  1.00  0.00\n", 
          xM+za*R[2][0], yM+za*R[2][1], zM+za*R[2][2]);
	xa = -TENSOR_LENGTH;
	ya =  TENSOR_LENGTH;
	za = -TENSOR_LENGTH;
	fprintf(F_pdb, "ATOM  90020  OO  TSR  9002%12.3f%8.3f%8.3f  1.00  0.00\n", xM, yM, zM);
  fprintf(F_pdb, "ATOM  90021  OX  TSR  9002%12.3f%8.3f%8.3f  1.00  0.00\n", 
					xM+xa*R[0][0], yM+xa*R[0][1], zM+xa*R[0][2]);
  fprintf(F_pdb, "ATOM  90022  OY  TSR  9002%12.3f%8.3f%8.3f  1.00  0.00\n", 
					xM+ya*R[1][0], yM+ya*R[1][1], zM+ya*R[1][2]);
  fprintf(F_pdb, "ATOM  90023  OZ  TSR  9002%12.3f%8.3f%8.3f  1.00  0.00\n", 
					xM+za*R[2][0], yM+za*R[2][1], zM+za*R[2][2]);
	xa = -TENSOR_LENGTH;
	ya = -TENSOR_LENGTH;
	za =  TENSOR_LENGTH;
	fprintf(F_pdb, "ATOM  90030  OO  TSR  9003%12.3f%8.3f%8.3f  1.00  0.00\n", xM, yM, zM);
  fprintf(F_pdb, "ATOM  90031  OX  TSR  9003%12.3f%8.3f%8.3f  1.00  0.00\n", 
					xM+xa*R[0][0], yM+xa*R[0][1], zM+xa*R[0][2]);
  fprintf(F_pdb, "ATOM  90032  OY  TSR  9003%12.3f%8.3f%8.3f  1.00  0.00\n", 
					xM+ya*R[1][0], yM+ya*R[1][1], zM+ya*R[1][2]);
  fprintf(F_pdb, "ATOM  90033  OZ  TSR  9003%12.3f%8.3f%8.3f  1.00  0.00\n", 
					xM+za*R[2][0], yM+za*R[2][1], zM+za*R[2][2]);
	fprintf(F_pdb, "TER\n");
    
	for(j = 0; j < _pdb_str->n_model; j++){
		if(_pdb_str->n_model > 1){
			fprintf(F_pdb, "MODEL\t%d\n", j + 1);
		}
		for(i = 0; i < _pdb_str->n_atom; i++){
			fprintf(F_pdb, "%s%7d %4s %-4s %4d  %10.3lf %7.3lf %7.3lf %5.2lf %5.2lf\n", 
					"ATOM", _pdb_str->tab_atom[i].atom_num, _pdb_str->tab_atom[i].atom_name,
					_pdb_str->tab_atom[i].residue_name, _pdb_str->tab_atom[i].residue_num,
					_pdb_str->tab_atom[i].atom_coo[j].x,
					_pdb_str->tab_atom[i].atom_coo[j].y,
					_pdb_str->tab_atom[i].atom_coo[j].z, 1.00, 0.00
					);
		}
		if(_pdb_str->n_model > 1){
			fprintf(F_pdb, "ENDMDL\n");
		}
	}
	fprintf(F_pdb, "END\n");
}

void print_tensor(molmol_data * mm_data){
	double Dx, Dy, Dz, Ax, Rh;
	double x, y, z;
	double a, b, g;
	double rot[3][3];
	a = mm_data->a;
	b = mm_data->b;
	g = mm_data->g;
	Ax = mm_data->Ax;
	Rh = mm_data->Rh;
	x = mm_data->X;
	y = mm_data->Y;
	z = mm_data->Z;
	filZX_or_YZ(rot, a, b, g);
		
	Dx = -Ax/3.0 + Rh/2.0;
	Dy = -Ax/3.0 - Rh/2.0;
	Dz = 2.0/3.0*Ax;
	fprintf(stderr, " alpha = %12.3lf\n", a);
	fprintf(stderr, " beta  = %12.3lf\n", b);
	fprintf(stderr, " gamma = %12.3lf\n", g);
	fprintf(stderr, " x     = %12.3lf\n", x);
	fprintf(stderr, " y     = %12.3lf\n", y);
	fprintf(stderr, " z     = %12.3lf\n", z);
	fprintf(stderr, " Ax    = %12.3lf\n", Ax);
	fprintf(stderr, " Rh    = %12.3lf\n", Rh);
	fprintf(stderr, " Dx    = %12.3lf\n", Dx);
	fprintf(stderr, " Dy    = %12.3lf\n", Dy);
	fprintf(stderr, " Dz    = %12.3lf\n", Dz);
	fprintf(stderr, "\n");
	print_matrix(rot);
	
}

void write_mac(FILE * F_mac, molmol_data * mm_data, char * name_pot, char * name_pdb,double pcs_max,
			   int color)
{
    double iso1;
    double iso2;
    double iso3;
	char color_p1[20];
	char color_p2[20];
	char color_n1[20];
	char color_n2[20];
	char blue[6] = "0 0 1";
	char green[6] = "0 1 0";
	char red[6] = "1 0 0";
	char dark[20] = "0.1 0.1 0.1";
	char black[6] = "0 0 0";
	char name_pdb_no_ext_temp[1000];
	char * name_pdb_no_ext;
	char * color_n, *color_p;
	sprintf(color_n1, "1 0 0");
	sprintf(color_p1, "0 0 1");
	sprintf(color_n2, "1 1 0");
	sprintf(color_p2, "0 1 1");
	
	sprintf(name_pdb_no_ext_temp, name_pdb);
	name_pdb_no_ext = strrchr(name_pdb_no_ext_temp, '/');
	//name_pdb_no_ext_temp[strlen(name_pdb) - 4] = '\0';
	name_pdb_no_ext_temp[strlen(name_pdb) - 4] = '0';
	name_pdb_no_ext_temp[strlen(name_pdb) - 3] = '0';
	name_pdb_no_ext_temp[strlen(name_pdb) - 2] = '1';
	name_pdb_no_ext_temp[strlen(name_pdb) - 1] = '\0';
	if(name_pdb_no_ext == NULL){
		box_error("error while trying to generate\n the MOLMOL macro file", NULL);
		return;
	}
	name_pdb_no_ext += 1;
	if(color == 0){
		color_n = color_n1;
		color_p = color_p1;
	}
	else{
		color_n = color_n2;
		color_p = color_p2;
	}
	
    iso3 = pcs_max;
    iso2 = pcs_max *4;
    iso1 = pcs_max *9;
    
    fprintf(F_mac, "#Molmol macro for tensor with values:\n");
    fprintf(F_mac, "#alpha: %.3lf beta: %.3lf gamma: %.3lf Ax: %.3lf Rh: %.3lf x: %.3lf y: %.3lf z: %.3lf\n"
            , mm_data->a, mm_data->b, mm_data->g, mm_data->Ax, mm_data->Rh, mm_data->X
            , mm_data->Y, mm_data->Z);
    fprintf(F_mac, "#Corresponding potential file should be in %s\n", name_pot);
    fprintf(F_mac, "define DrawIso\n");
    //fprintf(F_mac, "\t$file = '%s' \"potential file\"\n", name_pot);
	//fprintf(F_mac, "\t$file2 = '%s' \"tensor pdb file (without extension)\"\n", name_pdb_no_ext);
	//fprintf(F_mac, "\t$file3 = '%s' \"tensor pdb file (with extension)\"\n", name_pdb);
    //fprintf(F_mac, "\t$iso1 = '%.5lf' \"isosurface for ppm = \"\n", iso1);
	fprintf(F_mac, "\t$iso2 = '%.5lf' \"isosurface for ppm = \"\n", iso2);
    fprintf(F_mac, "\t$iso3 = '%.5lf' \"isosurface for ppm = \"\n", iso3);
	
    fprintf(F_mac, "\tSelectMol 'mol.name = \"%s\"'\n", name_pdb_no_ext); 
    //fprintf(F_mac, "\tMaterialInit 0.2 0.8 0.9 30 0 0.6 1\n"); 
    //fprintf(F_mac, "\tColorInit %s\n", color_n); 
    //fprintf(F_mac, "\tAddIsosurface '%s' -$iso1\n", name_pot);
    //fprintf(F_mac, "\tColorInit %s\n", color_p);
    //fprintf(F_mac, "\tAddIsosurface '%s' $iso1\n", name_pot);
    fprintf(F_mac, "\tMaterialInit 0.2 0.8 0.9 30 0 0.4 1\n"); 
	fprintf(F_mac, "\tColorInit %s\n", color_n);
	fprintf(F_mac, "\tAddIsosurface '%s' -$iso2\n", name_pot);
	fprintf(F_mac, "\tColorInit %s\n", color_p);
	fprintf(F_mac, "\tAddIsosurface '%s' $iso2\n", name_pot);
    fprintf(F_mac, "\tMaterialInit 0.2 0.8 0.9 30 0 0.15 1\n"); 
	fprintf(F_mac, "\tColorInit %s\n", color_n);
	fprintf(F_mac, "\tAddIsosurface '%s' -$iso3\n", name_pot);
	fprintf(F_mac, "\tColorInit %s\n", color_p);
	fprintf(F_mac, "\tAddIsosurface '%s' $iso3\n", name_pot);
    fprintf(F_mac, "\tMaterialInit 0.2 0.8 0.4 30 0 1 1\n");
	fprintf(F_mac, "\tColorInit %s\n", black);
	fprintf(F_mac, "end\n\n");
	//fprintf(F_mac, "InitAll\n");
	fprintf(F_mac, "MaterialInit 0.2 0.8 0.4 30 0 1 1\n");
	fprintf(F_mac, "ColorInit %s\n", black);
	fprintf(F_mac, "ReadPdb %s\n", name_pdb);
	fprintf(F_mac, "SelectMol 'mol.name = \"%s\"'\n", name_pdb_no_ext); 
	fprintf(F_mac, "MoveInit\n");
	
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OX\")&mol.name = \"%s\" & res.num = 9000'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"X\"\n");
	fprintf(F_mac, "ColorAtom %s\n", red);
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OY\")&mol.name = \"%s\" & res.num = 9000'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"Y\"\n");
	fprintf(F_mac, "ColorAtom %s\n", green);
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OZ\")&mol.name = \"%s\" & res.num = 9000'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"Z\"\n");
	fprintf(F_mac, "ColorAtom %s\n", blue);
	
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OX\")&mol.name = \"%s\" & res.num = 9001'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"X\"\n");
	fprintf(F_mac, "ColorAtom %s\n", red);
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OY\")&mol.name = \"%s\" & res.num = 9001'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"Y\"\n");
	fprintf(F_mac, "ColorAtom %s\n", green);
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OZ\")&mol.name = \"%s\" & res.num = 9001'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"Z\"\n");
	fprintf(F_mac, "ColorAtom %s\n", blue);
	
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OX\")&mol.name = \"%s\" & res.num = 9002'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"X\"\n");
	fprintf(F_mac, "ColorAtom %s\n", red);
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OY\")&mol.name = \"%s\" & res.num = 9002'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"Y\"\n");
	fprintf(F_mac, "ColorAtom %s\n", green);
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OZ\")&mol.name = \"%s\" & res.num = 9002'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"Z\"\n");
	fprintf(F_mac, "ColorAtom %s\n", blue);
	
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OX\")&mol.name = \"%s\" & res.num = 9003'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"X\"\n");
	fprintf(F_mac, "ColorAtom %s\n", red);
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OY\")&mol.name = \"%s\" & res.num = 9003'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"Y\"\n");
	fprintf(F_mac, "ColorAtom %s\n", green);
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\" | atom.name = \"OZ\")&mol.name = \"%s\" & res.num = 9003'\n", name_pdb_no_ext); 
	fprintf(F_mac, "AddBond\n");
	fprintf(F_mac, "StyleAtom sphere\n");
	fprintf(F_mac, "RadiusAtom 0.2\n");
	fprintf(F_mac, "LabelAtom \"Z\"\n");
	fprintf(F_mac, "ColorAtom %s\n", blue);
	
	fprintf(F_mac, "SelectAtom '(atom.name = \"OO\")&mol.name = \"%s\"'\n", name_pdb_no_ext); 
	fprintf(F_mac, "LabelAtom \"O\"\n");
	fprintf(F_mac, "ColorAtom %s\n", dark);
	fprintf(F_mac, "StyleAtom invisible\n");
	
	fprintf(F_mac, "SelectRes 'res.num = 9000'\n");
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OZ\")& (mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", blue);
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OY\") &(mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", green);
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OX\")& (mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", red);
	
	fprintf(F_mac, "SelectRes 'res.num = 9001'\n");
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OZ\")& (mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", blue);
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OY\") &(mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", green);
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OX\")& (mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", red);
	
	fprintf(F_mac, "SelectRes 'res.num = 9002'\n");
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OZ\")& (mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", blue);
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OY\") &(mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", green);
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OX\")& (mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", red);
	
	fprintf(F_mac, "SelectRes 'res.num = 9003'\n");
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OZ\")& (mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", blue);
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OY\") &(mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", green);
	fprintf(F_mac, "SelectBond '(atom1.name = \"OO\" & atom2.name = \"OX\")& (mol.name = \"%s\")'\n", name_pdb_no_ext); 
	fprintf(F_mac, "StyleBond neon\n");
	fprintf(F_mac, "RadiusBond 0.1\n");
	fprintf(F_mac, "ColorBond %s\n", red);
	
	fprintf(F_mac, "DrawIso\n");
	fprintf(F_mac, "SelectMol 'mol.name != \"%s\"'\n", name_pdb_no_ext);
	fprintf(F_mac, "SelectAtom '(atom.name != \"OO\") &  (atom.name != \"OZ\") & (atom.name != \"OY\") & (atom.name != \"OX*\")'\n");
	fprintf(F_mac, "SelectRes 'res.num < 9000'\n");
	fprintf(F_mac, "SelectBond 'res.num < 9000'\n");
	//////////////////////////
}

double write_pot_ZX_or_YZ(FILE * F_out, molmol_data * mm_data){
  double rot[3][3];
	double a, b, g;
	a = mm_data->a;
	b = mm_data->b;
	g = mm_data->g;
  filZX_or_YZ(rot, a, b, g);
  return(calc_pot_molmol(rot, F_out, mm_data));
}

double calc_pot_molmol(double rot[3][3], FILE * F_out, molmol_data * mm_data){

  double x_s, y_s, z_s;
  double x_ss, y_ss, z_ss;
  int x_ns, y_ns, z_ns;  
  double x_H, y_H, z_H;
  double x2_H, y2_H, z2_H;
  double x_t, y_t, z_t;
  double fac, r2, r5, PCS;
  int i, j, k, ii; 
  double compt;
  double X, Y, Z;
  double x_c, y_c, z_c;  
  double Dax, Drh;
  double pcs_max;

  //char tmp_buf[50];	
  pcs_max = 0; 
    
  Dax = mm_data->Ax;
  Drh = mm_data->Rh;  
    
  x_ns = mm_data->x_step_n;
  y_ns = mm_data->y_step_n;  
  z_ns = mm_data->z_step_n;
   
  x_ss = mm_data->x_step_size;
  y_ss = mm_data->y_step_size;
  z_ss = mm_data->z_step_size;  
    
  X = mm_data->X;
  Y = mm_data->Y;
  Z = mm_data->Z;
    
  x_s = mm_data->x_start ;
  y_s = mm_data->y_start ;
  z_s = mm_data->z_start ;

  fprintf(F_out, "% 8.3lf %4.0d %1.3lf\n", x_s, x_ns, x_ss);
  fprintf(F_out, "% 8.3lf %4.0d %1.3lf\n", y_s, y_ns, y_ss);
  fprintf(F_out, "% 8.3lf %4.0d %1.3lf\n", z_s, z_ns, z_ss);

  compt = 0;
  ii = 0;

  fac = 1.0/12.0*M_PI;

  x_t = rot[0][0] * X + rot[0][1] * Y + rot[0][2] * Z; 
  y_t = rot[1][0] * X + rot[1][1] * Y + rot[1][2] * Z; 
  z_t = rot[2][0] * X + rot[2][1] * Y + rot[2][2] * Z; 
	
  //setvbuf(F_out, buffer_write, _IOFBF, BUFFER_SIZE);

    z_c = z_s;
    for(i = 0; i < z_ns; i++) {
        y_c = y_s;
        for( j = 0; j < y_ns; j++ ) {
            x_c = x_s;
            for( k = 0; k < x_ns; k++ ) {
                x_H = rot[0][0]*x_c + rot[0][1] * y_c + rot[0][2] * z_c; 
                y_H = rot[1][0]*x_c + rot[1][1] * y_c + rot[1][2] * z_c; 
                z_H = rot[2][0]*x_c + rot[2][1] * y_c + rot[2][2] * z_c; 
                x_H -= x_t;
                y_H -= y_t;
                z_H -= z_t;
                x2_H = x_H * x_H;
                y2_H = y_H * y_H;
                z2_H = z_H * z_H;
                r2 = x2_H + y2_H + z2_H;
                r5 = r2*r2*sqrt(r2);
                if(epsilon_molmol_r2 < r2){
                    PCS = FACT_USI * 1.0 /* * fac */ /r5*( Dax*(2*z2_H-x2_H-y2_H) + 3.0/2.0*Drh*(x2_H - y2_H));
//THE fact_usi take into account (i) the 1/(12PI) (ii)
//the fact that length are in Angstrom and Xax in 10-32m3
//and (iii) that PCS are in ppm (10^6)
                    
                    if( (i == 0)||(j == 0)||(k == 0)||(i ==  (z_ns-1))||(j == (y_ns-1))||(k == (x_ns-1))){
                        //we are on the surface of the cube
                        //we save the biggeest PCS values for isovalue.
                        if (fabs(PCS) > pcs_max){
                            pcs_max = fabs(PCS);
                        }
                    }
                }
                else{
                    PCS = 0.0; 
                }

                fprintf(F_out, "% .3e ", PCS);
                ii++;
                if( ii ==  10){
                    ii = 0;

                    fprintf(F_out, "\n");
                }
                compt++;
                x_c += x_ss;
            }      
            y_c += y_ss;
        }
        z_c += z_ss;
    }
    
  //printf("%.1lf\n", compt);
  //printf("should be %d\n", x_ns * y_ns * z_ns);
  return(pcs_max);  
}

//angle given in degree please
int filZ_Y_Z(double rot[3][3], double a, double b, double g){ /*euler angles from spin in degree*/
  double ca, cb, cg;  
  double sa, sb, sg; 

  ca = cos(a*M_PI/180.0);
  cb = cos(b*M_PI/180.0);
  cg = cos(g*M_PI/180.0);

  sa = sin(a*M_PI/180.0);
  sb = sin(b*M_PI/180.0);
  sg = sin(g*M_PI/180.0);

  rot[0][0] =  cg * cb * ca  -  sg * sa;
  rot[0][1] =  cg * cb * sa  +  sg * ca;
  rot[0][2] = -cg * sb;
  rot[1][0] = -sg * cb * ca  -  cg * sa;
  rot[1][1] = -sg * cb * sa  +  cg * ca;
  rot[1][2] =  sg * sb;
  rot[2][0] =  sb * ca;
  rot[2][1] =  sb * sa;
  rot[2][2] =  cb;
  return(1);  
}

//angle given in degree please
int filZ_X_Z(double rot[3][3], double a, double b, double g){ /*euler angles from mathematica/mathworld, in degree*/
  double ca, cb, cg;  
  double sa, sb, sg; 

  ca = cos(a*M_PI/180.0);
  cb = cos(b*M_PI/180.0);
  cg = cos(g*M_PI/180.0);

  sa = sin(a*M_PI/180.0);
  sb = sin(b*M_PI/180.0);
  sg = sin(g*M_PI/180.0);

  rot[0][0] =  cg * ca  -  cb * sa * sg;
  rot[0][1] =  cg * sa  +  cb * ca * sg;
  rot[0][2] =  sg * sb;
  rot[1][0] = -sg * ca  -  cb * sa * cg;
  rot[1][1] = -sg * sa  +  cb * ca * cg;
  rot[1][2] =  cg * sb;
  rot[2][0] =  sb * sa;
  rot[2][1] = -sb * ca;
  rot[2][2] =  cb;
  return(1);  
}  

//angle given in degree please
int filZX_or_YZ(double rot[3][3], double a, double b, double g){
    if(EULER_CONVENTION == ZXZ_CONVENTION){
        return(filZ_X_Z(rot, a, b, g));
    }
    if(EULER_CONVENTION == ZYZ_CONVENTION){
        return(filZ_Y_Z(rot, a, b, g));
    }
    fprintf(stderr, "WARNING, no convention found (ZXZ or ZYZ). see common.h\n");
    return(-1);
}



void start_vm(void){
    if(_pdb_str == NULL){
        box_error( "You should first load a pdb file.\n", NULL);
        return;
    }
		uncheck_all((GtkTreeModel *) _model_tl, COLUMN_USED_TL);
    gtk_widget_show(_window_iso_molmol);
}

void init_vm(void){
    GladeXML *gxml;
    gxml = glade_get_widget_tree(_window_iso_molmol);
    GtkTreeView * treeview;
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview11");
    add_columns_tl(treeview, TENSOR_ANY);
    gtk_tree_view_set_model ((GtkTreeView *)treeview, GTK_TREE_MODEL (_model_tl));
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
	set_multiple_selection(GTK_TREE_VIEW (treeview));
    gtk_widget_hide(_window_iso_molmol);
}


void add_manual_vm(void){
    GladeXML *gxml;
    double x, y, z, Ax, Rh, a, b, g;
    
    gxml = glade_get_widget_tree(_window_iso_molmol);

    x = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_x5"));
    y = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_y5"));
    z = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_z5"));
    a = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_alpha5"));
    b = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_beta5"));
    g = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_gamma5"));
    Ax = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Ax5"));
    Rh = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Rh5"));
	
    add_to_tensor_listing(Ax, Rh, x, y, z, a, b, g, MANUAL_TENSOR, -1,
						   0, 0, 0, 0, 0, 0, 0, 0);
}

void cancel_vm(void){
    gtk_widget_hide(_window_iso_molmol);
}

void copy_vm(void){
	  GladeXML *gxml;  
	  GtkTreeView *treeview;

  gxml = glade_get_widget_tree(_window_iso_molmol);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview11"); 
  copy_tensor_from_list(gxml, treeview, 5);	

}

void remove_vm(void){
	
	GtkTreeView *treeview;
	GladeXML *gxml; 
 
  gxml = glade_get_widget_tree(_window_iso_molmol);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview11");  
  remove_tensor_from_list(gxml, treeview, 5);	
	
}

void save_vm(GtkButton * button){
  	GtkWidget *dialog;
  	GtkFileFilter *filter1;
  	GtkFileFilter *filter4;
  	gint result;
    molmol_data mm_data;
	char name_mac[500];
	int result2;
	
	result2 = read_tensor_from_gui(&mm_data, (GtkWidget*) button, MOLMOL_);
	
	if(result2 == -1){
		return;
	}
    
  dialog =
    gtk_file_chooser_dialog_new
    ("In which file do you want to save your MOLMOL macro?", NULL,
     GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
		
	gtk_window_set_icon_from_file ((GtkWindow *)dialog, NUMBAT_ICON_FILE, NULL);
  gtk_window_set_transient_for((GtkWindow*) dialog, (GtkWindow*)_window_iso_molmol);

  filter1 = gtk_file_filter_new ();
  filter4 = gtk_file_filter_new ();
  gtk_file_filter_add_pattern (filter1, "*.mac");
  gtk_file_filter_set_name (filter1, "*.mac macro for MOLMOL");
  gtk_file_filter_add_pattern (filter4, "*.*");
  gtk_file_filter_set_name (filter4, "*.* ALL");
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter1);
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter4);

  result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
    {
    case GTK_RESPONSE_ACCEPT:
      {
	char *filename;
	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		  
	unique_tensor(&mm_data);
	sprintf(name_mac, filename);
	write_for_molmol(&mm_data, name_mac, 0);
	print_tensor(&mm_data); 
	g_free (filename);
	gtk_widget_hide(_window_iso_molmol);
	  }
      break;
    default:
      break;
    }
  gtk_object_destroy((GtkObject *)filter1);
  gtk_object_destroy((GtkObject *)filter4);
  gtk_widget_destroy (dialog); 
    
}



int read_tensor_from_gui(molmol_data * mm_data, GtkWidget* widget, int MOL_or_PyM){
	GtkSpinButton * s_x_start;
    GtkSpinButton * s_y_start;
    GtkSpinButton * s_z_start;
    GtkSpinButton * s_n_step;
    GtkSpinButton * s_step_size;
	GtkTreeModel *model;
    double x, y, z, Ax, Rh, a, b, g;
    gboolean my_bool;
    GtkTreeIter iter;
    gboolean is_toggle;
    GladeXML *gxml;	
	
	gxml = glade_get_widget_tree(widget);
	
	model = (GtkTreeModel *) _model_tl;
    if(model == NULL){
		fprintf(stderr, "problem with model\n");
        return(-1);
    }
    is_toggle = FALSE;
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (model, &iter, COLUMN_USED_TL, &is_toggle, -1); 
        if (is_toggle == TRUE){
            break;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model, &iter);
    }
    
    if(is_toggle == FALSE){
        box_error("Exactly one tensor should be selected for calculation.", NULL);
        return(-1);
    }
	
	
    gtk_tree_model_get(model, &iter, COLUMN_X_TL, &x, -1);
    gtk_tree_model_get(model, &iter, COLUMN_Y_TL, &y, -1);
    gtk_tree_model_get(model, &iter, COLUMN_Z_TL, &z, -1);
    gtk_tree_model_get(model, &iter, COLUMN_AX_TL, &Ax, -1);
    gtk_tree_model_get(model, &iter, COLUMN_RH_TL, &Rh, -1);
    gtk_tree_model_get(model, &iter, COLUMN_ALPHA_TL, &a, -1);
    gtk_tree_model_get(model, &iter, COLUMN_BETA_TL, &b, -1);
    gtk_tree_model_get(model, &iter, COLUMN_GAMMA_TL, &g, -1);
    
	if(MOL_or_PyM == MOLMOL_){
    	s_x_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_x_start");
    	s_y_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_y_start");
    	s_z_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_z_start");
    	s_n_step = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_n_step");
    	s_step_size = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_step_size");
	}
	else{ // PYMOL_
		s_x_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_x_start2");
   	 	s_y_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_y_start2");
    	s_z_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_z_start2");
    	s_n_step = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_n_step2");
    	s_step_size = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_step_size2");
	}

    mm_data->a = a;
    mm_data->b = b;
    mm_data->g = g;
    mm_data->X = x;
    mm_data->Y = y;
    mm_data->Z = z;
    mm_data->Ax = Ax;
    mm_data->Rh = Rh;
    mm_data->x_start = (double) gtk_spin_button_get_value(s_x_start);
    mm_data->y_start = (double) gtk_spin_button_get_value(s_y_start);
    mm_data->z_start = (double) gtk_spin_button_get_value(s_z_start);
    mm_data->x_step_size = (double) gtk_spin_button_get_value(s_step_size);
    mm_data->y_step_size = (double) gtk_spin_button_get_value(s_step_size);
    mm_data->z_step_size = (double) gtk_spin_button_get_value(s_step_size);
    mm_data->x_step_n = (int) gtk_spin_button_get_value(s_n_step);
    mm_data->y_step_n = (int) gtk_spin_button_get_value(s_n_step);
    mm_data->z_step_n = (int) gtk_spin_button_get_value(s_n_step);
	
	return(1);
}

void write_for_molmol(molmol_data * mm_data , char * name_mac, int color){
	double pcs_max;
	FILE * F_pot;
    FILE * F_mac;
	FILE * F_pdb;
    char name_pot[500];
	char name_pdb[500];
	sprintf(name_pot, "%s.pot",  name_mac);
	sprintf(name_pdb, "%s.pdb",  name_mac);
	
	if (!(F_pot = g_fopen(name_pot , "w"))) {
        fprintf(stderr, "can't open %s\n", name_pot);
        return;
    }
    
    if (!(F_mac = g_fopen(name_mac , "w"))) {
        fprintf(stderr, "can't open %s\n", name_mac);
        fclose(F_pot);
        return;
    }
	
	if (!(F_pdb = g_fopen(name_pdb , "w"))) {
        fprintf(stderr, "can't open %s\n", name_pdb);
        fclose(F_pot);
		fclose(F_mac);
        return;
    }
    

    pcs_max = write_pot_ZX_or_YZ(F_pot, mm_data);

    
    write_mac(F_mac, mm_data, name_pot, name_pdb, pcs_max, color);
	write_pdb_tensor(F_pdb, mm_data);
    
    fclose(F_pot);
    fclose(F_mac);
    fclose(F_pdb);	
}

void ok_vm(GtkButton * button){

	GtkEntry * entry;
	GladeXML *gxml; 
    molmol_data mm_data;

	char name_mac[500];
    char prog[1500];
	int result;
    gboolean result_bool;
    GPid pid;
    gchar *argv[4];
    GError * error = NULL;
	
	result = read_tensor_from_gui(&mm_data, (GtkWidget*) button, MOLMOL_);
	if(result == -1){
		return;
	}
	gtk_widget_hide(_window_iso_molmol);
	unique_tensor(&mm_data);
	sprintf(name_mac, "./molmol_isosurface.mac");
	write_for_molmol(&mm_data, name_mac, 0);
	print_tensor(&mm_data);

    fprintf(stderr, "trying to launch MOLMOL with the command:\n");
	
	gxml = glade_get_widget_tree(_window_iso_molmol);    
  	entry = (GtkEntry *) glade_xml_get_widget (gxml, "entry_molmol"); 

    sprintf(prog, gtk_entry_get_text(entry));
    argv[0] = prog;
    argv[1] = "-f";
    argv[2] = name_mac;
    argv[3] = NULL;
    fprintf(stderr, "%s %s %s", argv[0], argv[1], argv[2]);

    result_bool =  g_spawn_async(
                        NULL,
                        argv,
                        NULL,
                        G_SPAWN_SEARCH_PATH,
                        NULL,
                        NULL,
                        &pid,
                        &error);

    g_spawn_close_pid(pid);                                                     
    
    if (error != NULL){
        fprintf (stderr, "Error message: %s\n", error->message);
        g_error_free (error);
    } 
    else{
        
    }
    return;
}

void update_vm_gride_end(GtkSpinButton * spin){
    GtkSpinButton * s_x_start;
    GtkSpinButton * s_y_start;
    GtkSpinButton * s_z_start;
    GtkSpinButton * s_n_step;
    GtkSpinButton * s_step_size;
    GtkLabel * l_x_end;
    GtkLabel * l_y_end;
    GtkLabel * l_z_end;
    int n_step;
    double step_size;
    char val_char[50];
    double end;
    
    GladeXML *gxml;	
    
	gxml = glade_get_widget_tree((GtkWidget*) spin);
    s_x_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_x_start");
    s_y_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_y_start");
    s_z_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_z_start");
    s_n_step = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_n_step");
    s_step_size = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_step_size");
    l_x_end = (GtkLabel *) glade_xml_get_widget(gxml, "label_x_end");
    l_y_end = (GtkLabel *) glade_xml_get_widget(gxml, "label_y_end");
    l_z_end = (GtkLabel *) glade_xml_get_widget(gxml, "label_z_end");
    
    n_step = (int) gtk_spin_button_get_value(s_n_step);
    step_size = (double) gtk_spin_button_get_value(s_step_size);
    
    end = gtk_spin_button_get_value(s_x_start) + n_step * step_size;
    sprintf(val_char, "%.3lf", end);
    gtk_label_set_text(l_x_end, val_char);
    end = gtk_spin_button_get_value(s_y_start) + n_step * step_size;
    sprintf(val_char, "%.3lf", end);
    gtk_label_set_text(l_y_end, val_char);
    end = gtk_spin_button_get_value(s_z_start) + n_step * step_size;
    sprintf(val_char, "%.3lf", end);
    gtk_label_set_text(l_z_end, val_char);
}

void update_vm_gride(GtkButton * button){
    GtkSpinButton * s_x_start;
    GtkSpinButton * s_y_start;
    GtkSpinButton * s_z_start;
    GtkSpinButton * s_n_step;
    GtkSpinButton * s_step_size;
    GtkTreeModel *model;
    gboolean my_bool;
    GtkTreeIter iter;
    gboolean is_toggle;
    double x, y, z;
    
    GladeXML *gxml;	
    
	gxml = glade_get_widget_tree((GtkWidget*) button);
    
    s_x_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_x_start");
    s_y_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_y_start");
    s_z_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_z_start");
    s_n_step = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_n_step");
    s_step_size = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_step_size");
        
    model = (GtkTreeModel *) _model_tl;
    if(model == NULL){
        return;
    }
    is_toggle = FALSE;
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (model, &iter, COLUMN_USED_TL, &is_toggle, -1); 
        if (is_toggle == TRUE){
            break;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model, &iter);
    }
    if(is_toggle == FALSE){
        box_error("Exactly one tensor should be selected for calculation.\n", NULL);
        return;
    }
    gtk_tree_model_get(model, &iter, COLUMN_X_TL, &x, -1);
    gtk_tree_model_get(model, &iter, COLUMN_Y_TL, &y, -1);
    gtk_tree_model_get(model, &iter, COLUMN_Z_TL, &z, -1);
    
    gtk_spin_button_set_value(s_x_start, x -40.0);
    gtk_spin_button_set_value(s_y_start, y -40.0);
    gtk_spin_button_set_value(s_z_start, z -40.0);
    gtk_spin_button_set_value(s_n_step, 80);
    gtk_spin_button_set_value(s_step_size, 1.0);

}
