/*
 * pcs.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>
#include <glib/gstdio.h>
#include "pdb.h"
#include "common.h"

#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "select_display.h"
#include "methyl.h"
#include "error_analysis.h"

extern GtkWidget *_window;

extern GArray *_str_CH3; //Where we save the methyl association

//~ those four function are there to be able to distinguish
//~ from which tab (PCS1...PCS4) the signal comes from
//~ couldn't find an elegant way to do it :(
void cell_edited_0 (GtkCellRendererText *cell, const gchar *path_string,
             const gchar *new_text, gpointer data) {
    cell_edited_n (cell, path_string, new_text, data, 0);  
}
void cell_edited_1 (GtkCellRendererText *cell, const gchar *path_string,
             const gchar *new_text, gpointer data) {
    cell_edited_n (cell, path_string, new_text, data, 1);  
}
void cell_edited_2 (GtkCellRendererText *cell, const gchar *path_string,
             const gchar *new_text, gpointer data) {
    cell_edited_n (cell, path_string, new_text, data, 2);  
}
void cell_edited_3 (GtkCellRendererText *cell, const gchar *path_string,
             const gchar *new_text, gpointer data) {
    cell_edited_n (cell, path_string, new_text, data, 3);  
}

//~ what happens when a cell (tolerance or experimental pcs is edited?
void cell_edited_n (GtkCellRendererText *cell, const gchar *path_string,
             const gchar *new_text, gpointer data, int n_button)
{
  GtkTreeModel *model = (GtkTreeModel *)data;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
  GtkTreeIter iter;

  int res_num;
  gchar * atom_name; 
  gint column = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), "column"));
  gtk_tree_model_get_iter (model, &iter, path);
	
  double old_value, new_value;
	new_value = 0;
	old_value = 0;

  switch (column)
    {
    case COLUMN_PCS_EXP:
      {
        gint i;
        int j;
        double old_exp;
        gboolean is_used;  
        gboolean is_useable; 
        gtk_tree_model_get (model, &iter, column, &old_exp, -1);
		old_value = old_exp;
        i = gtk_tree_path_get_indices (path)[0];
        gtk_tree_model_get (model, &iter, COLUMN_ATOM_NAME, &atom_name, -1);
        gtk_tree_model_get (model, &iter, COLUMN_RESIDUE_NUM, &res_num, -1);
        j = find_in_pcs(res_num, atom_name, _all_pcs[n_button]);
        if((strcmp(new_text, "?") == 0)||(strcmp(new_text, "nan") == 0)){
            g_array_index (_all_pcs[n_button], Item_pcs, j).pcs_exp = EXEPTION_VALUE;
			new_value = EXEPTION_VALUE;
        }
        else {  
            g_array_index (_all_pcs[n_button], Item_pcs, j).pcs_exp = atof(new_text);
			new_value = atof(new_text);
        }
        
        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                        g_array_index (_all_pcs[n_button], Item_pcs, j).pcs_exp, -1);
        is_useable = FALSE;  
        if((g_array_index (_all_pcs[n_button], Item_pcs, j).pcs_exp != EXEPTION_VALUE)){
            is_useable = TRUE;     
        }
        gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_USEABLE,
                    is_useable, -1);  
        g_array_index (_all_pcs[n_button], Item_pcs, j).useable = is_useable;
        is_used =  g_array_index (_all_pcs[n_button], Item_pcs, j).used;
        g_array_index (_all_pcs[n_button], Item_pcs, j).used = is_used && is_useable;
        if(g_array_index (_all_pcs[n_button], Item_pcs, j).used == FALSE){
           gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_USED, FALSE, -1); 
        }
      }
      break;
    case COLUMN_PCS_TOL:
      {
        gint i;
        int j;
        double old_tol;
        double new_tol;           
        gtk_tree_model_get (model, &iter, column, &old_tol, -1);
		old_value = old_tol;
        i = gtk_tree_path_get_indices (path)[0];
        gtk_tree_model_get (model, &iter, COLUMN_ATOM_NAME, &atom_name, -1);
        gtk_tree_model_get (model, &iter, COLUMN_RESIDUE_NUM, &res_num, -1);   
        j = find_in_pcs(res_num, atom_name, _all_pcs[n_button]);
        new_tol =  atof(new_text);
        if (new_tol < 0){
            new_tol = 0;
        }
		new_value = new_tol;
        g_array_index (_all_pcs[n_button], Item_pcs, j).pcs_tol = new_tol;
        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                        g_array_index (_all_pcs[n_button], Item_pcs, j).pcs_tol, -1);
      }
      break;
    }
  gtk_tree_path_free (path);
	if(new_value != old_value){
		// THE PCS HAS BEEN CHANGED, PREVIOUSLY CLACULATED TENSOR
		//ARE NOT VALID ANYMORE FOR ERROR ANALYSIS:
		clear_tensor_valid(n_button);
	}
}

void used_toggled_0 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    used_toggled_n (cell, path_str, data, 0);
}
void used_toggled_1 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    used_toggled_n (cell, path_str, data, 1);
}
void used_toggled_2 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    used_toggled_n (cell, path_str, data, 2);
}
void used_toggled_3 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    used_toggled_n (cell, path_str, data, 3);
}

void
used_toggled_n (GtkCellRendererToggle *cell,
    gchar *path_str, gpointer data, int n_button)
{
  GtkTreeModel *model = (GtkTreeModel *)data;
  GtkTreeIter  iter;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
  gboolean used;
  gboolean useable;
  int res_num;
  gchar * atom_name;  
  int index;
  /* get toggled iter */
  gtk_tree_model_get_iter (model, &iter, path);
  gtk_tree_model_get (model, &iter, COLUMN_USED, &used, -1);
  gtk_tree_model_get (model, &iter, COLUMN_USEABLE, &useable, -1);
  gtk_tree_model_get (model, &iter, COLUMN_ATOM_NAME, &atom_name, -1);
  gtk_tree_model_get (model, &iter, COLUMN_RESIDUE_NUM, &res_num, -1);  
  index = find_in_pcs(res_num, atom_name, _all_pcs[n_button]);
  if(index == -1){
      return;
  }
  /* do something with the value */
  if(useable == TRUE){
    used ^= 1;
	  // THE PCS HAS BEEN CHANGED, PREVIOUSLY CLACULATED TENSOR
	//ARE NOT VALID ANYMORE FOR ERROR ANALYSIS:
	clear_tensor_valid(n_button);
  } else{
      used = FALSE;
  }
  /* set new value */
  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_USED, used, -1);
  g_array_index (_all_pcs[n_button], Item_pcs, index).used = used;
  /* clean up */
  gtk_tree_path_free (path);
}


//~ Since the column is created before the model, the signal about
//~ the modification of the list must be connected after the creation of the model,
//~ by this function for the toogle.
void connect_signal_columns(GtkTreeModel * model, GtkTreeView *treeview, int n_button){
    GList * list_column;
    const gchar * title;
    GtkTreeViewColumn * column;
    guint i;

    list_column = gtk_tree_view_get_columns (treeview);
    for(i = 0; i < g_list_length(list_column); i++){
        column = (GtkTreeViewColumn *) g_list_nth_data (list_column, i);
        title = gtk_tree_view_column_get_title(column);
        if(strcmp(title, TITLE_COLUMN_USED)==0){
            connect_signal_column(model, column, col_used, n_button);
        }
        if(strcmp(title, TITLE_COLUMN_PCS_EXP)==0){
            connect_signal_column(model, column, col_exp, n_button);
        }
        if(strcmp(title, TITLE_COLUMN_PCS_TOL)==0){
            connect_signal_column(model, column, col_tol, n_button);
        }
    }
    g_list_free(list_column);
}


void 
connect_signal_column(GtkTreeModel * model, GtkTreeViewColumn * column, int title_type, int n_button){
    static int k_toggle_static;
    static int k_exp_static;
    static int k_tol_static;
    static gulong handler_toggle_id[N_PCS_ARRAY];
    static gulong handler_exp_id[N_PCS_ARRAY];
    static gulong handler_tol_id[N_PCS_ARRAY];
    GList * list_renderer;
    GtkCellRenderer *renderer; 
    int k;
    void (*to_connect_used) (GtkCellRendererToggle *, gchar *,gpointer);
    void (*to_connect_edited) (GtkCellRendererText *, const gchar *,
             const gchar *, gpointer);
    list_renderer = gtk_tree_view_column_get_cell_renderers (column);
    if(g_list_length(list_renderer)==0){
        g_list_free(list_renderer);
        return;
    }
    renderer = (GtkCellRenderer *) g_list_nth_data(list_renderer, 0);
    
    switch (n_button){
        case 0:{
        to_connect_used = used_toggled_0;
        to_connect_edited = cell_edited_0;   
        }
        break;
        case 1:{
        to_connect_used = used_toggled_1;
        to_connect_edited = cell_edited_1;
        }
        break;
        case 2:{
        to_connect_used = used_toggled_2;
        to_connect_edited = cell_edited_2;
        }
        break;
        case 3:{
        to_connect_used = used_toggled_3;
        to_connect_edited = cell_edited_3;
        }
        break;
        default:{
            return;
        }
        break;
    }
    switch(title_type){
        case col_used:{ //USED CONNECTION TO SIGNAL
            for(k = 0; k < N_PCS_ARRAY; k++){
              if(k != n_button){
                  continue;
              }
              if(handler_toggle_id[k] == 0){
                  continue;
              }
              if (g_signal_handler_is_connected (renderer, handler_toggle_id[k])){
                  g_signal_handler_disconnect (renderer, handler_toggle_id[k]);
                handler_toggle_id[k] = g_signal_connect (renderer, "toggled",
                             G_CALLBACK (to_connect_used), model);
                g_list_free(list_renderer);
                return;
              }
            }
            handler_toggle_id[k_toggle_static] = g_signal_connect (renderer, "toggled",
                             G_CALLBACK (to_connect_used), model);
            k_toggle_static++;
            k_toggle_static = k_toggle_static % N_PCS_ARRAY; //NEW
            g_list_free(list_renderer);
            return;
        }
        break;        
        case col_exp:{ //EXPERIMENTAL PCS CONNECTION TO SIGNAL
            for(k = 0; k < N_PCS_ARRAY; k++){
              if(handler_exp_id[k] == 0){
                  continue;
              }
              if (g_signal_handler_is_connected (renderer, handler_exp_id[k])){
                  g_signal_handler_disconnect (renderer, handler_exp_id[k]);
                handler_exp_id[k] = g_signal_connect (renderer, "edited",
                             G_CALLBACK (to_connect_edited), model);
                g_list_free(list_renderer);
                return;
              }
            }
            handler_exp_id[k_exp_static] = g_signal_connect (renderer, "edited",
                             G_CALLBACK (to_connect_edited), model);
            k_exp_static++;
            k_exp_static = k_exp_static % N_PCS_ARRAY; 
            g_list_free(list_renderer);
            return;
        }
        break;
        case col_tol:{ //TOLERANCE PCS CONNECTION TO SIGNAL
            for(k = 0; k < N_PCS_ARRAY; k++){
              if(handler_tol_id[k] == 0){
                  continue;
              }
              if (g_signal_handler_is_connected (renderer, handler_tol_id[k])){
                  g_signal_handler_disconnect (renderer, handler_tol_id[k]);
                handler_tol_id[k] = g_signal_connect (renderer, "edited",
                             G_CALLBACK (to_connect_edited), model);
                g_list_free(list_renderer);
                return;
              }
            }
            handler_tol_id[k_tol_static] = g_signal_connect (renderer, "edited",
                             G_CALLBACK (to_connect_edited), model);
            k_tol_static++;
            k_tol_static = k_tol_static % N_PCS_ARRAY; 
            g_list_free(list_renderer);
            return;
        }
        break;        
    }
}

static void  
change_column_properties(GtkTreeViewColumn *column){
  gtk_tree_view_column_set_reorderable (GTK_TREE_VIEW_COLUMN (column),
				   TRUE);
  gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN (column),
				   TRUE);
  gtk_tree_view_column_set_sort_indicator (GTK_TREE_VIEW_COLUMN (column),
				   FALSE);
}

static void polish_float_cell_data_function (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
    
     double value;
     gint column_id;
     gchar buf[40];
    column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (renderer), "column"));
    gtk_tree_model_get(model, iter, column_id, &value, -1);
    if(value == EXEPTION_VALUE){
        g_snprintf(buf, sizeof(buf), "?");
    }
    else{
        g_snprintf(buf, sizeof(buf), "%.3f", value);
    }
    g_object_set(renderer, "text", buf, NULL);
    
}

// NOT YET USED, IT IS IF I USE FLOAT FOR ATOM NUM....

static void polish_atom_num_cell_data_function (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
    int value;
    int atom_num;
    int n_button;
	int * n_but;
    gint column_id;
    gchar buf[20];
    n_but = (int *) (user_data);
	n_button = *n_but;
    column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (renderer), "column"));
    gtk_tree_model_get(model, iter, column_id, &value, -1);
    gtk_tree_model_get(model, iter, COLUMN_ATOM_NUM, &atom_num, -1);
    if(is_atom_dumby(atom_num, n_button)){
        value = value - DUMMY_OFFSET;
        g_snprintf(buf, sizeof(buf), "%d_d", (int) value);
        g_object_set(renderer, "text", buf, NULL);
    }
}


// Display special colors for methyl atoms and dumby atoms.
static void polish_atom_name_cell_data_function (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
    char * name;
    gint column_id;
	int * n_but;
    int n_button;
    int atom_num;
	n_but = (int *) (user_data);
	n_button = *n_but;
    column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (renderer), "column"));
    gtk_tree_model_get(model, iter, column_id, &name, -1);
    gtk_tree_model_get(model, iter, COLUMN_ATOM_NUM, &atom_num, -1);
    if( is_atom_dumby(atom_num, n_button) == TRUE){
        g_object_set (renderer, "cell-background", "#B3BEFB", NULL);
        g_object_set (renderer, "cell-background-set", TRUE, NULL);
    }
    else{
        if( is_atom_methyl(atom_num, n_button) == TRUE ){
            g_object_set (renderer, "cell-background", "#CFFCCB", NULL);
            g_object_set (renderer, "cell-background-set", TRUE, NULL);
        }
        else{
            g_object_set (renderer, "cell-background-set", FALSE, NULL);
        }
    }
    g_free(name);
}

//add the columns in the treeview (but no population here)
static void
add_columns_pcs (GtkTreeView *treeview, int n_button)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  int * n_but;
	n_but = malloc(sizeof(int));
	*n_but = n_button;
  /* column for used toggles */
  renderer = gtk_cell_renderer_toggle_new ();
		g_object_set (G_OBJECT (renderer), "indicator-size", TICK_SIZE, NULL);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_USED)); 
  column = gtk_tree_view_column_new_with_attributes (TITLE_COLUMN_USED,
						     renderer,
						     "active", COLUMN_USED,
						     NULL);
  gtk_tree_view_append_column (treeview, column);

  /* column for atom_name */
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_ATOM_NAME));   
  column = gtk_tree_view_column_new_with_attributes ("Atom",
						     renderer,
						     "text",
						     COLUMN_ATOM_NAME,
						     NULL);
  change_column_properties(column);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_ATOM_NAME);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_atom_name_cell_data_function, (gpointer) (n_but), NULL);
  gtk_tree_view_append_column (treeview, column);
    
  /* column for residue_name */
  renderer = gtk_cell_renderer_text_new ();  
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_RESIDUE_NAME));  
  column = gtk_tree_view_column_new_with_attributes ("Residue",
						     renderer,
						     "text",
						     COLUMN_RESIDUE_NAME,
						     NULL);
  change_column_properties(column);  
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_RESIDUE_NAME);
  gtk_tree_view_append_column (treeview, column);

  /* column for atom_num */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_ATOM_NUM));  
  column = gtk_tree_view_column_new_with_attributes ("#Atom",
						     renderer,
						     "text",
						     COLUMN_ATOM_NUM,
						     NULL);
  change_column_properties(column);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_ATOM_NUM);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_atom_num_cell_data_function, (gpointer)(n_but), NULL); 
  gtk_tree_view_append_column (treeview, column);

  /* column for residue_num */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_RESIDUE_NUM));  
  column = gtk_tree_view_column_new_with_attributes ("#Residue",
						     renderer,
						     "text",
						     COLUMN_RESIDUE_NUM,
						     NULL);
  change_column_properties(column);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_RESIDUE_NUM);
  gtk_tree_view_append_column (treeview, column);
    
		
  /* column for pcs_exp */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set (renderer,
                "editable", TRUE,
                NULL);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_PCS_EXP));  
  column = gtk_tree_view_column_new_with_attributes (TITLE_COLUMN_PCS_EXP,
						     renderer,
						     "text",
						     COLUMN_PCS_EXP,
						     NULL);

  change_column_properties(column);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_PCS_EXP);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function, NULL, NULL);  
  gtk_tree_view_append_column (treeview, column); 
    
  /* column for pcs_calc */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_PCS_CALC));  
  column = gtk_tree_view_column_new_with_attributes ("PCS calc.",
						     renderer,
						     "text",
						     COLUMN_PCS_CALC,
						     NULL);
      change_column_properties(column);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_PCS_CALC);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function, NULL, NULL); 
  gtk_tree_view_append_column (treeview, column);
		
		    /* column for pcs_tol */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set (renderer,
                "editable", TRUE,
                NULL); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_PCS_TOL));
  column = gtk_tree_view_column_new_with_attributes (TITLE_COLUMN_PCS_TOL,
						     renderer,
						     "text",
						     COLUMN_PCS_TOL,
						     NULL);
  change_column_properties(column);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_PCS_TOL);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function, NULL, NULL); 
  gtk_tree_view_append_column (treeview, column);
		
		    /* column for pcs_dev */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_PCS_DEV));
  column = gtk_tree_view_column_new_with_attributes ("PCS dev.",
						     renderer,
						     "text",
						     COLUMN_PCS_DEV,
						     NULL);
  change_column_properties(column);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_PCS_DEV);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function, NULL, NULL); 
  gtk_tree_view_append_column (treeview, column);
    
      /* column for racs */ //RACS_MOD
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_RACS));  
  column = gtk_tree_view_column_new_with_attributes ("RACS",
						     renderer,
						     "text",
						     COLUMN_RACS,
						     NULL);
  change_column_properties(column);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_RACS);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function, NULL, NULL); 
  gtk_tree_view_append_column (treeview, column);
    
    
    /* column for dist_center */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_DIST_CENTER));  
  change_column_properties(column);
  column = gtk_tree_view_column_new_with_attributes ("Dist. center",
						     renderer,
						     "text",
						     COLUMN_DIST_CENTER,
						     NULL);
      change_column_properties(column);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_DIST_CENTER);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function, NULL, NULL); 
  gtk_tree_view_append_column (treeview, column);
    
}

gint
compare_atom_num_func (GtkTreeModel *model,
                          GtkTreeIter  *a,
                          GtkTreeIter  *b,
                          gpointer      userdata)
{
    gint num1, num2;
    gint num_bis1, num_bis2;

    gtk_tree_model_get(model, a, 0, &num1, -1);
    gtk_tree_model_get(model, b, 0, &num2, -1);
    num_bis1 = num1;
    num_bis2 = num2;
    
    if(num1 >= DUMMY_OFFSET){
        num1 = num1 - DUMMY_OFFSET;
    }
    if(num2 >= DUMMY_OFFSET){
        num2 = num2 - DUMMY_OFFSET;
    }
    if(num1 == num2){
        if(num_bis1 == num_bis2){
            return(0);  
        }
        if(num_bis1 > num_bis2){
            return(-1);  
        }
        return(1);
    }
    if(num1 > num2){
        return(1);
    }
    return(-1);
}

//model to display the PCS list and population
static GtkTreeModel *
create_model (int n_button)
{
  gint i = 0;
  GtkListStore *store;
  GtkTreeIter iter;


  /* create list store */
  store = gtk_list_store_new (
                  NUM_ITEM_COLUMNS,
                  G_TYPE_INT,
			      			G_TYPE_STRING,
                  G_TYPE_INT,
                  G_TYPE_STRING,
			      			G_TYPE_DOUBLE,
			      			G_TYPE_DOUBLE,
                  G_TYPE_DOUBLE,   //RACS_MODE         
			      			G_TYPE_DOUBLE,
                  G_TYPE_DOUBLE,
                  G_TYPE_DOUBLE,            
			      			G_TYPE_BOOLEAN,
                  G_TYPE_BOOLEAN,
                  G_TYPE_BOOLEAN
                              );

  /* add data to the list store */
  if(_all_pcs[n_button] == NULL){
			goto empty_model;
  }
  for (i = 0; i <  _all_pcs[n_button]->len; i++)
    {
       if( g_array_index (_all_pcs[n_button], Item_pcs, i).displayed == FALSE){
           continue;
       }
        double pcs_exp;
        pcs_exp = g_array_index (_all_pcs[n_button], Item_pcs, i).pcs_exp;
        
        gtk_list_store_append (store, &iter);
        gtk_list_store_set (store, &iter,
        COLUMN_ATOM_NUM, g_array_index (_all_pcs[n_button], Item_pcs, i).atom_num,
        COLUMN_ATOM_NAME, g_array_index (_all_pcs[n_button], Item_pcs, i).atom_name,
        COLUMN_RESIDUE_NUM, g_array_index (_all_pcs[n_button], Item_pcs, i).residue_num,
        COLUMN_RESIDUE_NAME, g_array_index (_all_pcs[n_button], Item_pcs, i).residue_name,
        COLUMN_PCS_EXP, g_array_index (_all_pcs[n_button], Item_pcs, i).pcs_exp,
        COLUMN_PCS_CALC, g_array_index (_all_pcs[n_button], Item_pcs, i).pcs_calc,
        COLUMN_RACS, g_array_index (_all_pcs[n_button], Item_pcs, i).racs,
        COLUMN_PCS_DEV, g_array_index (_all_pcs[n_button], Item_pcs, i).pcs_dev,
        COLUMN_DIST_CENTER, g_array_index (_all_pcs[n_button], Item_pcs, i).dist_center,
        COLUMN_PCS_TOL, g_array_index (_all_pcs[n_button], Item_pcs, i).pcs_tol,
        COLUMN_USED, g_array_index (_all_pcs[n_button], Item_pcs, i).used,
        COLUMN_USEABLE, g_array_index (_all_pcs[n_button], Item_pcs, i).useable,
        COLUMN_DISPLAYED, g_array_index (_all_pcs[n_button], Item_pcs, i).displayed,
			  -1);
    }
    
		empty_model:
  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), COLUMN_ATOM_NUM,      
                        compare_atom_num_func, GINT_TO_POINTER(COLUMN_ATOM_NUM), NULL);    
  return GTK_TREE_MODEL (store);
}

// attache the model to the columns of treeview
// Once the pdb has been correctly read, this is the function to call
void display_pcs_list (int n_button, int serial_tensor)
{
    GtkTreeModel *model;
    GtkWidget *treeview;
    GladeXML *gxml;
    char tree_char[50];
    int i;
    gxml = glade_get_widget_tree(_window);
    if(_all_pcs[n_button] == NULL){
        return;
    }
    for(i = 0; i < _all_pcs[n_button]->len; i++){
        if(pass_filter_display(g_array_index (_all_pcs[n_button], Item_pcs , i).atom_name,
                               g_array_index (_all_pcs[n_button], Item_pcs , i).residue_name,
                               n_button, i) == 1){
            g_array_index (_all_pcs[n_button], Item_pcs , i).displayed = TRUE;
        }
        else{
            g_array_index (_all_pcs[n_button], Item_pcs , i).displayed = FALSE;
        }
    }
    sprintf(tree_char, "treeview_PCS%d", n_button+1);
		treeview = glade_xml_get_widget (gxml, tree_char);
      /* create tree model */
      model = create_model (n_button);
      /* connect signal */
      connect_signal_columns((GtkTreeModel *) model, (GtkTreeView *)treeview, n_button);
      /* create tree view */
      gtk_tree_view_set_model ((GtkTreeView *)treeview, GTK_TREE_MODEL (model));
      gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
      gtk_tree_view_set_search_column (GTK_TREE_VIEW (treeview),
				       COLUMN_ATOM_NUM);
      g_object_unref (G_OBJECT (model));
	switch(serial_tensor){
		case -1:{
			break;
		}
		case -2:{
			char status_name[50];
			GtkStatusbar *statusbar;
			sprintf(status_name, "statusbar_PCS%d", n_button+1);
			statusbar = (GtkStatusbar *) glade_xml_get_widget (gxml, status_name);
			gtk_statusbar_pop (statusbar, 1);
			gtk_statusbar_push (statusbar, 1, "");
			break;
		}
		default:{
			char txt[100];
			char status_name[50];
			GtkStatusbar *statusbar;
			sprintf(status_name, "statusbar_PCS%d", n_button+1);
			statusbar = (GtkStatusbar *) glade_xml_get_widget (gxml, status_name);
			gtk_statusbar_pop (statusbar, 1);
			sprintf(txt, "PCS calculated with tensor id: %d", serial_tensor);
			gtk_statusbar_push (statusbar, 1, txt);
			break;
		}
	}
}

int pass_filter_select(char * atom_name, char * residue_name, int n_button, int 
index_in_pcs){
    int i;
    gboolean bool_1, bool_2;
    bool_1 = bool_2 = FALSE;
    GArray * arr;
    arr = _all_pcs[n_button];
    //if not useable, we want to return -1 anyway
    if(g_array_index (arr, Item_pcs, index_in_pcs).useable == FALSE){
        return(-1);
    }

    for (i = 0; i < _pdb_str->n_atom_type; i++){
        if((_select_display_pcs [n_button][SELECT_ATOM][i] == TRUE)&&
           (strcmp(_pdb_str->tab_atom_type[i], atom_name)== 0)){
            bool_1 = TRUE;
        }
    }    
    for (i = 0; i < _pdb_str->n_residue_type; i++){
        if((_select_display_pcs [n_button][SELECT_RESIDUE][i] == TRUE)&&
           (strcmp(_pdb_str->tab_residue_type[i], residue_name)== 0)){
            bool_2 = TRUE;
        }
    }
    if(bool_1 && bool_2){
        return(1);
    }
    //Ok, now we deal separatly with dummy atom cases.
    if(g_array_index (arr, Item_pcs, index_in_pcs).is_dumby == TRUE){
        for (i = 0; i < _pdb_str->n_atom_type; i++){
            if((_select_display_pcs [n_button][SELECT_ATOM][i] == TRUE)&&
            (strcmp(_pdb_str->tab_atom_type[i], DUMMY_NAME)== 0)){
                return(1);
            }
        } 
    }
    return(-1);
}

int pass_filter_display(char * atom_name, char * residue_name, int n_button, int 
index_in_pcs){
    int i;
    gboolean bool_1, bool_2;
    bool_1 = bool_2 = FALSE;
    for (i = 0; i < _pdb_str->n_atom_type; i++){
        if((_select_display_pcs [n_button][DISPLAY_ATOM][i] == TRUE)&&
           (strcmp(_pdb_str->tab_atom_type[i], atom_name)== 0)){
            bool_1 = TRUE;
        }
    }    
    for (i = 0; i < _pdb_str->n_residue_type; i++){
        if((_select_display_pcs [n_button][DISPLAY_RESIDUE][i] == TRUE)&&
           (strcmp(_pdb_str->tab_residue_type[i], residue_name)== 0)){
            bool_2 = TRUE;
        }
    }
    if(bool_1 && bool_2){
        return(1);
    }
    
    //If the PCS is used, we want to display it anyway!
    if(g_array_index (_all_pcs[n_button], Item_pcs , index_in_pcs).used == TRUE){
        return(1);
    }
    
        //Ok, now we deal separatly with dummy atom cases.
    bool_1 = FALSE;
    if(g_array_index (_all_pcs[n_button], Item_pcs, index_in_pcs).is_dumby == TRUE){
        for (i = 0; i < _pdb_str->n_atom_type; i++){
            if((_select_display_pcs [n_button][DISPLAY_ATOM][i] == TRUE)&&
            (strcmp(_pdb_str->tab_atom_type[i], DUMMY_NAME)== 0)){
                bool_1 = TRUE;
            }
        } 
    }
    if(bool_1 && bool_2){
        return(1);
    }   
    return(-1);
}


void make_not_methyl(int atom_num, int n_button){
    int i;
    if(atom_num < 0){
        return;
    }
    //fprintf(stderr, "unmake methyl %d\n", atom_num);
    for(i = 0; i < _all_pcs[n_button]->len; i++){
        if(g_array_index (_all_pcs[n_button], Item_pcs, i).atom_num == atom_num){
           g_array_index (_all_pcs[n_button], Item_pcs, i).is_methyl = FALSE;
           g_array_index (_all_pcs[n_button], Item_pcs, i).methyl_1 = -1;
           g_array_index (_all_pcs[n_button], Item_pcs, i).methyl_2 = -1;
           g_array_index (_all_pcs[n_button], Item_pcs, i).methyl_3 = -1;
        }
    }
}

//~ update_pcs_list look at _str_pcs[n_button]
//~ and update the _all_pcs according to the new file of pcs loaded,
//~ or according to the methyl group association.
//~ so it is called only when a pcs is loaded, or when a methyl
//~ group is loaded
//~ if a new PCS is loaded, overwrite will be true
//~ if methyl group association modified, overwrite will be false
void update_pcs_list(int n_button, gboolean overwrite){
    int i;
    int index;
    int ind;
    int p;
    Item_pcs dumby;
    char * str_pcs;
    Item_pcs * foo; //ref to a _all_pcs Item
    Item_pcs * foo2;
    int len;
	
	//0.if overwrite is TRUE, reset all pcs and exp and calc to zero.
	if(overwrite == TRUE){
		for(i = 0; i < _all_pcs[n_button]->len; i++){
			{
				foo = &g_array_index(_all_pcs[n_button], Item_pcs, i);
				(*foo).pcs_exp = EXEPTION_VALUE;
				(*foo).pcs_tol = 0.0;
				(*foo).pcs_dev = EXEPTION_VALUE;
				(*foo).useable = FALSE;
				(*foo).used = FALSE;
				(*foo).displayed = FALSE;
				(*foo).pcs_calc = EXEPTION_VALUE;
				(*foo).dist_center = EXEPTION_VALUE;
                (*foo).racs = EXEPTION_VALUE; //RACS_MOD
			}
		}
	}
        
    //1. remove all dumby atoms where the methyl definition has been changed
    for(i = 0; i < _all_pcs[n_button]->len; i++){
        if(g_array_index (_all_pcs[n_button], Item_pcs, i).is_dumby == TRUE){
            if(is_name_dumby(g_array_index (_all_pcs[n_button], Item_pcs, i).atom_name)){
                //continue;
            }
            if(is_intact_dumby(g_array_index (_all_pcs[n_button], Item_pcs, i).atom_name)){
                continue;
            }
            g_array_remove_index(_all_pcs[n_button], i);
            i = i - 1; //because we removed one atom

        }
    }

    for(i = 0; i < _all_pcs[n_button]->len; i++){
        foo = &g_array_index(_all_pcs[n_button], Item_pcs, i);
        if((*foo).is_dumby == TRUE){
            (*foo).is_methyl= FALSE;
            (*foo).methyl_1 = -1;
            (*foo).methyl_2 = -1;
            (*foo).methyl_3 = -1;
        }
    }
    //2. add the new dumby atom and update the _all_pcs
    // according to methyl
    len = _all_pcs[n_button]->len; //Because we don't want to deal with newly inserted atoms!
    for(i = 0; i < len; i++){
        foo = &g_array_index(_all_pcs[n_button], Item_pcs, i);
        if((*foo).is_dumby == TRUE){
        }
        else{
           (*foo).is_dumby = FALSE; //useless
           (*foo).is_methyl = FALSE;
        
        (*foo).methyl_1 = -1;
        (*foo).methyl_2 = -1;
        (*foo).methyl_3 = -1;
        }
        str_pcs = NULL;
        p = what_methyl_position((*foo).atom_name, &str_pcs);
        if(str_pcs != NULL){
            switch(p){ //it is a methyl, let's do actions
                case 1: {
                    (*foo).methyl_1 =  0;
                    break;
                }
                case 2: {
                    (*foo).methyl_2 =  0;
                    break;
                }
                case 3: {
                    (*foo).methyl_3 =  0;
                    break;
                }
            }
            if(p > 0){ //it is a methyl, let's do actions
                (*foo).is_methyl = TRUE;
                ind = find_in_pcs((*foo).residue_num, str_pcs, _all_pcs[n_button]);
                if (ind == -1){ //not found : create a new dumby
                    dumby.atom_num = (*foo).atom_num + DUMMY_OFFSET;
                    dumby.residue_num = (*foo).residue_num;
                    strcpy(dumby.atom_name, str_pcs);
                    strcpy(dumby.residue_name, (*foo).residue_name);
                    dumby.useable = FALSE;
                    dumby.used = FALSE;
                    dumby.displayed = TRUE;
                    dumby.pcs_exp = EXEPTION_VALUE;
                    dumby.pcs_calc = EXEPTION_VALUE;
                    dumby.pcs_tol = 0.0;
                    dumby.pcs_dev = EXEPTION_VALUE;
                    dumby.dist_center = EXEPTION_VALUE;
                    dumby.is_dumby = TRUE;
                    dumby.is_methyl = FALSE;
                    dumby.methyl_1 = -1;
                    dumby.methyl_2 = -1;
                    dumby.methyl_3 = -1;
                    dumby.racs = EXEPTION_VALUE;
                    dumby.is_racs = FALSE;
                    dumby.racs_O = RACS_INVALID;
                    dumby.racs_C = RACS_INVALID;
                    dumby.racs_N = RACS_INVALID;
                    dumby.racs_H = RACS_INVALID;
                    dumby.racs_CA = RACS_INVALID;
                    dumby.racs_type = RACS_INVALID;
                    switch(p){
                        //IF I NEED A POINTER FROM METHYL TO DUMBY: HERE!
                        case 1: {
                        dumby.methyl_1 = i;//(*foo).atom_num;
                        break;
                        }
                        case 2: {
                        dumby.methyl_2 = i;//(*foo).atom_num;
                        break;
                        }
                        case 3: {
                        dumby.methyl_3 = i;//(*foo).atom_num;
                        break;
                        }
                    }
                    g_array_append_vals (_all_pcs[n_button], &dumby, 1);

                }
                else{
                    foo2 = &g_array_index(_all_pcs[n_button], Item_pcs, ind);
                    switch(p){
                        //IF I NEED A POINTER FROM METHYL TO DUMBY: HERE!
                        case 1: {
                            (*foo2).methyl_1 = i;//(*foo).atom_num;
                            break;
                        }
                        case 2: {
                            (*foo2).methyl_2 = i;//(*foo).atom_num;
                            break;
                        }
                        case 3: {
                            (*foo2).methyl_3 = i;//(*foo).atom_num;
                            break;
                        }
                    }
                }
            }
        }
    }
    
    
    
    //3. remove incomplete dumby atoms and update corresponding "wrong" methyl
    for(i = 0; i < _all_pcs[n_button]->len; i++){
        foo = &g_array_index(_all_pcs[n_button], Item_pcs, i);
        if((*foo).is_dumby == TRUE){ //IT IS A DUMBY...
            if(((*foo).methyl_1 == -1) ||
               ((*foo).methyl_2 == -1) ||
               ((*foo).methyl_3 == -1)){ //IT IS INCOMPLETE dumby, REMOVE!!!!
                    int atom_num_methyl;
                    if((*foo).methyl_1 !=-1){
                        atom_num_methyl = _pdb_str->tab_atom[(*foo).methyl_1].atom_num;
                        make_not_methyl(atom_num_methyl, n_button);
                    }
                    if((*foo).methyl_2 !=-1){
                        atom_num_methyl = _pdb_str->tab_atom[(*foo).methyl_2].atom_num;
                        make_not_methyl(atom_num_methyl, n_button);
                    }
                    if((*foo).methyl_3 !=-1){
                        atom_num_methyl = _pdb_str->tab_atom[(*foo).methyl_3].atom_num;
                        make_not_methyl(atom_num_methyl, n_button);
                    }
                    g_array_remove_index(_all_pcs[n_button], i);
                    i = i - 1; //because we removed one atom
               }
        }
    }
    
    //4 populate with pcs_file value (if OVERWRITE is TRUE)
    if(overwrite == TRUE){
		int total_PCS;
		total_PCS = 0;
        if(_str_pcs[n_button] == NULL){
            return;
        }
        for(i = 0; i < _str_pcs[n_button]->len; i++){
            index = find_in_pcs(g_array_index (_str_pcs[n_button], experiment_pcs, i).residue_num
                        ,g_array_index (_str_pcs[n_button], experiment_pcs, i).atom_name,
                            _all_pcs[n_button]);
            if(index < 0){
				fprintf(stderr, "\tWARNING residue *%d* atom *%s* not found in PDB file\n",
						g_array_index (_str_pcs[n_button], experiment_pcs, i).residue_num,
						g_array_index (_str_pcs[n_button], experiment_pcs, i).atom_name
						);
				fprintf(stderr, "\tIf this is a \"CH3\" PCS, you will have to define the methyl group in Numbat interface\n");               
                continue;
            }
			total_PCS ++;
            foo = &g_array_index(_all_pcs[n_button], Item_pcs, index);
            //if((overwrite)||((*foo).pcs_exp == EXEPTION_VALUE)){
            foo->pcs_exp = g_array_index (_str_pcs[n_button], experiment_pcs, i).pcs_value;
			foo->pcs_tol = g_array_index (_str_pcs[n_button], experiment_pcs, i).pcs_tolerance; //

            //}
            //DANGER about methyl, really?
            if(((*foo).pcs_exp != EXEPTION_VALUE))
            {
                foo->used = TRUE;
                foo->useable = TRUE;
            }
            //ARE YOU SURE???
            else{
                foo->used = FALSE;
                foo->useable = FALSE;
            }
            //END ARE YOU SURE
        }
		fprintf(stderr, "%d PCS have been loaded in the interface\n", total_PCS);
    }
    else{ //overwritte is FALSE (methyl update)
        //We don't want to touch any PCS exept methyl and dumby!
        //Methyl:   1. don't touch exp
        //          2. used = FALSE (whatever useable
        //
        //Dumby:    1. update exp if possible and if it is EXEPTION_VALUE
        //          2. used = TRUE if exp != EXEPTION_VALUE
        //
        if(_str_pcs[n_button] == NULL){
            return;
        }
        for(i = 0; i < _str_pcs[n_button]->len; i++){
            index = find_in_pcs(g_array_index (_str_pcs[n_button], experiment_pcs, i).residue_num
                        ,g_array_index (_str_pcs[n_button], experiment_pcs, i).atom_name,
                            _all_pcs[n_button]);
            if(index < 0){
                continue;
            }
            foo = &g_array_index(_all_pcs[n_button], Item_pcs, index);
            
            if(foo->is_dumby ==  TRUE){
                foo->pcs_exp = g_array_index (_str_pcs[n_button], experiment_pcs, i).pcs_value;
				foo->pcs_tol = g_array_index (_str_pcs[n_button], experiment_pcs, i).pcs_tolerance;//

                if(foo->pcs_exp != EXEPTION_VALUE){
                    foo->used = TRUE;
                    foo->useable = TRUE;
                }
                else{
                    foo->used = FALSE;
                    foo->useable = FALSE;
                }
            }
            if(foo->is_methyl ==  TRUE){
                foo->used = FALSE;
                foo->useable = FALSE;
                if(foo->pcs_exp != EXEPTION_VALUE){
                    foo->useable = TRUE;
                }
            }
        }
    }
}

//~ return the indice inside the pcs_array of the looked atom
//~ or -1 if not found
int find_in_pcs(int res_num, char * atom_name, GArray * pcs_array){
    int i;
    if(pcs_array == NULL){
        return(-2);
    }
    for(i = 0; i < pcs_array->len; i++){
        if(g_array_index(pcs_array, Item_pcs, i).residue_num == res_num){
            if(strcmp(g_array_index(pcs_array, Item_pcs, i).atom_name, atom_name) == 0){
                return(i);
            }
        }
    }
    return(-1);
}

// if the name given is dumby, return TRUE
gboolean is_name_dumby(char * name){
    int i;
    char * a4;
    if(_str_CH3 == NULL){
        return(FALSE);
    }
    for(i = 0; i < _str_CH3->len; i++){
        a4 = g_array_index (_str_CH3, structure_CH3 , i).atom_pcs;
        if(strcmp(a4, name) == 0){
            return(TRUE);
        }
    }
    return(FALSE);
}

// if the atom given is dumby, return TRUE
gboolean is_atom_dumby(int atom_num, int n_button){
    int i;
    if(_all_pcs[n_button] == NULL){
        return(FALSE);
    }
    for(i = 0; i < _all_pcs[n_button]->len; i++){
        if(g_array_index (_all_pcs[n_button], Item_pcs, i).atom_num == atom_num){
            return(g_array_index (_all_pcs[n_button], Item_pcs, i).is_dumby);
        }
    }
    return(FALSE);
}

// if the atom given is methyl, return TRUE
gboolean is_atom_methyl(int atom_num, int n_button){
    int i;
    if(_all_pcs[n_button] == NULL){
        return(FALSE);
    }
    for(i = 0; i < _all_pcs[n_button]->len; i++){
        if(g_array_index (_all_pcs[n_button], Item_pcs, i).atom_num == atom_num){
            return(g_array_index (_all_pcs[n_button], Item_pcs, i).is_methyl);
        }
    }
    return(FALSE);
}



// if the name given is a newly defined dumby, return FALSE
gboolean is_intact_dumby(char * name){
    int i;
    char * a4;
    if(_str_CH3 == NULL){
        return(FALSE);
    }
    for(i = 0; i < _str_CH3->len; i++){
        a4 = g_array_index (_str_CH3, structure_CH3 , i).atom_pcs;
        if(strcmp(a4, name) == 0){
            return(g_array_index (_str_CH3, structure_CH3 , i).intact);
        }
    }
    return(FALSE);
}

//~ will find if it is methyl 1, 2 or 3 in the methyl assocation structure_CH3
//~ and also update the reference pcs_name to the correct value
int what_methyl_position(char * atom_name, char ** pcs_name){
    int i;
    char * a1;
    char * a2;
    char * a3;
    char * a4;
    *pcs_name = NULL;
    if(_str_CH3 == NULL){
        return(-1);
    }
    for(i = 0; i < _str_CH3->len; i++){
        a1 = g_array_index (_str_CH3, structure_CH3 , i).atom1;
        a2 = g_array_index (_str_CH3, structure_CH3 , i).atom2;
        a3 = g_array_index (_str_CH3, structure_CH3 , i).atom3;
        a4 = g_array_index (_str_CH3, structure_CH3 , i).atom_pcs;
        if((strcmp(a1, atom_name)) == 0){
            *pcs_name = a4;
            return(1);
        }
        if((strcmp(a2, atom_name)) == 0){
            *pcs_name = a4;
            return(2);
        }
        if((strcmp(a3, atom_name)) == 0){
            *pcs_name = a4;
            return(3);
        }
    }
    return(-1);
}

void populate_n_button_pcs(int n_button){
    int j;
    Item_pcs foo;
    if(_all_pcs[n_button] != NULL){
        g_array_free(_all_pcs[n_button], TRUE);
    }
    _all_pcs[n_button]= g_array_sized_new (FALSE, FALSE, sizeof (Item_pcs), 1);
    for(j = 0; j < _pdb_str->n_atom; j++){
        foo.atom_num = _pdb_str->tab_atom[j].atom_num;
        foo.residue_num = _pdb_str->tab_atom[j].residue_num;
        strcpy(foo.atom_name, _pdb_str->tab_atom[j].atom_name);
        strcpy(foo.residue_name, _pdb_str->tab_atom[j].residue_name);
        foo.useable = FALSE;
        foo.used = FALSE;
        foo.displayed = TRUE;
        foo.pcs_exp = EXEPTION_VALUE;
        foo.pcs_calc = EXEPTION_VALUE;
        foo.pcs_tol = 0.0;
        foo.pcs_dev = EXEPTION_VALUE;
        foo.dist_center = EXEPTION_VALUE;
        //ADDENDA FOR METHYL
        foo.is_dumby = FALSE;
        foo.is_methyl = FALSE;
        foo.methyl_1 = -1;
        foo.methyl_2 = -1;
        foo.methyl_3 = -1;
        foo.racs = EXEPTION_VALUE; //RACS_MOD
        foo.is_racs = FALSE;
        foo.racs_O = RACS_INVALID;
        foo.racs_C = RACS_INVALID;
        foo.racs_N = RACS_INVALID;
        foo.racs_H = RACS_INVALID;
        foo.racs_CA = RACS_INVALID;
        foo.racs_type = RACS_INVALID;
        g_array_append_vals (_all_pcs[n_button], &foo, 1);
    }
    if(_str_CH3 == NULL){
        return;
    }
}


//~ only called when a new pdb is loaded!
//~ this function create a new array to store all pcs (of course, not experimental at the begining)
//~ and populate it for all atom of the pdb.
void populate_all_pcs(void){
    int i;

    for(i = 0; i< N_PCS_ARRAY; i++){
        populate_n_button_pcs(i);
    }
    
}

int make_link_for_racs_n_button(int n_button, racs_atom_name* racs_atom_na){
    int i;
    Item_pcs *foo;
    int index1, index2, index3;
    
    if(_all_pcs[n_button] == NULL){
        return(-1);
    }

    for(i = 0; i < _all_pcs[n_button]->len; i++){
        foo = &g_array_index(_all_pcs[n_button], Item_pcs, i);
        (*foo).is_racs = FALSE;
        if(strcmp((*foo).atom_name, racs_atom_na->atom_C) == 0){//C
            index1 = find_in_pcs((*foo).residue_num, racs_atom_na->atom_O, _all_pcs[n_button]);
            index2 = find_in_pcs((*foo).residue_num + 1, racs_atom_na->atom_N, _all_pcs[n_button]);
            if((index1 != -1)&& (index2 != -1)){
                (*foo).racs_O = index1;
                (*foo).racs_N = index2;
                (*foo).is_racs = TRUE;
                (*foo).racs_type = RACS_FOR_C;
            }
            else{
                (*foo).is_racs = FALSE;
                //(*foo).racs_type = RACS_INVALID;
            }
            continue;
        }
        if(strcmp((*foo).atom_name, racs_atom_na->atom_N) == 0){//N
            index1 = find_in_pcs((*foo).residue_num - 1, racs_atom_na->atom_C, _all_pcs[n_button]);
            index2 = find_in_pcs((*foo).residue_num, racs_atom_na->atom_CA, _all_pcs[n_button]);
            index3 = find_in_pcs((*foo).residue_num, racs_atom_na->atom_H, _all_pcs[n_button]);
            if((index1 != -1)&& (index2 != -1)){
                (*foo).racs_C = index1;
                (*foo).racs_CA = index2;
                (*foo).is_racs = TRUE;
                (*foo).racs_type = RACS_FOR_N_OFFSET;
                if(index3 != -1){
                    (*foo).racs_H = index3;
                    (*foo).racs_type = RACS_FOR_N_NATURAL;
                }
            }
            else{
                (*foo).is_racs = FALSE;
            }
            continue;
        }
        if(strcmp((*foo).atom_name, racs_atom_na->atom_H) == 0){//H
            index1 = find_in_pcs((*foo).residue_num, racs_atom_na->atom_N, _all_pcs[n_button]);
            index2 = find_in_pcs((*foo).residue_num-1, racs_atom_na->atom_C, _all_pcs[n_button]);
            if((index1 != -1)&& (index2 != -1)){
                (*foo).racs_N = index1;
                (*foo).racs_C = index2;
                (*foo).is_racs = TRUE;
                (*foo).racs_type = RACS_FOR_H;
            }
            else{
                (*foo).is_racs = FALSE;
            }
            continue;
        }
    }
    return(1);
}


void precalculate_CSA_tensor_n_button(int n_button, racs_data * racs_d){
    int i;
    Item_pcs *foo;
    int index_racs_C, index_racs_H, index_racs_CA, index_racs_N, index_racs_O;
    int j;
    if(_all_pcs[n_button] == NULL){
        return;
    }
    for(j = 0; j < _all_pdb_selector_pcs[n_button]->len; j++){
        for(i = 0; i < _all_pcs[n_button]->len; i++){
            foo = &g_array_index(_all_pcs[n_button], Item_pcs, i);
            if((*foo).is_racs == TRUE){
            
                double x1, y1, z1, x2, y2, z2, x3, y3, z3, beta, angle_offset;
                int racs_type;
                int compter_check;
                x1= y1= z1= x2= y2= z2= x3= y3= z3= beta= angle_offset = 0.0;
                racs_type = -1;
                compter_check = 0;
            
                index_racs_C = g_array_index(_all_pcs[n_button], Item_pcs, i).racs_C;
                index_racs_H = g_array_index(_all_pcs[n_button], Item_pcs, i).racs_H;
                index_racs_CA = g_array_index(_all_pcs[n_button], Item_pcs, i).racs_CA;
                index_racs_N = g_array_index(_all_pcs[n_button], Item_pcs, i).racs_N;
                index_racs_O = g_array_index(_all_pcs[n_button], Item_pcs, i).racs_O;
            
                if((index_racs_O != RACS_INVALID) && (index_racs_N != RACS_INVALID)){ //C
                    angle_offset = 0.0;
                    racs_type = RACS_FOR_C;
                    beta = racs_d->beta_C;
                
                    x1 = _pdb_str->tab_atom[i].atom_coo[j].x; //C
                    y1 = _pdb_str->tab_atom[i].atom_coo[j].y;
                    z1 = _pdb_str->tab_atom[i].atom_coo[j].z;
                
                    x2 = _pdb_str->tab_atom[index_racs_N].atom_coo[j].x; //N
                    y2 = _pdb_str->tab_atom[index_racs_N].atom_coo[j].y;
                    z2 = _pdb_str->tab_atom[index_racs_N].atom_coo[j].z;
                
                    x3 = _pdb_str->tab_atom[index_racs_O].atom_coo[j].x; //O
                    y3 = _pdb_str->tab_atom[index_racs_O].atom_coo[j].y;
                    z3 = _pdb_str->tab_atom[index_racs_O].atom_coo[j].z;
                    compter_check++;
                }
            
                if((index_racs_N != RACS_INVALID) && (index_racs_C != RACS_INVALID)){ //H
                    angle_offset = 0.0;
                    racs_type = RACS_FOR_H;
                    beta = racs_d->beta_H;
                    
                    x1 = _pdb_str->tab_atom[i].atom_coo[j].x; //H
                    y1 = _pdb_str->tab_atom[i].atom_coo[j].y;
                    z1 = _pdb_str->tab_atom[i].atom_coo[j].z;
                    
                    x2 = _pdb_str->tab_atom[index_racs_N].atom_coo[j].x; //N
                    y2 = _pdb_str->tab_atom[index_racs_N].atom_coo[j].y;
                    z2 = _pdb_str->tab_atom[index_racs_N].atom_coo[j].z;
                    
                    x3 = _pdb_str->tab_atom[index_racs_C].atom_coo[j].x; //C
                    y3 = _pdb_str->tab_atom[index_racs_C].atom_coo[j].y;
                    z3 = _pdb_str->tab_atom[index_racs_C].atom_coo[j].z;
                    compter_check++;
                    } 
                    
                if((index_racs_C != RACS_INVALID) && (index_racs_H != RACS_INVALID)){ //N_natural
                    angle_offset = 0.0;
                    racs_type = RACS_FOR_N_NATURAL;
                    beta = racs_d->beta_N;
                    
                    x1 = _pdb_str->tab_atom[i].atom_coo[j].x; //N
                    y1 = _pdb_str->tab_atom[i].atom_coo[j].y;
                    z1 = _pdb_str->tab_atom[i].atom_coo[j].z;
                    
                    x2 = _pdb_str->tab_atom[index_racs_H].atom_coo[j].x; //H
                    y2 = _pdb_str->tab_atom[index_racs_H].atom_coo[j].y;
                    z2 = _pdb_str->tab_atom[index_racs_H].atom_coo[j].z;
                    
                    x3 = _pdb_str->tab_atom[index_racs_C].atom_coo[j].x; //C
                    y3 = _pdb_str->tab_atom[index_racs_C].atom_coo[j].y;
                    z3 = _pdb_str->tab_atom[index_racs_C].atom_coo[j].z;
                    
                    compter_check++;
                }
                else{
                    if((index_racs_C != RACS_INVALID) && (index_racs_CA != RACS_INVALID)){ //N_offset
                        angle_offset = 120.47*M_PI/180.0;
                        racs_type = RACS_FOR_N_OFFSET;
                        beta = racs_d->beta_N;
                        
                        x1 = _pdb_str->tab_atom[i].atom_coo[j].x; //N
                        y1 = _pdb_str->tab_atom[i].atom_coo[j].y;
                        z1 = _pdb_str->tab_atom[i].atom_coo[j].z;
                        
                        x2 = _pdb_str->tab_atom[index_racs_CA].atom_coo[j].x; //CA
                        y2 = _pdb_str->tab_atom[index_racs_CA].atom_coo[j].y;
                        z2 = _pdb_str->tab_atom[index_racs_CA].atom_coo[j].z;
                        
                        x3 = _pdb_str->tab_atom[index_racs_C].atom_coo[j].x; //C
                        y3 = _pdb_str->tab_atom[index_racs_C].atom_coo[j].y;
                        z3 = _pdb_str->tab_atom[index_racs_C].atom_coo[j].z;
                        compter_check++;
                    }
                }
                    
                if(racs_type == -1){
                    fprintf (stderr, "problem with racs_type\n");
                    return;
                }
            
                if(compter_check != 1){
                    fprintf(stderr, "PROBLEm WITh CSA_matrix pre-calculation\n");
                    return;
                }
                else{
                    double CSA_matrix[3][3];
                
                    int result_csa;
                    result_csa = calculate_CSA_matrix(CSA_matrix,
                                            x1, y1, z1, 
                                            x2, y2, z2, 
                                            x3, y3, z3,
                                            beta, angle_offset, racs_type);
                    if(result_csa != -1){
                        atom_coordinate * foo_coo;
                        
                        foo_coo = &(_pdb_str->tab_atom[i].atom_coo[j]);
                        
                        (*foo_coo).CSA_tensor[0][0] = CSA_matrix[0][0];
                        (*foo_coo).CSA_tensor[0][1] = CSA_matrix[0][1];
                        (*foo_coo).CSA_tensor[0][2] = CSA_matrix[0][2];
                        (*foo_coo).CSA_tensor[1][0] = CSA_matrix[1][0];
                        (*foo_coo).CSA_tensor[1][1] = CSA_matrix[1][1];
                        (*foo_coo).CSA_tensor[1][2] = CSA_matrix[1][2];
                        (*foo_coo).CSA_tensor[2][0] = CSA_matrix[2][0];
                        (*foo_coo).CSA_tensor[2][1] = CSA_matrix[2][1];
                        (*foo_coo).CSA_tensor[2][2] = CSA_matrix[2][2];
                    }
                    else{
                         fprintf(stderr, "problem with pre-calculation of CSA tensor\n");
                    }
                } 
            }
        }
    }
}
void delet_link_for_racs_n_button(int n_button){
    int i;
    Item_pcs *foo;
    
    if(_all_pcs[n_button] == NULL){
        return;
    }

    for(i = 0; i < _all_pcs[n_button]->len; i++){
        foo = &g_array_index(_all_pcs[n_button], Item_pcs, i);
        (*foo).racs = EXEPTION_VALUE; //RACS_MOD
        (*foo).is_racs = FALSE;
        (*foo).racs_O = RACS_INVALID;
        (*foo).racs_C = RACS_INVALID;
        (*foo).racs_N = RACS_INVALID;
        (*foo).racs_H = RACS_INVALID;
        (*foo).racs_CA = RACS_INVALID;
        (*foo).racs_type = RACS_INVALID;
    }
}

//~ when a racs definition has been done, I try to
//~ make the racs linkage
//~ called by the GUI of racs when OK is clicked

void make_link_for_racs(racs_atom_name* racs_atom_na){
    int i;
    for(i = 0; i< N_PCS_ARRAY; i++){
        delet_link_for_racs_n_button(i);
        make_link_for_racs_n_button(i, racs_atom_na);
    }
}

void precalculate_CSA_tensor(racs_data * racs_d){
    int i;
    for(i = 0; i< N_PCS_ARRAY; i++){
        precalculate_CSA_tensor_n_button(i, racs_d);
    }
}

//~ When the user click on "Load" from one of the PCS tab,
//~ it display the input file box, and call the function
//~ to read the data inside the PCS file to the _dat_str
void 
load_PCS(int n_button)
{
  GtkWidget *dialog;
  GtkFileFilter *filter1;
  GtkFileFilter *filter2;
  GtkFileFilter *filter3;
  GtkFileFilter *filter4;
  gint result;
    
    if(_pdb_str == NULL){
        box_error( "You should first load a pdb file.\n", NULL);
        return;
    }

  dialog =
    gtk_file_chooser_dialog_new
    ("Choose a data file containing PCS values", NULL,
     GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
		
	gtk_window_set_icon_from_file ((GtkWindow *)dialog, NUMBAT_ICON_FILE, NULL);
  gtk_window_set_transient_for((GtkWindow*) dialog, (GtkWindow*)_window);

  filter1 = gtk_file_filter_new ();
  filter2 = gtk_file_filter_new ();
  filter3 = gtk_file_filter_new ();
  filter4 = gtk_file_filter_new ();
  gtk_file_filter_add_pattern (filter1, "*.tbl");
  gtk_file_filter_set_name (filter1, "*.tbl Xplor for PCS");
  //gtk_file_filter_add_pattern (filter2, "*.pales");
  //gtk_file_filter_set_name (filter2, "*.pales PALES for PCS");
  gtk_file_filter_add_pattern (filter3, "*.npc");
  gtk_file_filter_set_name (filter3, "*.npc Numbat for PCS");
  gtk_file_filter_add_pattern (filter4, "*.*");
  gtk_file_filter_set_name (filter4, "*.* ALL (auto-format recognition)");
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter3);
  //gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter2);
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter1);
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter4);

  result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
    {
    case GTK_RESPONSE_ACCEPT:
      {
	char *filename;
	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    result = read_pcs_file(n_button, filename);
	if(result >=0){
		fprintf(stderr, "%d PCS values read in the file %s\n",result, filename);
	}
	g_free (filename);
	  }
      break;
    default:
      break;
    }
  gtk_object_destroy((GtkObject *)filter1);
  gtk_object_destroy((GtkObject *)filter2);
  gtk_object_destroy((GtkObject *)filter3);
  gtk_object_destroy((GtkObject *)filter4);
  gtk_widget_destroy (dialog);
    
    if(result < 0){
        return;
    }
    
	clear_tensor_valid(n_button);
    update_CH3_tree_after_new_pcs();
    update_pcs_list(n_button, TRUE);
    display_pcs_list(n_button, -2);
    
}

//~ should be used only at initialisation
void init_PCS(void){
    int i;
    GtkWidget *treeview;
    GladeXML *gxml;
		GtkTreeModel * model;
    gxml = glade_get_widget_tree(_window);
    char tree_char[50];
    for (i = 0; i < N_PCS_ARRAY; i++){

        _str_pcs[i] = NULL; 
        _all_pcs[i] = NULL; 
        sprintf(tree_char, "treeview_PCS%d", i+1);
	      treeview = glade_xml_get_widget (gxml, tree_char);
        add_columns_pcs (GTK_TREE_VIEW (treeview), i);
				model = create_model (i);
				gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), model);
    }
}

//~ clear data of given PCS before reloading an other file of data
void free_array_PCS(GArray * arr){
    if(arr != NULL){
        g_array_free(arr, TRUE);
    }
}

//create a new array for PCS
GArray * new_array_PCS(void){
    return( g_array_sized_new (FALSE, FALSE, sizeof (experiment_pcs), 1));
}


//~ read the file of experimental pcs
int
read_pcs_file(int n_button, char * name){
    FILE *f_pcs;
    int format;
    GArray * result;

    //First we try to open the file
  f_pcs = g_fopen (name, "r");
  if (f_pcs == NULL)
    {
      box_error ("Error, can't open the file %s\nno pcs loaded", name);
      return (-1);
    }
    //Second we check what format it is:
	format = get_format(name);

    switch(format){
        case FORMAT_XPLOR:{
            result = read_xplor_pcs(f_pcs, n_button, name);
        }
        break;
        case FORMAT_PALES:{
            //result = read_pales_pcs(f_pcs, n_button, name);
			result = NULL;
        }
        break;
        case FORMAT_NUMBAT:{
             result = read_numbat_pcs(f_pcs, n_button, name);
        }
        break;
		case FORMAT_UNKNOWN:{
			 result = read_numbat_pcs(f_pcs, n_button, name);
			if(result == NULL){
				result = read_xplor_pcs(f_pcs, n_button, name);
			}
        }
        break;
		default:{
			result = NULL;
			break;
		}
    }
    fclose(f_pcs);
    if(result == NULL){
		box_error("error, file %s of unrecognized format\n", name);
        return(-1);
    }
    //COMMIT
    free_array_PCS(_str_pcs[n_button]);
    _str_pcs[n_button] = result;
	set_name_label_pcs(n_button, name);
    return(_str_pcs[n_button]->len);
	//return(1);
}

//~ read a experimental pcs file assuming xplor format (.tbl)
//~ store the values in the corresponding _str_pcs.
GArray * read_xplor_pcs(FILE * f_pcs, int n_button, char * name){
  int res_num;
  double pcs_val;
  double pcs_tol;
  char atom_na[MAX_ATOM_NAME_LENGTH];
  char junk_string[40];	
  int state;
  GArray * array;
  experiment_pcs exp_pcs;
  int r;
  
  array = new_array_PCS();
  fseek (f_pcs, 0, SEEK_SET);
  state = 0;

  while (fgets (line, MAX_CAR, f_pcs) != NULL)
    {

      if ((strncmp (line, "#", 1) == 0) ||(strncmp (line, "!", 1) == 0)
		  ||(strncmp (line, "{*", 2) == 0))
        {
        	continue;
        }
		if(is_line_empty(line) == TRUE){
			continue;
		}
		
		sscanf(line, "%s", junk_string);
		switch(state){
			case 0:{
				if((strcmp(junk_string, "assign") == 0)||(strcmp(junk_string, "ASSIgn") == 0)){
					state = 1;
				}
				else{
					free_array_PCS(array);
					return(NULL);
				}
				break;
			}
			case 4:{
				r = sscanf(line, "%*[ \t(]%*s%d%*s%*s%*[ \t]%[^) ]%*s%lf%lf", &res_num, atom_na, &pcs_val, &pcs_tol);
				if(r != 4){
					free_array_PCS(array);
					return(NULL);
				}
				exp_pcs.residue_num = res_num;
        		exp_pcs.pcs_value = pcs_val;
        		exp_pcs.pcs_tolerance = pcs_tol;
        		strcpy(exp_pcs.atom_name, atom_na);
        		g_array_append_vals (array, &exp_pcs, 1);
				state = 0;
				break;
			}
			default:{
				state ++;
			}
		}
    }
	if(state != 0){
		free_array_PCS(array);
		return(NULL);
	}
	
	if(array->len == 0){
		free_array_PCS(array);
		return(NULL);
	}
    return(array);
}

//~ read a experimental pcs file assuming numbat format (.npc)
//~ store the values in the corresponding _str_pcs.
GArray *  read_numbat_pcs(FILE * f_pcs, int n_button, char * name){
  int res_num;
  double pcs_val;
  double pcs_tol;
  char atom_na[MAX_ATOM_NAME_LENGTH];
  int n;
  GArray * array;
  experiment_pcs exp_pcs;
  
  array = new_array_PCS();
  fseek (f_pcs, 0, SEEK_SET);

  while (fgets (line, MAX_CAR, f_pcs) != NULL)
    {
      if ((strncmp (line, "#", 1) == 0) || (strncmp (line, " #", 2) == 0)|| (strncmp (line, "\t#", 2) == 0))
        {
        continue;
        }
		if(is_line_empty(line) == TRUE){
			continue;
		}
        n = 0;
        n = sscanf(line, "%d%s%lf%lf", &res_num, atom_na, &pcs_val, &pcs_tol);
		if(n != 4){
			free_array_PCS(array);
			return(NULL);
		}
        exp_pcs.residue_num = res_num;
        exp_pcs.pcs_value = pcs_val;
        exp_pcs.pcs_tolerance = pcs_tol;
        strcpy(exp_pcs.atom_name, atom_na);
        g_array_append_vals (array, &exp_pcs, 1);
    }
    return(array);
}


//~ print on stderr the pcs information
int print_full_pcs_structure(GArray * local_str_pcs[]){
    int i;
    int j;
    for(i = 0; i<N_PCS_ARRAY; i++){
        if(local_str_pcs[i] == NULL){
            continue;
        }
        for (j = 0; j < local_str_pcs[i]->len; j++){
            fprintf(stderr, "%d\t%s\t%lf\t%lf\n", 
                    g_array_index (local_str_pcs[i], experiment_pcs , j).residue_num,
                    g_array_index (local_str_pcs[i], experiment_pcs , j).atom_name,
                    g_array_index (local_str_pcs[i], experiment_pcs , j).pcs_value,
                    g_array_index (local_str_pcs[i], experiment_pcs , j).pcs_tolerance
                    );
        }
    }
  return(1);  
}

//~ look for the given PCS information (atom name and residue number)
//~ if there is a match in the pdb. return the index on it or
//~ -1 if not
int find_pcs_in_pdb(int res_num, char * atom_na){
    int i;
    if(_pdb_str == NULL){
        return(-2);
    }
    for(i = 0; i < _pdb_str->n_atom; i++){
        if(_pdb_str->tab_atom[i].residue_num == res_num){
            if(strcmp(_pdb_str->tab_atom[i].atom_name, atom_na) == 0){
                return(i);
            }
        }
    }
    return(-1);
}

//~ When someone click on save, we create the window file chooser
void save_PCS(GtkButton * button, gpointer user_data, int n_button){
  GtkWidget *dialog;
  GtkFileFilter *filter1;
  GtkFileFilter *filter2;
  GtkFileFilter *filter3;
  GtkFileFilter *filter4;
  gint result;
    
  if(_pdb_str == NULL){
      box_error( "You should first load a pdb file.\n", NULL);
      return;
  }  

    
  dialog =
    gtk_file_chooser_dialog_new
    ("In which file do you want to save your PCS data list?", NULL,
     GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
		
	gtk_window_set_icon_from_file ((GtkWindow *)dialog, NUMBAT_ICON_FILE, NULL);
  gtk_window_set_transient_for((GtkWindow*) dialog, (GtkWindow*)_window);

  filter1 = gtk_file_filter_new ();
  filter2 = gtk_file_filter_new ();
  filter3 = gtk_file_filter_new ();
  filter4 = gtk_file_filter_new ();
  gtk_file_filter_add_pattern (filter1, "*.tbl");
  gtk_file_filter_set_name (filter1, "*.tbl Xplor-NIH format for PCS");
	/*
  gtk_file_filter_add_pattern (filter2, "*.pales");
  gtk_file_filter_set_name (filter2, "*.pales PALES-like format for PCS");
	*/
  gtk_file_filter_add_pattern (filter3, "*.npc");
  gtk_file_filter_set_name (filter3, "*.npc Numbat format for PCS");
  gtk_file_filter_add_pattern (filter4, "*.lst");
  gtk_file_filter_set_name (filter4, "*.lst All PCS information saved");
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter3);
  //gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter2);
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter1);
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter4);

  result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
    {
    case GTK_RESPONSE_ACCEPT:
      {
	char *filename;
	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
  result = save_pcs_file(n_button, filename);
	g_free (filename);
	  }
      break;
    default:
      break;
    }
  gtk_object_destroy((GtkObject *)filter1);
  gtk_object_destroy((GtkObject *)filter2);
  gtk_object_destroy((GtkObject *)filter3);
  gtk_object_destroy((GtkObject *)filter4);
  gtk_widget_destroy (dialog);    
}


//~ save in pcs_file.
int save_pcs_file(int n_button, char * name){
    FILE * f_save;
    int result;
    int format;
    f_save = g_fopen (name, "w");
  if (f_save == NULL)
    {
      box_error ("Error, can't open the file %s\nno pcs saved", name);
      return (-1);
    }
    //Second we check what format it is:
    format = get_format(name);
    switch(format){
        case FORMAT_XPLOR:{
            result = save_xplor_pcs(f_save, n_button, name);
        }
        break;
        case FORMAT_PALES:{
            //result = save_pales_pcs(f_save, n_button, name);
        }
        break;
        case FORMAT_NUMBAT:{
             result = save_numbat_pcs(f_save, n_button, name);
        }
        break;
        case FORMAT_LST:{
             result = save_lst_pcs(f_save, n_button, name);
        }
        break;
		default: {
			box_error("Warning, format of file %s \nurecognized\n", name);
		}
    }
    fclose(f_save);
    return(1);
}

int save_numbat_pcs(FILE * f_save, int n_button, char *name){
    int i;
    GArray * arr;
    
    arr = _all_pcs[n_button];
    for (i = 0; i< arr->len; i++){
        if( g_array_index (arr, Item_pcs , i).displayed == TRUE){
            if(g_array_index (arr, Item_pcs , i).pcs_exp != EXEPTION_VALUE){
                fprintf(f_save, "%4d\t%5s\t% 10.3lf\t% 10.3lf\n", 
                    g_array_index (arr, Item_pcs , i).residue_num,
                    g_array_index (arr, Item_pcs , i).atom_name,
                    g_array_index (arr, Item_pcs , i).pcs_exp,
                    g_array_index (arr, Item_pcs , i).pcs_tol
                    );
            }
        }
    }
    return(1);
}

int save_lst_pcs(FILE * f_save, int n_button, char *name){
    int i;
    GArray * arr;
    
    arr = _all_pcs[n_button];
    fprintf(f_save, "#atom_num atom_name residue_num residue_name \
pcs_exp pcs_tol pcs_calc pcs_dev dist_center\n");
    for (i = 0; i< arr->len; i++){
        if( g_array_index (arr, Item_pcs , i).displayed == TRUE){
            fprintf(f_save, "%4d\t%5s\t%4d\t%5s\t% 10.3lf\t% 10.3lf\t% 10.3lf\t% 10.3lf\t% 10.3lf\n",
                    g_array_index (arr, Item_pcs , i).atom_num,
                    g_array_index (arr, Item_pcs , i).atom_name,
                    g_array_index (arr, Item_pcs , i).residue_num,
                    g_array_index (arr, Item_pcs , i).residue_name,
                    g_array_index (arr, Item_pcs , i).pcs_exp,
                    g_array_index (arr, Item_pcs , i).pcs_tol,
                    g_array_index (arr, Item_pcs , i).pcs_calc,
                    g_array_index (arr, Item_pcs , i).pcs_dev,
                    g_array_index (arr, Item_pcs , i).dist_center
                    );
        }
    }
    return(1);
}


int save_xplor_pcs(FILE * f_save, int n_button, char *name){
    int i;
    GArray * arr;
    arr = _all_pcs[n_button];
    for (i = 0; i< arr->len; i++){
        if( g_array_index (arr, Item_pcs , i).displayed == TRUE){
            if(g_array_index (arr, Item_pcs , i).pcs_exp != EXEPTION_VALUE){
				fprintf( f_save, "\
assign ( resid 500  and name OO )\n\
       ( resid 500  and name Z )\n\
       ( resid 500  and name X )\n\
       ( resid 500  and name Y )\n\
       ( resid %3d  and name %2s )   %.5lf  %.5lf\n", 
						g_array_index (arr, Item_pcs , i).residue_num,
						g_array_index (arr, Item_pcs , i).atom_name,
						g_array_index (arr, Item_pcs , i).pcs_exp,
						g_array_index (arr, Item_pcs , i).pcs_tol
                		);
            }
        }
    }
    return(1);
	
}

