/*
 * pcs.h
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#define N_PCS_ARRAY 4 //THIS should match the number of pcs tab in the GUI!!
#define EXEPTION_VALUE -999.000
#define TITLE_COLUMN_USED "Use?"
#define TITLE_COLUMN_PCS_EXP "PCS exp."
#define TITLE_COLUMN_PCS_TOL "PCS tol."
#define DUMMY_OFFSET 100000

#define RACS_INVALID -1
//#define RACS_VALID -2


//~ Item_PCS is used internally by the GUI to display in
//~ the treeview the PCS.
typedef struct
{
  gint atom_num;
  gchar atom_name[MAX_ATOM_NAME_LENGTH];
  gint residue_num;
  gchar residue_name[MAX_RESIDUE_NAME_LENGTH];
  gdouble pcs_exp;
  gdouble pcs_calc;
  gdouble pcs_dev;
  gdouble dist_center;
  gdouble pcs_tol;
  gboolean useable;
  gboolean used;
  gboolean displayed;
  gboolean is_dumby;
  gboolean is_methyl;  
  gint methyl_1; //it is an index, not atom num
  gint methyl_2; //it is an index, not atom num
  gint methyl_3; //it is an index, not atom num
    
  gboolean is_racs;
  gint racs_O;  //it is an index, not atom num
  gint racs_C;  //it is an index, not atom num
  gint racs_N;  //it is an index, not atom num
  gint racs_CA; //it is an index, not atom num
  gint racs_H;  //it is an index, not atom num
  gint racs_type;
  //gdouble CSA_tensor[3][3];  //Must be on pdb...
  gdouble racs;  
}
Item_pcs;

//~ this enum is used internally by the GUI to display atom name
enum
{
  COLUMN_ATOM_NUM,
  COLUMN_ATOM_NAME,
  COLUMN_RESIDUE_NUM,
  COLUMN_RESIDUE_NAME,
  COLUMN_PCS_EXP,
  COLUMN_PCS_CALC,
  COLUMN_RACS,  
  COLUMN_PCS_DEV,
  COLUMN_DIST_CENTER,
  COLUMN_PCS_TOL,
  COLUMN_USEABLE, 
  COLUMN_USED,
  COLUMN_DISPLAYED,  
  NUM_ITEM_COLUMNS
};

GArray *_all_pcs[N_PCS_ARRAY]; //ALL PCS FROM THE PDB
GArray *_str_pcs[N_PCS_ARRAY]; //DON'T EDIT, store the pcs data FROM FILE.

//experiment pcs is used to store information about the file of experimental pcs
typedef struct
{
    int residue_num;
    char atom_name[MAX_ATOM_NAME_LENGTH];
    double pcs_value;
    double pcs_tolerance;
}
experiment_pcs;


enum {
    col_used,
    col_exp,
    col_tol
};


void 
load_PCS(int n_button);

void init_PCS(void);

void 
populate_n_button_pcs(int n_button);

int
read_pcs_file(int n_button, char * name);
GArray * read_xplor_pcs(FILE * f_pcs, int n_button, char * name);
GArray * read_numbat_pcs(FILE * f_pcs, int n_button,char * name);
int find_pcs_in_pdb(int res_num, char * atom_na);
int find_in_pcs(int res_num, char * atom_name, GArray * pcs_array);
void populate_all_pcs(void);
void free_array_PCS(GArray * arr);
gboolean is_atom_dumby(int atom_num, int n_button);
int pass_filter_display(char * atom_name, char * residue_name, int n_button, int index_in_pcs);
void display_pcs_list (int n_button, int serial_tensor);
int print_full_pcs_structure(GArray * local_str_pcs[]);
void connect_signal_column(GtkTreeModel * model, GtkTreeViewColumn *        column, int title_type, int n_button);
void
cell_edited_n (GtkCellRendererText *cell, const gchar *path_string,
             const gchar *new_text, gpointer data, int n_button);
void
used_toggled_n (GtkCellRendererToggle *cell,
    gchar *path_str, gpointer data, int n_button);

int pass_filter_select(char * atom_name, char * residue_name, int n_button, int 
index_in_pcs);
int save_pcs_file(int n_button, char * name);

int save_numbat_pcs(FILE * f_save, int n_button, char *name);

void save_PCS(GtkButton * button, gpointer user_data, int n_button);

void update_pcs_list(int n_button, gboolean overwrite);
gboolean is_name_dumby(char * name);
gboolean is_intact_dumby(char * name);
int what_methyl_position(char * atom_name, char ** pcs_name);
gboolean is_atom_methyl(int atom_num, int n_button);

int save_lst_pcs(FILE * f_save, int n_button, char *name);
int save_xplor_pcs(FILE * f_save, int n_button, char *name);
void make_link_for_racs(racs_atom_name* racs_atom_na);
void precalculate_CSA_tensor_n_button(int n_button, racs_data * racs_d);
void precalculate_CSA_tensor(racs_data * racs_d);
