/*
 * pcsfit_metal.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
//#include <malloc.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <glade/glade.h>
#include "pdb.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "error_analysis.h"
#include "pcsfit_metal.h"
#include "common.h"
#include "tensor_list.h"
#include "molmol_iso.h"

#define N 10000
#define MAX_LINE_LEN 200
#define W_MAGNITUDE 1000.0
#define W_ANGLE 10000.0
#define W_POSITION 50000.0
#define MAX_ITER_SINGLE 1000000
#define MAX_ITER_MULTI 1000000

#define _DELTA_ANGLE 0.001
#define _DELTA_MAGN 1.0
#define _DELTA_COO 0.001


double SinglePCS_old(double R[3][3], double x, double y, double z, double xM, double yM, double zM, double Xax, double Xrh)
{
    double  x0, y0, z0, x1, y1, z1, r2, r, r5;

    double  tmp, pcs;
    
    
    x0 = x - xM;
    y0 = y - yM;
    z0 = z - zM;
    
    x1 = x0*R[0][0] + y0*R[0][1] + z0*R[0][2];
    y1 = x0*R[1][0] + y0*R[1][1] + z0*R[1][2];
    z1 = x0*R[2][0] + y0*R[2][1] + z0*R[2][2];
    
    r2 = x1*x1 + y1*y1 + z1*z1;
    r = sqrt(r2);
    r5 = r2*r2*r;
	
    tmp = 1.0/r5;                      
    pcs = FACT_USI * tmp*(Xax * (3.0*z1*z1 - r2) + Xrh * 1.5*(x1*x1-y1*y1));
    
//THE fact_usi take into account (i) the 1/(12PI) (ii)
//the fact that length are in Angstrom and Xax in 10-32m3
//and (iii) that PCS are in ppm (10^6)
    return pcs;
}

double calculate_RACS2(double chi_matrix[3][3],
                       double CSA_matrix[3][3],
                       double Xax, double Xrh, 
                       double sigma11, double sigma22, double sigma33,
                       double B0, double T
                       /*,double x1, double y1, double z1*/)
{
    double racs;
    double temp_scal;
    /*
    double Xxx_axe[3];
    double Xyy_axe[3];
    double Xzz_axe[3];
    */
    /*
    double sigma11_axe[3];
    double sigma22_axe[3];
    double sigma33_axe[3];
    */
    
    //OPTIMIZATION:
    //1. calculate k_factor much earlier
    //2. calculate Xx, Xy, Xz much earlier
    //4. Try to preweigth the matrices with sqrt(sigmaii)
    // and sqrt(chixx)
    
    double k_factor;
    
    double Xx, Xy, Xz;

    Xx = -Xax/3.0 + Xrh/2.0;
	Xy = -Xax/3.0 - Xrh/2.0;
	Xz = 2.0/3.0*Xax;
    
    k_factor = B0*B0/(15*CONSTANTE_MU0*CONSTANTE_k_BOLTZ*T) * pow(10, -32);
    
    /*
    Xxx_axe[0] = chi_matrix[0][0];
    Xxx_axe[1] = chi_matrix[0][1];
    Xxx_axe[2] = chi_matrix[0][2];
    
    Xyy_axe[0] = chi_matrix[1][0];
    Xyy_axe[1] = chi_matrix[1][1];
    Xyy_axe[2] = chi_matrix[1][2];
    
    Xzz_axe[0] = chi_matrix[2][0];
    Xzz_axe[1] = chi_matrix[2][1];
    Xzz_axe[2] = chi_matrix[2][2];
    */
    /*
    sigma11_axe[0] = CSA_matrix[0][0];
    sigma11_axe[1] = CSA_matrix[1][0];
    sigma11_axe[2] = CSA_matrix[2][0];
    
    sigma22_axe[0] = CSA_matrix[0][1];
    sigma22_axe[1] = CSA_matrix[1][1];
    sigma22_axe[2] = CSA_matrix[2][1];
    
    sigma33_axe[0] = CSA_matrix[0][2];
    sigma33_axe[1] = CSA_matrix[1][2];
    sigma33_axe[2] = CSA_matrix[2][2];
    */
    
    racs = 0;
    
    temp_scal = dot_product(CSA_matrix[0], chi_matrix[0]);
    racs+=sigma11*temp_scal*temp_scal*Xx;
    
    temp_scal = dot_product(CSA_matrix[1], chi_matrix[0]);
    racs+=sigma22*temp_scal*temp_scal*Xx;
    
    temp_scal = dot_product(CSA_matrix[2], chi_matrix[0]);
    racs+=sigma33*temp_scal*temp_scal*Xx;
    
    
    temp_scal = dot_product(CSA_matrix[0], chi_matrix[1]);
    racs+=sigma11*temp_scal*temp_scal*Xy;
    
    temp_scal = dot_product(CSA_matrix[1], chi_matrix[1]);
    racs+=sigma22*temp_scal*temp_scal*Xy;
    
    temp_scal = dot_product(CSA_matrix[2], chi_matrix[1]);
    racs+=sigma33*temp_scal*temp_scal*Xy;
    
    
    temp_scal = dot_product(CSA_matrix[0], chi_matrix[2]);
    racs+=sigma11*temp_scal*temp_scal*Xz;
    
    temp_scal = dot_product(CSA_matrix[1], chi_matrix[2]);
    racs+=sigma22*temp_scal*temp_scal*Xz;
    
    temp_scal = dot_product(CSA_matrix[2], chi_matrix[2]);
    racs+=sigma33*temp_scal*temp_scal*Xz;
    /*
    temp_scal = dot_product(sigma11_axe, chi_matrix[0]);
    racs+=sigma11*temp_scal*temp_scal*Xx;
    
    temp_scal = dot_product(sigma22_axe, chi_matrix[0]);
    racs+=sigma22*temp_scal*temp_scal*Xx;
    
    temp_scal = dot_product(sigma33_axe, chi_matrix[0]);
    racs+=sigma33*temp_scal*temp_scal*Xx;
    
    
    temp_scal = dot_product(sigma11_axe, chi_matrix[1]);
    racs+=sigma11*temp_scal*temp_scal*Xy;
    
    temp_scal = dot_product(sigma22_axe, chi_matrix[1]);
    racs+=sigma22*temp_scal*temp_scal*Xy;
    
    temp_scal = dot_product(sigma33_axe, chi_matrix[1]);
    racs+=sigma33*temp_scal*temp_scal*Xy;
    
    
    temp_scal = dot_product(sigma11_axe, chi_matrix[2]);
    racs+=sigma11*temp_scal*temp_scal*Xz;
    
    temp_scal = dot_product(sigma22_axe, chi_matrix[2]);
    racs+=sigma22*temp_scal*temp_scal*Xz;
    
    temp_scal = dot_product(sigma33_axe, chi_matrix[2]);
    racs+=sigma33*temp_scal*temp_scal*Xz;
    */
    /*
    double cst;
    cst = 1.0;
    fprintf(stderr, "RACS calculated: %lf with the CSA and TSR:\n", k_factor*racs);
    fprintf(stderr, "ATOM  80000  S00 CSA  8000%12.3f%8.3f%8.3f  1.00  0.00\n", x1, y1, z1);
    fprintf(stderr, "ATOM  80001  S11 CSA  8000%12.3f%8.3f%8.3f  1.00  0.00\n", x1+ cst* CSA_matrix[0][0], y1+ cst* CSA_matrix[1][0], z1+ cst* CSA_matrix[2][0]);
    fprintf(stderr, "ATOM  80002  S22 CSA  8000%12.3f%8.3f%8.3f  1.00  0.00\n", x1+ cst* CSA_matrix[0][1], y1+ cst* CSA_matrix[1][1], z1+ cst* CSA_matrix[2][1]);
    fprintf(stderr, "ATOM  80003  S33 CSA  8000%12.3f%8.3f%8.3f  1.00  0.00\n", x1+ cst* CSA_matrix[0][2], y1+ cst* CSA_matrix[1][2], z1+ cst* CSA_matrix[2][2]);
    fprintf(stderr, "ATOM  80004  TOO TSR  8000%12.3f%8.3f%8.3f  1.00  0.00\n", x1, y1, z1);
    fprintf(stderr, "ATOM  80005  TXX TSR  8000%12.3f%8.3f%8.3f  1.00  0.00\n", x1+ cst* chi_matrix[0][0], y1+ cst* chi_matrix[0][1], z1+ cst* chi_matrix[0][2]);
    fprintf(stderr, "ATOM  80006  TYY TSR  8000%12.3f%8.3f%8.3f  1.00  0.00\n", x1+ cst* chi_matrix[1][0], y1+ cst* chi_matrix[1][1], z1+ cst* chi_matrix[1][2]);
    fprintf(stderr, "ATOM  80007  TZZ TSR  8000%12.3f%8.3f%8.3f  1.00  0.00\n", x1+ cst* chi_matrix[2][0], y1+ cst* chi_matrix[2][1], z1+ cst* chi_matrix[2][2]);
    */
    return (-1.0*k_factor*racs);
}

/*----------------------------------------------------------------------*/
double SinglePCS_new(double R_chi[3][3], double xM, double yM, double zM, double Xax, double Xrh, int index_global, int n_button, int * index_on_pdb, int * index_on_pcs, double * offset_noise_x, double * offset_noise_y, double* offset_noise_z, gboolean use_racs, double * racs, racs_data * racs_d)
/*----------------------------------------------------------------------*/
{

    double  pcs;
    double x, y, z;
    double x_clean, y_clean, z_clean;
    int i;
    double pcs_total;
    double racs_total;
    int compteur;
    double racs_local;
    double noise_x, noise_y, noise_z;
    double sigma11, sigma22, sigma33;
    
    //OPTIMIZATION: 
    //1. DOn't read into an array, calling function 
    //should provide directly the double value of error
    //2. Precalculate k_factor in calculate_single_tensor and calculate_multiple_tensor

    noise_x = noise_y = noise_z = 0;
    if(offset_noise_x != NULL){
        noise_x = offset_noise_x[index_global];
    }
    if(offset_noise_y != NULL){
        noise_y = offset_noise_y[index_global];
    }
    if(offset_noise_z != NULL){
        noise_z = offset_noise_z[index_global];
    }

    compteur = 0;
    pcs_total = 0;
    racs_total = 0;
    
    //fprintf(stderr, "start SinglePCS_new\n");
    for(i = 0; i < _all_pdb_selector_pcs[n_button]->len; i++){
        if(g_array_index (_all_pdb_selector_pcs[n_button], pdb_selector , i).is_used == FALSE){
            continue;
        }
        racs_local = 0.0;
        if(g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[index_global]).is_dumby == FALSE){
            
            x_clean = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].x;
            y_clean = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].y;
            z_clean = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].z;
            x = x_clean + noise_x;
            y = y_clean + noise_y;
            z = z_clean + noise_z;
            pcs = SinglePCS_old(R_chi, x, y, z, xM, yM, zM, Xax, Xrh);

            if(use_racs){ //USE RACS
                if(g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[index_global]).is_racs == TRUE){

                    racs_local = 0.0;
                    
                    switch(g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[index_global]).racs_type){
                        case RACS_FOR_C:{
                            sigma11 = racs_d->sigma_C_11;
                            sigma22 = racs_d->sigma_C_22;
                            sigma33 = racs_d->sigma_C_33;
                        }
                        break;
                        case RACS_FOR_H:{
                            sigma11 = racs_d->sigma_H_11;
                            sigma22 = racs_d->sigma_H_22;
                            sigma33 = racs_d->sigma_H_33;
                        }
                        break;
                        
                        case RACS_FOR_N_NATURAL:{
                            sigma11 = racs_d->sigma_N_11;
                            sigma22 = racs_d->sigma_N_22;
                            sigma33 = racs_d->sigma_N_33;
                        }
                        break;
                        
                        case RACS_FOR_N_OFFSET:{
                            sigma11 = racs_d->sigma_N_11;
                            sigma22 = racs_d->sigma_N_22;
                            sigma33 = racs_d->sigma_N_33;
                        }
                        break;
                        default:{
                            sigma11 = 0;
                            sigma22 = 0;
                            sigma33 = 0;
                        }
                    }
                    //double CSA_matrix[3][3];
                    //int result_csa;
                    /*
                    CSA_matrix[0][0] = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].CSA_tensor[0][0];
                    CSA_matrix[0][1] = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].CSA_tensor[0][1];
                    CSA_matrix[0][2] = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].CSA_tensor[0][2];
                    CSA_matrix[1][0] = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].CSA_tensor[1][0];
                    CSA_matrix[1][1] = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].CSA_tensor[1][1];
                    CSA_matrix[1][2] = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].CSA_tensor[1][2];
                    CSA_matrix[2][0] = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].CSA_tensor[2][0];
                    CSA_matrix[2][1] = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].CSA_tensor[2][1];
                    CSA_matrix[2][2] = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].CSA_tensor[2][2];
                    */
                    racs_local = calculate_RACS2(R_chi,
                                                _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].CSA_tensor, //CSA_matrix,
                                                Xax, Xrh,
                                                sigma11, sigma22, sigma33,
                                                racs_d->B0,  racs_d->T
                                                );
                    
                }
            }//end use racs
            pcs += racs_local;
        }
        else{
            int ind;
            double pcs1, pcs2, pcs3;
            ind = g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[index_global]).methyl_1;
            x = _pdb_str->tab_atom[ind].atom_coo[i].x + noise_x;
            y = _pdb_str->tab_atom[ind].atom_coo[i].y + noise_y;
            z = _pdb_str->tab_atom[ind].atom_coo[i].z + noise_z;
            pcs1 = SinglePCS_old(R_chi, x, y, z, xM, yM, zM, Xax, Xrh);
            
            ind = g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[index_global]).methyl_2;
            x = _pdb_str->tab_atom[ind].atom_coo[i].x + noise_x;
            y = _pdb_str->tab_atom[ind].atom_coo[i].y + noise_y;
            z = _pdb_str->tab_atom[ind].atom_coo[i].z + noise_z;
            pcs2 = SinglePCS_old(R_chi, x, y, z, xM, yM, zM, Xax, Xrh);
            
            ind = g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[index_global]).methyl_3;
            x = _pdb_str->tab_atom[ind].atom_coo[i].x + noise_x;
            y = _pdb_str->tab_atom[ind].atom_coo[i].y + noise_y;
            z = _pdb_str->tab_atom[ind].atom_coo[i].z + noise_z;
            pcs3 = SinglePCS_old(R_chi, x, y, z, xM, yM, zM, Xax, Xrh);
            pcs = (pcs1 + pcs2 + pcs3)/3.0;
        }
        pcs_total += pcs;
        racs_total += racs_local;
        compteur = compteur + 1;
    }
    //fprintf(stderr, "end SinglePCS_new\n");
    if(racs != NULL){
        *racs = racs_total/compteur;
    }
    
    return (pcs_total/compteur);
}

void get_correct_parameters(double * a, double *b, double *g, double *x, 
                            double *y, double *z, double *Ax, double *Rh, 
                            const gsl_vector * v, void *data)
{
    int i;
    gboolean * bitmap_opt;
    int * index_v;
    index_v =  ((struct PCSdata3 *) data)->index_v;
    bitmap_opt = ((struct PCSdata3 *) data)->bitmap_opt;
    
    i = 0;
    if(bitmap_opt[i] == FALSE){
        *a = ((struct PCSdata3 *) data)->a;
    }
    else{ if(index_v[i] == -1) {fprintf(stderr, "pbm with index_v\n");}
        *a = gsl_vector_get (v, index_v[i]);
    }
    i++;
    
    if(bitmap_opt[i] == FALSE){
        *b = ((struct PCSdata3 *) data)->b;
    }
    else{ if(index_v[i] == -1) {fprintf(stderr, "pbm with index_v\n");}
        *b = gsl_vector_get (v, index_v[i]);
    }
    i++;
    
        if(bitmap_opt[i] == FALSE){
        *g = ((struct PCSdata3 *) data)->g;
    }
    else{ if(index_v[i] == -1) {fprintf(stderr, "pbm with index_v\n");}
        *g = gsl_vector_get (v, index_v[i]);
    }
    i++;
    
        if(bitmap_opt[i] == FALSE){
        *x = ((struct PCSdata3 *) data)->xM;
    }
    else{ if(index_v[i] == -1) {fprintf(stderr, "pbm with index_v\n");}
        *x = gsl_vector_get (v, index_v[i]);
    }
    i++;
    
        if(bitmap_opt[i] == FALSE){
        *y = ((struct PCSdata3 *) data)->yM;
    }
    else{ if(index_v[i] == -1) {fprintf(stderr, "pbm with index_v\n");}
        *y = gsl_vector_get (v, index_v[i]);
    }
    i++;
    
        if(bitmap_opt[i] == FALSE){
        *z = ((struct PCSdata3 *) data)->zM;
    }
    else{ if(index_v[i] == -1) {fprintf(stderr, "pbm with index_v\n");}
        *z = gsl_vector_get (v, index_v[i]);
    }
    i++;
    
    if(bitmap_opt[i] == FALSE){
        *Ax = ((struct PCSdata3 *) data)->Ax;
    }
    else{ if(index_v[i] == -1) {fprintf(stderr, "pbm with index_v\n");}
        *Ax = gsl_vector_get (v, index_v[i]);
    }
    i++;
    
    if(bitmap_opt[i] == FALSE){
        *Rh = ((struct PCSdata3 *) data)->Rh;
    }
    else{ if(index_v[i] == -1) {fprintf(stderr, "pbm with index_v\n");}
        *Rh = gsl_vector_get (v, index_v[i]);
    }
    i++;
}


void get_correct_parameters_multi(double * a, double *b, double *g, double *x, 
                            double *y, double *z, double *Ax, double *Rh, 
                            const gsl_vector * v, void *data)
{
    int i, gg;
    gboolean * g_button_tab = ((struct PCSdata4 *) data)->g_button_tab;
    int * index_v_multi =  ((struct PCSdata4 *) data)->index_v_multi;
    gboolean ** bitmap_opt = ((struct PCSdata4 *) data)->bitmap_opt;
    gboolean * bitmap_opt_center = ((struct PCSdata4 *) data)->bitmap_opt_center;
    double ** x_init = ((struct PCSdata4 *) data)->x_init;
    double * x_init_center = ((struct PCSdata4 *) data)->x_init_center;
    
    for(gg = 0; gg < N_PCS_ARRAY; gg++){
        if(g_button_tab[gg] == FALSE){
            continue;
        }
        i = 0;
        if(bitmap_opt[gg][i] == FALSE){
            a[gg] = x_init[gg][i];
        }
        else{ if(index_v_multi[gg*5+i] == -1) {fprintf(stderr, "pbm with index_v\n");}
            a[gg] = gsl_vector_get (v, index_v_multi[gg*5+i]);
        }
        i++;
        if(bitmap_opt[gg][i] == FALSE){
            b[gg] = x_init[gg][i];
        }
        else{ if(index_v_multi[gg*5+i] == -1) {fprintf(stderr, "pbm with index_v\n");}
            b[gg] = gsl_vector_get (v, index_v_multi[gg*5+i]);
        }
        i++;
        if(bitmap_opt[gg][i] == FALSE){
            g[gg] = x_init[gg][i];
        }
        else{ if(index_v_multi[gg*5+i] == -1) {fprintf(stderr, "pbm with index_v\n");}
            g[gg] = gsl_vector_get (v, index_v_multi[gg*5+i]);
        }
        i++;
        
        if(bitmap_opt_center[0] == FALSE){
            x[gg] = x_init_center[0];
        }
        else{ if(index_v_multi[N_PCS_ARRAY*5+0] == -1) {fprintf(stderr, "pbm with index_v\n");}
            x[gg] = gsl_vector_get (v, index_v_multi[N_PCS_ARRAY*5+0]);
        }
        if(bitmap_opt_center[1] == FALSE){
            y[gg] = x_init_center[1];
        }
        else{ if(index_v_multi[N_PCS_ARRAY*5+1] == -1) {fprintf(stderr, "pbm with index_v\n");}
            y[gg] = gsl_vector_get (v, index_v_multi[N_PCS_ARRAY*5+1]);
        }
        if(bitmap_opt_center[2] == FALSE){
            z[gg] = x_init_center[2];
        }
        else{ if(index_v_multi[N_PCS_ARRAY*5+2] == -1) {fprintf(stderr, "pbm with index_v\n");}
            z[gg] = gsl_vector_get (v, index_v_multi[N_PCS_ARRAY*5+2]);
        }
        
        if(bitmap_opt[gg][i+3] == FALSE){
            Ax[gg] = x_init[gg][i+3];
        }
        else{ if(index_v_multi[gg*5+i] == -1) {fprintf(stderr, "pbm with index_v\n");}
            Ax[gg] = gsl_vector_get (v, index_v_multi[gg*5+i]);
        }
        i++;
        if(bitmap_opt[gg][i+3] == FALSE){
            Rh[gg] = x_init[gg][i+3];
        }
        else{ if(index_v_multi[gg*5+i] == -1) {fprintf(stderr, "pbm with index_v\n");}
            Rh[gg] = gsl_vector_get (v, index_v_multi[gg*5+i]);
        }
    }
}

/*----------------------------------------------------------------------*/
int print_f_unified (const gsl_vector * v, void *data, gsl_vector * f, char **label)
/*----------------------------------------------------------------------*/
{
    int   i;
    double tmp, PCS_calc, d;
    size_t n = ((struct PCSdata3 *)data)->n;
    double *PCS = ((struct PCSdata3 *)data)->PCS;
	//double *TOL = ((struct PCSdata3 *)data)->TOL;
    int n_button = ((struct PCSdata3*) data)->n_button;
    int * index_on_pdb = ((struct PCSdata3 *) data)->index_on_pdb;
    int * index_on_pcs = ((struct PCSdata3 *) data)->index_on_pcs;
    gboolean use_racs = ((struct PCSdata3 *) data)->use_racs;
    double alpha, beta, gamma, Xax, Xrh, xM, yM, zM;
    racs_data * racs_d = ((struct PCSdata3 *) data)->racs_d;
    
    get_correct_parameters(&alpha, &beta, &gamma, &xM, &yM, &zM, &Xax, &Xrh, v, data);

    double R_chi[3][3];
    
    filZX_or_YZ(R_chi, alpha*180/M_PI, beta*180/M_PI, gamma*180/M_PI);
    
    fprintf(stdout, "#  no  PCS_calc  PCS_exp   distance  delta_PCS  delta_PCS^2  label\n");
    for (i = 0; i < n; i++) {
        /*
        tmp = x[i] - xM[i];
        d = tmp*tmp;
        tmp = y[i] - yM[i];
        d += tmp*tmp;
        tmp = z[i] - zM[i];
        d += tmp*tmp;
        */
        d = calculate_average_distance_family(index_on_pdb, index_on_pcs, i, n_button, xM, yM, zM);
        
        PCS_calc = SinglePCS_new(R_chi, xM, yM, zM, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs, 0, 0, 0, use_racs, NULL, racs_d);
        //tmp = fmax(fabs(PCS_calc - PCS[i])-TOL[i], 0);
		//ANYWAY I DON'T USE THIS FUNCTION
        tmp = PCS_calc - PCS[i];
        fprintf(stdout, "%5i %10.5f %10.5f %10.5f %10.5f %10.5f      %s\n", i+1, PCS_calc, PCS[i], d, tmp, tmp*tmp, label[i]);
    }

    return GSL_SUCCESS;
}



double calculate_parameter_deviation_for_f(boundarie * my_boundarie, double  value, double  w){
    double result;
    double dev_min;
    double dev_max;
    dev_min = dev_max = 0;
    if( value > my_boundarie->max){
        dev_max = value - my_boundarie->max;
    }
    
    if( value < my_boundarie->min){
        dev_min = my_boundarie->min - value;
    }
    result = dev_min + dev_max;
    result = sqrt(w) * result;
    return(result);
}

int expb_f_unified (const gsl_vector * v, void *data, gsl_vector * f)
{
    size_t i;
    int my_i;
    double tmp, PCS_calc;
    size_t n = ((struct PCSdata3 *)data)->n;
    double *PCS = ((struct PCSdata3 *)data)->PCS;
	double *PCS_OFFSET = ((struct PCSdata3 *)data)->PCS_OFFSET;
	double *TOL = ((struct PCSdata3 *)data)->TOL;
    int n_button = ((struct PCSdata3 *) data)->n_button;
    int * index_on_pdb = ((struct PCSdata3 *) data)->index_on_pdb;
    int * index_on_pcs = ((struct PCSdata3 *) data)->index_on_pcs;
    gboolean * bitmap_bound =  ((struct PCSdata3 *) data)->bitmap_bound;
    boundarie my_boundarie;
    double alpha, beta, gamma, Xax, Xrh, xM, yM, zM;
    int n_bound = ((struct PCSdata3*) data)->n_bound;
	double PCS_exp;
    double *offset_noise_x, *offset_noise_y, *offset_noise_z;
	double *PDB_OFFSET_X = ((struct PCSdata3 *)data)->PDB_OFFSET_X;
	double *PDB_OFFSET_Y = ((struct PCSdata3 *)data)->PDB_OFFSET_Y;
	double *PDB_OFFSET_Z = ((struct PCSdata3 *)data)->PDB_OFFSET_Z;
	gboolean * SUBSET_ERROR = ((struct PCSdata3 *)data)->SUBSET_ERROR;
	gboolean use_racs = ((struct PCSdata3 *) data)->use_racs;
    racs_data * racs_d = ((struct PCSdata3 *) data)->racs_d;
    double R_chi[3][3];
    
    get_correct_parameters(&alpha, &beta, &gamma, &xM, &yM, &zM, &Xax, &Xrh, v, data);
    filZX_or_YZ(R_chi, alpha*180/M_PI, beta*180/M_PI, gamma*180/M_PI);
    
    for (i = 0; i < n; i++) {
		if(SUBSET_ERROR[i] == FALSE){
			gsl_vector_set (f, i, 0.0);
			continue;
		}
		
		offset_noise_x = PDB_OFFSET_X;
		offset_noise_y = PDB_OFFSET_Y;
		offset_noise_z = PDB_OFFSET_Z;
        /*
        double SinglePCS_new(   double phi, double theta, double psi, 
                        double xM, double yM, double zM, 
                        double Xax, double Xrh, 
                        int index_global, int n_button, 
                        int * index_on_pdb, int * index_on_pcs, 
                        double * offset_noise_x, double * offset_noise_y, double* offset_noise_z, 
                        gboolean use_racs, double * racs, racs_data * racs_d)
        */
        PCS_calc = SinglePCS_new(R_chi, xM, yM, zM, Xax, Xrh
                                 , i, n_button, index_on_pdb, index_on_pcs, offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d);
        //tmp = PCS_calc - PCS[i];
		
		PCS_exp = PCS[i] + PCS_OFFSET[i];
		tmp = my_fmax(fabs(PCS_calc - PCS_exp) - TOL[i], 0);
		if(PCS_calc < PCS_exp){
			tmp *= -1.0;
		}
        gsl_vector_set (f, i, tmp);
    }
    my_i = 0;
    
    
    for(i = 0; i < 8; i++){
        if(bitmap_bound[i] == TRUE){
            tmp = 0;
            switch(i){
                case 0: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_a;
                    tmp = calculate_parameter_deviation_for_f(&my_boundarie, alpha/M_PI*180.0, W_ANGLE);
                } break;
                case 1: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_b;
                    tmp = calculate_parameter_deviation_for_f(&my_boundarie, beta/M_PI*180.0, W_ANGLE);
                } break;
                case 2: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_g;
                    tmp = calculate_parameter_deviation_for_f(&my_boundarie, gamma/M_PI*180.0, W_ANGLE);
                } break;
                case 3: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_x;
                    tmp = calculate_parameter_deviation_for_f(&my_boundarie, xM, W_POSITION);
                } break;
                case 4: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_y;
                    tmp = calculate_parameter_deviation_for_f(&my_boundarie, yM, W_POSITION);
                } break;
                case 5: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_z;
                    tmp = calculate_parameter_deviation_for_f(&my_boundarie, zM, W_POSITION);
                } break;
                case 6: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_Ax;
                    tmp = calculate_parameter_deviation_for_f(&my_boundarie, Xax, W_MAGNITUDE);
                } break;
                case 7: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_Rh;
                    tmp = calculate_parameter_deviation_for_f(&my_boundarie, Xrh, W_MAGNITUDE);
                } break;
            }
            gsl_vector_set (f, n + my_i, tmp);
            my_i++;
        }
    }
    if(n_bound != my_i){
        fprintf(stderr, "my_bound different from my_i %d %d\n", n_bound, my_i);
    }

    return GSL_SUCCESS;
    
}


int expb_f_unified_multi (const gsl_vector * v, void *data, gsl_vector * f)
{
    boundarie my_boundarie;
    int my_i;
    int i;
    double tmp, PCS_calc;
    double alpha[N_PCS_ARRAY], beta[N_PCS_ARRAY], gamma[N_PCS_ARRAY], Xax[N_PCS_ARRAY]
        , Xrh[N_PCS_ARRAY], xM[N_PCS_ARRAY], yM[N_PCS_ARRAY], zM[N_PCS_ARRAY];
    int gg;
    gboolean ** bitmap_bound =  ((struct PCSdata4 *) data)->bitmap_bound;
    gboolean * bitmap_bound_center =  ((struct PCSdata4 *) data)->bitmap_bound_center;
    gboolean * g_button_tab = ((struct PCSdata4*) data)->g_button_tab;
    int ** index_on_pdb = ((struct PCSdata4 *) data)->index_on_pdb;
    int ** index_on_pcs = ((struct PCSdata4 *) data)->index_on_pcs;
    double **PCS = ((struct PCSdata4 *)data)->PCS;
	double **PCS_OFFSET = ((struct PCSdata4 *)data)->PCS_OFFSET;
	double **TOL = ((struct PCSdata4 *)data)->TOL;
    int * n = ((struct PCSdata4 *)data)->n;
    //int * index_v_multi = ((struct PCSdata4 *)data)->index_v_multi;
    int * index_bound_multi = ((struct PCSdata4 *)data)->index_bound_multi;
    int i_total;
    int n_total;
    n_total = 0;
	double PCS_exp;
    double * offset_noise_x, *offset_noise_y, *offset_noise_z;
	double **PDB_OFFSET_X = ((struct PCSdata4 *)data)->PDB_OFFSET_X;
	double **PDB_OFFSET_Y = ((struct PCSdata4 *)data)->PDB_OFFSET_Y;
	double **PDB_OFFSET_Z = ((struct PCSdata4 *)data)->PDB_OFFSET_Z;
	gboolean ** SUBSET_ERROR = ((struct PCSdata4 *)data)->SUBSET_ERROR;
    gboolean *use_racs_t = ((struct PCSdata4 *) data)->use_racs;
	gboolean use_racs;
    racs_data * racs_d = ((struct PCSdata4 *) data)->racs_d;
    get_correct_parameters_multi(alpha, beta, gamma, xM, yM, zM, Xax, Xrh, v, data);
    
    double R_chi[3][3];
    
    i_total = 0;
    for(gg = 0; gg < N_PCS_ARRAY; gg++){
        
        filZX_or_YZ(R_chi, alpha[gg]*180/M_PI, beta[gg]*180/M_PI, gamma[gg]*180/M_PI);
        
        use_racs = use_racs_t[gg];
        if(g_button_tab[gg] == FALSE){
            continue;
        }
        n_total += n[gg];
        for (i = 0; i < n[gg]; i++) {
			if(SUBSET_ERROR[gg][i] == FALSE){
				gsl_vector_set (f, i_total, 0.0);
            	i_total++;
				continue;
			}
			offset_noise_x = PDB_OFFSET_X[gg];
			offset_noise_y = PDB_OFFSET_Y[gg];
			offset_noise_z = PDB_OFFSET_Z[gg];
			
            PCS_calc = SinglePCS_new(R_chi, xM[gg], yM[gg], zM[gg], Xax[gg], Xrh[gg]
                                 , i, gg, index_on_pdb[gg], index_on_pcs[gg], offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d);
            //tmp = PCS_calc - PCS[gg][i];
			PCS_exp = PCS[gg][i] + PCS_OFFSET[gg][i];
			tmp = my_fmax(fabs(PCS_calc - PCS_exp) - TOL[gg][i], 0);
			if(PCS_calc < PCS_exp){
				tmp *= -1.0;
			}
            gsl_vector_set (f, i_total, tmp);
            i_total++;
        }
    }
    
    
    my_i = -999;
    for(gg = 0; gg < N_PCS_ARRAY; gg++){
        if(g_button_tab[gg] == FALSE){
            continue;
        }
        for(i = 0; i < 8; i++){
            if((bitmap_bound[gg][i] == TRUE)&&((i!=3)&&(i!=4)&&(i!=5))){
                tmp = 0;
                switch(i){
                    case 0: { 
                        my_boundarie = _all_boundaries_pcs[gg]->b_a;
                        tmp = calculate_parameter_deviation_for_f(&my_boundarie, alpha[gg]/M_PI*180.0, W_ANGLE);
                        my_i = 5*gg + i;
                    } break;
                    case 1: { 
                        my_boundarie = _all_boundaries_pcs[gg]->b_b;
                        tmp = calculate_parameter_deviation_for_f(&my_boundarie, beta[gg]/M_PI*180.0, W_ANGLE);
                        my_i = 5*gg + i;
                    } break;
                    case 2: { 
                        my_boundarie = _all_boundaries_pcs[gg]->b_g;
                        tmp = calculate_parameter_deviation_for_f(&my_boundarie, gamma[gg]/M_PI*180.0, W_ANGLE);
                        my_i = 5*gg + i;
                    } break;
                    case 6: { 
                        my_boundarie = _all_boundaries_pcs[gg]->b_Ax;
                        tmp = calculate_parameter_deviation_for_f(&my_boundarie, Xax[gg], W_MAGNITUDE);
                        my_i = 5*gg + i-3;
                    } break;
                    case 7: { 
                        my_boundarie = _all_boundaries_pcs[gg]->b_Rh;
                        tmp = calculate_parameter_deviation_for_f(&my_boundarie, Xrh[gg], W_MAGNITUDE);
                        my_i = 5*gg + i-3;
                    } break;
                }
                if(index_bound_multi[my_i] == -1){
                    fprintf(stderr, "Problem with index_bound_multi\n");
                    exit(0);
                }
                gsl_vector_set (f, n_total + index_bound_multi[my_i], tmp);
            }
        }
        for(i = 3; i < 6; i++){
            if(bitmap_bound_center[i-3] == TRUE){
                tmp = 0;
                switch(i){
                    case 3: { 
                        my_boundarie = _multiple_pcs_boundaries_pcs->b_x;
                        tmp = calculate_parameter_deviation_for_f(&my_boundarie, xM[gg], W_POSITION);
                        my_i = 5*N_PCS_ARRAY + 0;
                    } break;
                    case 4: { 
                        my_boundarie = _multiple_pcs_boundaries_pcs->b_y;
                        tmp = calculate_parameter_deviation_for_f(&my_boundarie, yM[gg], W_POSITION);
                        my_i = 5*N_PCS_ARRAY + 1;
                    } break;
                    case 5: { 
                        my_boundarie = _multiple_pcs_boundaries_pcs->b_z;
                        tmp = calculate_parameter_deviation_for_f(&my_boundarie, zM[gg], W_POSITION);
                        my_i = 5*N_PCS_ARRAY + 2;
                    } break;
                }
                if(index_bound_multi[my_i] == -1){
                    fprintf(stderr, "Problem with index_bound_multi\n");
                    exit(0);
                }
                gsl_vector_set (f, n_total + index_bound_multi[my_i], tmp);
            }
        }
    }
    
    return GSL_SUCCESS;
}


double calculate_parameter_deviation_for_deriv_f(boundarie * my_boundarie, double  value, double  w){
    //double result;
    double dev_min;
    double dev_max;
    dev_min = dev_max = 0;
    if( value > my_boundarie->max){
        //dev_max = fabs(value - my_boundarie->max);
        return(sqrt(w));
    }
    
    if( value < my_boundarie->min){
        //dev_min = fabs(my_boundarie->min - value);
        return(-sqrt(w));
        
    }
    //result = dev_min + dev_max;
    return(0);
}

/*----------------------------------------------------------------------*/
int expb_df_unified_multi (const gsl_vector * v, void *data, gsl_matrix * J)
/*----------------------------------------------------------------------*/
{
    int i;
    //int j;
    double tmp, tmp2;//, delta;
    /*
    int n_button = ((struct PCSdata3 *) data)->n_button;
    gboolean * bitmap_bound =  ((struct PCSdata3 *) data)->bitmap_bound;
    int n_bound = ((struct PCSdata3 *) data)->n_bound;
    int n_opt = ((struct PCSdata3 *) data)->n_opt;
    */
    boundarie my_boundarie;
    int my_i;
    
    
    int * n = ((struct PCSdata4 *)data)->n;
    int ** index_on_pdb = ((struct PCSdata4 *) data)->index_on_pdb;
    int ** index_on_pcs = ((struct PCSdata4 *) data)->index_on_pcs;
    int * index_v_multi =  ((struct PCSdata4 *) data)->index_v_multi;
    int * index_bound_multi = ((struct PCSdata4 *)data)->index_bound_multi;
    gboolean ** bitmap_opt =  ((struct PCSdata4 *) data)->bitmap_opt;
    gboolean * bitmap_opt_center =  ((struct PCSdata4 *) data)->bitmap_opt_center;
    gboolean * g_button_tab = ((struct PCSdata4 *) data)->g_button_tab;
    gboolean ** bitmap_bound =  ((struct PCSdata4 *) data)->bitmap_bound;
    gboolean * bitmap_bound_center =  ((struct PCSdata4 *) data)->bitmap_bound_center;
    double alpha[N_PCS_ARRAY], beta[N_PCS_ARRAY], gamma[N_PCS_ARRAY], Xax[N_PCS_ARRAY], 
        Xrh[N_PCS_ARRAY], xM[N_PCS_ARRAY], yM[N_PCS_ARRAY], zM[N_PCS_ARRAY];
    int gg;
    int i_total;
    int hh, kk;
    int hh_max, kk_max;
	double *offset_noise_x, *offset_noise_y, *offset_noise_z;
	double **PDB_OFFSET_X = ((struct PCSdata4 *)data)->PDB_OFFSET_X;
	double **PDB_OFFSET_Y = ((struct PCSdata4 *)data)->PDB_OFFSET_Y;
	double **PDB_OFFSET_Z = ((struct PCSdata4 *)data)->PDB_OFFSET_Z;
    gboolean ** SUBSET_ERROR = ((struct PCSdata4 *)data)->SUBSET_ERROR;
	gboolean *use_racs_t = ((struct PCSdata4 *) data)->use_racs;
    gboolean use_racs;
    racs_data * racs_d = ((struct PCSdata4 *) data)->racs_d;
    get_correct_parameters_multi(alpha, beta, gamma, xM, yM, zM, Xax, Xrh, v, data);
    
    double R_chi[3][3];
    double R_chi_plus_d_alpha[3][3];
    double R_chi_plus_d_beta[3][3];
    double R_chi_plus_d_gamma[3][3];
    double R_chi_minus_d_alpha[3][3];
    double R_chi_minus_d_beta[3][3];
    double R_chi_minus_d_gamma[3][3];
    


    
    //CALCULATE ROW SIZE OF J and COLUMN size of J
    hh_max = 0;
    kk_max = 0;
    for(gg = 0; gg < N_PCS_ARRAY; gg++){
        use_racs = use_racs_t[gg];
        if(g_button_tab[gg] == FALSE){
            continue;
        }
        hh_max += n[gg];
    }
    for(i = 0; i < (N_PCS_ARRAY*5+3); i++){
        if(index_v_multi[i] != -1){
            kk_max ++;
        }
        if(index_bound_multi[i] != -1){
            hh_max ++;
        }
    }
    
    //INIT J with 0
    for(hh = 0; hh < hh_max; hh++){
        for(kk = 0; kk < kk_max; kk++){
            gsl_matrix_set (J, hh, kk, 0.0);
        }
    }
    
    i_total = 0;
    for(gg = 0; gg < N_PCS_ARRAY; gg++){
        
        filZX_or_YZ(R_chi, alpha[gg]*180/M_PI, beta[gg]*180/M_PI, gamma[gg]*180/M_PI);
        
        filZX_or_YZ(R_chi_plus_d_alpha, (alpha[gg] + _DELTA_ANGLE)*180/M_PI, beta[gg]*180/M_PI, gamma[gg]*180/M_PI);
        filZX_or_YZ(R_chi_plus_d_beta, alpha[gg]*180/M_PI, (beta[gg] + _DELTA_ANGLE)*180/M_PI, gamma[gg]*180/M_PI);
        filZX_or_YZ(R_chi_plus_d_gamma, alpha[gg]*180/M_PI, beta[gg]*180/M_PI, (gamma[gg] + _DELTA_ANGLE)*180/M_PI);
    
        filZX_or_YZ(R_chi_minus_d_alpha, (alpha[gg] - _DELTA_ANGLE)*180/M_PI, beta[gg]*180/M_PI, gamma[gg]*180/M_PI);
        filZX_or_YZ(R_chi_minus_d_beta, alpha[gg]*180/M_PI, (beta[gg] - _DELTA_ANGLE)*180/M_PI, gamma[gg]*180/M_PI);
        filZX_or_YZ(R_chi_minus_d_gamma, alpha[gg]*180/M_PI, beta[gg]*180/M_PI, (gamma[gg] - _DELTA_ANGLE)*180/M_PI);
        
        
        use_racs = use_racs_t[gg];
        if(g_button_tab[gg] == FALSE){
            continue;
        }
        for (i = 0; i < n[gg]; i++) {
            tmp2 = 1.0;
        	offset_noise_x = PDB_OFFSET_X[gg];
			offset_noise_y = PDB_OFFSET_Y[gg];
			offset_noise_z = PDB_OFFSET_Z[gg];
			
			if(SUBSET_ERROR[gg][i] == FALSE){
				tmp2 = 0.0; //easier that way, even though not the most efficient
			}
			
            //alpha, beta, gamma
            //delta = 0.001;
            if(bitmap_opt[gg][0] == TRUE){
                tmp = (SinglePCS_new(R_chi_plus_d_alpha, xM[gg], yM[gg], zM[gg], Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
                    SinglePCS_new(R_chi_minus_d_alpha, xM[gg], yM[gg], zM[gg], Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_ANGLE);
                if(index_v_multi[5*gg + 0] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
                gsl_matrix_set (J, i_total, index_v_multi[5*gg + 0], tmp*tmp2);
            }
            if(bitmap_opt[gg][1] == TRUE){
                tmp = (SinglePCS_new(R_chi_plus_d_beta, xM[gg], yM[gg], zM[gg], Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
                    SinglePCS_new(R_chi_minus_d_beta, xM[gg], yM[gg], zM[gg], Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_ANGLE);
                if(index_v_multi[5*gg + 1] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
                gsl_matrix_set (J, i_total, index_v_multi[5*gg + 1], tmp*tmp2);
            }
            if(bitmap_opt[gg][2] == TRUE){
                tmp = (SinglePCS_new(R_chi_plus_d_gamma, xM[gg], yM[gg], zM[gg], Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
                    SinglePCS_new(R_chi_minus_d_gamma, xM[gg], yM[gg], zM[gg], Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_ANGLE);
                if(index_v_multi[5*gg + 2] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
                gsl_matrix_set (J, i_total, index_v_multi[5*gg + 2], tmp*tmp2);
            }
            
            //x, y, z
            //delta = 0.001;
            if(bitmap_opt_center[0] == TRUE){
                tmp = (SinglePCS_new(R_chi, xM[gg]+_DELTA_COO, yM[gg], zM[gg], Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
                    SinglePCS_new(R_chi, xM[gg]-_DELTA_COO, yM[gg], zM[gg], Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_COO);
                if(index_v_multi[5*N_PCS_ARRAY + 0] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
                gsl_matrix_set (J, i_total, index_v_multi[5*N_PCS_ARRAY + 0], tmp*tmp2);
            }
            if(bitmap_opt_center[1] == TRUE){
                tmp = (SinglePCS_new(R_chi, xM[gg], yM[gg]+_DELTA_COO, zM[gg], Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
                    SinglePCS_new(R_chi, xM[gg], yM[gg]-_DELTA_COO, zM[gg], Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_COO);
                if(index_v_multi[5*N_PCS_ARRAY + 1] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
                gsl_matrix_set (J, i_total, index_v_multi[5*N_PCS_ARRAY + 1], tmp*tmp2);
            }
            if(bitmap_opt_center[2] == TRUE){
                tmp = (SinglePCS_new(R_chi, xM[gg], yM[gg], zM[gg]+_DELTA_COO, Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
                    SinglePCS_new(R_chi, xM[gg], yM[gg], zM[gg]-_DELTA_COO, Xax[gg], Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_COO);
                if(index_v_multi[5*N_PCS_ARRAY + 2] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
                gsl_matrix_set (J, i_total, index_v_multi[5*N_PCS_ARRAY + 2], tmp*tmp2);
            }
            
            //Ax, Rh
            //delta = 1.0;
            if(bitmap_opt[gg][6] == TRUE){
                tmp = (SinglePCS_new(R_chi, xM[gg], yM[gg], zM[gg], Xax[gg]+_DELTA_MAGN, Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
                    SinglePCS_new(R_chi, xM[gg], yM[gg], zM[gg], Xax[gg]-_DELTA_MAGN, Xrh[gg], i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_MAGN);
                if(index_v_multi[5*gg + 3] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
                gsl_matrix_set (J, i_total, index_v_multi[5*gg + 3], tmp*tmp2);
            }
            if(bitmap_opt[gg][7] == TRUE){
                tmp = (SinglePCS_new(R_chi, xM[gg], yM[gg], zM[gg], Xax[gg], Xrh[gg]+_DELTA_MAGN, i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
                    SinglePCS_new(R_chi, xM[gg], yM[gg], zM[gg], Xax[gg], Xrh[gg]-_DELTA_MAGN, i, gg, index_on_pdb[gg], index_on_pcs[gg] , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_MAGN);
                if(index_v_multi[5*gg + 4] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
                gsl_matrix_set (J, i_total, index_v_multi[5*gg + 4], tmp*tmp2);
            }
            i_total++;
        }
    }
    
    
    for(gg = 0; gg < N_PCS_ARRAY; gg++){
        if(g_button_tab[gg] == FALSE){
            continue;
        }
        my_i = 999;
        for(i = 0; i < 8; i++){
            if((bitmap_bound[gg][i] == TRUE)&&((i!=3)&&(i!=4)&&(i!=5))){
                tmp = 0;
                switch(i){
                    case 0: { 
                        my_boundarie = _all_boundaries_pcs[gg]->b_a;
                        tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, alpha[gg]/M_PI*180.0, W_ANGLE);
                        my_i = 5*gg + i;
                    } break;
                    case 1: { 
                        my_boundarie = _all_boundaries_pcs[gg]->b_b;
                        tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, beta[gg]/M_PI*180.0, W_ANGLE);
                        my_i = 5*gg + i;
                    } break;
                    case 2: { 
                        my_boundarie = _all_boundaries_pcs[gg]->b_g;
                        tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, gamma[gg]/M_PI*180.0, W_ANGLE);
                        my_i = 5*gg + i;
                    } break;
                    case 6: { 
                        my_boundarie = _all_boundaries_pcs[gg]->b_Ax;
                        tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, Xax[gg], W_MAGNITUDE);
                        my_i = 5*gg + i - 3;
                    } break;
                    case 7: { 
                        my_boundarie = _all_boundaries_pcs[gg]->b_Rh;
                        tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, Xrh[gg], W_MAGNITUDE);
                        my_i = 5*gg + i - 3;
                    } break;
                }
                gsl_matrix_set (J, i_total + index_bound_multi[my_i], index_v_multi[my_i], tmp);
            }
        }
        for(i = 3; i < 6; i++){
            if(bitmap_bound_center[i-3] == TRUE){
                tmp = 0;
                switch(i){
                    case 3: { 
                        my_boundarie = _multiple_pcs_boundaries_pcs->b_x;
                        tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, xM[gg], W_POSITION);
                        my_i = 5*N_PCS_ARRAY+ 0;
                    } break;
                    case 4: { 
                        my_boundarie = _multiple_pcs_boundaries_pcs->b_y;
                        tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, yM[gg], W_POSITION);
                        my_i = 5*N_PCS_ARRAY+ 1;
                    } break;
                    case 5: { 
                        my_boundarie = _multiple_pcs_boundaries_pcs->b_z;
                        tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, zM[gg], W_POSITION);
                        my_i = 5*N_PCS_ARRAY+ 2;
                    } break;
                }
                gsl_matrix_set (J, i_total + index_bound_multi[my_i], index_v_multi[my_i], tmp);
            }
        }
    }

    return GSL_SUCCESS;
}


/*----------------------------------------------------------------------*/
int expb_df_unified (const gsl_vector * v, void *data, gsl_matrix * J)
/*----------------------------------------------------------------------*/
{
    size_t i;
    int j;
    double tmp, tmp2;//, delta_magn, delat_angle, delta_coor;
    size_t n = ((struct PCSdata3 *)data)->n;
    int n_button = ((struct PCSdata3 *) data)->n_button;
    int * index_on_pdb = ((struct PCSdata3 *) data)->index_on_pdb;
    int * index_on_pcs = ((struct PCSdata3 *) data)->index_on_pcs;
    gboolean * bitmap_opt =  ((struct PCSdata3 *) data)->bitmap_opt;
    int * index_v =  ((struct PCSdata3 *) data)->index_v;
    gboolean * bitmap_bound =  ((struct PCSdata3 *) data)->bitmap_bound;
    int n_bound = ((struct PCSdata3 *) data)->n_bound;
    int n_opt = ((struct PCSdata3 *) data)->n_opt;
    boundarie my_boundarie;
    int my_i;
    double alpha, beta, gamma, Xax, Xrh, xM, yM, zM;
	double *offset_noise_x, *offset_noise_y, *offset_noise_z;
	double *PDB_OFFSET_X = ((struct PCSdata3 *)data)->PDB_OFFSET_X;
	double *PDB_OFFSET_Y = ((struct PCSdata3 *)data)->PDB_OFFSET_Y;
	double *PDB_OFFSET_Z = ((struct PCSdata3 *)data)->PDB_OFFSET_Z;
	gboolean * SUBSET_ERROR = ((struct PCSdata3 *)data)->SUBSET_ERROR;
	gboolean use_racs = ((struct PCSdata3 *) data)->use_racs;
    racs_data * racs_d = ((struct PCSdata3 *) data)->racs_d;
    get_correct_parameters(&alpha, &beta, &gamma, &xM, &yM, &zM, &Xax, &Xrh, v, data);
    
    double R_chi_plus_d_alpha[3][3];
    double R_chi_plus_d_beta[3][3];
    double R_chi_plus_d_gamma[3][3];
    double R_chi_minus_d_alpha[3][3];
    double R_chi_minus_d_beta[3][3];
    double R_chi_minus_d_gamma[3][3];
    
    double R_chi[3][3];
    
    filZX_or_YZ(R_chi_plus_d_alpha, (alpha + _DELTA_ANGLE)*180/M_PI, beta*180/M_PI, gamma*180/M_PI);
    filZX_or_YZ(R_chi_plus_d_beta, alpha*180/M_PI, (beta + _DELTA_ANGLE)*180/M_PI, gamma*180/M_PI);
    filZX_or_YZ(R_chi_plus_d_gamma, alpha*180/M_PI, beta*180/M_PI, (gamma + _DELTA_ANGLE)*180/M_PI);
    
    filZX_or_YZ(R_chi_minus_d_alpha, (alpha - _DELTA_ANGLE)*180/M_PI, beta*180/M_PI, gamma*180/M_PI);
    filZX_or_YZ(R_chi_minus_d_beta, alpha*180/M_PI, (beta - _DELTA_ANGLE)*180/M_PI, gamma*180/M_PI);
    filZX_or_YZ(R_chi_minus_d_gamma, alpha*180/M_PI, beta*180/M_PI, (gamma - _DELTA_ANGLE)*180/M_PI);
    
    filZX_or_YZ(R_chi, alpha*180/M_PI, beta*180/M_PI, gamma*180/M_PI);
    
    for (i = 0; i < n; i++) {
        tmp2 = 1.0;
		if(SUBSET_ERROR[i] == FALSE){
			tmp2 = 0.0; //easier that way, even though not the most efficient
		}
		offset_noise_x = PDB_OFFSET_X;
		offset_noise_y = PDB_OFFSET_Y;
		offset_noise_z = PDB_OFFSET_Z;
        //alpha, beta, gamma
        //delta = 0.001;
        if(bitmap_opt[0] == TRUE){
            tmp = (SinglePCS_new(R_chi_plus_d_alpha, xM, yM, zM, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
               SinglePCS_new(R_chi_minus_d_alpha, xM, yM, zM, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_ANGLE);
            if(index_v[0] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
            gsl_matrix_set (J, i, index_v[0], tmp*tmp2);
        }
        if(bitmap_opt[1] == TRUE){
            tmp = (SinglePCS_new(R_chi_plus_d_beta, xM, yM, zM, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
                SinglePCS_new(R_chi_minus_d_beta, xM, yM, zM, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_ANGLE);
            if(index_v[1] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
            gsl_matrix_set (J, i, index_v[1], tmp*tmp2);
        }
        if(bitmap_opt[2] == TRUE){
            tmp = (SinglePCS_new(R_chi_plus_d_gamma, xM, yM, zM, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
                SinglePCS_new(R_chi_minus_d_gamma, xM, yM, zM, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_ANGLE);
            if(index_v[2] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
            gsl_matrix_set (J, i, index_v[2], tmp*tmp2);
        }

        //x, y, z
        //delta = 0.001;
        if(bitmap_opt[3] == TRUE){
            tmp = (SinglePCS_new(R_chi, xM+_DELTA_COO, yM, zM, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
               SinglePCS_new(R_chi, xM-_DELTA_COO, yM, zM, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_COO);
            if(index_v[3] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
            gsl_matrix_set (J, i, index_v[3], tmp*tmp2);
        }
        if(bitmap_opt[4] == TRUE){
            tmp = (SinglePCS_new(R_chi, xM, yM+_DELTA_COO, zM, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
               SinglePCS_new(R_chi, xM, yM-_DELTA_COO, zM, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_COO);
            if(index_v[4] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
            gsl_matrix_set (J, i, index_v[4], tmp*tmp2);
        }
        if(bitmap_opt[5] == TRUE){
            tmp = (SinglePCS_new(R_chi, xM, yM, zM+_DELTA_COO, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
               SinglePCS_new(R_chi, xM, yM, zM-_DELTA_COO, Xax, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_COO);
            if(index_v[5] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
            gsl_matrix_set (J, i, index_v[5], tmp*tmp2);
        }
        
        //Ax, Rh
        //delta = 1.0;
        if(bitmap_opt[6] == TRUE){
            tmp = (SinglePCS_new(R_chi, xM, yM, zM, Xax+_DELTA_MAGN, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
               SinglePCS_new(R_chi, xM, yM, zM, Xax-_DELTA_MAGN, Xrh, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_MAGN);
            if(index_v[6] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
            gsl_matrix_set (J, i, index_v[6], tmp*tmp2);
        }
        if(bitmap_opt[7] == TRUE){
            tmp = (SinglePCS_new(R_chi, xM, yM, zM, Xax, Xrh+_DELTA_MAGN, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d) - 
               SinglePCS_new(R_chi, xM, yM, zM, Xax, Xrh-_DELTA_MAGN, i, n_button, index_on_pdb, index_on_pcs , offset_noise_x, offset_noise_y, offset_noise_z, use_racs, NULL, racs_d)) / (2*_DELTA_MAGN);
            if(index_v[7] == -1){ fprintf(stderr, "Big problem with index_v!\n");}
            gsl_matrix_set (J, i, index_v[7], tmp*tmp2);
        }
    }
    //for boundaries
    
    for(i = 0; i < n_bound; i++){
        for(j = 0; j < n_opt; j++){
            gsl_matrix_set (J, n + i, j, 0.0);
        }
    }
    
    my_i = 0;
    for(i = 0; i < 8; i++){ 
        if(bitmap_bound[i] == TRUE){
            tmp = 0;
            switch(i){
                case 0: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_a;
                    tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, alpha/M_PI*180.0, W_ANGLE);
                } break;
                case 1: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_b;
                    tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, beta/M_PI*180.0, W_ANGLE);
                } break;
                case 2: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_g;
                    tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, gamma/M_PI*180.0, W_ANGLE);
                } break;
                case 3: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_x;
                    tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, xM, W_POSITION);
                } break;
                case 4: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_y;
                    tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, yM, W_POSITION);
                } break;
                case 5: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_z;
                    tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, zM, W_POSITION);
                } break;
                case 6: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_Ax;
                    tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, Xax, W_MAGNITUDE);
                } break;
                case 7: { 
                    my_boundarie = _all_boundaries_pcs[n_button]->b_Rh;
                    tmp = calculate_parameter_deviation_for_deriv_f(&my_boundarie, Xrh, W_MAGNITUDE);
                } break;
            }
            gsl_matrix_set (J, n + my_i, index_v[i], tmp);
            my_i++;
        }
    }

    return GSL_SUCCESS;
}


/*----------------------------------------------------------------------*/
int expb_fdf_unified (const gsl_vector * x, void *data, gsl_vector * f, gsl_matrix * J)
/*----------------------------------------------------------------------*/
{
  expb_f_unified (x, data, f);
  expb_df_unified (x, data, J);

  return GSL_SUCCESS;
}

/*----------------------------------------------------------------------*/
int expb_fdf_unified_multi (const gsl_vector * x, void *data, gsl_vector * f, gsl_matrix * J)
/*----------------------------------------------------------------------*/
{
  expb_f_unified_multi (x, data, f);
  expb_df_unified_multi (x, data, J);

  return GSL_SUCCESS;
}


/*----------------------------------------------------------------------*/
void print_state (int iter, gsl_multifit_fdfsolver * s, 
				  int status, GtkTextBuffer* buffer)
/*----------------------------------------------------------------------*/
{
  GtkTextIter iterbuf;
  char ligne[100];

	
	if(status != -999){
		sprintf (ligne, "iter: %10i |f(x)| = %.9lf  status = %s\n", iter, gsl_blas_dnrm2 (s->f)
		   ,gsl_strerror (status));
	}
	else{
		sprintf (ligne, "iter: %10i |f(x)| = %.9lf\n", iter, gsl_blas_dnrm2 (s->f));
	}
	//printf(ligne);
	gtk_text_buffer_get_iter_at_offset(buffer,&iterbuf,0);
	gtk_text_buffer_insert(buffer, &iterbuf, ligne, -1);
	
}



/*----------------------------------------------------------------------*/
char *parse_labelfield(char *line)
/*----------------------------------------------------------------------*/
{
    char  *pos;
    int   n, i;

    n = 0;
    pos = line;
    for (i = 0; i < 4; i++) {
        while (*pos == ' ')
            ++pos;
        while ((*pos != '\n') && (*pos != ' ') && (*pos != '\t'))
            ++pos;
        if (*pos == '\n')
            return NULL;
    }
    while (*pos == ' ')
        ++pos;
    return pos;
}


/*----------------------------------------------------------------------*/
int PrintTensorCoord(double phi, double theta, double psi, double ax, double rh, double xM, double yM, double zM)
/*----------------------------------------------------------------------*/
{
    

    //FILE *fpipe;
    //char line[MAX_LINE_LEN], command[MAX_LINE_LEN];
    //int  i, j;
    //double  x, y, z;
    //double  xs, ys, zs;
    double  xa, ya, za;

    //double  sint, sins, sinp, cost, coss, cosp;

  double R[3][3];
    
    filZX_or_YZ(R, phi, theta, psi);
    
/*
    sint = sin(M_PI*theta/180.0);
    sins = sin(M_PI*psi/180.0);
    sinp = sin(M_PI*phi/180.0);
    cost = cos(M_PI*theta/180.0);
    coss = cos(M_PI*psi/180.0);
    cosp = cos(M_PI*phi/180.0);
*/
/*
    sprintf(command, "./compare_tensor %lf %lf %lf %lf %lf 2> /dev/null", phi, theta, psi, ax, rh);

    if ( !(fpipe = (FILE*)popen(command,"r")) ) {  // If fpipe is NULL
        fprintf(stderr, "Problems with pipe\n");
        exit(1);
    }


    xs = ys = zs = 0;
    fgets( line, MAX_LINE_LEN, fpipe);
    fgets( line, MAX_LINE_LEN, fpipe);
    for (i = 0; i < 6; i++) {
        sscanf(&line[8], "%lf%lf", &ax, &rh);
        z = 2*ax/3.0;
        y = -rh/2.0 - ax/3.0;
        x = rh/2.0 - ax/3.0;
        if ((z > y) && (y > x)) {
            zs = z;
            ys = y;
            xs = x;
        }
        for (j = 0; j < 6; j++)
            fgets( line, MAX_LINE_LEN, fpipe);
    }
    pclose(fpipe);
*/
        
        //WARNING, here we print the transpose matrix
	
    fprintf(stdout, "\n");
    fprintf(stdout, "+----------------------------------------------------------------+\n");
    fprintf(stdout, "|                Tensor frame                                    |\n");
    fprintf(stdout, "+----------------------------------------------------------------+\n\n");
    fprintf(stdout, "                        x              y              z\n");
    fprintf(stdout, "tensor-x: %15.5f%15.5f%15.5f\n"
            , R[0][0], R[0][1], R[0][2]);
    fprintf(stdout, "tensor-y: %15.5f%15.5f%15.5f\n"
            , R[1][0], R[1][1], R[1][2]);
    fprintf(stdout, "tensor-z: %15.5f%15.5f%15.5f\n"
            , R[2][0], R[2][1], R[2][2] );
    //fprintf(stdout, "l: %15.5f%15.5f%15.5f\n\n", xs, ys, zs);


    xa = 5;
    ya = 5;
    za = 5;
    fprintf(stdout, "REMARK --- Cut-and-paste these lines to the end of the PDB file to visualise the tensor frame ---\n");
    fprintf(stdout, "REMARK  Tensor coordinate frame centred on metal ion and with axis length of %lf Angstrom\n", xa);
    fprintf(stdout, "ATOM  10000  OO  TSR  1000%12.3f%8.3f%8.3f  1.00  0.00\n", xM, yM, zM);
    fprintf(stdout, "ATOM  10001 1OX  TSR  1000%12.3f%8.3f%8.3f  1.00  0.00\n"
            , xM+xa*R[0][0], yM+xa*R[0][1], zM+xa*R[0][2]);
    fprintf(stdout, "ATOM  10002 1OY  TSR  1000%12.3f%8.3f%8.3f  1.00  0.00\n"
            , xM+ya*R[1][0], yM+ya*R[1][1], zM+ya*R[1][2]);
    fprintf(stdout, "ATOM  10003 1OZ  TSR  1000%12.3f%8.3f%8.3f  1.00  0.00\n"
            , xM+za*R[2][0], yM+za*R[2][1], zM+za*R[2][2]);
    xa = -5;
    ya = -5;
    za = -5;
    fprintf(stdout, "ATOM  10004 2OX  TSR  1000%12.3f%8.3f%8.3f  1.00  0.00\n"
            , xM+xa*R[0][0], yM+xa*R[0][1], zM+xa*R[0][2]);
    fprintf(stdout, "ATOM  10005 2OY  TSR  1000%12.3f%8.3f%8.3f  1.00  0.00\n"
            , xM+ya*R[1][0], yM+ya*R[1][1], zM+ya*R[1][2]);
    fprintf(stdout, "ATOM  10006 2OZ  TSR  1000%12.3f%8.3f%8.3f  1.00  0.00\n"
            , xM+za*R[2][0], yM+za*R[2][1], zM+za*R[2][2]);
    fprintf(stdout, "CONECT100001000110002\n");
    fprintf(stdout, "CONECT100001000110003\n");
    fprintf(stdout, "CONECT100001000410005\n");
    fprintf(stdout, "CONECT100001000410006\n");
	fprintf(stdout, "\n");
    return 0;
}

int check_pcs_loaded(int n_button){
    char a[3];  
    if(_all_pcs[n_button] == NULL){
        sprintf(a, "%d", n_button+1); 
        fprintf(stderr, "You need first to load a PCS file in experiment %s\n", a);
        box_error( "error, You need first to load a PCS file in experiment %s\n", a);
        return(-1);
    }
    return(0);
}


/*----------------------------------------------------------------------*/
int calculate_single_tensor(int n_button,
					 error_control_data * err_ctrl,
					 tensor my_tensor_multi[N_PCS_ARRAY])
/*----------------------------------------------------------------------*/
{
    const gsl_multifit_fdfsolver_type *T;
    gsl_multifit_fdfsolver *s;
	int n_sample;
    int status;
    int i, iter = 0;
    int n_bis, /*j,*/ p;    /* p = number of parameters */
    int n_bis_p;
    int my_k;
    int n_bound;
    //char   **label;
    double PCS[N], x[N], y[N], z[N], TOL[N];
	double PCS_OFFSET[N], PDB_OFFSET_X[N], PDB_OFFSET_Y[N], PDB_OFFSET_Z[N];
	gboolean SUBSET_ERROR[N];
    gboolean bitmap_opt[8];
    gboolean bitmap_bound[8];
    int index_v[8];
    //double xM[N], yM[N], zM[N];
    //double xM, yM, zM, Ax, Rh, a, b, g;
    double x_init[8];
    double x_init_smaller[8];
    int * index_on_pdb = NULL;
    int * index_on_pcs = NULL;
	gsl_rng * my_rand;
	my_rand = NULL;
	//int serial;
	gboolean use_racs;
    racs_data racs_d;
    
    use_racs = is_use_racs_PCS(n_button);
    get_racs_parameters(&racs_d);
	//fprintf(stderr, "start calculate_single_tensor\n");
	
    if( _all_pcs[n_button] == NULL){
        if(check_pcs_loaded(n_button) == -1){
            return(-1);
        }
    }
    
    {
        int compt;
        compt = 0;
        for(i = 0; i< _all_pdb_selector_pcs[n_button]->len; i++){
            if(g_array_index(_all_pdb_selector_pcs[n_button], pdb_selector, i).is_used 
               == TRUE){
                   compt ++;
                   break;
               }
        }
        if( compt == 0){
            box_error("You should use at least one pdb model\n", NULL);
            return (-1);
        }
    }
    
    for(i = 0; i < 8; i++){
        index_v[i] = -1;
        bitmap_opt[i] = FALSE;
        bitmap_bound[i] = FALSE;
    }
	
    
    p = 0;
    i = 0;
    if(_all_boundaries_pcs[n_button]->b_a.type != TYPE_FIXED) { index_v[i] = p; p++; bitmap_opt[0] = TRUE;}
    i++;
    if(_all_boundaries_pcs[n_button]->b_b.type != TYPE_FIXED) { index_v[i] = p; p++; bitmap_opt[1] = TRUE;}
    i++;
    if(_all_boundaries_pcs[n_button]->b_g.type != TYPE_FIXED) { index_v[i] = p; p++; bitmap_opt[2] = TRUE;}
    i++;
    if(_all_boundaries_pcs[n_button]->b_x.type != TYPE_FIXED) { index_v[i] = p; p++; bitmap_opt[3] = TRUE;}
    i++;
    if(_all_boundaries_pcs[n_button]->b_y.type != TYPE_FIXED) { index_v[i] = p; p++; bitmap_opt[4] = TRUE;}
    i++;
    if(_all_boundaries_pcs[n_button]->b_z.type != TYPE_FIXED) { index_v[i] = p; p++; bitmap_opt[5] = TRUE;}
    i++;
    if(_all_boundaries_pcs[n_button]->b_Ax.type != TYPE_FIXED){ index_v[i] = p; p++; bitmap_opt[6] = TRUE;}
    i++;
    if(_all_boundaries_pcs[n_button]->b_Rh.type != TYPE_FIXED){ index_v[i] = p; p++; bitmap_opt[7] = TRUE;}
    
    n_bound = 0;
    if(_all_boundaries_pcs[n_button]->b_a.type == TYPE_BOUNDARIES) { bitmap_bound[0] = TRUE; n_bound++;}
    if(_all_boundaries_pcs[n_button]->b_b.type == TYPE_BOUNDARIES) { bitmap_bound[1] = TRUE; n_bound++;}
    if(_all_boundaries_pcs[n_button]->b_g.type == TYPE_BOUNDARIES) { bitmap_bound[2] = TRUE; n_bound++;}
    if(_all_boundaries_pcs[n_button]->b_x.type == TYPE_BOUNDARIES) { bitmap_bound[3] = TRUE; n_bound++;}
    if(_all_boundaries_pcs[n_button]->b_y.type == TYPE_BOUNDARIES) { bitmap_bound[4] = TRUE; n_bound++;}
    if(_all_boundaries_pcs[n_button]->b_z.type == TYPE_BOUNDARIES) { bitmap_bound[5] = TRUE; n_bound++;}
    if(_all_boundaries_pcs[n_button]->b_Ax.type == TYPE_BOUNDARIES){ bitmap_bound[6] = TRUE; n_bound++;}
    if(_all_boundaries_pcs[n_button]->b_Rh.type == TYPE_BOUNDARIES){ bitmap_bound[7] = TRUE; n_bound++;}

	if(err_ctrl == NULL){ // ERROR OFF
    	x_init[0] = _all_boundaries_pcs[n_button]->b_a.start;
    	x_init[1] = _all_boundaries_pcs[n_button]->b_b.start;
    	x_init[2] = _all_boundaries_pcs[n_button]->b_g.start;
   	 	x_init[3] = _all_boundaries_pcs[n_button]->b_x.start;
    	x_init[4] = _all_boundaries_pcs[n_button]->b_y.start;
    	x_init[5] = _all_boundaries_pcs[n_button]->b_z.start;
    	x_init[6] = _all_boundaries_pcs[n_button]->b_Ax.start;
    	x_init[7] = _all_boundaries_pcs[n_button]->b_Rh.start; 
    
    	x_init[0] *= M_PI/180.0;
    	x_init[1] *= M_PI/180.0;
    	x_init[2] *= M_PI/180.0;
    
    	fprintf(stdout, "\n");
    	fprintf(stdout, "+----------------------------------------------------------------+\n");
    	fprintf(stdout, "|                       Init value for tensor search             |\n");
    	fprintf(stdout, "+----------------------------------------------------------------+\n\n");
    	fprintf(stdout, "Ax start\t %lf\t\toptimized? %d\n", x_init[6], bitmap_opt[6]);
    	fprintf(stdout, "Rh start\t %lf\t\toptimized? %d\n", x_init[7], bitmap_opt[7]);
    	fprintf(stdout, "x  start\t %lf\t\toptimized? %d\n", x_init[3], bitmap_opt[3]);
    	fprintf(stdout, "y  start\t %lf\t\toptimized? %d\n", x_init[4], bitmap_opt[4]);
    	fprintf(stdout, "z  start\t %lf\t\toptimized? %d\n", x_init[5], bitmap_opt[5]);
    	fprintf(stdout, "a  start\t %lf\t\toptimized? %d\n", x_init[0]/M_PI*180.0, bitmap_opt[0]);
    	fprintf(stdout, "b  start\t %lf\t\toptimized? %d\n", x_init[1]/M_PI*180.0, bitmap_opt[1]);
    	fprintf(stdout, "g  start\t %lf\t\toptimized? %d\n", x_init[2]/M_PI*180.0, bitmap_opt[2]); 
	}
	else { // ERROR ON
		int my_n_butt;
		int i;
		my_n_butt = -1;
		for(i = 0; i < N_PCS_ARRAY; i++){
			if((err_ctrl->use_data_PCS)[i] == TRUE){
				my_n_butt = i;
			}
		}
		if(my_n_butt == -1){
			fprintf(stderr, "Problem with err_analysis\n");
			return(-1);
		}
		x_init[0] = my_tensor_multi[my_n_butt].a;
		x_init[1] = my_tensor_multi[my_n_butt].b;
    	x_init[2] = my_tensor_multi[my_n_butt].g;
   	 	x_init[3] = my_tensor_multi[my_n_butt].x;
    	x_init[4] = my_tensor_multi[my_n_butt].y;
    	x_init[5] = my_tensor_multi[my_n_butt].z;
    	x_init[6] = my_tensor_multi[my_n_butt].ax;
    	x_init[7] = my_tensor_multi[my_n_butt].rh; 
	}
    
    n_bis = 0;
    {
        int i;
        for(i = 0; i < _all_pcs[n_button]->len; i++){
            if(g_array_index (_all_pcs[n_button], Item_pcs, i).used == TRUE){
                n_bis++;
            }
        }
    }
    if (n_bis > N) {
        fprintf(stderr, "N too small\n");
        fprintf(stderr, "Numbat can't calculate tensor with a number of PCS bigger than %d\n", N);
        fprintf(stderr, "You need to change the variable N in  src/pcsfit_metal.c and recompile\n");
        return(-1);
    }
    
    if (p > n_bis) {
		char text[200];
        sprintf(text, "You need more than %d PCS values to optimize %d parameters\n\
                (you have only %d PCS values)\n", p, p, n_bis);
		box_error( text, NULL);
        return(-1);
    }
    
    if(p == 0){ //Nothing to optimize...
        fprintf(stderr, "nothing to optimize\n");
        box_error( "No paramter to optimize", NULL);
        return(-1);
    }
    
    if(index_on_pdb != NULL){
        free(index_on_pdb);
    }
    index_on_pdb = (int *) calloc(n_bis, sizeof(int));
    if(index_on_pdb == NULL){
        fprintf(stderr, "couldn't allocate memory to create index_on_pdb %d\n", n_bis);
        exit(-1);
    }
    
    if(index_on_pcs != NULL){
        free(index_on_pcs);
    }
    index_on_pcs = (int *) calloc(n_bis, sizeof(int));
    if(index_on_pcs == NULL){
        fprintf(stderr, "couldn't allocate memory to create index_on_pcs\n");
        exit(-1);
    }
    
    //////////////////////////
    n_bis_p = n_bis + n_bound;
    ///////////////////////////
    struct PCSdata3 d = { n_bis, PCS, TOL, x, y, z, 
        x_init[0], x_init[1], x_init[2], 
        x_init[3], x_init[4], x_init[5], 
        x_init[6], x_init[7], 
        n_button, 
        index_on_pdb, 
        index_on_pcs,
        index_v, 
        bitmap_opt,
        bitmap_bound, 
        p, //n_opt
        n_bound,
		err_ctrl,
		PCS_OFFSET,
		PDB_OFFSET_X,
		PDB_OFFSET_Y,
		PDB_OFFSET_Z,
		SUBSET_ERROR,
        use_racs,
        & racs_d
    }; 

    gsl_multifit_function_fdf f;
   
    {
        int k;
        int kk;
        kk = 0;
        for(k = 0; k < 8; k++){
            if( bitmap_opt[k] ==  TRUE){
                x_init_smaller[kk] = x_init[k];
                kk++;
            }
        }
        if(kk != p){
            fprintf(stderr, "WARNING problem with number of parameters to optimize\n");
        }
    }

    gsl_vector_view v = gsl_vector_view_array (x_init_smaller, p);

    const gsl_rng_type * type;
    gsl_rng * r;

    gsl_rng_env_setup();

    type = gsl_rng_default;
    r = gsl_rng_alloc (type);

	
    f.f = &expb_f_unified;
    f.df = &expb_df_unified;
    f.fdf = &expb_fdf_unified;
    //f.n = n;
    ///////////////////////////////////////////
    f.n = n_bis_p; //now I add the optimized tensor parameters for boundaries purpose
    ///////////////////////////////////////////
    f.p = p;
    f.params = &d;

    /* This is the data to be fitted */
    
    /*label = (char **) malloc(n_bis*sizeof(char *));
    if (label == NULL) {
        fprintf(stderr, "Error allocating label space\n");
        exit(1);
    }
	*/
    
    { 
        int t;
        for(t = 0; t < n_bis; t++){
            //label[t] = NULL;
        }
    }
    
    my_k = 0;
    for (i = 0; i < n_bis; i++) {
        //fprintf(stderr, "iteration %d\n", i);
        int index_pdb;
        while(g_array_index (_all_pcs[n_button], Item_pcs, my_k).used != TRUE){
            my_k++;
            if(my_k >= _all_pcs[n_button]->len){
                fprintf(stderr, "BIG_ERROR\n");
                    if(index_on_pdb != NULL){
                        free(index_on_pdb);
                    }
                    if(index_on_pcs != NULL){
                        free(index_on_pcs);
                    }
                return(-1);
            }
        }
		
		PCS_OFFSET[i] = 0.0;
		PDB_OFFSET_X[i] = 0.0;
		PDB_OFFSET_Y[i] = 0.0;
		PDB_OFFSET_Z[i] = 0.0;
		SUBSET_ERROR[i] = TRUE;
        PCS[i] = g_array_index (_all_pcs[n_button], Item_pcs, my_k).pcs_exp;
		TOL[i] = g_array_index (_all_pcs[n_button], Item_pcs, my_k).pcs_tol;
        index_on_pcs[i] = my_k;
        if(g_array_index (_all_pcs[n_button], Item_pcs, my_k).is_dumby == FALSE){
            index_pdb = find_pcs_in_pdb(g_array_index (_all_pcs[n_button], Item_pcs, my_k).residue_num
                        ,g_array_index (_all_pcs[n_button], Item_pcs, my_k).atom_name );
        }
        else { //it is a dummy atom 
           index_pdb = -2;
        }
        my_k++;  
        if(index_pdb == -1){
            fprintf(stderr, "ANOTHER BIG_ERROR\n");
            if(index_on_pdb != NULL){
                free(index_on_pdb);
            }
            if(index_on_pcs != NULL){
                free(index_on_pcs);
            }
            return(-1);
        }
        index_on_pdb[i] = index_pdb;

        char stemp[11];
        if(index_pdb == -2){
            sprintf(stemp, "%s_%d", g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[i]).atom_name,
                g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[i]).residue_num);
        }
        else {
            sprintf(stemp, "%s_%d", _pdb_str->tab_atom[index_pdb].atom_name,
            _pdb_str->tab_atom[index_pdb].residue_num);
        }
        //pos = parse_labelfield(line);
		/*
        label[i] = (char *) malloc(10*sizeof(char));
        if (label[i] == NULL) {
            fprintf(stderr, "Error allocating label space\n");
            exit(1);
        }
		*/
        //strcpy(label[i], stemp);
        //fprintf(stderr, "%lf\t%lf\t%lf\t%lf\t%s\n", PCS[i], x[i], y[i], z[i], label[i]);
    } // loop on i
	
	fprintf(stdout, "\n");
    fprintf(stdout, "+----------------------------------------------------------------+\n");
    fprintf(stdout, "|                       Optimisation output                      |\n");
    fprintf(stdout, "+----------------------------------------------------------------+\n\n");
	

	GtkTextBuffer* buffer;
	buffer = NULL;
	progressbar_init(&buffer);
	
	n_sample = 1;
	if(err_ctrl != NULL){
		n_sample = err_ctrl->n_sample;
	}
	
	switch(n_sample){
		case 1:{ //error OFF
			T = gsl_multifit_fdfsolver_lmsder;
    		s = gsl_multifit_fdfsolver_alloc (T, n_bis_p, p);
    		gsl_multifit_fdfsolver_set (s, &f, &v.vector);
			
			print_state (iter, s , -999, buffer);
    		printf ("\n");
			
			do {
       	 		iter++;
                status = gsl_multifit_fdfsolver_iterate (s);
				if(iter % 10 == 0){
                    print_state (iter, s, status, buffer);
					gboolean still_running = progressbar_update(1.0);
					if(still_running == FALSE){
						break;
					}
				}
                status = gsl_multifit_test_delta (s->dx, s->x, 1e-6, 1e-6);
    		}
    		while (status == GSL_CONTINUE && iter < MAX_ITER_SINGLE);
			print_state (iter, s, status, buffer);
			gsl_matrix *covar = gsl_matrix_alloc (p, p);
   	 		gsl_multifit_covar (s->J, 0.0, covar);
		
			postproceed_single(s, covar, n_bis, p, bitmap_opt,index_v,
					   x_init, n_button, use_racs, &racs_d);
			fprintf(stdout, "\n");
    		printf ("status = %s\n", gsl_strerror (status));
            
            //1 Get the tensor found.
            //2 translate it into UTR
            //3
            //gsl_multifit_fdfsolver_set (s, &f, &v.vector); //for UTR error
            
            
			gsl_multifit_fdfsolver_free (s);
    		gsl_matrix_free (covar);
			
		}
		break;
		default:{ //error ON
			tensor_analysis * t_a;
			gsl_rng * my_rand;
			int j;
			gboolean still_running = TRUE;
			int iter_total = 0;
			tensor my_tensor_loc;
			GtkTextIter iterbuf;
  			char ligne[100];
			my_rand = gsl_rng_alloc (gsl_rng_ranlxs0);
    		gsl_rng_set (my_rand, err_ctrl->seed);
			t_a = new_tensor_analysis(1, n_sample);
			
			//fprintf(stderr, "noise for str: %lf\n",err_ctrl->noise_structure);  
			for(i = 0; i < n_sample; i++){
				for (j = 0; j < n_bis; j++) {
					PCS_OFFSET[j] = gsl_ran_gaussian(my_rand, err_ctrl->noise_data_PCS[n_button]);
					PDB_OFFSET_X[j] = gsl_ran_gaussian(my_rand, err_ctrl->noise_structure);
					PDB_OFFSET_Y[j] = gsl_ran_gaussian(my_rand, err_ctrl->noise_structure);
					PDB_OFFSET_Z[j] = gsl_ran_gaussian(my_rand, err_ctrl->noise_structure);
					fill_bitmap_single_subset(SUBSET_ERROR, n_bis, my_rand, err_ctrl->subset_percentage);
				}
				
				int iter = 0;
				T = gsl_multifit_fdfsolver_lmsder;
    			s = gsl_multifit_fdfsolver_alloc (T, n_bis_p, p);
    			gsl_multifit_fdfsolver_set (s, &f, &v.vector);
				do {
       	 			iter++;
					iter_total ++;
                    status = gsl_multifit_fdfsolver_iterate (s);
					if(iter_total % 10 == 1){
						still_running = progressbar_update(1.0);
						if(still_running == FALSE){
							break;
						}
					}
                    status = gsl_multifit_test_delta (s->dx, s->x, 1e-6, 1e-6);
    			}
    			while (status == GSL_CONTINUE && iter < MAX_ITER_SINGLE);
				if(still_running == FALSE){
					break;
				}
				
				gsl_matrix *covar = gsl_matrix_alloc (p, p);
   	 			gsl_multifit_covar (s->J, 0.0, covar);
				
				get_tensor_single_from_covar(bitmap_opt, x_init,
						  s, index_v, &my_tensor_loc);
				store_tensor_analysis(t_a, 1, i, &my_tensor_loc, gsl_blas_dnrm2(s->f));
				sprintf(ligne, "%3d", i+1);
				fprintf(stderr, ligne);
				gtk_text_buffer_get_iter_at_offset(buffer,&iterbuf,0);
				gtk_text_buffer_insert(buffer, &iterbuf, ligne, -1);
				
				sprintf (ligne, "\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf", 
						 my_tensor_loc.a, my_tensor_loc.b, my_tensor_loc.g,
						 my_tensor_loc.x, my_tensor_loc.y, my_tensor_loc.z,
						 my_tensor_loc.ax, my_tensor_loc.rh,
						 gsl_blas_dnrm2 (s->f)
						 );
				fprintf(stderr, ligne);
				//gtk_text_buffer_get_iter_at_offset(buffer,&iterbuf,0);
				gtk_text_buffer_insert(buffer, &iterbuf, ligne, -1);
				
				sprintf(ligne, "\n");
				fprintf(stderr, ligne);
				//gtk_text_buffer_get_iter_at_offset(buffer,&iterbuf,0);
				gtk_text_buffer_insert(buffer, &iterbuf, ligne, -1);
                
                
				
				gsl_multifit_fdfsolver_free (s);
    			gsl_matrix_free (covar);
				
			}// i (n_sample)
			gsl_rng_free (my_rand);
			statistic_tensor_analysis(t_a);
            prepare_sanson(t_a);
			free_tensor_analysis(t_a);
		}
		break;
	} //switch

	progressbar_finish();
    if(n_sample !=1){ //ERROR ON
        start_sanson();
    }
	//FREE DATAs
    gsl_rng_free (r);
    if(index_on_pdb != NULL){
        free(index_on_pdb);
    }
    if(index_on_pcs != NULL){
        free(index_on_pcs);
    }
    //free_label(label, n_bis);
    return 0;
}

void fill_bitmap_multiple_subset(gboolean ** SUBSET_ERROR, int n_bis[N_PCS_ARRAY]
								 , gsl_rng * my_rand, double subset_percentage,
								 gboolean g_button_tab[N_PCS_ARRAY])
{
	gboolean * table_bool;
	int i, j;
	int offset_i;
	int n_total;
	double total_true_d;
	int total_true;
	
	n_total = 0;
	for(i = 0; i < N_PCS_ARRAY; i++){
		if(g_button_tab[i] == TRUE){
			n_total+= n_bis[i];
		}
	}
	table_bool = (gboolean *) calloc(n_total, sizeof(gboolean));
	
	total_true_d = n_total *subset_percentage / 100.0;
	total_true = ceil(total_true_d);
	
	for(i = 0; i < total_true; i++){
		table_bool[i] = TRUE;
	}
	for(i = total_true; i < n_total; i++){
		table_bool[i] = FALSE;
	}
	
	gsl_ran_shuffle (my_rand, table_bool, n_total, sizeof(gboolean));	
	
	offset_i = 0;
	for(j = 0; j < N_PCS_ARRAY; j++){
		if(g_button_tab[j] == TRUE){
			for(i = 0 ; i < n_bis[j]; i++){
				SUBSET_ERROR[j][i] = table_bool[i + offset_i];
			}
			offset_i += n_bis[j];
		}
	}
	
	for(j = 0; j < N_PCS_ARRAY; j++){
		if(g_button_tab[j] == TRUE){
			//fprintf(stderr, "table %d\n", j);
			for(i = 0 ; i < n_bis[j]; i++){
				//fprintf(stderr, "%d\n", SUBSET_ERROR[j][i]);
			}
		}
	}
	
	free(table_bool);
}

void fill_bitmap_single_subset(gboolean * SUBSET_ERROR, 
							   int n_bis, gsl_rng * my_rand, 
							   double subset_percentage)
{
	double total_true_d;
	int total_true;
	int i;
	
	total_true_d = n_bis *subset_percentage / 100.0;
	total_true = ceil(total_true_d);
	
	for(i = 0; i < total_true; i++){
		SUBSET_ERROR[i] = TRUE;
	}
	for(i = total_true; i < n_bis; i++){
		SUBSET_ERROR[i] = FALSE;
	}
	gsl_ran_shuffle (my_rand, SUBSET_ERROR, n_bis, sizeof(gboolean));							      
}


int get_tensor_multiple_from_covar(gboolean * g_button_tab, gsl_multifit_fdfsolver *s,
								   gboolean * bitmap_opt[N_PCS_ARRAY], gboolean bitmap_opt_center[3],
								   double x_init_center[3], double * x_init[N_PCS_ARRAY],
								   int index_v_multi[N_PCS_ARRAY * 5 + 3], 
								   tensor my_tensor_loc[N_PCS_ARRAY]
								   )
{
	int g, j;
	
#define FIT_MULTI(i) gsl_vector_get(s->x, index_v_multi[i])
	
	for(g = 0; g < N_PCS_ARRAY; g++){
           	double alpha_, beta_, gamma_;
            int ii;
            if(g_button_tab[g] == FALSE){
                continue;
            }
            
            if(bitmap_opt_center[0] == TRUE){
                my_tensor_loc[g].x = FIT_MULTI(5*N_PCS_ARRAY + 0);
            }
            else{
                my_tensor_loc[g].x = x_init_center[0];
            }
            if(bitmap_opt_center[1] == TRUE){
                my_tensor_loc[g].y = FIT_MULTI(5*N_PCS_ARRAY + 1);
            }
            else{
                my_tensor_loc[g].y = x_init_center[1];
            }
            if(bitmap_opt_center[1] == TRUE){
                my_tensor_loc[g].z = FIT_MULTI(5*N_PCS_ARRAY + 2);
            }
            else{
                my_tensor_loc[g].z = x_init_center[2];
            }
            //alpha
            if(bitmap_opt[g][0] == TRUE){
                alpha_ = 180/M_PI*FIT_MULTI(g*5+0);
            }
            else{
                alpha_ = 180/M_PI*x_init[g][0];
            }
            j = (int) (alpha_ / 360.0);
            if (alpha_ > 0)
                alpha_ -= j*360;
            else
                alpha_ -= (j-1)*360;
        
            //beta
            if(bitmap_opt[g][1] == TRUE){
                beta_ = 180/M_PI*FIT_MULTI(g*5+1);
            }
            else{
                beta_ = 180/M_PI*x_init[g][1];
            }
            j = (int) (beta_ / 360.0);
            if (beta_ > 0)
                beta_ -= j*360;
            else
                beta_ -= (j-1)*360;
    
            //gamma
            if(bitmap_opt[g][2] == TRUE){
                gamma_ = 180/M_PI*FIT_MULTI(g*5+2);
            }
            else{
                gamma_ = 180/M_PI*x_init[g][2];
            }
            j = (int) (gamma_ / 360.0);
            if (gamma_ > 0)
                gamma_ -= j*360;
            else
                gamma_ -= (j-1)*360;
        
            ii = 0;
            if(bitmap_opt[g][ii] == TRUE){
                my_tensor_loc[g].a = alpha_;
            }
            else{
                my_tensor_loc[g].a = 180/M_PI*x_init[g][ii];
            }
            ii++;
            if(bitmap_opt[g][ii] == TRUE){
                my_tensor_loc[g].b = beta_;
            }
            else{
                my_tensor_loc[g].b = 180/M_PI*x_init[g][ii];
            }
            ii++;
            if(bitmap_opt[g][ii] == TRUE){
                my_tensor_loc[g].g = gamma_;
            }
            else{       
                my_tensor_loc[g].g = 180/M_PI*x_init[g][ii];
            }
            ii++;
            if(bitmap_opt[g][ii+3] == TRUE){
                my_tensor_loc[g].ax = FIT_MULTI(g*5+ii);
            }
            else{ 
                my_tensor_loc[g].ax = x_init[g][ii+3];
            }
            ii++;
            if(bitmap_opt[g][ii+3] == TRUE){
                my_tensor_loc[g].rh = FIT_MULTI(g*5+ii);
            }
            else{    
                my_tensor_loc[g].rh = x_init[g][ii+3];
            }
        }
	return(0);
}


int get_tensor_single_from_covar(gboolean bitmap_opt[8], double x_init[8],
						  gsl_multifit_fdfsolver *s,
						  int index_v[8],
						  tensor * my_tensor_loc
						  )
{
		int j, ii;
		double alpha_, beta_, gamma_;
	
#define FIT(i) gsl_vector_get(s->x, index_v[i])
	
	   if(bitmap_opt[0] == TRUE){
            alpha_ = 180/M_PI*FIT(0);
        }
        else{
            alpha_ = 180/M_PI*x_init[0];
        }
        j = (int) (alpha_ / 360.0);
        if (alpha_ > 0)
            alpha_ -= j*360;
        else
            alpha_ -= (j-1)*360;
        
        //beta
        if(bitmap_opt[1] == TRUE){
            beta_ = 180/M_PI*FIT(1);
        }
        else{
            beta_ = 180/M_PI*x_init[1];
        }
        j = (int) (beta_ / 360.0);
        if (beta_ > 0)
            beta_ -= j*360;
        else
            beta_ -= (j-1)*360;
    
        //gamma
        if(bitmap_opt[2] == TRUE){
            gamma_ = 180/M_PI*FIT(2);
        }
        else{
            gamma_ = 180/M_PI*x_init[2];
        }
        j = (int) (gamma_ / 360.0);
        if (gamma_ > 0)
            gamma_ -= j*360;
        else
            gamma_ -= (j-1)*360;
        
        ii = 0;
        if(bitmap_opt[ii] == TRUE){
            my_tensor_loc->a = alpha_;
        }
        else{
            my_tensor_loc->a = 180/M_PI*x_init[ii];
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            my_tensor_loc->b = beta_;
        }
        else{
            my_tensor_loc->b = 180/M_PI*x_init[ii];
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            my_tensor_loc->g = gamma_;
        }
        else{     
            my_tensor_loc->g = 180/M_PI*x_init[ii];
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            my_tensor_loc->x = FIT(ii);
        }
        else{
            my_tensor_loc->x = x_init[ii];
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            my_tensor_loc->y = FIT(ii);
        }
        else{
            my_tensor_loc->y = x_init[ii];
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            my_tensor_loc->z = FIT(ii);
        }
        else{
            my_tensor_loc->z = x_init[ii];
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            my_tensor_loc->ax = FIT(ii);
        }
        else{
            my_tensor_loc->ax = x_init[ii];
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            my_tensor_loc->rh = FIT(ii);
        }
        else{  
            my_tensor_loc->rh = x_init[ii];
        }
	return(0);
}

//POSTPROCEED doesn't change any of the given argument.
int postproceed_single(gsl_multifit_fdfsolver *s, gsl_matrix *covar, 
					   int n_bis, int p,
					   gboolean bitmap_opt[8], int index_v[8],
					   double x_init[8], int n_button, gboolean use_racs
                       ,racs_data * racs_d)
{
        double chi = gsl_blas_dnrm2(s->f);
        double dof = n_bis - p;
        double c = GSL_MAX_DBL(1, chi / sqrt(dof)); 
        double alpha_, beta_, gamma_;
        int ii;
        double alpha_f, beta_f, gamma_f, Ax_f, Rh_f, xM_f, yM_f, zM_f;
        double alpha_e, beta_e, gamma_e, Ax_e, Rh_e, xM_e, yM_e, zM_e;
		int j;
		int serial;
#define FIT(i) gsl_vector_get(s->x, index_v[i])
#define ERR(i) sqrt(gsl_matrix_get(covar,index_v[i],index_v[i]))
	
        fprintf(stdout, "\n");
        fprintf(stdout, "+----------------------------------------------------------------+\n");
        fprintf(stdout, "|           Quality of fit and final tensor parameters           |\n");
        fprintf(stdout, "+----------------------------------------------------------------+\n\n");
        //fprintf(stderr, "c: %lf\n", c);
        //fprintf(stderr, "DO IT FOR MULTIPLE PCS TOO CHRISTOPHE!\n");
        printf("chisq/dof = %g\n",  pow(chi, 2.0) / dof);
        c = chi / sqrt(dof);
        //fprintf(stderr, "NEW c: %lf\n", c);
        //alpha
        if(bitmap_opt[0] == TRUE){
            alpha_ = 180/M_PI*FIT(0);
        }
        else{
            alpha_ = 180/M_PI*x_init[0];
        }
        j = (int) (alpha_ / 360.0);
        if (alpha_ > 0)
            alpha_ -= j*360;
        else
            alpha_ -= (j-1)*360;
        
        //beta
        if(bitmap_opt[1] == TRUE){
            beta_ = 180/M_PI*FIT(1);
        }
        else{
            beta_ = 180/M_PI*x_init[1];
        }
        j = (int) (beta_ / 360.0);
        if (beta_ > 0)
            beta_ -= j*360;
        else
            beta_ -= (j-1)*360;
    
        //gamma
        if(bitmap_opt[2] == TRUE){
            gamma_ = 180/M_PI*FIT(2);
        }
        else{
            gamma_ = 180/M_PI*x_init[2];
        }
        j = (int) (gamma_ / 360.0);
        if (gamma_ > 0)
            gamma_ -= j*360;
        else
            gamma_ -= (j-1)*360;

        //c = chi*chi/dof;
        
        
        ii = 0;
        if(bitmap_opt[ii] == TRUE){
            printf ("alpha = %12.5f\t+/- %.5f\n", alpha_, c*ERR(ii)*180/M_PI);
            alpha_f = alpha_;
            alpha_e = c*ERR(ii)*180/M_PI;
        }
        else{
            printf ("alpha = %.5f\n", 180/M_PI*x_init[ii]);
            alpha_f = 180/M_PI*x_init[ii];
            alpha_e = 0;
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            printf ("beta  = %12.5f\t+/- %.5f\n", beta_, c*ERR(ii)*180/M_PI);
            beta_f = beta_;
            beta_e = c*ERR(ii)*180/M_PI;
        }
        else{
            printf ("beta  = %.5f\n", 180/M_PI*x_init[ii]);
            beta_f = 180/M_PI*x_init[ii];
            beta_e = 0;
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            printf ("gamma = %12.5f\t+/- %.5f\n", gamma_, c*ERR(ii)*180/M_PI);
            gamma_f = gamma_;
            gamma_e = c*ERR(ii)*180/M_PI;
        }
        else{
            printf ("gamma = %.5f\n", 180/M_PI*x_init[ii]);        
            gamma_f = 180/M_PI*x_init[ii];
            gamma_e = 0;
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            printf ("x     = %12.5f\t+/- %.5f\n", FIT(ii), c*ERR(ii));
            xM_f = FIT(ii);
            xM_e = c*ERR(ii);
        }
        else{
            printf ("x     = %.5f\n", x_init[ii]);
            xM_f = x_init[ii];
            xM_e = 0;
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            printf ("y     = %12.5f\t+/- %.5f\n", FIT(ii), c*ERR(ii));
            yM_f = FIT(ii);
            yM_e = c*ERR(ii);
        }
        else{
            printf ("y     = %.5f\n", x_init[ii]);  
            yM_f = x_init[ii];
            yM_e = 0;
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            printf ("z     = %12.5f\t+/- %.5f\n", FIT(ii), c*ERR(ii));
            zM_f = FIT(ii);
            zM_e = c*ERR(ii);
        }
        else{
            printf ("z     = %.5f\n", x_init[ii]); 
            zM_f = x_init[ii];
            zM_e = 0;
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            printf ("Ax    = %12.5f\t+/- %.5f\n", FIT(ii), c*ERR(ii));
            Ax_f = FIT(ii);
            Ax_e = c*ERR(ii);
        }
        else{
            printf ("Ax    = %.5f\n", x_init[ii]);   
            Ax_f = x_init[ii];
            Ax_e = 0;
        }
        ii++;
        if(bitmap_opt[ii] == TRUE){
            printf ("Rh    = %12.5f\t+/- %.5f\n", FIT(ii), c*ERR(ii));
            Rh_f = FIT(ii);
            Rh_e = c*ERR(ii);
        }
        else{
            printf ("Rh    = %.5f\n", x_init[ii]);     
            Rh_f = x_init[ii];
            Rh_e = 0;
        }
        
        PrintTensorCoord(alpha_f, beta_f, gamma_f, Ax_f, Rh_f, xM_f, yM_f, zM_f);
        
        free_tensor(_all_tensor_pcs[n_button]);
        _all_tensor_pcs[n_button] = new_tensor();
        _all_tensor_pcs[n_button]->ax = Ax_f;
        _all_tensor_pcs[n_button]->rh = Rh_f;
        _all_tensor_pcs[n_button]->a = alpha_f/180*M_PI;
        _all_tensor_pcs[n_button]->b = beta_f/180*M_PI;
        _all_tensor_pcs[n_button]->g = gamma_f/180*M_PI;
        _all_tensor_pcs[n_button]->x = xM_f;
        _all_tensor_pcs[n_button]->y = yM_f;
        _all_tensor_pcs[n_button]->z = zM_f;
        _all_tensor_pcs[n_button]->err_ax = Ax_e;
        _all_tensor_pcs[n_button]->err_rh = Rh_e;
        _all_tensor_pcs[n_button]->err_a = alpha_e/180*M_PI;
        _all_tensor_pcs[n_button]->err_b = beta_e/180*M_PI;
        _all_tensor_pcs[n_button]->err_g = gamma_e/180*M_PI;
        _all_tensor_pcs[n_button]->err_x = xM_e;
        _all_tensor_pcs[n_button]->err_y = yM_e;
        _all_tensor_pcs[n_button]->err_z = zM_e;
        serial = add_to_tensor_listing(Ax_f, Rh_f, xM_f, yM_f, zM_f, alpha_f, beta_f, gamma_f, 
							  CALCULATED_TENSOR, n_button+1
							  ,Ax_e, Rh_e, xM_e, yM_e, zM_e
                           , alpha_e/180*M_PI, beta_e/180*M_PI, gamma_e/180*M_PI);
		display_tensor(n_button, SINGLE, serial, FALSE);
		append_tensor_valid(n_button, serial);
	
	calculate_pcs(_all_tensor_pcs[n_button], n_button, UPDATE_PCS_CALC, 0.0, 1, use_racs,
                  racs_d);
    display_pcs_list(n_button, serial);
	return(serial);
}

void free_label(char   **label, int n){
    int i;
    if (label == NULL){
        return;
    }
    for(i = 0; i < n; i++){
        if(label[i] != NULL){
            free(label[i]);
        }
    }
    free(label);
}

void free_index_on_pxx_multi(int ** index_on_pxx){
    int g;
    for (g = 0; g < N_PCS_ARRAY; g++){
        if(index_on_pxx[g] != NULL){
            free(index_on_pxx[g]);
        }
    }
}

void free_bitmap(gboolean ** bitmap){
    int g;
    for (g = 0; g < N_PCS_ARRAY; g++){
        if(bitmap[g] != NULL){
            free(bitmap[g]);
        }
    }
}

void free_double( double ** array){
    int g;
    for (g = 0; g < N_PCS_ARRAY; g++){
        if(array[g] != NULL){
            free(array[g]);
        }
    }
}

void start_calculate_multiple_tensor(GtkButton * button){
    GladeXML *gxml;
    gboolean g_button_tab[N_PCS_ARRAY];
    GtkToggleButton *toggle_button1;
    GtkToggleButton *toggle_button2;
    GtkToggleButton *toggle_button3;
    GtkToggleButton *toggle_button4;
    GtkRadioButton * radio1;
    GtkRadioButton * radio2;
    GtkRadioButton * radio3;
    GtkRadioButton * radio4;
    int copy_from;
    
    if(_pdb_str == NULL){
        box_error( "You should first load a pdb file.\n", NULL);
        return;
    }

    gxml = glade_get_widget_tree((GtkWidget * )button);
    
    toggle_button1 = (GtkToggleButton *) glade_xml_get_widget (gxml, "check_PCS1_MULTI");
    toggle_button2 = (GtkToggleButton *) glade_xml_get_widget (gxml, "check_PCS2_MULTI");
    toggle_button3 = (GtkToggleButton *) glade_xml_get_widget (gxml, "check_PCS3_MULTI");
    toggle_button4 = (GtkToggleButton *) glade_xml_get_widget (gxml, "check_PCS4_MULTI");
    
    radio1 = (GtkRadioButton *) glade_xml_get_widget (gxml, "radiobutton_PCS1_MULTI");
    radio2 = (GtkRadioButton *) glade_xml_get_widget (gxml, "radiobutton_PCS2_MULTI");
    radio3 = (GtkRadioButton *) glade_xml_get_widget (gxml, "radiobutton_PCS3_MULTI");
    radio4 = (GtkRadioButton *) glade_xml_get_widget (gxml, "radiobutton_PCS4_MULTI");
    
    
    g_button_tab[0] = gtk_toggle_button_get_active(toggle_button1);
    g_button_tab[1] = gtk_toggle_button_get_active(toggle_button2);
    g_button_tab[2] = gtk_toggle_button_get_active(toggle_button3);
    g_button_tab[3] = gtk_toggle_button_get_active(toggle_button4);
    
    copy_from = 0;
    if(gtk_toggle_button_get_active((GtkToggleButton *)radio1)){
        copy_from = 0;
    }
    if(gtk_toggle_button_get_active((GtkToggleButton *)radio2)){
        copy_from = 1;
    }
    if(gtk_toggle_button_get_active((GtkToggleButton *)radio3)){
        copy_from = 2;
    }
    if(gtk_toggle_button_get_active((GtkToggleButton *)radio4)){
        copy_from = 3;
    }
    
    _multiple_pcs_boundaries_pcs->b_x.type = _all_boundaries_pcs[copy_from]->b_x.type;
    _multiple_pcs_boundaries_pcs->b_y.type = _all_boundaries_pcs[copy_from]->b_y.type;
    _multiple_pcs_boundaries_pcs->b_z.type = _all_boundaries_pcs[copy_from]->b_z.type;
    
    _multiple_pcs_boundaries_pcs->b_x.start = _all_boundaries_pcs[copy_from]->b_x.start;
    _multiple_pcs_boundaries_pcs->b_y.start = _all_boundaries_pcs[copy_from]->b_y.start;
    _multiple_pcs_boundaries_pcs->b_z.start = _all_boundaries_pcs[copy_from]->b_z.start;
    
    _multiple_pcs_boundaries_pcs->b_x.min = _all_boundaries_pcs[copy_from]->b_x.min;
    _multiple_pcs_boundaries_pcs->b_y.min = _all_boundaries_pcs[copy_from]->b_y.min;
    _multiple_pcs_boundaries_pcs->b_z.min = _all_boundaries_pcs[copy_from]->b_z.min;
    
    _multiple_pcs_boundaries_pcs->b_x.max = _all_boundaries_pcs[copy_from]->b_x.max;
    _multiple_pcs_boundaries_pcs->b_y.max = _all_boundaries_pcs[copy_from]->b_y.max;
    _multiple_pcs_boundaries_pcs->b_z.max = _all_boundaries_pcs[copy_from]->b_z.max;
    
    //boundarie is already initiate in tensor.h
    //but I still have to copy the correct one!
    //AND ALSO TO CHECK EVERYTHING IS OKAY! 
    
    //fprintf(stderr, "boundari from %d\n", copy_from);
    calculate_multiple_tensor(g_button_tab, NULL, NULL);
}

void free_multiple_arrays(gboolean ** bitmap_opt, gboolean ** bitmap_bound,
                          int ** index_v,  double ** x_init)
{
    free_double(x_init);
    free_bitmap(bitmap_opt);
    free_bitmap(bitmap_bound);
    free_index_on_pxx_multi(index_v);
}

void free_x_y_z_PCS(double ** x, double ** y, double** z, double ** PCS, double ** TOL,
					double ** one, double ** two, double ** three, double ** offset_str,
					gboolean ** SUBSET_ERROR)
{
    free_double(x);
    free_double(y);
    free_double(z);
    free_double(PCS);
	free_double(TOL);
	free_double(one);
	free_double(two);
	free_double(three);
	free_double(offset_str);
	free_bitmap(SUBSET_ERROR);
	
}

/*----------------------------------------------------------------------*/
int calculate_multiple_tensor(gboolean * g_button_tab,
							  error_control_data * err_ctrl,
							  tensor my_tensor_multi[N_PCS_ARRAY]) // ONLY FOR MULTIPLE TENSOR
/*----------------------------------------------------------------------*/
{
    const gsl_multifit_fdfsolver_type *T;
    gsl_multifit_fdfsolver *s;
    int status;
    int i, iter = 0;
    int n_bis[N_PCS_ARRAY],/* j,*/ p[N_PCS_ARRAY];    /* p = number of parameters */
    int p_center;
    int n_bis_p_multi;
    int my_k;
    int n_bound[N_PCS_ARRAY];
    int n_bound_center;
    //char   **label[N_PCS_ARRAY];
    double * PCS[N_PCS_ARRAY], * x[N_PCS_ARRAY], * y[N_PCS_ARRAY],* z[N_PCS_ARRAY];
	gboolean * SUBSET_ERROR[N_PCS_ARRAY];
	double * TOL[N_PCS_ARRAY];
	double * PCS_OFFSET[N_PCS_ARRAY];
	double * PDB_OFFSET_X[N_PCS_ARRAY];
	double * PDB_OFFSET_Y[N_PCS_ARRAY];
	double * PDB_OFFSET_Z[N_PCS_ARRAY];
    gboolean * bitmap_opt[N_PCS_ARRAY];//[8];
    gboolean bitmap_opt_center[3];
    gboolean * bitmap_bound[N_PCS_ARRAY];//[8];
    gboolean bitmap_bound_center[3];
    int index_bound_multi[N_PCS_ARRAY * 5 + 3];
    int * index_v[N_PCS_ARRAY];//[8];
    int index_v_center[3];
    double * x_init[N_PCS_ARRAY];//[8];
    double x_init_center[3];
    double x_init_smaller_multi[N_PCS_ARRAY * 5 + 3];
    int index_v_multi[N_PCS_ARRAY * 5 + 3];
    int * index_on_pdb[N_PCS_ARRAY];
    int * index_on_pcs[N_PCS_ARRAY];
    int g;
    int n_exp_used;
    int p_total;
    int n_bis_total;
	int serial_a[N_PCS_ARRAY];
	int n_sample;
	gsl_rng * my_rand;
    gboolean use_racs[N_PCS_ARRAY];
    racs_data racs_d;
    
    get_racs_parameters(&racs_d);
    for(g = 0; g < N_PCS_ARRAY; g++){
        use_racs[g] = is_use_racs_PCS(g);
    }
    //use_racs = FALSE;
	
	my_rand = NULL;
	
    //bar = get_bar_window_running();
       
    for(g = 0; g < N_PCS_ARRAY; g++){
        bitmap_opt[g] = (gboolean *) calloc(8, sizeof(gboolean));
        if(bitmap_opt[g] == NULL){
            fprintf(stderr, "couldn't allocate memory to create bitmap_opt\n");
            exit(-1);
        }
        bitmap_bound[g] = (gboolean *) calloc(8, sizeof(gboolean));
        if(bitmap_bound[g] == NULL){
            fprintf(stderr, "couldn't allocate memory to create bitmap_bound\n");
            exit(-1);
        }
        index_v[g] = (int *) calloc(8, sizeof(int));
        if(index_v[g] == NULL){
            fprintf(stderr, "couldn't allocate memory to create index_v\n");
            exit(-1);
        }
        x_init[g] = (double *) calloc(8, sizeof(double));
        if(index_v[g] == NULL){
            fprintf(stderr, "couldn't allocate memory to create x_init\n");
            exit(-1);
        }
    }
    
    //WE CHECK THAT AT LEAST ONE PCS experiment is used
    n_exp_used = 0;
    for(g = 0; g < N_PCS_ARRAY; g++){
        if(g_button_tab[g] == TRUE){
            n_exp_used ++;
        }
    }
    if(n_exp_used == 0){
        box_error("You should use choose at least one pcs experiment\n", NULL);
        free_multiple_arrays(bitmap_opt, bitmap_bound, index_v, x_init); 
        return(-1);
    }
    
    //WE CHECK THAT PCS data are loaded for each experiment used
    for(g = 0; g < N_PCS_ARRAY; g++){
        if(g_button_tab[g]==TRUE){
            if(check_pcs_loaded(g) == -1){
                free_multiple_arrays(bitmap_opt, bitmap_bound, index_v, x_init); 
                return(-1);
            }
        }
    }
    
    { //WE CHECK THAT AT LEAST ONE PDB MODEL IS LOADED
        int compt;
        compt = 0;
        for(i = 0; i< _multiple_pcs_pdb_selector_pcs->len; i++){
            if(g_array_index(_multiple_pcs_pdb_selector_pcs, pdb_selector, i).is_used 
               == TRUE){
                   compt ++;
                   break;
               }
        }
        if( compt == 0){
            box_error("You should use at least one pdb model\n", NULL);
            free_multiple_arrays(bitmap_opt, bitmap_bound, index_v, x_init); 
            return (-1);
        }
    }
    
    //WE INITIALISE SOME ARRAYS
    for(g = 0; g < N_PCS_ARRAY; g++){
        for(i = 0; i < 8; i++){
            index_v[g][i] = -1;
            bitmap_opt[g][i] = FALSE;
            bitmap_bound[g][i] = FALSE;
        }
        p[g] = 0;
        n_bound[g] = 0;
    }
    
    for(i = 0; i < 3; i++){
        index_v_center[i] = -1;
        bitmap_opt_center[i] = FALSE;
        bitmap_bound_center[i] = FALSE;
    }
    
    //WE POPULATE SOME ARRAYS
    i = 0;
    p_center = 0;
    if(_multiple_pcs_boundaries_pcs->b_x.type != TYPE_FIXED) { index_v_center[i] = p_center; p_center++; bitmap_opt_center[0] = TRUE;}
    i++;
    if(_multiple_pcs_boundaries_pcs->b_y.type != TYPE_FIXED) { index_v_center[i] = p_center; p_center++; bitmap_opt_center[1] = TRUE;}
    i++;
    if(_multiple_pcs_boundaries_pcs->b_z.type != TYPE_FIXED) { index_v_center[i] = p_center; p_center++; bitmap_opt_center[2] = TRUE;}
    
    n_bound_center = 0;
    if(_multiple_pcs_boundaries_pcs->b_x.type == TYPE_BOUNDARIES) { bitmap_bound_center[0] = TRUE; n_bound_center++;}
    if(_multiple_pcs_boundaries_pcs->b_y.type == TYPE_BOUNDARIES) { bitmap_bound_center[1] = TRUE; n_bound_center++;}
    if(_multiple_pcs_boundaries_pcs->b_z.type == TYPE_BOUNDARIES) { bitmap_bound_center[2] = TRUE; n_bound_center++;}
    
	if(err_ctrl == NULL){ // ERROR ON
    	x_init_center[0] = _multiple_pcs_boundaries_pcs->b_x.start;
    	x_init_center[1] = _multiple_pcs_boundaries_pcs->b_y.start;
    	x_init_center[2] = _multiple_pcs_boundaries_pcs->b_z.start;
	}
	else {
		int my_n_butt;
		int i;
		my_n_butt = -1;
		for(i = 0; i < N_PCS_ARRAY; i++){
			if((err_ctrl->use_data_PCS)[i] == TRUE){
				my_n_butt = i;
			}
		}
		if(my_n_butt == -1){
			fprintf(stderr, "Problem with err_analysis\n");
			return(-1);
		}
   	 	x_init_center[0] = my_tensor_multi[my_n_butt].x;
    	x_init_center[1] = my_tensor_multi[my_n_butt].y;
    	x_init_center[2] = my_tensor_multi[my_n_butt].z;
	}
    
    for(g = 0; g < N_PCS_ARRAY; g++){
        if(g_button_tab[g]){
            p[g] = 0;
            i = 0;
            if(_all_boundaries_pcs[g]->b_a.type != TYPE_FIXED) { index_v[g][i] = p[g]; p[g]++; bitmap_opt[g][0] = TRUE;}
            i++;
            if(_all_boundaries_pcs[g]->b_b.type != TYPE_FIXED) { index_v[g][i] = p[g]; p[g]++; bitmap_opt[g][1] = TRUE;}
            i++;
            if(_all_boundaries_pcs[g]->b_g.type != TYPE_FIXED) { index_v[g][i] = p[g]; p[g]++; bitmap_opt[g][2] = TRUE;}
            i++;
            i = i + 3;
            if(_all_boundaries_pcs[g]->b_Ax.type != TYPE_FIXED){ index_v[g][i] = p[g]; p[g]++; bitmap_opt[g][6] = TRUE;}
            i++;
            if(_all_boundaries_pcs[g]->b_Rh.type != TYPE_FIXED){ index_v[g][i] = p[g]; p[g]++; bitmap_opt[g][7] = TRUE;}
            
            n_bound[g] = 0;
            if(_all_boundaries_pcs[g]->b_a.type == TYPE_BOUNDARIES) { bitmap_bound[g][0] = TRUE; n_bound[g]++;}
            if(_all_boundaries_pcs[g]->b_b.type == TYPE_BOUNDARIES) { bitmap_bound[g][1] = TRUE; n_bound[g]++;}
            if(_all_boundaries_pcs[g]->b_g.type == TYPE_BOUNDARIES) { bitmap_bound[g][2] = TRUE; n_bound[g]++;}
            if(_all_boundaries_pcs[g]->b_Ax.type == TYPE_BOUNDARIES){ bitmap_bound[g][6] = TRUE; n_bound[g]++;}
            if(_all_boundaries_pcs[g]->b_Rh.type == TYPE_BOUNDARIES){ bitmap_bound[g][7] = TRUE; n_bound[g]++;}
 
			if(err_ctrl == NULL){ // ERROR OFF
            	x_init[g][0] = _all_boundaries_pcs[g]->b_a.start;
            	x_init[g][1] = _all_boundaries_pcs[g]->b_b.start;
            	x_init[g][2] = _all_boundaries_pcs[g]->b_g.start;
            	x_init[g][6] = _all_boundaries_pcs[g]->b_Ax.start;
            	x_init[g][7] = _all_boundaries_pcs[g]->b_Rh.start; 
            
            	x_init[g][0] *= M_PI/180.0;
            	x_init[g][1] *= M_PI/180.0;
            	x_init[g][2] *= M_PI/180.0;
            
            	fprintf(stdout, "\n");
            	fprintf(stdout, "+----------------------------------------------------------------+\n");
            	fprintf(stdout, "|     Init value for tensor search (experiment n %d)             |\n", g+1);
            	fprintf(stdout, "+----------------------------------------------------------------+\n\n");
            	fprintf(stdout, "Ax start\t %lf\t\toptimized? %d\n", x_init[g][6], bitmap_opt[g][6]);
            	fprintf(stdout, "Rh start\t %lf\t\toptimized? %d\n", x_init[g][7], bitmap_opt[g][7]);
            	fprintf(stdout, "a  start\t %lf\t\toptimized? %d\n", x_init[g][0]/M_PI*180.0, bitmap_opt[g][0]);
            	fprintf(stdout, "b  start\t %lf\t\toptimized? %d\n", x_init[g][1]/M_PI*180.0, bitmap_opt[g][1]);
            	fprintf(stdout, "g  start\t %lf\t\toptimized? %d\n", x_init[g][2]/M_PI*180.0, bitmap_opt[g][2]); 

			}
			else{// ERROR ON				
				x_init[g][0] = my_tensor_multi[g].a;
				x_init[g][1] = my_tensor_multi[g].b;
    			x_init[g][2] = my_tensor_multi[g].g;
    			x_init[g][6] = my_tensor_multi[g].ax;
    			x_init[g][7] = my_tensor_multi[g].rh;
			}
		}
    }
	if(err_ctrl == NULL){ // ERROR OFF
    	for(g = 0; g < N_PCS_ARRAY; g++){
        	if(g_button_tab[g]){
          	 	fprintf(stdout, "\n");
            	fprintf(stdout, "+----------------------------------------------------------------+\n");
            	fprintf(stdout, "|     Init value for tensor search (common parameters x, y, z)   |\n");
            	fprintf(stdout, "+----------------------------------------------------------------+\n\n");
            	fprintf(stdout, "x  start\t %lf\t\toptimized? %d\n", x_init_center[0], bitmap_opt_center[0]);
            	fprintf(stdout, "y  start\t %lf\t\toptimized? %d\n", x_init_center[1], bitmap_opt_center[1]);
            	fprintf(stdout, "z  start\t %lf\t\toptimized? %d\n", x_init_center[2], bitmap_opt_center[2]);
            	break;
        	}
    	}
	}
    
    //WE COUNT HOW MANY PCS VALUES WE HAVE
    for(g = 0; g < N_PCS_ARRAY; g++){
        n_bis[g] = 0;
        if(g_button_tab[g]){
            for(i = 0; i < _all_pcs[g]->len; i++){
                if(g_array_index (_all_pcs[g], Item_pcs, i).used == TRUE){
                    n_bis[g]++;
                }
            }
        }
    }
    
    //WE CHECK WE HAVE ENOUGH PCS VALUES. AND THAT N IS BIG ENOUGH

    n_bis_total = 0;
    p_total = 0;
    for(g = 0; g < N_PCS_ARRAY; g++){
        if(g_button_tab[g]){
            /* NO NEED CAUSE NOW IT IS DYNAMIC ALLOCATION
            if (n_bis[g] > N) {
                fprintf(stderr, "N too small\n");
                fprintf(stderr, "Numbat can't calculate tensor with a number of PCS bigger than %d\n", N);
                fprintf(stderr, "You need to change the variable N in src/pcsfit_metal.c and recompile\n");
                free_multiple_arrays(bitmap_opt, bitmap_bound, index_v, x_init); 
                return(-1);
            }
            */
            if (p[g] > n_bis[g]) {
                fprintf(stderr, "For experiment %d, You need more than %d PCS values to optimize %d parameters\n\
                                (you have only %d PCS values)\n",g+1, p[g], p[g], n_bis[g]);
                free_multiple_arrays(bitmap_opt, bitmap_bound, index_v, x_init); 
                return(-1);
            }
            if( 0 == n_bis[g]){
                fprintf(stderr, "No PCS data used in experiment %d\n", g+1);
                free_multiple_arrays(bitmap_opt, bitmap_bound, index_v, x_init); 
                return(-1);
            }
            n_bis_total += n_bis[g];
            p_total += p[g];
        }
    }
    
    p_total += p_center;
    if(p_total > n_bis_total){
        fprintf(stderr, "Globaly, you need more than %d PCS values to optimize %d parameters\n\
                        (you have only %d PCS values)\n", p_total, p_total, n_bis_total);
        free_multiple_arrays(bitmap_opt, bitmap_bound, index_v, x_init); 
        return(-1);
    }

    if(p_center == 0){ //Nothing to optimize...
        fprintf(stderr, "No tensor coordinate parameter to optimize\n");
        box_error( "No tensor coordinate parameter to optimize", NULL);
        free_multiple_arrays(bitmap_opt, bitmap_bound, index_v, x_init); 
        return(-1);
    }

    //WE ALLOCATE SOME ARRAYS
    for(g = 0; g < N_PCS_ARRAY; g++){
        index_on_pdb[g] = NULL;
        if(g_button_tab[g]){
            index_on_pdb[g] = (int *) calloc(n_bis[g], sizeof(int));
            if(index_on_pdb[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create index_on_pdb %d\n", n_bis[g]);
                exit(-1);
            }
        }
        index_on_pcs[g] = NULL;
        if(g_button_tab[g]){
            index_on_pcs[g] = (int *) calloc(n_bis[g], sizeof(int));
            if(index_on_pcs[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create index_on_pcs\n");
                exit(-1);
            }
        }
    }
    
    //////////////////////////
    n_bis_p_multi = 0; // n_bis + n_bound;
    for(g = 0; g < N_PCS_ARRAY; g++){
        if(g_button_tab[g]){
            n_bis_p_multi += n_bis[g];
            n_bis_p_multi += n_bound[g];
        }
    }
    n_bis_p_multi += n_bound_center;
    ///////////////////////////
    
    struct PCSdata4 d = { 
        n_bis, 
        PCS,
		TOL,
        x, 
        y, 
        z,
        index_on_pdb, 
        index_on_pcs,
        g_button_tab,  
        x_init,  
        index_v, 
        bitmap_opt,
        bitmap_bound,
        n_bound,
        p,  
        x_init_center,  
        index_v_center, 
        bitmap_opt_center,
        bitmap_bound_center, 
        n_bound_center,
        p_center,
        index_v_multi,
        index_bound_multi,
		err_ctrl,
		PCS_OFFSET,
		PDB_OFFSET_X,
		PDB_OFFSET_Y,
		PDB_OFFSET_Z,
		SUBSET_ERROR,
        use_racs,
        & racs_d
    }; 

    gsl_multifit_function_fdf f;
    
    {//POPULATE X_INIT_SMALLER and INDEX_V_MULTI
        for(i = 0; i < (N_PCS_ARRAY * 5 + 3); i++){
            index_v_multi[i] = -1;
        }
        int k;
        int kk;
        int ii;
        kk = 0;
        ii = 0;
        for(g = 0; g < N_PCS_ARRAY; g++){
            if(g_button_tab[g]){
                for(k = 0; k < 8; k++){
                    if((k == 3)||(k == 4)||(k == 5)){
                        continue;
                    }
                    if( bitmap_opt[g][k] == TRUE){
                        x_init_smaller_multi[kk] = x_init[g][k];
                        index_v_multi[ii] = kk;
                        kk++;
                    }
                    ii++;
                }
            }
            else{
                ii+=5;
            }
        }
        if(bitmap_opt_center[0]){ x_init_smaller_multi[kk] = x_init_center[0]; index_v_multi[ii] = kk; kk++;}
        ii++;
        if(bitmap_opt_center[1]){ x_init_smaller_multi[kk] = x_init_center[1]; index_v_multi[ii] = kk; kk++; }
        ii++;
        if(bitmap_opt_center[2]){ x_init_smaller_multi[kk] = x_init_center[2]; index_v_multi[ii] = kk; kk++; }
        if(kk != p_total){
            fprintf(stderr, "WARNING problem with number of parameters to optimize\n");
            exit(-1);
        }
    }
    
    {//POPULATE INDEX_BOUND_MULTI
        for(i = 0; i < N_PCS_ARRAY * 5 + 3; i++){
            index_bound_multi[i] = -1;
        }
        int k;
        int kk;
        int ii;
        kk = 0;
        ii = 0;
        for(g = 0; g < N_PCS_ARRAY; g++){
            if(g_button_tab[g]){
                for(k = 0; k < 8; k++){
                    if((k == 3)||(k == 4)||(k == 5)){
                        continue;
                    }
                    if( bitmap_bound[g][k] == TRUE){
                        index_bound_multi[ii] = kk;
                        kk++;
                    }
                    ii++;
                }
            }
            else{
                ii+=5;
            }
        }
        if(bitmap_bound_center[0]){ index_bound_multi[ii] = kk; kk++;}
        ii++;
        if(bitmap_bound_center[1]){ index_bound_multi[ii] = kk; kk++; }
        ii++;
        if(bitmap_bound_center[2]){ index_bound_multi[ii] = kk; kk++; }
    }
    
    gsl_vector_view v = gsl_vector_view_array (x_init_smaller_multi, p_total);

    const gsl_rng_type * type;
   	gsl_rng * r;

    gsl_rng_env_setup();

    type = gsl_rng_default;
    r = gsl_rng_alloc (type);

    f.f = &expb_f_unified_multi;    
    f.df = &expb_df_unified_multi;  
    f.fdf = &expb_fdf_unified_multi;
    f.n = n_bis_p_multi;
    f.p = p_total;
    f.params = &d;

    for(g = 0; g < N_PCS_ARRAY; g++){
        //label[g] = NULL;
    }
    // ALLOCATOIN OF PCS,TOL, x, y, z
    for(g = 0; g < N_PCS_ARRAY; g++){
        PCS[g] = NULL;
		PCS_OFFSET[g] = NULL;
		PDB_OFFSET_X[g] = NULL;
		PDB_OFFSET_Y[g] = NULL;
		PDB_OFFSET_Z[g] = NULL;
		SUBSET_ERROR[g] = NULL;
		TOL[g] = NULL;
        x[g] = NULL;
        y[g] = NULL;
        z[g] = NULL;
        if(g_button_tab[g]){
            PCS[g] = (double *) calloc(n_bis[g], sizeof(double));
			
            if(PCS[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create PCS %d\n", n_bis[g]);
                exit(-1);
            }
			PCS_OFFSET[g] = (double *) calloc(n_bis[g], sizeof(double));
			if(PCS_OFFSET[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create PCS_OFFSET %d\n", n_bis[g]);
                exit(-1);
            }
			PDB_OFFSET_X[g] = (double *) calloc(n_bis[g], sizeof(double));
			if(PDB_OFFSET_X[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create PDB_OFFSET_X %d\n", n_bis[g]);
                exit(-1);
            }
			PDB_OFFSET_Y[g] = (double *) calloc(n_bis[g], sizeof(double));
			if(PDB_OFFSET_Y[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create PDB_OFFSET_Y %d\n", n_bis[g]);
                exit(-1);
            }
			PDB_OFFSET_Z[g] = (double *) calloc(n_bis[g], sizeof(double));
			if(PDB_OFFSET_Z[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create PDB_OFFSET_Z %d\n", n_bis[g]);
                exit(-1);
            }
			
			SUBSET_ERROR[g] = (gboolean *) calloc(n_bis[g], sizeof(gboolean));
			if(SUBSET_ERROR[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create SUBSET_ERROR %d\n", n_bis[g]);
                exit(-1);
            }
			
			TOL[g] = (double *) calloc(n_bis[g], sizeof(double));
            if(TOL[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create TOL %d\n", n_bis[g]);
                exit(-1);
            }
            x[g] = (double *) calloc(n_bis[g], sizeof(double));
            if(x[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create x %d\n", n_bis[g]);
                exit(-1);
            }
            y[g] = (double *) calloc(n_bis[g], sizeof(double));
            if(y[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create y %d\n", n_bis[g]);
                exit(-1);
            }
            z[g] = (double *) calloc(n_bis[g], sizeof(double));
            if(z[g] == NULL){
                fprintf(stderr, "couldn't allocate memory to create z %d\n", n_bis[g]);
                exit(-1);
            }
        }
    }
    
    //I STILL NEED TO CHECK VALIDITY AFTER THAT PART
    
    /* This is the data to be fitted */
    for(g = 0; g < N_PCS_ARRAY; g++){
        if(g_button_tab[g]){
			/*
            label[g] = (char **) malloc(n_bis[g]*sizeof(char *));
            if (label[g] == NULL) {
                fprintf(stderr, "Error allocating label space\n");
                exit(1);
            }
            */
            { 
                int t;
                for(t = 0; t < n_bis[g]; t++){
                    //label[g][t] = NULL;
                }
            }
            
            my_k = 0;
            for (i = 0; i < n_bis[g]; i++) {
                int index_pdb;
                while(g_array_index(_all_pcs[g], Item_pcs, my_k).used != TRUE){
                    my_k++;
                    if(my_k >= _all_pcs[g]->len){
                        fprintf(stderr, "BIG_ERROR\n");
                        free_index_on_pxx_multi(index_on_pdb);
                        free_index_on_pxx_multi(index_on_pcs);
                        free_multiple_arrays(bitmap_opt, bitmap_bound, index_v, x_init); 
                        free_x_y_z_PCS(x, y, z, PCS, TOL, PDB_OFFSET_X, PDB_OFFSET_Y,
									   PDB_OFFSET_Z, PCS_OFFSET, SUBSET_ERROR);
                        return(-1);
                    }
                }
                PCS_OFFSET[g][i] = 0.0;
				PDB_OFFSET_X[g][i] = 0.0;
				PDB_OFFSET_Y[g][i] = 0.0;
				PDB_OFFSET_Z[g][i] = 0.0;
				SUBSET_ERROR[g][i] = TRUE;
                PCS[g][i] = g_array_index (_all_pcs[g], Item_pcs, my_k).pcs_exp;
				TOL[g][i] = g_array_index (_all_pcs[g], Item_pcs, my_k).pcs_tol;
                index_on_pcs[g][i] = my_k;
                if(g_array_index (_all_pcs[g], Item_pcs, my_k).is_dumby == FALSE){
                    index_pdb = find_pcs_in_pdb(g_array_index (_all_pcs[g], Item_pcs, my_k).residue_num
                                ,g_array_index (_all_pcs[g], Item_pcs, my_k).atom_name );
                }
                else { //it is a dummy atom 
                    index_pdb = -2;
                }
                my_k++;  
                if(index_pdb == -1){
                    fprintf(stderr, "ANOTHER BIG_ERROR\n");
                    free_index_on_pxx_multi(index_on_pdb);
                    free_index_on_pxx_multi(index_on_pcs);
                    free_multiple_arrays(bitmap_opt, bitmap_bound, index_v, x_init); 
                    free_x_y_z_PCS(x, y, z, PCS, TOL, PDB_OFFSET_X, PDB_OFFSET_Y,
									   PDB_OFFSET_Z, PCS_OFFSET, SUBSET_ERROR);
                    return(-1);
                }
                index_on_pdb[g][i] = index_pdb;
        
                char stemp[11];
                if(index_pdb == -2){
                    sprintf(stemp, "%s_%d", g_array_index(_all_pcs[g], Item_pcs, index_on_pcs[g][i]).atom_name,
                        g_array_index(_all_pcs[g], Item_pcs, index_on_pcs[g][i]).residue_num);
                }
                else {
                    sprintf(stemp, "%s_%d", _pdb_str->tab_atom[index_pdb].atom_name,
                    _pdb_str->tab_atom[index_pdb].residue_num);
                }
                /*label[g][i] = (char *) malloc(10*sizeof(char));
                if (label[g][i] == NULL) {
                    fprintf(stderr, "Error allocating label space\n");
                    exit(1);
                }
				*/
                //strcpy(label[g][i], stemp);
            } // loop on i
        }//if on g_button_tab[g]
    }//loop on g    
    
    	
	display_matrix_multi_init();
	
	fprintf(stdout, "\n");
    fprintf(stdout, "+----------------------------------------------------------------+\n");
    fprintf(stdout, "|                       Optimisation output                      |\n");
    fprintf(stdout, "+----------------------------------------------------------------+\n\n");
	

	GtkTextBuffer *buffer;
	buffer = NULL;
	progressbar_init(&buffer);
	
	n_sample = 1;
	if(err_ctrl != NULL){
		n_sample = err_ctrl->n_sample;
	}
	
	switch(n_sample){
		case 1:{ //ERROR OFF
			
			T = gsl_multifit_fdfsolver_lmsder;
    		s = gsl_multifit_fdfsolver_alloc (T, n_bis_p_multi, p_total);
    		gsl_multifit_fdfsolver_set (s, &f, &v.vector);
			print_state (iter, s, -999, buffer);
    		printf ("\n");
    		do {
        		iter++;
                status = gsl_multifit_fdfsolver_iterate (s);
                
				if(iter % 10 == 0){
                    print_state (iter, s, status, buffer);
					gboolean still_running = progressbar_update(1.0);
					if(still_running == FALSE){
						break;
					}
				}
                status = gsl_multifit_test_delta (s->dx, s->x, 1e-6, 1e-6);
        		
    		}
    		while (status == GSL_CONTINUE && iter < MAX_ITER_MULTI);
            print_state (iter, s, status, buffer);
            
    		gsl_matrix *covar = gsl_matrix_alloc (p_total, p_total);
    		gsl_multifit_covar (s->J, 0.0, covar);

			postproceed_multiple(s, covar,
						 n_bis, n_bis_total,
						 p, p_total,
						 bitmap_opt, bitmap_opt_center,
						 index_v, index_v_multi,
						 x_init, x_init_center,
						 g_button_tab, serial_a,
                         use_racs, &racs_d
						);

    		fprintf(stdout, "\n");
    		printf ("status = %s\n", gsl_strerror (status));
			gsl_multifit_fdfsolver_free (s);
    		gsl_matrix_free (covar);
		}
		break;
		default: {//ERROR_ON
			////
			int quick_i;
			gboolean still_running = TRUE;
			tensor_analysis * t_a;
			int iter_total = 0;
			int gg, j;
			tensor my_tensor_loc[N_PCS_ARRAY];
			GtkTextIter iterbuf;
  			char ligne[100];
			my_rand = gsl_rng_alloc (gsl_rng_ranlxs0);
    		gsl_rng_set (my_rand, err_ctrl->seed);
			
			//noise_to_add = gsl_ran_gaussian(my_rand, noise);
			
			t_a = new_tensor_analysis(n_exp_used, n_sample);
			
			for(i = 0; i < n_sample; i++){
				
				for(gg = 0; gg < N_PCS_ARRAY; gg++){
					if(g_button_tab[gg] == FALSE){
                		continue;
            		}
					for (j = 0; j < n_bis[gg]; j++) {
						PCS_OFFSET[gg][j] = gsl_ran_gaussian(my_rand, err_ctrl->noise_data_PCS[gg]);
						PDB_OFFSET_X[gg][j] = gsl_ran_gaussian(my_rand, err_ctrl->noise_structure);
						PDB_OFFSET_Y[gg][j] = gsl_ran_gaussian(my_rand, err_ctrl->noise_structure);
						PDB_OFFSET_Z[gg][j] = gsl_ran_gaussian(my_rand, err_ctrl->noise_structure);
					}
				}
				fill_bitmap_multiple_subset(SUBSET_ERROR, n_bis, my_rand
											, err_ctrl->subset_percentage, g_button_tab);
				
				int iter = 0;
				T = gsl_multifit_fdfsolver_lmsder;
    			s = gsl_multifit_fdfsolver_alloc (T, n_bis_p_multi, p_total);
				gsl_multifit_fdfsolver_set (s, &f, &v.vector);
				do {
       	 			iter++;
					iter_total ++;
                    status = gsl_multifit_fdfsolver_iterate (s);
					if(iter_total % 10 == 0){
						still_running = progressbar_update(1.0);
						if(still_running == FALSE){
							break;
						}
					}
                    status = gsl_multifit_test_delta (s->dx, s->x, 1e-6, 1e-6);
    			}
    			while (status == GSL_CONTINUE && iter < MAX_ITER_MULTI);
				if(still_running == FALSE){
					break;
				}
				
				gsl_matrix *covar = gsl_matrix_alloc (p_total, p_total);
    			gsl_multifit_covar (s->J, 0.0, covar);
				
				get_tensor_multiple_from_covar(g_button_tab, s,
								   bitmap_opt, bitmap_opt_center,
								   x_init_center, x_init,
								   index_v_multi, 
								   my_tensor_loc
								   );
				sprintf(ligne, "%3d", i+1);
				fprintf(stderr, ligne);
				gtk_text_buffer_get_iter_at_offset(buffer,&iterbuf,0);
				gtk_text_buffer_insert(buffer, &iterbuf, ligne, -1);
				quick_i = 1;
				for(gg = 0; gg < N_PCS_ARRAY; gg++){
					if(g_button_tab[gg] == FALSE){
                		continue;
            		}
					store_tensor_analysis(t_a, quick_i, i, &my_tensor_loc[gg],
                                          gsl_blas_dnrm2 (s->f));
					quick_i++;
					sprintf (ligne, "\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf", 
						 my_tensor_loc[gg].a, my_tensor_loc[gg].b, my_tensor_loc[gg].g,
						 my_tensor_loc[gg].x, my_tensor_loc[gg].y, my_tensor_loc[gg].z,
						 my_tensor_loc[gg].ax, my_tensor_loc[gg].rh,
						 gsl_blas_dnrm2 (s->f)
						 );
					fprintf(stderr, ligne);
					//gtk_text_buffer_get_iter_at_offset(buffer,&iterbuf,0);
					gtk_text_buffer_insert(buffer, &iterbuf, ligne, -1);
				}
				sprintf(ligne, "\n");
				fprintf(stderr, ligne);
				//gtk_text_buffer_get_iter_at_offset(buffer,&iterbuf,0);
				gtk_text_buffer_insert(buffer, &iterbuf, ligne, -1);
				
				gsl_multifit_fdfsolver_free (s);
    			gsl_matrix_free (covar);
				
			}// i (n_sample)
			gsl_rng_free (my_rand);
			statistic_tensor_analysis(t_a);
            prepare_sanson(t_a);
			free_tensor_analysis(t_a);
		}
		break;
	} // switch
	
	progressbar_finish();
    if(n_sample !=1){ //ERROR ON
        start_sanson();
    }
  //FREE
    gsl_rng_free (r);
    free_index_on_pxx_multi(index_on_pdb);
    free_index_on_pxx_multi(index_on_pcs);
    free_multiple_arrays(bitmap_opt, bitmap_bound, index_v, x_init);
    free_x_y_z_PCS(x, y, z, PCS, TOL, PDB_OFFSET_X, PDB_OFFSET_Y,
									   PDB_OFFSET_Z, PCS_OFFSET, SUBSET_ERROR);
    for(g = 0; g < N_PCS_ARRAY; g++){
        //free_label(label[g], n_bis[g]);
    }
	
    return 0;
}

//This function MODIFY none of the argument
int postproceed_multiple(gsl_multifit_fdfsolver *s, gsl_matrix *covar,
						 int n_bis[N_PCS_ARRAY], int n_bis_total,
						 int p[N_PCS_ARRAY], int p_total,
						 gboolean * bitmap_opt[N_PCS_ARRAY], gboolean bitmap_opt_center[3],
						 int * index_v[N_PCS_ARRAY], int index_v_multi[N_PCS_ARRAY * 5 + 3],
						 double * x_init[N_PCS_ARRAY], double x_init_center[3],
						 gboolean * g_button_tab, int serial_a[N_PCS_ARRAY],
						 gboolean use_racs[N_PCS_ARRAY]
                         , racs_data * racs_d)
{
	
	int g, j;
		
#define FIT_MULTI(i) gsl_vector_get(s->x, index_v_multi[i])
#define ERR_MULTI(i) sqrt(gsl_matrix_get(covar,index_v_multi[i],index_v_multi[i]))

    {
        double chi = gsl_blas_dnrm2(s->f);
        double dof = n_bis_total - p_total;
        double c = GSL_MAX_DBL(1, chi / sqrt(dof)); 
        double xM_f, yM_f, zM_f;
        double xM_e, yM_e, zM_e;
		int first_of_g;
        fprintf(stdout, "\n");
        fprintf(stdout, "+----------------------------------------------------------------+\n");
        fprintf(stdout, "|           Quality of fit and final tensor parameters           |\n");
        fprintf(stdout, "+----------------------------------------------------------------+\n\n");
        printf("chisq/dof = %g\n",  pow(chi, 2.0) / dof);
        //c = chi*chi/dof;
        c = chi / sqrt(dof);
		first_of_g = 0;
		for(g = N_PCS_ARRAY -1 ; g>=0; g--){
			if(g_button_tab[g] != FALSE){
                first_of_g = g;
            }
		}
        for(g = 0; g < N_PCS_ARRAY; g++){
            double alpha_, beta_, gamma_;
            int ii;
            double alpha_f, beta_f, gamma_f, Ax_f, Rh_f;
            double alpha_e, beta_e, gamma_e, Ax_e, Rh_e;
            if(g_button_tab[g] == FALSE){
                continue;
            }
            fprintf(stdout, "+----------------------------------------------------------------+\n");
            fprintf(stdout, "|           Tensor for experiment %d                             |\n", g+1);
            fprintf(stdout, "+----------------------------------------------------------------+\n\n");
            if(bitmap_opt_center[0] == TRUE){
                printf ("x     = %12.5f\t+/- %.5f\n", FIT_MULTI(5*N_PCS_ARRAY + 0), c*ERR_MULTI(5*N_PCS_ARRAY + 0));
                xM_f = FIT_MULTI(5*N_PCS_ARRAY + 0);
                xM_e = c*ERR_MULTI(5*N_PCS_ARRAY + 0);
            }
            else{
                printf ("x     = %.5f\n", x_init_center[0]);
                xM_f = x_init_center[0];
                xM_e = 0;
            }
            if(bitmap_opt_center[1] == TRUE){
                printf ("y     = %12.5f\t+/- %.5f\n", FIT_MULTI(5*N_PCS_ARRAY + 1), c*ERR_MULTI(5*N_PCS_ARRAY + 1));
                yM_f = FIT_MULTI(5*N_PCS_ARRAY + 1);
                yM_e = c*ERR_MULTI(5*N_PCS_ARRAY + 1);
            }
            else{
                printf ("y     = %.5f\n", x_init_center[1]);  
                yM_f = x_init_center[1];
                yM_e = 0;
            }
            if(bitmap_opt_center[1] == TRUE){
                printf ("z     = %12.5f\t+/- %.5f\n", FIT_MULTI(5*N_PCS_ARRAY + 2), c*ERR_MULTI(5*N_PCS_ARRAY + 2));
                zM_f = FIT_MULTI(5*N_PCS_ARRAY + 2);
                zM_e = c*ERR_MULTI(5*N_PCS_ARRAY + 2);
            }
            else{
                printf ("z     = %.5f\n", x_init_center[2]); 
                zM_f = x_init_center[2];
                zM_e = 0;
            }
            //alpha
            if(bitmap_opt[g][0] == TRUE){
                alpha_ = 180/M_PI*FIT_MULTI(g*5+0);
            }
            else{
                alpha_ = 180/M_PI*x_init[g][0];
            }
            j = (int) (alpha_ / 360.0);
            if (alpha_ > 0)
                alpha_ -= j*360;
            else
                alpha_ -= (j-1)*360;
        
            //beta
            if(bitmap_opt[g][1] == TRUE){
                beta_ = 180/M_PI*FIT_MULTI(g*5+1);
            }
            else{
                beta_ = 180/M_PI*x_init[g][1];
            }
            j = (int) (beta_ / 360.0);
            if (beta_ > 0)
                beta_ -= j*360;
            else
                beta_ -= (j-1)*360;
    
            //gamma
            if(bitmap_opt[g][2] == TRUE){
                gamma_ = 180/M_PI*FIT_MULTI(g*5+2);
            }
            else{
                gamma_ = 180/M_PI*x_init[g][2];
            }
            j = (int) (gamma_ / 360.0);
            if (gamma_ > 0)
                gamma_ -= j*360;
            else
                gamma_ -= (j-1)*360;
        
            ii = 0;
            if(bitmap_opt[g][ii] == TRUE){
                printf ("alpha = %12.5f\t+/- %.5f\n", alpha_, c*ERR_MULTI(g*5+ii)*180/M_PI);
                alpha_f = alpha_;
                alpha_e = c*ERR_MULTI(g*5+ii)*180/M_PI;
            }
            else{
                printf ("alpha = %.5f\n", 180/M_PI*x_init[g][ii]);
                alpha_f = 180/M_PI*x_init[g][ii];
                alpha_e = 0;
            }
            ii++;
            if(bitmap_opt[g][ii] == TRUE){
                printf ("beta  = %12.5f\t+/- %.5f\n", beta_, c*ERR_MULTI(g*5+ii)*180/M_PI);
                beta_f = beta_;
                beta_e = c*ERR_MULTI(g*5+ii)*180/M_PI;
            }
            else{
                printf ("beta  = %.5f\n", 180/M_PI*x_init[g][ii]);
                beta_f = 180/M_PI*x_init[g][ii];
                beta_e = 0;
            }
            ii++;
            if(bitmap_opt[g][ii] == TRUE){
                printf ("gamma = %12.5f\t+/- %.5f\n", gamma_, c*ERR_MULTI(g*5+ii)*180/M_PI);
                gamma_f = gamma_;
                gamma_e = c*ERR_MULTI(g*5+ii)*180/M_PI;
            }
            else{
                printf ("gamma = %.5f\n", 180/M_PI*x_init[g][ii]);        
                gamma_f = 180/M_PI*x_init[g][ii];
                gamma_e = 0;
            }
            ii++;
            if(bitmap_opt[g][ii+3] == TRUE){
                printf ("Ax    = %12.5f\t+/- %.5f\n", FIT_MULTI(g*5+ii), c*ERR_MULTI(g*5+ii));
                Ax_f = FIT_MULTI(g*5+ii);
                Ax_e = c*ERR_MULTI(g*5+ii);
            }
            else{
                printf ("Ax    = %.5f\n", x_init[g][ii+3]);   
                Ax_f = x_init[g][ii+3];
                Ax_e = 0;
            }
            ii++;
            if(bitmap_opt[g][ii+3] == TRUE){
                printf ("Rh    = %12.5f\t+/- %.5f\n", FIT_MULTI(g*5+ii), c*ERR_MULTI(g*5+ii));
                Rh_f = FIT_MULTI(g*5+ii);
                Rh_e = c*ERR_MULTI(g*5+ii);
            }
            else{
                printf ("Rh    = %.5f\n", x_init[g][ii+3]);     
                Rh_f = x_init[g][ii+3];
                Rh_e = 0;
            }
            PrintTensorCoord(alpha_f, beta_f, gamma_f, Ax_f, Rh_f, xM_f, yM_f, zM_f);
            free_tensor(_all_tensor_pcs[g]);
            _all_tensor_pcs[g] = new_tensor();
            _all_tensor_pcs[g]->ax = Ax_f;
            _all_tensor_pcs[g]->rh = Rh_f;
            _all_tensor_pcs[g]->a = alpha_f/180*M_PI;
            _all_tensor_pcs[g]->b = beta_f/180*M_PI;
            _all_tensor_pcs[g]->g = gamma_f/180*M_PI;
            _all_tensor_pcs[g]->x = xM_f;
            _all_tensor_pcs[g]->y = yM_f;
            _all_tensor_pcs[g]->z = zM_f;
            _all_tensor_pcs[g]->err_ax = Ax_e;
            _all_tensor_pcs[g]->err_rh = Rh_e;
            _all_tensor_pcs[g]->err_a = alpha_e/180*M_PI;
            _all_tensor_pcs[g]->err_b = beta_e/180*M_PI;
            _all_tensor_pcs[g]->err_g = gamma_e/180*M_PI;
            _all_tensor_pcs[g]->err_x = xM_e;
            _all_tensor_pcs[g]->err_y = yM_e;
            _all_tensor_pcs[g]->err_z = zM_e;
            int serial = add_to_tensor_listing(Ax_f, Rh_f, xM_f, yM_f, zM_f, alpha_f, beta_f, gamma_f,
								  CALCULATED_TENSOR_MULTI, g+1
								  ,Ax_e, Rh_e, xM_e, yM_e, zM_e
                           , alpha_e/180*M_PI, beta_e/180*M_PI, gamma_e/180*M_PI);
			serial_a[g] = serial;
			//append_tensor_valid(g, serial); NOW MULTI
			if(first_of_g == g){
				display_tensor(g, MULTIPLE, serial, TRUE);
				append_multi_tensor_valid(g, serial, TRUE);
			}
			else{
				display_tensor(g, MULTIPLE, serial, FALSE);
				append_multi_tensor_valid(g, serial, FALSE);
			}
        }
    }
	
	
	for(g = 0; g < N_PCS_ARRAY; g++){
        if(g_button_tab[g]){
            calculate_pcs(_all_tensor_pcs[g], g, UPDATE_PCS_CALC, 0.0, 1, use_racs[g],
                          racs_d);
            display_pcs_list(g, serial_a[g]);
        }
    }
	
	return(1);
}

