/*
 * pcsfit_metal.h
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */



struct PCSdata3
{
  size_t n;
  double *PCS;
  double *TOL;
  double *x, *y, *z;		//N 
  double a, b, g;
  double xM, yM, zM;
  double Ax, Rh;
  int n_button;
  int *index_on_pdb;		//n (n < N)
  int *index_on_pcs;		//n (n < N)
  int *index_v;			//8
  gboolean *bitmap_opt;		//8 
  gboolean *bitmap_bound;	//8 
  int n_opt;
  int n_bound;
  error_control_data *err_ctrl;
  double *PCS_OFFSET;
  double *PDB_OFFSET_X;
  double *PDB_OFFSET_Y;
  double *PDB_OFFSET_Z;
  gboolean *SUBSET_ERROR;
  gboolean use_racs;
  racs_data *racs_d;
} PCSdata3;


struct PCSdata4
{
  int *n;
  double **PCS;
  double **TOL;
  double **x;
  double **y;
  double **z;
  int **index_on_pdb;
  int **index_on_pcs;
  gboolean *g_button_tab;
  double **x_init;
  int **index_v;
  gboolean **bitmap_opt;
  gboolean **bitmap_bound;
  int *n_bound;
  int *n_opt;
  double *x_init_center;
  int *index_v_center;
  gboolean *bitmap_opt_center;
  gboolean *bitmap_bound_center;
  int n_bound_center;
  int p_center;
  int *index_v_multi;
  gboolean *index_bound_multi;
  error_control_data *err_ctrl;
  double **PCS_OFFSET;
  double **PDB_OFFSET_X;
  double **PDB_OFFSET_Y;
  double **PDB_OFFSET_Z;
  gboolean **SUBSET_ERROR;
  gboolean *use_racs;
  racs_data *racs_d;
} PCSdata4;






enum
{
  SINGLE,
  MULTIPLE
};

int calculate_single_tensor (int n_button, error_control_data * err_ctrl,
			     tensor my_tensor_multi[N_PCS_ARRAY]);


double SinglePCS_new (double R_chi[3][3], double xM, double yM, double zM,
		      double Xax, double Xrh, int index_global, int n_button,
		      int *index_on_pdb, int *index_on_pcs,
		      double *offset_noise_x, double *offset_noise_y,
		      double *offset_noise_z, gboolean use_racs, double *racs,
		      racs_data * racs_d);


int PrintTensorCoord (double phi, double theta, double psi, double ax,
		      double rh, double xM, double yM, double zM);
int calculate_multiple_tensor (gboolean * g_button_tab,
			       error_control_data * err_ctrl,
			       tensor my_tensor_multi[N_PCS_ARRAY]);
void start_calculate_multiple_tensor (GtkButton * button);





void fill_bitmap_single_subset (gboolean * SUBSET_ERROR,
				int n_bis, gsl_rng * my_rand,
				double subset_percentage);
int get_tensor_single_from_covar (gboolean bitmap_opt[8], double x_init[8],
				  gsl_multifit_fdfsolver * s,
				  int index_v[8], tensor * my_tensor_loc);

int postproceed_single (gsl_multifit_fdfsolver * s, gsl_matrix * covar,
			int n_bis, int p,
			gboolean bitmap_opt[8], int index_v[8],
			double x_init[8], int n_button,
			gboolean use_racs, racs_data * racs_d);

int postproceed_multiple (gsl_multifit_fdfsolver * s, gsl_matrix * covar,
			  int n_bis[N_PCS_ARRAY], int n_bis_total,
			  int p[N_PCS_ARRAY], int p_total,
			  gboolean * bitmap_opt[N_PCS_ARRAY],
			  gboolean bitmap_opt_center[3],
			  int *index_v[N_PCS_ARRAY],
			  int index_v_multi[N_PCS_ARRAY * 5 + 3],
			  double *x_init[N_PCS_ARRAY],
			  double x_init_center[3], gboolean * g_button_tab,
			  int serial_a[N_PCS_ARRAY],
			  gboolean use_racs[N_PCS_ARRAY], racs_data * racs_d);
int expb_df_unified (const gsl_vector * v, void *data, gsl_matrix * J);
int expb_f_unified (const gsl_vector * v, void *data, gsl_vector * f);
int expb_fdf_unified (const gsl_vector * x, void *data, gsl_vector * f,
		      gsl_matrix * J);
int expb_fdf_unified_multi (const gsl_vector * x, void *data, gsl_vector * f,
			    gsl_matrix * J);
int expb_f_unified_multi (const gsl_vector * v, void *data, gsl_vector * f);
int expb_df_unified_multi (const gsl_vector * v, void *data, gsl_matrix * J);
char *parse_labelfield (char *line);
int print_f_unified (const gsl_vector * v, void *data, gsl_vector * f,
		     char **label);
void print_state (int iter, gsl_multifit_fdfsolver * s, int status,
		  GtkTextBuffer * buffer);
void free_label (char **label, int n);
void cross_product (double vect1[3], double vect2[3], double vect_result[3]);
/*
double calculate_RACS (double x1, double y1, double z1,
		       double x2, double y2, double z2,
		       double x3, double y3, double z3,
		       double R[3][3],
		       double Xax, double Xrh,
		       double sigma11, double sigma22, double sigma33,
		       double beta, double angle_offset, int racs_type,
		       double B0, double T);
*/
