/*
 * pdb.h
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */


#define MAX_ATOM_TYPE 1000
#define MAX_RESIDUE_TYPE 100
#define MAX_ATOM_NAME_LENGTH 7
#define MAX_RESIDUE_NAME_LENGTH 5
#define MAX_CAR 1000
#define MAX_FILE_NAME 1000

//~ this structure is for atom coordinate
typedef struct Atom_coordinate
{
  double x;
  double y;
  double z;
  double junk;
  double CSA_tensor[3][3];	//Must be on pdb...
} atom_coordinate;


//~ this structure is for the atom
typedef struct Atom_table
{
  int atom_num;
  int residue_num;
  char atom_name[MAX_ATOM_NAME_LENGTH];
  char residue_name[MAX_RESIDUE_NAME_LENGTH];
  atom_coordinate *atom_coo;
} atom_table;


//~ this structure is for storing all information relative to
//~ the pdb file. shouldn't be modified once created (exept to delet it)
typedef struct Pdb_structure
{
  char file_name[1000];		//file_name
//    int f_valid; //boolean to say if the pdb structure is valid (useable) or not
  int n_model;			//number of MODEL in the pdb file (> 1 for NMR family)
  int n_atom;			//number of atom in each MODEL (all the same number!)
  int n_residue;		//number of residue in each MODEL (all the same number!)
  int n_residue_type;		//number of different residue type (ALA, VAL ...)
  int n_atom_type;		//number of different atom type
  char tab_atom_type[MAX_ATOM_TYPE][MAX_ATOM_NAME_LENGTH];	//list
  //of all atom type present in the MODEL
  char tab_residue_type[MAX_RESIDUE_TYPE][MAX_RESIDUE_NAME_LENGTH];	//list
  //of all residue type present in the MODEL 
  atom_table *tab_atom;
} pdb_structure;

pdb_structure *_pdb_str;

//~ Prototype declaration

int print_info_pdb_structure (void);

int read_pdb_file (char *name);

int count_pdb_atom (FILE * f_pdb);

int count_pdb_model (FILE * f_pdb);

pdb_structure *new_pdb_structure (int n_pdb_model, int n_pdb_atom);

int free_pdb_structure (pdb_structure * pdb_str);

int
populate_pdb_structure (int n_pdb_model, int n_pdb_atom, FILE * f_pdb,
			pdb_structure * local_pdb_str);

int init_pdb_structure (void);

int
store (char *line, int atom_num, int model_num,
       pdb_structure * local_pdb_str);

int populate_type (pdb_structure * local_pdb_str);

int print_full_pdb_structure (pdb_structure * local_pdb_str);

void load_PDB (void);

int find_residue_type_in_pdb (char *name);

int find_atom_type_in_pdb (char *name);
