/*
 * pymol_iso.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <glib.h>
#include <glib/gstdio.h>

//#define _ZXZ 1
//#define _ZYZ 0
#define epsilon_molmol_r2 0.001

#include "pdb.h"
#include "common.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "tensor_list.h"
#include "molmol_iso.h"
#include "unify_tensor.h"
#include "pymol_iso.h"

//#define BUFFER_SIZE 100000
//char buffer_write[BUFFER_SIZE];

extern GtkWidget *_window_iso_pymol;
extern GtkTreeModel * _model_tl;



double calc_pot_pymol(double rot[3][3], FILE * F_out, molmol_data * mm_data){

  double x_s, y_s, z_s;
  double x_ss, y_ss, z_ss;
  int x_ns, y_ns, z_ns;  
  double x_H, y_H, z_H;
  double x2_H, y2_H, z2_H;
  double x_t, y_t, z_t;
  double fac, r2, r5, PCS;
  int i, j, k, ii; 
  double compt;
  double X, Y, Z;
  double x_c, y_c, z_c;  
  double Dax, Drh;
  double pcs_max;
  double max_pcs, min_pcs;
	
  pcs_max = 0; 
  max_pcs = -99999;
  min_pcs = 99999;
    
  Dax = mm_data->Ax;
  Drh = mm_data->Rh;  
    
  x_ns = mm_data->x_step_n;
  y_ns = mm_data->y_step_n;  
  z_ns = mm_data->z_step_n;
   
  x_ss = mm_data->x_step_size;
  y_ss = mm_data->y_step_size;
  z_ss = mm_data->z_step_size;  
    
  X = mm_data->X;
  Y = mm_data->Y;
  Z = mm_data->Z;
    
  x_s = mm_data->x_start ;
  y_s = mm_data->y_start ;
  z_s = mm_data->z_start ;

  //fprintf(F_out, "% 8.3lf %4.0d %1.3lf\n", x_s, x_ns, x_ss);
  //fprintf(F_out, "% 8.3lf %4.0d %1.3lf\n", y_s, y_ns, y_ss);
  //fprintf(F_out, "% 8.3lf %4.0d %1.3lf\n", z_s, z_ns, z_ss);
  	fprintf(F_out, "\n       3 !NTITLE\n");
  	fprintf(F_out, " REMARKS ...\n");     
	fprintf(F_out, " REMARKS ...\n"); 
	fprintf(F_out, " REMARKS ...\n"); 
	fprintf(F_out, "%8d%8d%8d%8d%8d%8d%8d%8d%8d\n",
			x_ns, (int) x_s, x_ns + (int)x_s - 1,
			y_ns, (int) y_s, y_ns + (int)y_s - 1,
			z_ns, (int) z_s, z_ns + (int)z_s - 1
			);
	fprintf(F_out, "% .5E% .5E% .5E% .5E% .5E% .5E\n",
			/*x_s + */x_ns*x_ss,
			/*y_s + */y_ns*y_ss, 
			/*z_s + */z_ns*z_ss, 
			90.0, 90.0, 90.0);
	fprintf(F_out, "ZYX");

  compt = 0;
  ii = 0;

  fac = 1.0/12.0*M_PI;

  x_t = rot[0][0] * X + rot[0][1] * Y + rot[0][2] * Z; 
  y_t = rot[1][0] * X + rot[1][1] * Y + rot[1][2] * Z; 
  z_t = rot[2][0] * X + rot[2][1] * Y + rot[2][2] * Z; 
	
  //setvbuf(F_out, buffer_write, _IOFBF, BUFFER_SIZE);

    z_c = z_s;
    for(i = 0; i < z_ns; i++) {
		fprintf(F_out, "\n%8d\n", i);
		ii = 0;
        y_c = y_s;
        for( j = 0; j < y_ns; j++ ) {
            x_c = x_s;
            for( k = 0; k < x_ns; k++ ) {
                x_H = rot[0][0]*x_c + rot[0][1] * y_c + rot[0][2] * z_c; 
                y_H = rot[1][0]*x_c + rot[1][1] * y_c + rot[1][2] * z_c; 
                z_H = rot[2][0]*x_c + rot[2][1] * y_c + rot[2][2] * z_c; 
                x_H -= x_t;
                y_H -= y_t;
                z_H -= z_t;
                x2_H = x_H * x_H;
                y2_H = y_H * y_H;
                z2_H = z_H * z_H;
                r2 = x2_H + y2_H + z2_H;
                r5 = r2*r2*sqrt(r2);
                if(epsilon_molmol_r2 < r2){
                    PCS = FACT_USI * 1.0 /* * fac */ /r5*( Dax*(2*z2_H-x2_H-y2_H) + 3.0/2.0*Drh*(x2_H - y2_H));
//THE fact_usi take into account (i) the 1/(12PI) (ii)
//the fact that length are in Angstrom and Xax in 10-32m3
//and (iii) that PCS are in ppm (10^6)
					if (PCS > max_pcs){
                         max_pcs = PCS;
                    }
					if (PCS < min_pcs){
                         min_pcs = PCS;
                    }
					
                    if( (i == 0)||(j == 0)||(k == 0)||(i ==  (z_ns-1))||(j == (y_ns-1))||(k == (x_ns-1))){
                        //we are on the surface of the cube
                        //we save the biggeest PCS values for isovalue.
                        if (fabs(PCS) > pcs_max){
                            pcs_max = fabs(PCS);
                        }
                    }
                }
                else{
                    PCS = 0.0; 
                }
                fprintf(F_out, "% .5E", PCS);
                ii++;
                if( ii ==  6){
                    ii = 0;
                    fprintf(F_out, "\n");
                }
                compt++;
                x_c += x_ss;
            }      
            y_c += y_ss;
        }
        z_c += z_ss;
    }
	
  if(ii != 0){
  	fprintf(F_out, "\n");	
  }
	
  fprintf(F_out, "-9999\n");
  fprintf(F_out, "% .5E% .5E\n", min_pcs, max_pcs);
    
  return(pcs_max);  
}

void add_manual_vp(void){
    GladeXML *gxml;
    double x, y, z, Ax, Rh, a, b, g;
    
    gxml = glade_get_widget_tree(_window_iso_pymol);

    x = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_x2"));
    y = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_y2"));
    z = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_z2"));
    a = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_alpha2"));
    b = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_beta2"));
    g = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_gamma2"));
    Ax = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Ax2"));
    Rh = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Rh2"));
	
    add_to_tensor_listing(Ax, Rh, x, y, z, a, b, g, MANUAL_TENSOR, -1,
						   0, 0, 0, 0, 0, 0, 0, 0);
}

void start_vp(void){
    if(_pdb_str == NULL){
        box_error( "You should first load a pdb file.\n", NULL);
        return;
    }
	uncheck_all((GtkTreeModel *) _model_tl, COLUMN_USED_TL);
    gtk_widget_show(_window_iso_pymol);
}

void init_vp(void){
    GladeXML *gxml;
    gxml = glade_get_widget_tree(_window_iso_pymol);
    GtkTreeView * treeview;
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview4");
    add_columns_tl(treeview, TENSOR_ANY);
    gtk_tree_view_set_model ((GtkTreeView *)treeview, GTK_TREE_MODEL (_model_tl));
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
	set_multiple_selection(GTK_TREE_VIEW (treeview));
    gtk_widget_hide(_window_iso_pymol);
}

void update_vp_gride_end(GtkSpinButton * spin){
    GtkSpinButton * s_x_start;
    GtkSpinButton * s_y_start;
    GtkSpinButton * s_z_start;
    GtkSpinButton * s_n_step;
    GtkSpinButton * s_step_size;
    GtkLabel * l_x_end;
    GtkLabel * l_y_end;
    GtkLabel * l_z_end;
    int n_step;
    double step_size;
    char val_char[50];
    double end;
    
    GladeXML *gxml;	
    
	gxml = glade_get_widget_tree((GtkWidget*) spin);
    s_x_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_x_start2");
    s_y_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_y_start2");
    s_z_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_z_start2");
    s_n_step = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_n_step2");
    s_step_size = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_step_size2");
    l_x_end = (GtkLabel *) glade_xml_get_widget(gxml, "label_x_end2");
    l_y_end = (GtkLabel *) glade_xml_get_widget(gxml, "label_y_end2");
    l_z_end = (GtkLabel *) glade_xml_get_widget(gxml, "label_z_end2");
    
    n_step = (int) gtk_spin_button_get_value(s_n_step);
    step_size = (double) gtk_spin_button_get_value(s_step_size);
    
    end = gtk_spin_button_get_value(s_x_start) + n_step * step_size;
    sprintf(val_char, "%.3lf", end);
    gtk_label_set_text(l_x_end, val_char);
    end = gtk_spin_button_get_value(s_y_start) + n_step * step_size;
    sprintf(val_char, "%.3lf", end);
    gtk_label_set_text(l_y_end, val_char);
    end = gtk_spin_button_get_value(s_z_start) + n_step * step_size;
    sprintf(val_char, "%.3lf", end);
    gtk_label_set_text(l_z_end, val_char);
}

void update_vp_gride(GtkButton * button){
    GtkSpinButton * s_x_start;
    GtkSpinButton * s_y_start;
    GtkSpinButton * s_z_start;
    GtkSpinButton * s_n_step;
    GtkSpinButton * s_step_size;
    GtkTreeModel *model;
    gboolean my_bool;
    GtkTreeIter iter;
    gboolean is_toggle;
    double x, y, z;
    
    GladeXML *gxml;	
    
	gxml = glade_get_widget_tree((GtkWidget*) button);
    
    s_x_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_x_start2");
    s_y_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_y_start2");
    s_z_start = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_z_start2");
    s_n_step = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_n_step2");
    s_step_size = (GtkSpinButton *) glade_xml_get_widget(gxml, "spin_step_size2");
        
    model = (GtkTreeModel *) _model_tl;
    if(model == NULL){
        return;
    }
    is_toggle = FALSE;
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (model, &iter, COLUMN_USED_TL, &is_toggle, -1); 
        if (is_toggle == TRUE){
            break;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model, &iter);
    }
    if(is_toggle == FALSE){
        box_error("Exactly one tensor should be selected for calculation.", NULL);
        return;
    }
    gtk_tree_model_get(model, &iter, COLUMN_X_TL, &x, -1);
    gtk_tree_model_get(model, &iter, COLUMN_Y_TL, &y, -1);
    gtk_tree_model_get(model, &iter, COLUMN_Z_TL, &z, -1);
    
    gtk_spin_button_set_value(s_x_start, ((int)x) -40.0);
    gtk_spin_button_set_value(s_y_start, ((int)y) -40.0);
    gtk_spin_button_set_value(s_z_start, ((int)x) -40.0);
    gtk_spin_button_set_value(s_n_step, 80);
    gtk_spin_button_set_value(s_step_size, 1.0);

}

void cancel_vp(void){
    gtk_widget_hide(_window_iso_pymol);
}

void copy_vp(void){
	  GladeXML *gxml;  
	  GtkTreeView *treeview;

  gxml = glade_get_widget_tree(_window_iso_pymol);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview4"); 
  copy_tensor_from_list(gxml, treeview, 2);	

}

void remove_vp(void){
	
	GtkTreeView *treeview;
	GladeXML *gxml; 
 
  gxml = glade_get_widget_tree(_window_iso_pymol);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview4");  
  remove_tensor_from_list(gxml, treeview, 2);	
	
}


void ok_vp(GtkButton * button){
    //FILE *fpipe;

    molmol_data mm_data;
    char com[1500];
	char name_mac[500];
    char prog[1500];
	int result;
	GtkEntry * entry;
	GladeXML *gxml; 
    gboolean result_bool;
    GPid pid;
    gchar *argv[2];
    GError * error = NULL;
	//char * working_directory = "./";
	result = read_tensor_from_gui(&mm_data, (GtkWidget*) button, PYMOL_);
	if(result == -1){
		return;
	}
    /*
    print_all_64( mm_data.a,  mm_data.b,  mm_data.g, &mm_data);
    return;
    */
    
	gtk_widget_hide(_window_iso_pymol);
	unique_tensor(&mm_data);
	sprintf(name_mac, "./pymol_isosurface.pml");
	write_for_pymol(&mm_data, name_mac, 0); 
	print_tensor(&mm_data); 

    fprintf(stderr, "trying to launch PyMOL with the command:\n");
	
	gxml = glade_get_widget_tree(_window_iso_pymol);    
  	entry = (GtkEntry *) glade_xml_get_widget (gxml, "entry_pymol"); 
	sprintf(com, "%s %s", gtk_entry_get_text(entry), name_mac);
	fprintf(stderr, "%s\n", com);
    sprintf(prog, "%s", gtk_entry_get_text(entry));
    argv[0] = prog;
    argv[1] = name_mac;
    argv[2] = NULL;
    /* //NOT COMPATIBLE WITH WINDOWS
    if ( !(fpipe = (FILE*)popen(com,"r")) ) {  // If fpipe is NULL
        fprintf(stderr, "Problems with pipe\n");
        return;
    }
    */
    //fprintf(stderr, "%s %s %s\n", argv[0], argv[1], working_directory);
    
    result_bool =  g_spawn_async(
                        NULL,
                        argv,
                        NULL,
                        G_SPAWN_SEARCH_PATH,
                        NULL,
                        NULL,
                        &pid,
                        &error);

    g_spawn_close_pid(pid);                                                     
    
    if (error != NULL){
        fprintf (stderr, "Error message: %s\n", error->message);
        g_error_free (error);
    } 
    else{
        
    }
    
    return;
}



void save_vp(GtkButton * button){
  	GtkWidget *dialog;
  	GtkFileFilter *filter1;
  	GtkFileFilter *filter4;
  	gint result;
    molmol_data mm_data;
	char name_mac[500];
	int result2;
	
	result2 = read_tensor_from_gui(&mm_data, (GtkWidget*) button, PYMOL_);
	if(result2 == -1){
		return;
	}
    
  dialog =
    gtk_file_chooser_dialog_new
    ("In which file do you want to save your PyMOL macro?", NULL,
     GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
		
	gtk_window_set_icon_from_file ((GtkWindow *)dialog, NUMBAT_ICON_FILE, NULL);
  gtk_window_set_transient_for((GtkWindow*) dialog, (GtkWindow*)_window_iso_pymol);

  filter1 = gtk_file_filter_new ();
  filter4 = gtk_file_filter_new ();
  gtk_file_filter_add_pattern (filter1, "*.pml");
  gtk_file_filter_set_name (filter1, "*.pml macro for Pymol");
  gtk_file_filter_add_pattern (filter4, "*.*");
  gtk_file_filter_set_name (filter4, "*.* ALL");
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter1);
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter4);

  result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
    {
    case GTK_RESPONSE_ACCEPT:
      {
	char *filename;
	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		  
	unique_tensor(&mm_data);
	sprintf(name_mac, filename);
	write_for_pymol(&mm_data, name_mac, 0);
	print_tensor(&mm_data); 
	g_free (filename);
    gtk_widget_hide(_window_iso_pymol);
	  }
      break;
    default:
      break;
    }
  gtk_object_destroy((GtkObject *)filter1);
  gtk_object_destroy((GtkObject *)filter4);
  gtk_widget_destroy (dialog);    
    
}


void write_for_pymol(molmol_data * mm_data , char * name_mac, int color){
	double pcs_max;
	FILE * F_pot;
    FILE * F_mac;
	FILE * F_pdb;
    char name_pot[500];
	char name_pdb[500];
	sprintf(name_pot, "%s.xplor",  name_mac);
	sprintf(name_pdb, "%s.pdb",  name_mac);
	
	if (!(F_pot = g_fopen(name_pot , "w"))) {
        fprintf(stderr, "can't open %s\n", name_pot);
        return;
    }
    
    if (!(F_mac = g_fopen(name_mac , "w"))) {
        fprintf(stderr, "can't open %s\n", name_mac);
        fclose(F_pot);
        return;
    }
	
	if (!(F_pdb = g_fopen(name_pdb , "w"))) {
        fprintf(stderr, "can't open %s\n", name_pdb);
        fclose(F_pot);
		fclose(F_mac);
        return;
    }
    
    pcs_max = write_pot_ZX_or_YZ_PYMOL(F_pot, mm_data);

    
    write_mac_PYMOL(F_mac, mm_data, name_pot, name_pdb, pcs_max, color);
	write_pdb_tensor(F_pdb, mm_data);
    
    fclose(F_pot);
    fclose(F_mac);
    fclose(F_pdb);	
}



double write_pot_ZX_or_YZ_PYMOL(FILE * F_out, molmol_data * mm_data){
  double rot[3][3];
	double a, b, g;
	a = mm_data->a;
	b = mm_data->b;
	g = mm_data->g;
  filZX_or_YZ(rot, a, b, g);
  return(calc_pot_pymol(rot, F_out, mm_data));
}


void write_mac_PYMOL(FILE * F_mac, molmol_data * mm_data, char * name_pot, char * name_pdb, double pcs_max,
			   int color)
{
    double iso1;
    double iso2;
    double iso3;
	char name_pdb_no_ext_temp[1000];
	char * name_pdb_no_ext;

	
	sprintf(name_pdb_no_ext_temp, name_pdb);
	name_pdb_no_ext = strrchr(name_pdb_no_ext_temp, '/');
	name_pdb_no_ext_temp[strlen(name_pdb) - 4] = '\0';
	if(name_pdb_no_ext == NULL){
		box_error("error while trying to generate\n the PyMOL macro file", NULL);
		return;
	}
	name_pdb_no_ext += 1;
	
    iso3 = pcs_max;
    iso2 = pcs_max *4;
    iso1 = pcs_max *9;
    
    fprintf(F_mac, "#PyMOL macro for tensor with values:\n");
    fprintf(F_mac, "#alpha: %.3lf beta: %.3lf gamma: %.3lf Ax: %.3lf Rh: %.3lf x: %.3lf y: %.3lf z: %.3lf\n"
            , mm_data->a, mm_data->b, mm_data->g, mm_data->Ax, mm_data->Rh, mm_data->X
            , mm_data->Y, mm_data->Z);
    fprintf(F_mac, "#Corresponding potential file should be in %s\n", name_pot);
    fprintf(F_mac, "load %s, isomap, 1, xplor\n", name_pot);
	fprintf(F_mac, "isosurface iso1, isomap, %.4lf\n", iso2);
	fprintf(F_mac, "isosurface iso2, isomap, %.4lf\n", -1.0*iso2);
	fprintf(F_mac, "isosurface iso3, isomap, %.4lf\n", iso3);
	fprintf(F_mac, "isosurface iso4, isomap, %.4lf\n", -1.0*iso3);
	fprintf(F_mac, "set transparency, 0.5, iso1\n");
	fprintf(F_mac, "set transparency, 0.5, iso2\n");
	fprintf(F_mac, "set transparency, 0.7, iso3\n");
	fprintf(F_mac, "set transparency, 0.7, iso4\n");
	fprintf(F_mac, "set surface_color, blue, iso1\n");
	fprintf(F_mac, "set surface_color, red, iso2\n");
	fprintf(F_mac, "set surface_color, blue, iso3\n");
	fprintf(F_mac, "set surface_color, red, iso4\n");
	fprintf(F_mac, "color blue, iso1\n");
	fprintf(F_mac, "color red, iso2\n");
	fprintf(F_mac, "color blue, iso3\n");
	fprintf(F_mac, "color red, iso4\n");
	fprintf(F_mac, "load %s\n", name_pdb);
		
	fprintf(F_mac, "bond (name OX) and (object %s) and (residue 9000), (name OO) and (object %s) and (residue 9000)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "bond (name OY) and (object %s) and (residue 9000) , (name OO) and (object %s) and (residue 9000)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "bond (name OZ) and (object %s) and (residue 9000) , (name OO) and (object %s) and (residue 9000)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	
	fprintf(F_mac, "bond (name OX) and (object %s) and (residue 9001) , (name OO) and (object %s) and (residue 9001)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "bond (name OY) and (object %s) and (residue 9001) , (name OO) and (object %s) and (residue 9001)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "bond (name OZ) and (object %s) and (residue 9001) , (name OO) and (object %s) and (residue 9001)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	
	fprintf(F_mac, "bond (name OX) and (object %s) and (residue 9002) , (name OO) and (object %s) and (residue 9002)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "bond (name OY) and (object %s) and (residue 9002) , (name OO) and (object %s) and (residue 9002)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "bond (name OZ) and (object %s) and (residue 9002) , (name OO) and (object %s) and (residue 9002)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	
	fprintf(F_mac, "bond (name OX) and (object %s) and (residue 9003) , (name OO) and (object %s) and (residue 9003)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "bond (name OY) and (object %s) and (residue 9003) , (name OO) and (object %s) and (residue 9003)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "bond (name OZ) and (object %s) and (residue 9003) , (name OO) and (object %s) and (residue 9003)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
		
	fprintf(F_mac, "unbond (name OX) and (object %s), (name OY) and (object %s)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "unbond (name OX) and (object %s), (name OZ) and (object %s)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "unbond (name OY) and (object %s), (name OZ) and (object %s)\n"
			, name_pdb_no_ext, name_pdb_no_ext);

	fprintf(F_mac, "set_bond stick_radius, 0.1, (name OX) and (object %s) , (name OO) and (object %s)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "set_bond stick_radius, 0.1, (name OY) and (object %s) , (name OO) and (object %s)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "set_bond stick_radius, 0.1, (name OZ) and (object %s) , (name OO) and (object %s)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "set_bond stick_color, red, (name OX) and (object %s) , (name OO) and (object %s)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "set_bond stick_color, green, (name OY) and (object %s) , (name OO) and (object %s)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "set_bond stick_color, blue, (name OZ) and (object %s) , (name OO) and (object %s)\n"
			, name_pdb_no_ext, name_pdb_no_ext);
	fprintf(F_mac, "color yellow, (name OO) and (object %s)\n", name_pdb_no_ext);
	fprintf(F_mac, "color red, (name OX) and (object %s)\n", name_pdb_no_ext);
	fprintf(F_mac, "color green, (name OY) and (object %s)\n", name_pdb_no_ext);
	fprintf(F_mac, "color blue, (name OZ) and (object %s)\n", name_pdb_no_ext);
	fprintf(F_mac, "show sticks, object %s\n", name_pdb_no_ext);
	fprintf(F_mac, "label (name OX) and (object %s), \'X\'\n", name_pdb_no_ext);
	fprintf(F_mac, "label (name OY) and (object %s), \'Y\'\n", name_pdb_no_ext);
	fprintf(F_mac, "label (name OZ) and (object %s), \'Z\'\n", name_pdb_no_ext);
	fprintf(F_mac, "label (name OO) and (object %s), \'O\'\n", name_pdb_no_ext);
	fprintf(F_mac, "show labels, object %s\n", name_pdb_no_ext);
}
