/*
 * racs.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <math.h>

#include "pdb.h"
#include "methyl.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "tensor_list.h"
#include "common.h"

#define AXIS_ROT_MATRIX 1

extern GtkWidget *_window_RACS;
extern GtkWidget *_window_tensor_list;
GtkSpinButton *_spin_B0_RACS;
GtkSpinButton *_spin_T_RACS;
GtkSpinButton *_sigma_C_11;
GtkSpinButton *_sigma_C_22;
GtkSpinButton *_sigma_C_33;
GtkSpinButton *_sigma_N_11;
GtkSpinButton *_sigma_N_22;
GtkSpinButton *_sigma_N_33;
GtkSpinButton *_sigma_H_11;
GtkSpinButton *_sigma_H_22;
GtkSpinButton *_sigma_H_33;

GtkSpinButton *_beta_C;
GtkSpinButton *_beta_N;
GtkSpinButton *_beta_H;

GtkComboBox *_combobox_RACS_O;
GtkComboBox *_combobox_RACS_C;
GtkComboBox *_combobox_RACS_N;
GtkComboBox *_combobox_RACS_H;
GtkComboBox *_combobox_RACS_CA;

GtkCheckButton *_checkbutton_RACS_PCS1;
GtkCheckButton *_checkbutton_RACS_PCS2;
GtkCheckButton *_checkbutton_RACS_PCS3;
GtkCheckButton *_checkbutton_RACS_PCS4;
GtkCheckButton *_checkbutton_RACS_PCSGEN;

gboolean _IS_RACS_VALID = FALSE;

void
ok_racs (void)
{
  racs_atom_name racs_atom_na;
  racs_data racs_d;
  gtk_widget_hide (_window_RACS);
  get_racs_parameters (&racs_d);
  read_racs_atom_name (&racs_atom_na);
  make_link_for_racs (&racs_atom_na);
  precalculate_CSA_tensor (&racs_d);
  //ONLY IF THE RACS DEFINITION IS VALID
  fprintf (stderr,
	   "CHISTOPHE YOU HAVE TO CHECK THE VALIDITY OF RACS DEFINITION\n");
  validate_racs_toggle ();
}


void
cancel_racs (void)
{
  gtk_widget_hide (_window_RACS);
}

void
start_racs (void)
{
    
    if(_pdb_str == NULL){
        box_error( "You should first load a pdb file.\n", NULL);
        return;
    }
  gtk_widget_show (_window_RACS);
}

void
init_racs (void)
{
  GladeXML *gxml;
  GladeXML *gxml2;
  gxml = glade_get_widget_tree (_window_RACS);
  gxml2 = glade_get_widget_tree (_window_tensor_list);
  _spin_B0_RACS =
    (GtkSpinButton *) glade_xml_get_widget (gxml, "constante_B0_RACS");
  _spin_T_RACS =
    (GtkSpinButton *) glade_xml_get_widget (gxml, "constante_T_RACS");
  _sigma_C_11 = (GtkSpinButton *) glade_xml_get_widget (gxml, "sigma_C_11");
  _sigma_C_22 = (GtkSpinButton *) glade_xml_get_widget (gxml, "sigma_C_22");
  _sigma_C_33 = (GtkSpinButton *) glade_xml_get_widget (gxml, "sigma_C_33");
  _sigma_N_11 = (GtkSpinButton *) glade_xml_get_widget (gxml, "sigma_N_11");
  _sigma_N_22 = (GtkSpinButton *) glade_xml_get_widget (gxml, "sigma_N_22");
  _sigma_N_33 = (GtkSpinButton *) glade_xml_get_widget (gxml, "sigma_N_33");
  _sigma_H_11 = (GtkSpinButton *) glade_xml_get_widget (gxml, "sigma_H_11");
  _sigma_H_22 = (GtkSpinButton *) glade_xml_get_widget (gxml, "sigma_H_22");
  _sigma_H_33 = (GtkSpinButton *) glade_xml_get_widget (gxml, "sigma_H_33");

  _beta_C = (GtkSpinButton *) glade_xml_get_widget (gxml, "beta_C");
  _beta_N = (GtkSpinButton *) glade_xml_get_widget (gxml, "beta_N");
  _beta_H = (GtkSpinButton *) glade_xml_get_widget (gxml, "beta_H");

  _combobox_RACS_O =
    (GtkComboBox *) glade_xml_get_widget (gxml, "combobox_RACS_O");
  _combobox_RACS_C =
    (GtkComboBox *) glade_xml_get_widget (gxml, "combobox_RACS_C");
  _combobox_RACS_N =
    (GtkComboBox *) glade_xml_get_widget (gxml, "combobox_RACS_N");
  _combobox_RACS_H =
    (GtkComboBox *) glade_xml_get_widget (gxml, "combobox_RACS_H");
  _combobox_RACS_CA =
    (GtkComboBox *) glade_xml_get_widget (gxml, "combobox_RACS_CA");

  gtk_combo_box_set_active (_combobox_RACS_O, 0);
  gtk_combo_box_set_active (_combobox_RACS_C, 0);
  gtk_combo_box_set_active (_combobox_RACS_N, 0);
  gtk_combo_box_set_active (_combobox_RACS_H, 0);
  gtk_combo_box_set_active (_combobox_RACS_CA, 0);

  _checkbutton_RACS_PCS1 =
    (GtkCheckButton *) glade_xml_get_widget (gxml, "checkbutton_RACS_PCS1");
  _checkbutton_RACS_PCS2 =
    (GtkCheckButton *) glade_xml_get_widget (gxml, "checkbutton_RACS_PCS2");
  _checkbutton_RACS_PCS3 =
    (GtkCheckButton *) glade_xml_get_widget (gxml, "checkbutton_RACS_PCS3");
  _checkbutton_RACS_PCS4 =
    (GtkCheckButton *) glade_xml_get_widget (gxml, "checkbutton_RACS_PCS4");

  _checkbutton_RACS_PCSGEN =
    (GtkCheckButton *) glade_xml_get_widget (gxml2,
					     "checkbutton_RACS_PCSGEN");
}

// to be called when a new pdb is loaded
void
new_pdb_lodaded_for_racs (void)
{

  GtkTreeModel *model;
  model = create_atom_list_model ();
  gtk_combo_box_set_model (_combobox_RACS_O, model);
  gtk_combo_box_set_model (_combobox_RACS_C, model);
  gtk_combo_box_set_model (_combobox_RACS_N, model);
  gtk_combo_box_set_model (_combobox_RACS_H, model);
  gtk_combo_box_set_model (_combobox_RACS_CA, model);
  gtk_combo_box_set_active (_combobox_RACS_O, 0);
  gtk_combo_box_set_active (_combobox_RACS_C, 0);
  gtk_combo_box_set_active (_combobox_RACS_N, 0);
  gtk_combo_box_set_active (_combobox_RACS_H, 0);
  gtk_combo_box_set_active (_combobox_RACS_CA, 0);
  g_object_unref (model);
  invalidate_racs_toggle ();
}

void
invalidate_racs_toggle (void)
{
  gtk_toggle_button_set_active ((GtkToggleButton *) _checkbutton_RACS_PCS1,
				FALSE);
  gtk_toggle_button_set_active ((GtkToggleButton *) _checkbutton_RACS_PCS2,
				FALSE);
  gtk_toggle_button_set_active ((GtkToggleButton *) _checkbutton_RACS_PCS3,
				FALSE);
  gtk_toggle_button_set_active ((GtkToggleButton *) _checkbutton_RACS_PCS4,
				FALSE);
  gtk_toggle_button_set_active ((GtkToggleButton *) _checkbutton_RACS_PCSGEN,
				FALSE);
  _IS_RACS_VALID = FALSE;
}

void
validate_racs_toggle (void)
{
  _IS_RACS_VALID = TRUE;
}

void
toggle_racs (GtkToggleButton * togglebutton)
{
  if (_IS_RACS_VALID == FALSE)
    {
      gtk_toggle_button_set_active (togglebutton, FALSE);
    }
}

void
get_racs_parameters (racs_data * racs_d)
{
  double temp;
  temp = gtk_spin_button_get_value (_spin_B0_RACS);
  B0_from_Hz_2_T (&temp);	//CONVERSION
  racs_d->B0 = temp;
  racs_d->T = gtk_spin_button_get_value (_spin_T_RACS);

  racs_d->sigma_C_11 = gtk_spin_button_get_value (_sigma_C_11);
  racs_d->sigma_C_22 = gtk_spin_button_get_value (_sigma_C_22);
  racs_d->sigma_C_33 = gtk_spin_button_get_value (_sigma_C_33);

  racs_d->sigma_N_11 = gtk_spin_button_get_value (_sigma_N_11);
  racs_d->sigma_N_22 = gtk_spin_button_get_value (_sigma_N_22);
  racs_d->sigma_N_33 = gtk_spin_button_get_value (_sigma_N_33);

  racs_d->sigma_H_11 = gtk_spin_button_get_value (_sigma_H_11);
  racs_d->sigma_H_22 = gtk_spin_button_get_value (_sigma_H_22);
  racs_d->sigma_H_33 = gtk_spin_button_get_value (_sigma_H_33);

  racs_d->beta_C = gtk_spin_button_get_value (_beta_C) * M_PI / 180.0;
  racs_d->beta_N = gtk_spin_button_get_value (_beta_N) * M_PI / 180.0;
  racs_d->beta_H = gtk_spin_button_get_value (_beta_H) * M_PI / 180.0;
}


int
read_racs_atom_name (racs_atom_name * racs_atom_na)
{
  GtkTreeIter iter;
  GtkTreeModel *mod;
  gchar *test;


  gtk_combo_box_get_active_iter (_combobox_RACS_O, &iter);
  mod = gtk_combo_box_get_model (GTK_COMBO_BOX (_combobox_RACS_O));
  gtk_tree_model_get (mod, &iter, COLUMN_ATOM_TEXT, &test, -1);
  sprintf (racs_atom_na->atom_O, test);

  gtk_combo_box_get_active_iter (_combobox_RACS_C, &iter);
  mod = gtk_combo_box_get_model (GTK_COMBO_BOX (_combobox_RACS_C));
  gtk_tree_model_get (mod, &iter, COLUMN_ATOM_TEXT, &test, -1);
  sprintf (racs_atom_na->atom_C, test);

  gtk_combo_box_get_active_iter (_combobox_RACS_N, &iter);
  mod = gtk_combo_box_get_model (GTK_COMBO_BOX (_combobox_RACS_N));
  gtk_tree_model_get (mod, &iter, COLUMN_ATOM_TEXT, &test, -1);
  sprintf (racs_atom_na->atom_N, test);

  gtk_combo_box_get_active_iter (_combobox_RACS_H, &iter);
  mod = gtk_combo_box_get_model (GTK_COMBO_BOX (_combobox_RACS_H));
  gtk_tree_model_get (mod, &iter, COLUMN_ATOM_TEXT, &test, -1);
  sprintf (racs_atom_na->atom_H, test);

  gtk_combo_box_get_active_iter (_combobox_RACS_CA, &iter);
  mod = gtk_combo_box_get_model (GTK_COMBO_BOX (_combobox_RACS_CA));
  gtk_tree_model_get (mod, &iter, COLUMN_ATOM_TEXT, &test, -1);
  sprintf (racs_atom_na->atom_CA, test);

  g_free (test);

  return (1);
}

gboolean
is_use_racs_generate (void)
{
  return (gtk_toggle_button_get_active
	  ((GtkToggleButton *) _checkbutton_RACS_PCSGEN));
}

gboolean
is_use_racs_PCS (int n_button)
{
  switch (n_button)
    {
    case 0:
      {
	return (gtk_toggle_button_get_active
		((GtkToggleButton *) _checkbutton_RACS_PCS1));
      }
      break;
    case 1:
      {
	return (gtk_toggle_button_get_active
		((GtkToggleButton *) _checkbutton_RACS_PCS2));
      }
      break;
    case 2:
      {
	return (gtk_toggle_button_get_active
		((GtkToggleButton *) _checkbutton_RACS_PCS3));
      }
      break;
    case 3:
      {
	return (gtk_toggle_button_get_active
		((GtkToggleButton *) _checkbutton_RACS_PCS4));
      }
      break;
    }
  return (FALSE);
}



void
auto_name (void)
{

  focus_on_name (_combobox_RACS_O, "O", COLUMN_ATOM_TEXT);
  focus_on_name (_combobox_RACS_H, "H", COLUMN_ATOM_TEXT);
	focus_on_name (_combobox_RACS_H, "HN", COLUMN_ATOM_TEXT);
  focus_on_name (_combobox_RACS_N, "N", COLUMN_ATOM_TEXT);
  focus_on_name (_combobox_RACS_C, "C", COLUMN_ATOM_TEXT);
  focus_on_name (_combobox_RACS_CA, "CA", COLUMN_ATOM_TEXT);

}

void
normalize (double vect[3])
{
  double x, y, z, r, r2, one_over_r;
  x = vect[0];
  y = vect[1];
  z = vect[2];
  r2 = x * x + y * y + z * z;
  r = sqrt (r2);
  one_over_r = 1.0 / r;
  vect[0] = vect[0] * one_over_r;
  vect[1] = vect[1] * one_over_r;
  vect[2] = vect[2] * one_over_r;
}

void
cross_product (double vect1[3], double vect2[3], double vect_result[3])
{
  vect_result[0] = vect1[1] * vect2[2] - vect1[2] * vect2[1];
  vect_result[1] = vect1[2] * vect2[0] - vect1[0] * vect2[2];
  vect_result[2] = vect1[0] * vect2[1] - vect1[1] * vect2[0];
}

double
dot_product (double v1[3], double v2[3])
{
  return (v1[0] * v2[0] + v1[1] * v2[1] + v1[2] * v2[2]);
}

//angle given in radian
//From wikipedia maybe it is actually left handed,
//check the http://www.cprogramming.com/tutorial/3d/rotation.html website
void
fil_rot_axis_matrix (double R[3][3], double axe[3], double angle)
{

  double axe2[3];
  double x, y, z;
  double c, s, C, xs, ys, zs, xC, yC, zC, xyC, yzC, zxC;
  axe2[0] = axe[0];
  axe2[1] = axe[1];
  axe2[2] = axe[2];
  normalize (axe2);
  x = axe[0];
  y = axe[1];
  z = axe[2];
  angle = angle;		//*M_PI/180.0;
  c = cos (angle);
  s = sin (angle);
  C = 1 - c;
  xs = x * s;
  ys = y * s;
  zs = z * s;
  xC = x * C;
  yC = y * C;
  zC = z * C;
  xyC = x * yC;
  yzC = y * zC;
  zxC = z * xC;

  if (AXIS_ROT_MATRIX == 1)
    {				//looks left handed
      R[0][0] = x * xC + c;
      R[0][1] = xyC - zs;
      R[0][2] = zxC + ys;
      R[1][0] = xyC + zs;
      R[1][1] = y * yC + c;
      R[1][2] = yzC - xs;
      R[2][0] = zxC - ys;
      R[2][1] = yzC + xs;
      R[2][2] = z * zC + c;
    }
  //DOESN'T MAKE DIFFERENCE ANYWAY
  else
    {				//looks rigth handed
      R[0][0] = x * xC + c;
      R[0][1] = xyC + zs;
      R[0][2] = zxC - ys;
      R[1][0] = xyC - zs;
      R[1][1] = y * yC + c;
      R[1][2] = yzC + xs;
      R[2][0] = zxC + ys;
      R[2][1] = yzC - xs;
      R[2][2] = z * zC + c;
    }
}

/************************************************************/
void
calculate_CSA_matrix_H (double CSA_matrix[3][3],
			double x1, double y1, double z1,
			double x2, double y2, double z2,
			double x3, double y3, double z3, double beta)
{
  double sigma_no_rot[3];
  double temp[3];
  double sigma_perp[3];
  double R_axis[3][3];
  double sigma_rot[3];
  double sigma_last[3];

  sigma_no_rot[0] = x2 - x1;
  sigma_no_rot[1] = y2 - y1;
  sigma_no_rot[2] = z2 - z1;

  normalize (sigma_no_rot);
  temp[0] = x3 - x1;
  temp[1] = y3 - y1;
  temp[2] = z3 - z1;

  cross_product (sigma_no_rot, temp, sigma_perp);
  normalize (sigma_perp);

  sigma_no_rot[0] = -1.0 * sigma_no_rot[0];
  sigma_no_rot[1] = -1.0 * sigma_no_rot[1];
  sigma_no_rot[2] = -1.0 * sigma_no_rot[2];

  beta = -beta;

  fil_rot_axis_matrix (R_axis, sigma_perp, beta);

  sigma_rot[0] =
    sigma_no_rot[0] * R_axis[0][0] + sigma_no_rot[1] * R_axis[0][1] +
    sigma_no_rot[2] * R_axis[0][2];
  sigma_rot[1] =
    sigma_no_rot[0] * R_axis[1][0] + sigma_no_rot[1] * R_axis[1][1] +
    sigma_no_rot[2] * R_axis[1][2];
  sigma_rot[2] =
    sigma_no_rot[0] * R_axis[2][0] + sigma_no_rot[1] * R_axis[2][1] +
    sigma_no_rot[2] * R_axis[2][2];
  normalize (sigma_rot);

  cross_product (sigma_perp, sigma_rot, sigma_last);
  normalize (sigma_last);

  CSA_matrix[0][0] = sigma_perp[0];
  CSA_matrix[0][1] = sigma_perp[1];
  CSA_matrix[0][2] = sigma_perp[2];

  CSA_matrix[1][0] = sigma_last[0];
  CSA_matrix[1][1] = sigma_last[1];
  CSA_matrix[1][2] = sigma_last[2];

  CSA_matrix[2][0] = sigma_rot[0];
  CSA_matrix[2][1] = sigma_rot[1];
  CSA_matrix[2][2] = sigma_rot[2];

}

/************************************************************/
void
calculate_CSA_matrix_C (double CSA_matrix[3][3],
			double x1, double y1, double z1,
			double x2, double y2, double z2,
			double x3, double y3, double z3, double beta)
{
  double sigma_no_rot[3];
  double temp[3];
  double sigma_perp[3];
  double R_axis[3][3];
  double sigma_rot[3];
  double sigma_last[3];

  sigma_no_rot[0] = x2 - x1;
  sigma_no_rot[1] = y2 - y1;
  sigma_no_rot[2] = z2 - z1;

  normalize (sigma_no_rot);
  temp[0] = x3 - x1;
  temp[1] = y3 - y1;
  temp[2] = z3 - z1;

  cross_product (sigma_no_rot, temp, sigma_perp);
  normalize (sigma_perp);

  fil_rot_axis_matrix (R_axis, sigma_perp, beta);

  sigma_rot[0] =
    sigma_no_rot[0] * R_axis[0][0] + sigma_no_rot[1] * R_axis[0][1] +
    sigma_no_rot[2] * R_axis[0][2];
  sigma_rot[1] =
    sigma_no_rot[0] * R_axis[1][0] + sigma_no_rot[1] * R_axis[1][1] +
    sigma_no_rot[2] * R_axis[1][2];
  sigma_rot[2] =
    sigma_no_rot[0] * R_axis[2][0] + sigma_no_rot[1] * R_axis[2][1] +
    sigma_no_rot[2] * R_axis[2][2];
  normalize (sigma_rot);
  cross_product (sigma_perp, sigma_rot, sigma_last);
  normalize (sigma_last);

  CSA_matrix[0][0] = sigma_rot[0];
  CSA_matrix[0][1] = sigma_rot[1];
  CSA_matrix[0][2] = sigma_rot[2];

  CSA_matrix[1][0] = sigma_last[0];
  CSA_matrix[1][1] = sigma_last[1];
  CSA_matrix[1][2] = sigma_last[2];

  CSA_matrix[2][0] = sigma_perp[0];
  CSA_matrix[2][1] = sigma_perp[1];
  CSA_matrix[2][2] = sigma_perp[2];

}

/************************************************************/
void
calculate_CSA_matrix_N_NATURAL (double CSA_matrix[3][3],
				double x1, double y1, double z1,
				double x2, double y2, double z2,
				double x3, double y3, double z3, double beta)
{
  double sigma_no_rot[3];
  double temp[3];
  double sigma_perp[3];
  double R_axis[3][3];
  double sigma_rot[3];
  double sigma_last[3];

  sigma_no_rot[0] = x2 - x1;
  sigma_no_rot[1] = y2 - y1;
  sigma_no_rot[2] = z2 - z1;


  normalize (sigma_no_rot);
  temp[0] = x3 - x1;
  temp[1] = y3 - y1;
  temp[2] = z3 - z1;

  cross_product (sigma_no_rot, temp, sigma_perp);
  normalize (sigma_perp);

  sigma_perp[0] = -1.0 * sigma_perp[0];
  sigma_perp[1] = -1.0 * sigma_perp[1];
  sigma_perp[2] = -1.0 * sigma_perp[2];

  beta = -beta;

  fil_rot_axis_matrix (R_axis, sigma_perp, beta);

  sigma_rot[0] =
    sigma_no_rot[0] * R_axis[0][0] + sigma_no_rot[1] * R_axis[0][1] +
    sigma_no_rot[2] * R_axis[0][2];
  sigma_rot[1] =
    sigma_no_rot[0] * R_axis[1][0] + sigma_no_rot[1] * R_axis[1][1] +
    sigma_no_rot[2] * R_axis[1][2];
  sigma_rot[2] =
    sigma_no_rot[0] * R_axis[2][0] + sigma_no_rot[1] * R_axis[2][1] +
    sigma_no_rot[2] * R_axis[2][2];
  normalize (sigma_rot);

  cross_product (sigma_perp, sigma_rot, sigma_last);
  normalize (sigma_last);

  CSA_matrix[0][0] = sigma_rot[0];
  CSA_matrix[0][1] = sigma_rot[1];
  CSA_matrix[0][2] = sigma_rot[2];

  CSA_matrix[1][0] = sigma_perp[0];
  CSA_matrix[1][1] = sigma_perp[1];
  CSA_matrix[1][2] = sigma_perp[2];

  CSA_matrix[2][0] = -sigma_last[0];
  CSA_matrix[2][1] = -sigma_last[1];
  CSA_matrix[2][2] = -sigma_last[2];

}

/************************************************************/
void
calculate_CSA_matrix_N_OFFSET (double CSA_matrix[3][3],
			       double x1, double y1, double z1,
			       double x2, double y2, double z2,
			       double x3, double y3, double z3, double beta)
{
  double sigma_no_rot[3];
  double temp[3];
  double sigma_perp[3];
  double R_axis[3][3];
  double sigma_rot[3];
  double sigma_last[3];

  sigma_no_rot[0] = x2 - x1;
  sigma_no_rot[1] = y2 - y1;
  sigma_no_rot[2] = z2 - z1;

  normalize (sigma_no_rot);
  temp[0] = x3 - x1;
  temp[1] = y3 - y1;
  temp[2] = z3 - z1;

  cross_product (sigma_no_rot, temp, sigma_perp);
  normalize (sigma_perp);

  beta = -beta;

  fil_rot_axis_matrix (R_axis, sigma_perp, beta);

  sigma_rot[0] =
    sigma_no_rot[0] * R_axis[0][0] + sigma_no_rot[1] * R_axis[0][1] +
    sigma_no_rot[2] * R_axis[0][2];
  sigma_rot[1] =
    sigma_no_rot[0] * R_axis[1][0] + sigma_no_rot[1] * R_axis[1][1] +
    sigma_no_rot[2] * R_axis[1][2];
  sigma_rot[2] =
    sigma_no_rot[0] * R_axis[2][0] + sigma_no_rot[1] * R_axis[2][1] +
    sigma_no_rot[2] * R_axis[2][2];
  normalize (sigma_rot);

  cross_product (sigma_perp, sigma_rot, sigma_last);
  normalize (sigma_last);

  CSA_matrix[0][0] = sigma_rot[0];
  CSA_matrix[0][1] = sigma_rot[1];
  CSA_matrix[0][2] = sigma_rot[2];

  CSA_matrix[1][0] = sigma_perp[0];
  CSA_matrix[1][1] = sigma_perp[1];
  CSA_matrix[1][2] = sigma_perp[2];

  CSA_matrix[2][0] = -sigma_last[0];
  CSA_matrix[2][1] = -sigma_last[1];
  CSA_matrix[2][2] = -sigma_last[2];
}

/************************************************************/
int
calculate_CSA_matrix (double CSA_matrix[3][3],
		      double x1, double y1, double z1,
		      double x2, double y2, double z2,
		      double x3, double y3, double z3,
		      double beta, double angle_offset, int racs_type)
{

  switch (racs_type)
    {
    case RACS_FOR_H:
      {
	calculate_CSA_matrix_H (CSA_matrix,
				x1, y1, z1, x2, y2, z2, x3, y3, z3, beta);
      }
      break;
    case RACS_FOR_C:
      {
	calculate_CSA_matrix_C (CSA_matrix,
				x1, y1, z1, x2, y2, z2, x3, y3, z3, beta);
      }
      break;
    case RACS_FOR_N_NATURAL:
      {
	calculate_CSA_matrix_N_NATURAL (CSA_matrix,
					x1, y1, z1,
					x2, y2, z2, x3, y3, z3, beta);
      }
      break;
    case RACS_FOR_N_OFFSET:
      {
	calculate_CSA_matrix_N_OFFSET (CSA_matrix,
				       x1, y1, z1,
				       x2, y2, z2,
				       x3, y3, z3, beta + angle_offset);
      }
      break;
    default:
      {
	return (-1);
      }
    }
  return (1);
}
