/*
 * racs.h
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

enum{ 
    RACS_FOR_C,
    RACS_FOR_H,
    RACS_FOR_N_NATURAL,
    RACS_FOR_N_OFFSET
};

typedef struct racs_data {
    double B0;
    double T;
    double sigma_C_11;
    double sigma_C_22;
    double sigma_C_33;
    double sigma_N_11;
    double sigma_N_22;
    double sigma_N_33;
    double sigma_H_11;
    double sigma_H_22;
    double sigma_H_33;
    double beta_C; //stored in rad
    double beta_N;
    double beta_H;
} racs_data;

typedef struct racs_atom_name {
    char atom_O[MAX_ATOM_NAME_LENGTH];
    char atom_C[MAX_ATOM_NAME_LENGTH];
    char atom_N[MAX_ATOM_NAME_LENGTH];
    char atom_H[MAX_ATOM_NAME_LENGTH];
    char atom_CA[MAX_ATOM_NAME_LENGTH];
} racs_atom_name;

void cancel_racs(void);
void start_racs(void);
void init_racs(void);
void ok_racs(void);
void new_pdb_lodaded_for_racs(void);
void invalidate_racs_toggle(void);
void validate_racs_toggle(void);
void toggle_racs(GtkToggleButton *togglebutton);
void get_racs_parameters(racs_data * racs_data_values);
int read_racs_atom_name(racs_atom_name * racs_atom_na);
gboolean is_use_racs_generate(void);
gboolean is_use_racs_PCS(int n_button);
void auto_name(void);
double dot_product(double v1[3], double v2[3]);
int calculate_CSA_matrix(double CSA_matrix[3][3],
                    double x1, double y1, double z1, 
                    double x2, double y2, double z2, 
                    double x3, double y3, double z3,
                    double beta, double angle_offset, int racs_type);
