/*
 * rdc.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <math.h>
#include <glib/gstdio.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>

#include "pdb.h"
#include "methyl.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "tensor_list.h"
#include "common.h"
#include "error_analysis.h"
#include "pcsfit_metal.h"
#include "molmol_iso.h"
#include "unify_tensor.h"
#include "rdc.h"

#define PLUS_MINUS 1

extern GtkWidget *_window_rdc;
extern GtkTreeModel * _model_tl;

GtkSpinButton *_spin_S;
GtkSpinButton *_spin_B0;
GtkSpinButton *_spin_T;
GtkSpinButton *_spin_G1;
GtkSpinButton *_spin_G2;
GtkComboBox * _box_atom1;
GtkComboBox * _box_atom2;
GtkComboBox * _box_plus_minus_residue;
GtkComboBox * _box_copy_from_pdb_selector;

enum{
	COLUMN_PLUS_MINUS_STRING,
	COLUMN_PLUS_MINUS_INT,
	NUM_COLUMNS_PLUS_MINUS
};

//to be called at startup
void  init_rdc(void){
	GladeXML *gxml;	
	GtkTreeModel * model;
	GtkTreeView * treeview;
	gxml = glade_get_widget_tree(_window_rdc);
	_spin_S = (GtkSpinButton *) glade_xml_get_widget (gxml, "constante_S");
	_spin_B0 = (GtkSpinButton *) glade_xml_get_widget (gxml, "constante_B0");
	_spin_T = (GtkSpinButton *)	glade_xml_get_widget (gxml, "constante_T");
	_spin_G1 = (GtkSpinButton *) glade_xml_get_widget (gxml, "constante_G1");
	_spin_G2 = (GtkSpinButton *) glade_xml_get_widget (gxml, "constante_G2");
	_box_atom1 = (GtkComboBox *)glade_xml_get_widget (gxml, "combobox_RDC_ATOM1");
	_box_atom2 = (GtkComboBox *)glade_xml_get_widget (gxml, "combobox_RDC_ATOM2");
	_box_plus_minus_residue = (GtkComboBox *)glade_xml_get_widget (gxml, "combobox_RDC_RES");
	_box_copy_from_pdb_selector = (GtkComboBox *)glade_xml_get_widget (gxml, "combobox_copy_from_pdb_selector");
	gtk_spin_button_set_value(_spin_G1, 267.522);
	gtk_spin_button_set_value(_spin_G2, -27.126);
	model = create_plus_minus_model();
	gtk_combo_box_set_model(_box_plus_minus_residue, model);
	gtk_combo_box_set_active(_box_plus_minus_residue, PLUS_MINUS);
	g_object_unref (model);
	
	model = create_copy_from_pdb_selector();
	gtk_combo_box_set_model(_box_copy_from_pdb_selector, model);
	gtk_combo_box_set_active(_box_copy_from_pdb_selector, 0);
	g_object_unref (model);
	
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview6");
    add_columns_tl(treeview, TENSOR_ANY);
    gtk_tree_view_set_model ((GtkTreeView *)treeview, GTK_TREE_MODEL (_model_tl));
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
	set_multiple_selection(GTK_TREE_VIEW (treeview));
	
}


GtkTreeModel *
create_copy_from_pdb_selector (void)
{
  gint i = 0;
  GtkListStore *model;
  GtkTreeIter iter;

  model = gtk_list_store_new (NUM_COLUMNS_PLUS_MINUS, G_TYPE_STRING,
							  G_TYPE_INT);

    for (i = 0; i < N_PCS_ARRAY; i++)
    {
      char str_from[6];
      sprintf(str_from, "PCS%d", i + 1);
      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter,
                          COLUMN_PLUS_MINUS_STRING, str_from,
						  COLUMN_PLUS_MINUS_INT, i+1,
                          -1);
    }
  return GTK_TREE_MODEL (model);
}

// to be called when a new pdb is loaded
void new_pdb_lodaded_for_rdc(void){
	GtkTreeModel * model;
	model = create_atom_list_model ();
	gtk_combo_box_set_model(_box_atom1, model);
	gtk_combo_box_set_model(_box_atom2, model);
    focus_on_name(_box_atom1, "HN", COLUMN_ATOM_TEXT);
    focus_on_name(_box_atom1, "H", COLUMN_ATOM_TEXT);
    focus_on_name(_box_atom2, "N", COLUMN_ATOM_TEXT);
	g_object_unref (model);
}

GtkTreeModel * create_plus_minus_model(void){
  gint i = 0;
  GtkListStore *model;
  GtkTreeIter iter;
  model = gtk_list_store_new (NUM_COLUMNS_PLUS_MINUS, 
							  G_TYPE_STRING,
							  G_TYPE_INT
							  );

    for (i = 0; i < 2*PLUS_MINUS + 1; i++)
    {
		char txt[5];
		sprintf(txt, "%+d", i - PLUS_MINUS);
		
      	gtk_list_store_append (model, &iter);
      	gtk_list_store_set (model, &iter,
						  COLUMN_PLUS_MINUS_STRING, txt,
						  COLUMN_PLUS_MINUS_INT, i-PLUS_MINUS,
                          -1);
    }
  return GTK_TREE_MODEL (model);
}

//new tensor to add in the common tensor list
void add_manual_rdc(void){
    GladeXML *gxml;
    double x, y, z, Ax, Rh, a, b, g;
    
    gxml = glade_get_widget_tree(_window_rdc);

    x = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_x4"));
    y = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_y4"));
    z = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_z4"));
    a = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_alpha4"));
    b = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_beta4"));
    g = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_gamma4"));
    Ax = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Ax4"));
    Rh = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Rh4"));
	
    add_to_tensor_listing(Ax, Rh, x, y, z, a, b, g, MANUAL_TENSOR, -1,
						   0, 0, 0, 0, 0, 0, 0, 0);
}

//cancel button
void cancel_rdc(void){
    gtk_widget_hide(_window_rdc);
}

//copy tensor form the common tensor list
void copy_rdc(void){
	
  GladeXML *gxml;  
  GtkTreeView *treeview;

  gxml = glade_get_widget_tree(_window_rdc);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview6"); 
  copy_tensor_from_list(gxml, treeview, 4);	

}

//remove tensor from the common tensor list
void remove_rdc(void){
	
	GtkTreeView *treeview;
	GladeXML *gxml; 
 
  gxml = glade_get_widget_tree(_window_rdc);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview6");  
  remove_tensor_from_list(gxml, treeview, 4);	
	
}

//action: we generate and print rdc vals
void save_rdc(GtkButton * button){

	GtkWidget *dialog;
  	GtkFileFilter *filter1;
		GtkFileFilter *filter3;
  	GtkFileFilter *filter4;
  	gint result;
		int format;

	//char name_file[500];
	

	rdc_data r_data;
	tensor my_tensor;
	int resu1, resu2;
	
	resu1 = read_rdc_data(&r_data);
	if(resu1 < 0){
		box_error("You should use at least one pdb model\n", NULL);
		return;
	}
	
	resu2 = read_tensor_only_from_tl(_model_tl, &my_tensor);
	if(resu2 < 0){
		return;
	}
	
	  dialog =
    gtk_file_chooser_dialog_new
    ("In which file do you want to save your RDC values?", NULL,
     GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
		
	gtk_window_set_icon_from_file ((GtkWindow *)dialog, NUMBAT_ICON_FILE, NULL);
  gtk_window_set_transient_for((GtkWindow*) dialog, (GtkWindow*)_window_rdc);	

  filter1 = gtk_file_filter_new ();
  filter4 = gtk_file_filter_new ();
	filter3 = gtk_file_filter_new ();
		
	gtk_file_filter_add_pattern (filter3, "*.tbl");
  gtk_file_filter_set_name (filter3, "*.tbl Xplor-NIH format for RDC");
  gtk_file_filter_add_pattern (filter1, "*.tab");
  gtk_file_filter_set_name (filter1, "*.tab PALES-like format");
  gtk_file_filter_add_pattern (filter4, "*.*");
  gtk_file_filter_set_name (filter4, "*.* ALL");
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter1);
	gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter3);
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter4);

  result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
    {
    case GTK_RESPONSE_ACCEPT:
      {
	char *filename;
	FILE * file;
	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
	file = g_fopen(filename, "w");							
	if (file == NULL)
    {
      box_error ("Error, can't open the file %s\nno rdc saved", filename);
			g_free (filename);
      return;
  }
	format = get_format(filename);	
	switch(format){
        case FORMAT_XPLOR:{
            print_rdc_xplor(&r_data, &my_tensor, file);
						
        }
        break;
        case FORMAT_PALES:{
						print_rdc_pales(&r_data, &my_tensor, file);
        }
        break;
				default: {
						box_error("Warning, format of file %s \nurecognized\n", filename);
						g_free (filename);
						return;
				}
	}
	
	

	g_free (filename);  
	gtk_widget_hide(_window_rdc);
	fclose(file);	  
	  }
      break;
    default:
      break;
    }
  gtk_object_destroy((GtkObject *)filter1);
  gtk_object_destroy((GtkObject *)filter4);
  gtk_widget_destroy (dialog); 
	
    return;
}



void start_rdc(void){
    if(_pdb_str == NULL){
        box_error( "You should first load a pdb file.\n", NULL);
		return;
    }
	uncheck_all((GtkTreeModel *) _model_tl, COLUMN_USED_TL);
    gtk_widget_show(_window_rdc);
}

int read_rdc_data(rdc_data * r_data){
	GtkTreeIter iter;
	GtkTreeModel * mod;
	int plus_minus;
	int my_n_button;
	gchar * test;
	int i, n_mod;
    double temp;
	
	r_data->S = gtk_spin_button_get_value(_spin_S); 
	temp = gtk_spin_button_get_value(_spin_B0); 
    B0_from_Hz_2_T(&temp); //CONVERSION
    r_data->B0 = temp;
	r_data->g1 = gtk_spin_button_get_value(_spin_G1)*pow(10.0, 6); 
	r_data->g2 = gtk_spin_button_get_value(_spin_G2)*pow(10.0, 6);
	r_data->T = gtk_spin_button_get_value(_spin_T);
	
	gtk_combo_box_get_active_iter(_box_plus_minus_residue, &iter);
	mod = gtk_combo_box_get_model(GTK_COMBO_BOX(_box_plus_minus_residue));
	gtk_tree_model_get(mod, &iter, COLUMN_PLUS_MINUS_INT, &plus_minus, -1);
	r_data->res_next = plus_minus;
	
	gtk_combo_box_get_active_iter(_box_copy_from_pdb_selector, &iter);
	mod = gtk_combo_box_get_model(GTK_COMBO_BOX(_box_copy_from_pdb_selector));
	gtk_tree_model_get(mod, &iter, COLUMN_PLUS_MINUS_INT, &my_n_button, -1);
	my_n_button--;
	r_data->n_button = my_n_button;
	
	gtk_combo_box_get_active_iter(_box_atom1, &iter);
	mod = gtk_combo_box_get_model(GTK_COMBO_BOX(_box_atom1));
	gtk_tree_model_get(mod, &iter, COLUMN_ATOM_TEXT, &test, -1);
	sprintf(r_data->atom1, test);
	
	gtk_combo_box_get_active_iter(_box_atom2, &iter);
	mod = gtk_combo_box_get_model(GTK_COMBO_BOX(_box_atom2));
	gtk_tree_model_get(mod, &iter, COLUMN_ATOM_TEXT, &test, -1);
	sprintf(r_data->atom2, test);
	
	n_mod = 0;
	for(i = 0; i < _all_pdb_selector_pcs[my_n_button]->len; i++){
        if(g_array_index (_all_pdb_selector_pcs[my_n_button], pdb_selector , i).is_used == TRUE){
            n_mod = 1;
			break;
        }
	}
	if(n_mod == 0){
		return(-1);
	}
	return(1);
}

void print_rdc_pales(rdc_data * r_data, tensor * my_tensor, FILE * f){
	int j;
	int index_atom1, index_atom2;
	double rdc;
	int residue_num;
    for(j = 0; j < _pdb_str->n_atom; j++){
        if(strcmp(_pdb_str->tab_atom[j].atom_name, r_data->atom1) != 0){
            continue;
        }
        residue_num = _pdb_str->tab_atom[j].residue_num;
        index_atom1 = j;
        index_atom2 = find_pcs_in_pdb(residue_num + r_data->res_next, r_data->atom2);
		if(index_atom2 < 0){
            fprintf(stderr, "Couldn't find atom2 %s (%d)\n", r_data->atom2, residue_num + r_data->res_next);
			//continue;
		}
        if((index_atom1 < 0) || (index_atom2 < 0)){
            continue;
        }
		rdc = calculate_rdc(index_atom1, index_atom2, my_tensor, r_data);
		fprintf(f, "%5d %6s %6s %5d %6s %6s %9.3f %9.3f %.2f\n",
				residue_num, 
				_pdb_str->tab_atom[index_atom1].residue_name,
				r_data->atom1,
				residue_num + r_data->res_next,
				_pdb_str->tab_atom[index_atom2].residue_name,
				r_data->atom2,
				rdc,
				1.0,
				1.0);
	}
}

void print_rdc_xplor(rdc_data * r_data, tensor * my_tensor, FILE * f){
	int j;
	int index_atom1, index_atom2;
	double rdc;
	int residue_num;
		
  for(j = 0; j < _pdb_str->n_atom; j++){      
		if(strcmp(_pdb_str->tab_atom[j].atom_name, r_data->atom1) != 0){
     	continue;
    }
    residue_num = _pdb_str->tab_atom[j].residue_num;
    index_atom1 = j;
    index_atom2 = find_pcs_in_pdb(residue_num + r_data->res_next, r_data->atom2);

		if(index_atom2 < 0){
      fprintf(stderr, "Couldn't find atom2 %s (%d)\n", r_data->atom2, residue_num + r_data->res_next);
		}
    if((index_atom1 < 0) || (index_atom2 < 0)){
    	continue;
    }
		rdc = calculate_rdc(index_atom1, index_atom2, my_tensor, r_data);
		fprintf( f, "\
assign ( resid 500  and name OO )\n\
       ( resid 500  and name Z )\n\
       ( resid 500  and name X )\n\
       ( resid 500  and name Y )\n\
			 ( resid %3d  and name %2s )\n\
       ( resid %3d  and name %2s )   %.5lf  %.5lf\n", 
						residue_num,
						r_data->atom1,
						residue_num,
						r_data->atom2,
						rdc,
						0.0);
	}
}

/*----------------------------------------------------------------------*/
double calculate_rdc(int atom1, int atom2, tensor * my_tensor, rdc_data * r_data)
/*----------------------------------------------------------------------*/
{
    double  rdc;
    double x1, y1, z1;
	double x2, y2, z2;
	double a, b, g, Ax, Rh;
    int i;
    double rdc_total;
    int compteur;
	int n_button;
	double S, g1, g2, B0, T;
	double k_boltz, h_dirac;
	
	S = r_data->S;
	g1 = r_data->g1;
	g2 = r_data->g2;
	B0 = r_data->B0;
	T = r_data->T;
	n_button = r_data->n_button;
	
	h_dirac = CONSTANTE_h_dirac;
	k_boltz = CONSTANTE_k_BOLTZ;
	
    compteur = 0;
    rdc_total = 0;
	
	a = my_tensor->a;
	b = my_tensor->b;
	g = my_tensor->g;
	Ax = my_tensor->ax;
	Rh = my_tensor->rh;
	
    for(i = 0; i < _all_pdb_selector_pcs[n_button]->len; i++){
        if(g_array_index (_all_pdb_selector_pcs[n_button], pdb_selector , i).is_used == FALSE){
            continue;
        }
		
        x1 = _pdb_str->tab_atom[atom1].atom_coo[i].x;
        y1 = _pdb_str->tab_atom[atom1].atom_coo[i].y;
        z1 = _pdb_str->tab_atom[atom1].atom_coo[i].z;
		
		x2 = _pdb_str->tab_atom[atom2].atom_coo[i].x;
        y2 = _pdb_str->tab_atom[atom2].atom_coo[i].y;
        z2 = _pdb_str->tab_atom[atom2].atom_coo[i].z;
		
		rdc =  SingleRDC(a, b, g, x1, y1, z1, x2, y2, z2, Ax, Rh);
		rdc *= -S*g1*g2*h_dirac*B0*B0 / (8*15*M_PI*M_PI*k_boltz*T);
        rdc *= 0.01; //because Ax and r are not in
        //USI but in 10-32 m3 and 10-10m
		
        rdc_total += rdc;
        compteur = compteur + 1;
    }
    return (rdc_total/compteur);
}

double SingleRDC(double phi, double theta, double psi, double x1, double y1, double z1, double x2, double y2, double z2, double Xax, double Xrh)
{
    double  x_0, y_0, z_0, x_1, y_1, z_1, r2, r, r5;
    double  tmp, rdc;
    double R[3][3];
    
    filZX_or_YZ(R, phi*180/M_PI, theta*180/M_PI, psi*180/M_PI);
    
    x_0 = x2 - x1;
    y_0 = y2 - y1;
    z_0 = z2 - z1;

    x_1 = x_0*R[0][0] + y_0*R[0][1] + z_0*R[0][2];
    y_1 = x_0*R[1][0] + y_0*R[1][1] + z_0*R[1][2];
    z_1 = x_0*R[2][0] + y_0*R[2][1] + z_0*R[2][2];
    
    
    r2 = x_1*x_1 + y_1*y_1 + z_1*z_1;
    r = sqrt(r2);
    r5 = r2*r2*r;
    tmp = 1.0/r5;
    rdc = tmp*(Xax * (3.0*z_1*z_1 - r2) + Xrh * 1.5*(x_1*x_1-y_1*y_1));
    
    return rdc;
}
