/*
 * save_tensor.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <math.h>
#include <glib/gstdio.h>

#include "pdb.h"
#include "methyl.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "tensor_list.h"
#include "common.h"
#include "error_analysis.h"
#include "molmol_iso.h"
#include "unify_tensor.h"


extern GtkWidget *_window_save_tensor;
extern GtkTreeModel * _model_tl;

void init_save_tensor(void){
    GladeXML *gxml;
    gxml = glade_get_widget_tree(_window_save_tensor);
    GtkTreeView * treeview;
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview14");
    add_columns_tl(treeview, TENSOR_ANY_MULTI);
    gtk_tree_view_set_model ((GtkTreeView *)treeview, GTK_TREE_MODEL (_model_tl));
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
		set_multiple_selection(GTK_TREE_VIEW (treeview));
    gtk_widget_hide(_window_save_tensor);
}

void start_save_tensor(void){
    uncheck_all((GtkTreeModel *) _model_tl, COLUMN_USED_TL);
    gtk_widget_show(_window_save_tensor);
}

void cancel_save_tensor(void){
    gtk_widget_hide(_window_save_tensor);
}

void copy_save_tensor(void){
	  GladeXML *gxml;  
	  GtkTreeView *treeview;

  gxml = glade_get_widget_tree(_window_save_tensor);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview14"); 
  copy_tensor_from_list(gxml, treeview, 7);	

}

void remove_save_tensor(void){
	
	GtkTreeView *treeview;
	GladeXML *gxml; 
 
  gxml = glade_get_widget_tree(_window_save_tensor);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview14");  
  remove_tensor_from_list(gxml, treeview, 7);	
	
}

void add_manual_save_tensor(void){
    GladeXML *gxml;
    double x, y, z, Ax, Rh, a, b, g;
    
    gxml = glade_get_widget_tree(_window_save_tensor);

    x = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_x8"));
    y = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_y8"));
    z = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_z8"));
    a = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_alpha8"));
    b = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_beta8"));
    g = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_gamma8"));
    Ax = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Ax8"));
    Rh = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Rh8"));
	
    add_to_tensor_listing(Ax, Rh, x, y, z, a, b, g, MANUAL_TENSOR, -1,
						   0, 0, 0, 0, 0, 0, 0, 0);
}


int save_in_file(char* title, char* filter, char* filter_description, char *file){
    GtkWidget *dialog;
  	GtkFileFilter *filter1;
  	GtkFileFilter *filter4;
  	gint result;
    int to_return;

  dialog =
    gtk_file_chooser_dialog_new
    (title, NULL,
     GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
		
	gtk_window_set_icon_from_file ((GtkWindow *)dialog, NUMBAT_ICON_FILE, NULL);
  gtk_window_set_transient_for((GtkWindow*) dialog, (GtkWindow*)_window_save_tensor);

  filter1 = gtk_file_filter_new ();
  filter4 = gtk_file_filter_new ();
  gtk_file_filter_add_pattern (filter1, filter);
  gtk_file_filter_set_name (filter1, filter_description);
  gtk_file_filter_add_pattern (filter4, "*.*");
  gtk_file_filter_set_name (filter4, "*.* ALL");
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter1);
  gtk_file_chooser_add_filter ((GtkFileChooser *) dialog, filter4);

  result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
    {
    case GTK_RESPONSE_ACCEPT:
      {
        char * filename;  
        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
        sprintf(file, filename);
        to_return = 1;
        g_free (filename);
	  }
      break;
    default: to_return = -1;
      break;
    }
    
    gtk_object_destroy((GtkObject *)filter1);
    gtk_object_destroy((GtkObject *)filter4);
    gtk_widget_destroy (dialog); 
    return(to_return);
}


void write_sanson_coord_bis(FILE * F, double *angle[3], int number_of_tensor){
    
    int j;
    double coordinate[6];
    fprintf(stderr, "number_of_tensor %d\n", number_of_tensor);
    for(j = 0; j< number_of_tensor; j++){
        euler_2_sanson(angle[0][j], angle[1][j], angle[2][j],
                        coordinate, 1);
        fprintf(F, "% 9.3lf % 9.3lf % 9.3lf % 9.3lf % 9.3lf % 9.3lf\n",
                    coordinate[0], coordinate[1], coordinate[2],
                    coordinate[3], coordinate[4], coordinate[5]);
    }
    fprintf(F, "\n");
    for(j = 0; j< number_of_tensor; j++){
        euler_2_sanson(angle[0][j], angle[1][j], angle[2][j],
                        coordinate, -1);
        fprintf(F, "% 9.3lf % 9.3lf % 9.3lf % 9.3lf % 9.3lf % 9.3lf\n",
                    coordinate[0], coordinate[1], coordinate[2],
                    coordinate[3], coordinate[4], coordinate[5]);
    }
    fprintf(F, "\n");
}





int get_UTR_euler_list_from_treeview_selected(double *angle[3], int *number_of_tensor)
{
    
    //////
    GtkTreeIter iter;
    gboolean is_toggle;
	  gboolean my_bool;
    double a, b, g;
		double Xax, Xrh;
    int n_tensor;
    int i;
    n_tensor = 0;
    
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)_model_tl, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (_model_tl, &iter, COLUMN_USED_TL, &is_toggle, -1); 
        if (is_toggle == TRUE){
            n_tensor++;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)_model_tl, &iter);
    }
    
    if(n_tensor == 0){
        box_error("error, at least one tensor should be selected\n", NULL);
        return (-1);
    }
    
    *number_of_tensor = n_tensor;
    angle[0] = calloc(*number_of_tensor, sizeof(double));
    angle[1] = calloc(*number_of_tensor, sizeof(double));
    angle[2] = calloc(*number_of_tensor, sizeof(double));
    
    i = 0;
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)_model_tl, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (_model_tl, &iter, COLUMN_USED_TL, &is_toggle, -1); 
        if (is_toggle == TRUE){
						molmol_data mm_data;
            gtk_tree_model_get(_model_tl, &iter, COLUMN_ALPHA_TL, &(a), -1);
            gtk_tree_model_get(_model_tl, &iter, COLUMN_BETA_TL, &(b), -1);
            gtk_tree_model_get(_model_tl, &iter, COLUMN_GAMMA_TL, &(g), -1);
						gtk_tree_model_get(_model_tl, &iter, COLUMN_AX_TL, &(Xax), -1);
            gtk_tree_model_get(_model_tl, &iter, COLUMN_RH_TL, &(Xrh), -1);
						
						//convertion UTR
						mm_data.Ax = Xax;
						mm_data.Rh = Xrh;
						mm_data.a = a;
						mm_data.b = b;
						mm_data.g = g;
						unique_tensor(&mm_data);
						//convertion UTR
						
            angle[0][i] = mm_data.a; //a;//180*M_PI;
            angle[1][i] = mm_data.b; //b;//180*M_PI;
            angle[2][i] = mm_data.g; //g;//180*M_PI;
            i++;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)_model_tl, &iter);
    }
    
    return(1);
}

void save_for_sanson(void){
    char file[1500];
    FILE * F_sanson_plot;
    FILE * F_sanson_mac;
    FILE * F_world_plot;
    int result;
    double *angle[3];
    int number_of_tensor;
    
    angle[0] = angle[1] = angle[2] = NULL;
    result = save_in_file("In which file do you want to save your gnuplot macro?", "*.plt", "*.plt macro for gnuplot", file);

    if(result < 0){
        return;
    }
    number_of_tensor = 0;
    result = get_UTR_euler_list_from_treeview_selected(angle, &number_of_tensor);
    if(result < 0){
        return;
    }
    
    char name_F_sanson_plot[1500];
    char name_F_sanson_mac[1500];
    char name_F_world_plot[1500];
    
    sprintf(name_F_sanson_mac, file);
    sprintf(name_F_sanson_plot, "%s%s", file, ".txt");
    sprintf(name_F_world_plot,  "%s%s", file, ".world.txt");
        
    if (!(F_sanson_plot = g_fopen(name_F_sanson_plot , "w"))) {
        fprintf(stderr, "can't open %s\n", name_F_sanson_plot);
        return;
    }
    
    if (!(F_sanson_mac = g_fopen(name_F_sanson_mac , "w"))) {
        fprintf(stderr, "can't open %s\n", name_F_sanson_mac);
        fclose(F_sanson_plot);
        return;
    }
    
    if (!(F_world_plot = g_fopen(name_F_world_plot , "w"))) {
        fprintf(stderr, "can't open %s\n", name_F_world_plot);
        fclose(F_sanson_plot);
        fclose(F_sanson_mac);
        return;
    }
    
    write_sanson_coord_bis(F_sanson_plot, angle, number_of_tensor);
    write_sanson_mac(F_sanson_mac, name_F_sanson_plot, name_F_world_plot, 1);
    write_sanson_world(F_world_plot);
    
    if(angle[0] != NULL){
        g_free(angle[0]);
    }
    if(angle[1] != NULL){
        g_free(angle[1]);
    }
    if(angle[2] != NULL){
        g_free(angle[2]);
    }
    
    fclose(F_sanson_plot);
    fclose(F_sanson_mac);
    fclose(F_world_plot);
    gtk_widget_hide(_window_save_tensor);
}

int get_UTR_tensor_list_from_treeview_selected(tensor **result_tensors, int *number_of_tensor)
{
    GtkTreeIter iter;
    gboolean is_toggle;
	  gboolean my_bool;
    double a, b, g, x, y, z;
		double Xax, Xrh;
    int n_tensor;
    int i;
    n_tensor = 0;
		tensor * result_tensors_bis;
    result_tensors_bis = NULL;
		
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)_model_tl, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (_model_tl, &iter, COLUMN_USED_TL, &is_toggle, -1); 
        if (is_toggle == TRUE){
            n_tensor++;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)_model_tl, &iter);
    }
    
    if(n_tensor == 0){
        box_error("error, at least one tensor should be selected\n", NULL);
        return (-1);
    }
    
    *number_of_tensor = n_tensor;
    //*result_tensors = (tensor *) calloc(*number_of_tensor, sizeof(tensor));
    result_tensors_bis = (tensor *) calloc(*number_of_tensor, sizeof(tensor));
		
    i = 0;
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)_model_tl, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (_model_tl, &iter, COLUMN_USED_TL, &is_toggle, -1); 
        if (is_toggle == TRUE){
						molmol_data mm_data;
            gtk_tree_model_get(_model_tl, &iter, COLUMN_ALPHA_TL, &(a), -1);
            gtk_tree_model_get(_model_tl, &iter, COLUMN_BETA_TL, &(b), -1);
            gtk_tree_model_get(_model_tl, &iter, COLUMN_GAMMA_TL, &(g), -1);
						gtk_tree_model_get(_model_tl, &iter, COLUMN_AX_TL, &(Xax), -1);
            gtk_tree_model_get(_model_tl, &iter, COLUMN_RH_TL, &(Xrh), -1);
						gtk_tree_model_get(_model_tl, &iter, COLUMN_X_TL, &(x), -1);
						gtk_tree_model_get(_model_tl, &iter, COLUMN_Y_TL, &(y), -1);
						gtk_tree_model_get(_model_tl, &iter, COLUMN_Z_TL, &(z), -1);
						//convertion UTR
						mm_data.Ax = Xax;
						mm_data.Rh = Xrh;
						mm_data.a = a;
						mm_data.b = b;
						mm_data.g = g;
						unique_tensor(&mm_data);
						//convertion UTR
						
						(result_tensors_bis[i]).a = mm_data.a; 
            (result_tensors_bis[i]).b = mm_data.b;
            (result_tensors_bis[i]).g = mm_data.g;
						(result_tensors_bis[i]).ax = mm_data.Ax;
            (result_tensors_bis[i]).rh = mm_data.Rh;
						(result_tensors_bis[i]).x = x; 
            (result_tensors_bis[i]).y = y;
            (result_tensors_bis[i]).z = z;
            i++;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)_model_tl, &iter);
    }
		*result_tensors = result_tensors_bis;
    
    return(1);
}

void write_tensor_as_pdb(FILE * F_PDB, tensor * result_tensors, int * number_of_tensor){
		int i, j;
		double xa, ya, za, xM, yM, zM;
		double R[3][3];
		
		xa = TENSOR_LENGTH;
  	ya = TENSOR_LENGTH;
  	za = TENSOR_LENGTH;
		int offset_residue = 9000;
		int offset_atom = 90000;
		j = offset_atom;
		
		for (i = 0; i < *number_of_tensor; i++){
				filZX_or_YZ(R, (result_tensors[i]).a, (result_tensors[i]).b , (result_tensors[i]).g);
				
				
				xM = (result_tensors[i]).x;
				yM = (result_tensors[i]).y;
				zM = (result_tensors[i]).z;
					
				fprintf(F_PDB, "ATOM  %5d  OO  TSR  %4d%12.3f%8.3f%8.3f  1.00  0.00\n", j, i + offset_residue,
								xM, yM, zM);
				j++;
  			fprintf(F_PDB, "ATOM  %5d  OX  TSR  %4d%12.3f%8.3f%8.3f  1.00  0.00\n", j, i + offset_residue,
								xM+xa*R[0][0], yM+xa*R[0][1], zM+xa*R[0][2]);
				j++;
  			fprintf(F_PDB, "ATOM  %5d  OY  TSR  %4d%12.3f%8.3f%8.3f  1.00  0.00\n", j, i + offset_residue,
								xM+ya*R[1][0], yM+ya*R[1][1], zM+ya*R[1][2]);
				j++;
  			fprintf(F_PDB, "ATOM  %5d  OZ  TSR  %4d%12.3f%8.3f%8.3f  1.00  0.00\n", j, i + offset_residue,
								xM+za*R[2][0], yM+za*R[2][1], zM+za*R[2][2]);
				j++;
				fprintf(F_PDB, "TER\n");
		}
}

void write_tensor_as_txt(FILE * F_PDB, tensor * result_tensors, int * number_of_tensor){
		int i;
		fprintf(F_PDB, "#Axial Rhombic x y z alpha beta gamma\n");
		fprintf(F_PDB, "#10^-32m^3  10^-32m^3 10^-32m^3 Angstrom Angstrom Angstrom degree degree degree\n");
		for (i = 0; i < *number_of_tensor; i++){
				fprintf(stderr, "tensor saved: %d\n", i);
				fprintf(stderr, "ax: %lf\n", (result_tensors[i]).ax);
				fprintf(F_PDB, "% 15.3lf % 15.3lf % 15.3lf % 15.3lf % 15.3lf % 10.3lf % 10.3lf % 10.3lf\n",
								(result_tensors[i]).ax, (result_tensors[i]).rh,
								(result_tensors[i]).x, (result_tensors[i]).y, (result_tensors[i]).z,
								(result_tensors[i]).a, (result_tensors[i]).b, (result_tensors[i]).g);
		}
}

void save_for_pdb(void)
{
		char file[1500];
    FILE * F_PDB;
    tensor * result_tensors;
    int number_of_tensor;
		int result;
    
		result_tensors = NULL;

    result = save_in_file("In which PDB file do you want to save the list of tensors?", "*.pdb", "*.pdb", file);
    if(result < 0){
        return;
    }
		number_of_tensor = 0;
		result = get_UTR_tensor_list_from_treeview_selected(&result_tensors, &number_of_tensor);
    if(result < 0){
        return;
    }
		
		if (!(F_PDB = g_fopen(file , "w"))) {
        fprintf(stderr, "can't open %s\n", file);
        return;
    }
		
		write_tensor_as_pdb(F_PDB, result_tensors, &number_of_tensor);
		
		if(result_tensors != NULL){
				g_free(result_tensors);
		}
		fclose(F_PDB);
    gtk_widget_hide(_window_save_tensor);
}

void save_for_txt(void)
{
		char file[1500];
    FILE * F_TXT;
    tensor * result_tensors;
    int number_of_tensor;
		int result;
    
		result_tensors = NULL;

    result = save_in_file("In which txt file do you want to save the list of tensors?", "*.txt", "*.txt", file);
    if(result < 0){
        return;
    }
		number_of_tensor = 0;
		result = get_UTR_tensor_list_from_treeview_selected(&result_tensors, &number_of_tensor);
    if(result < 0){
        return;
    }
		if (!(F_TXT = g_fopen(file , "w"))) {
        fprintf(stderr, "can't open %s\n", file);
        return;
    }
		
		write_tensor_as_txt(F_TXT, result_tensors, &number_of_tensor);
		
		if(result_tensors != NULL){
				g_free(result_tensors);
		}
		fclose(F_TXT);
    gtk_widget_hide(_window_save_tensor);
}
