/*
 * select_display.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include "pdb.h"
#include "methyl.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "error_analysis.h"
#include "select_display.h"
#include "common.h"

extern GtkWidget *_window_select;
extern GtkWidget *_window_display;

char HELP_DISPLAY[] = "display";
char HELP_SELECT[] = "select";

enum
{
  COLUMN_NUMBER_TEXT,
  NUM_NUMBER_COLUMNS
};

void prepare_copy_from(GtkWidget * win, char * label, int n_button){
    
    GladeXML *gxml;
    GtkWidget *copy_from_box;
    GtkTreeModel *copy_from_model;
    
    gxml = glade_get_widget_tree((GtkWidget *) win);

    /*combo box */
    copy_from_model = create_copy_from_model (n_button); //code in methyl.c
    copy_from_box = glade_xml_get_widget (gxml, label);
    gtk_combo_box_set_model((GtkComboBox *) copy_from_box, copy_from_model);
    
    g_object_unref (copy_from_model);
}

void prepare_label(GtkWidget * win, int n_button, char * label_name)
{
    char a[5];
    GtkLabel * label;
    GladeXML *gxml;
    
    gxml = glade_get_widget_tree((GtkWidget *) win);
    sprintf(a, "%d", n_button);
    label = (GtkLabel *) glade_xml_get_widget (gxml, label_name);
    gtk_label_set_text (label, a);
}

//to be called only at initialisation.
void init_select_display(void){
    int i;
    int j;
    for(i = 0; i < N_PCS_ARRAY; i++){
        for(j = 0; j < NUM_BOOLEAN_TAB; j++){
        _select_display_pcs[i][j] = NULL; 
        }
    }
}


//to be called when a new pdb is loaded.
void free_array_select_display(void){
    int i;
    int j;
    for(i = 0; i < N_PCS_ARRAY; i++){
        for(j = 0; j < NUM_BOOLEAN_TAB; j++){
            if(_select_display_pcs[i][j] != NULL){
                free(_select_display_pcs[i][j]);
                _select_display_pcs[i][j] = NULL;
            }
        }
    }
}

//to be called when a new pdb is loaded.
void create_array_select_display(void){
    int i;
    int j;
    free_array_select_display();
    for(i = 0; i < N_PCS_ARRAY; i++){
        _select_display_pcs[i][DISPLAY_RESIDUE] = 
            calloc(_pdb_str->n_residue_type, sizeof(gboolean));
        _select_display_pcs[i][SELECT_RESIDUE] = 
            calloc(_pdb_str->n_residue_type, sizeof(gboolean));
        for(j = 0; j < _pdb_str->n_residue_type; j++){
            _select_display_pcs[i][SELECT_RESIDUE][j] = TRUE;
            _select_display_pcs[i][DISPLAY_RESIDUE][j] = TRUE;
        }
        _select_display_pcs[i][DISPLAY_ATOM] = 
            calloc(_pdb_str->n_atom_type, sizeof(gboolean));
        _select_display_pcs[i][SELECT_ATOM] = 
            calloc(_pdb_str->n_atom_type, sizeof(gboolean));
                for(j = 0; j < _pdb_str->n_atom_type; j++){
            _select_display_pcs[i][SELECT_ATOM][j] = TRUE;
            _select_display_pcs[i][DISPLAY_ATOM][j] = TRUE;
        }
    }
}


//~ Function called by callback when one want to select
//~ with the select button
void
run_select_PCS(int n_button){
    
    if(_pdb_str == NULL){
        box_error( "You should first load a pdb file.\n", NULL);
        return;
    }
    prepare_copy_from(_window_select, "copy_from_select", n_button);
    prepare_label(_window_select, n_button, "label_flag_select");
    populate_select_display(_window_select, n_button, SELECT);
    gtk_widget_show(_window_select);
}

void
run_display_PCS(int n_button){
    
    if(_pdb_str == NULL){
        box_error( "You should first load a pdb file.\n", NULL);
        return;
    }
    prepare_copy_from(_window_display, "copy_from_display", n_button);
    prepare_label(_window_display, n_button, "label_flag_display");
    populate_select_display(_window_display, n_button, DISPLAY);
    gtk_widget_show(_window_display);
}

//~ what happens when someone click on a toggle of one of the treeview?
//~ we update the treeview.
void select_display_toggled_n (GtkCellRendererToggle *cell, gchar *path_str,gpointer data, int n_button, int type_select_display){

  GtkTreeModel *model = (GtkTreeModel *)data;
  GtkTreeIter  iter;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
  gboolean used;
    
  gtk_tree_model_get_iter (model, &iter, path);
  gtk_tree_model_get (model, &iter, COLUMN_SELECT_DISPLAY_CHECK, &used, -1);
  used ^= 1;
    
  /* set new value */
  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_SELECT_DISPLAY_CHECK, used, -1);
  /* clean up */
  gtk_tree_path_free (path);  
}

//~ what we have to do once the new structure for display has been stored
void update_display(int n_button){
    display_pcs_list (n_button, -1);
}

//~what we have to do once the new structure for select has been stored.
void update_select(int n_button){
    int i;
    GArray * arr;
    arr = _all_pcs[n_button];
	gboolean old_check;
	gboolean new_check;
	gboolean need_to_clean_tensor_valid;
    need_to_clean_tensor_valid = FALSE;
    if(arr == NULL){
        return;
    }
    for (i = 0; i < arr->len; i++){
        if(pass_filter_select(g_array_index (_all_pcs[n_button], Item_pcs , i).atom_name,
                               g_array_index (_all_pcs[n_button], Item_pcs , i).residue_name,
                               n_button, i) == 1){
			old_check = g_array_index (arr, Item_pcs, i).used;
            g_array_index (arr, Item_pcs, i).used = TRUE;
			new_check = TRUE;
			if(old_check != new_check){
				need_to_clean_tensor_valid = TRUE;
			}
        } else {
			old_check = g_array_index (arr, Item_pcs, i).used;
            g_array_index (arr, Item_pcs, i).used = FALSE;
			new_check = FALSE;
			if(old_check != new_check){
				need_to_clean_tensor_valid = TRUE;
			}
        }
    }
	if(need_to_clean_tensor_valid == TRUE){
		// TENSOR FOR ERROR ANALYSIS NOT VALID ANYMORE
		clear_tensor_valid(n_button);
	}
    display_pcs_list (n_button, -1);
}

//~ what happens when someone click on apply?
//~ we store the information in the data structure, 
//~ end we call the update_select or update_display function.
void select_display_apply_n(int n_button, int type_select_display){
    GtkWidget * window;
    GladeXML *gxml;
    GtkTreeView * treeview_atom;
    GtkTreeView * treeview_residue;
    GtkListStore *model_atom;
    GtkListStore *model_residue;
    GtkTreeIter iter_atom;
    GtkTreeIter iter_residue;
    gboolean my_bool;
    gboolean * my_bool_array_atom;
    gboolean * my_bool_array_residue;
    
    if(type_select_display == SELECT){
        window = _window_select;
        gxml = glade_get_widget_tree((GtkWidget *) window);
        treeview_atom = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_atom_select");
        treeview_residue = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_residue_select");
    }
    else{
        window = _window_display;
        gxml = glade_get_widget_tree((GtkWidget *) window);
        treeview_atom = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_atom_display");
        treeview_residue = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_residue_display");
        
    }
    model_atom = (GtkListStore *) gtk_tree_view_get_model(treeview_atom);
    model_residue = (GtkListStore *) gtk_tree_view_get_model(treeview_residue);

    my_bool_array_atom = NULL;
    my_bool_array_residue = NULL;
    
    switch(type_select_display){
        case DISPLAY:{
            my_bool_array_atom = _select_display_pcs [n_button][DISPLAY_ATOM];
            my_bool_array_residue = _select_display_pcs [n_button][DISPLAY_RESIDUE];
        } break;
        case SELECT:{
            my_bool_array_atom = _select_display_pcs [n_button][SELECT_ATOM];
            my_bool_array_residue = _select_display_pcs [n_button][SELECT_RESIDUE];
        }
        break;
    }
    if (my_bool_array_atom == NULL){
         gtk_widget_hide(window);
          return;
    }
    if (my_bool_array_residue == NULL){
         gtk_widget_hide(window);
          return;
    }
	
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model_atom, &iter_atom);
    while (my_bool)
    {
      gchar *str_name;
      gboolean   checked;
	  gboolean   old_checked;	
      int index;  
      gtk_tree_model_get ((GtkTreeModel *)model_atom, &iter_atom, 
                          COLUMN_SELECT_DISPLAY_CHECK, &checked,
                          COLUMN_SELECT_DISPLAY_NAME, &str_name,
                          -1);
      index = find_atom_type_in_pdb(str_name);
      if(index == -1){
          fprintf(stderr, "Panic, mismatch with atom type\n");
          return;
      }
	  old_checked = my_bool_array_atom[index];
      my_bool_array_atom[index] = checked;
	  if(old_checked != checked){
		  //fprintf(stderr, "change in check (atom)\n");
	  }
      g_free (str_name);
      my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model_atom, &iter_atom);
    }
	
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model_residue, &iter_residue);
	while (my_bool)
    {
      gchar *str_name;
      gboolean   checked;
	  gboolean old_checked;		
      int index;  
      gtk_tree_model_get ((GtkTreeModel *)model_residue, &iter_residue, 
                          COLUMN_SELECT_DISPLAY_CHECK, &checked,
                          COLUMN_SELECT_DISPLAY_NAME, &str_name,
                          -1); 
      index = find_residue_type_in_pdb(str_name);
      if(index == -1){
          fprintf(stderr, "Panic, mismatch with residue type\n");
          return;
      }
	  old_checked = my_bool_array_residue[index];
      my_bool_array_residue[index] = checked;
	  if(old_checked != checked){
		  //fprintf(stderr, "change in check (residue)\n");
	  }
      g_free (str_name);
      my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model_residue, &iter_residue);
    }
	
    switch(type_select_display){
        case DISPLAY:{
            update_display(n_button);
        }
        break;
        case SELECT:{
            update_select(n_button);
        }
        break;
    }
    
    gtk_widget_hide(window);
}

void
cancel_select(void)
{
    gtk_widget_hide(_window_select);
}

void
cancel_display(void)
{
    gtk_widget_hide(_window_display);
}

void apply_display(void)
{
    int n_button;
    n_button = get_flag_display_select (_window_display, DISPLAY);
    select_display_apply_n(n_button, DISPLAY);
}

void apply_select(void)
{
    int n_button;
    n_button = get_flag_display_select (_window_display, SELECT);
    select_display_apply_n(n_button, SELECT);
}


void
all_button_clicked(char * name_treeview){
    GtkTreeView * treeview;
    GtkListStore *model;
    gboolean my_bool;
    GtkTreeIter iter;
    GladeXML *gxml;
    
    gxml = glade_get_widget_tree((GtkWidget *) _window_select);
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, name_treeview);
    model = (GtkListStore *) gtk_tree_view_get_model (treeview);
    if(model == NULL){
        return;
    }
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model, &iter);
    while (my_bool)
    {
        gtk_list_store_set (GTK_LIST_STORE (model), &iter,              COLUMN_SELECT_DISPLAY_CHECK, TRUE, -1); 
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model, &iter);
    }
}

void
none_button_clicked(char * name_treeview){
    GtkTreeView * treeview;
    GtkListStore *model;
    gboolean my_bool;
    GtkTreeIter iter;
    GladeXML *gxml;
    
    gxml = glade_get_widget_tree((GtkWidget *) _window_select);
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, name_treeview);
    model = (GtkListStore *) gtk_tree_view_get_model (treeview);
    if(model == NULL){
        return;
    }
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model, &iter);
    while (my_bool)
    {
        gtk_list_store_set (GTK_LIST_STORE (model), &iter,              COLUMN_SELECT_DISPLAY_CHECK, FALSE, -1); 
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model, &iter);
    }
}


void
all_residue_select()
{
    all_button_clicked("treeview_residue_select");
}
void
none_residue_select()
{
    none_button_clicked("treeview_residue_select");
}
void
all_atom_select()
{
    all_button_clicked("treeview_atom_select");
}
void
none_atom_select()
{
    none_button_clicked("treeview_atom_select");
}

void
all_residue_display()
{
    all_button_clicked("treeview_residue_display");
}
void
none_residue_display()
{
    none_button_clicked("treeview_residue_display");
}
void
all_atom_display()
{
    all_button_clicked("treeview_atom_display");
}
void
none_atom_display()
{
    none_button_clicked("treeview_atom_display");
}


GtkTreeModel * 
create_atom_list_model_ds(int n_button, int type_select_display){
  gint i = 0;
  GtkListStore *model_atom;
  GtkTreeIter iter;
  gboolean * my_bool_array_atom;  
  switch(type_select_display){
        case DISPLAY:{
            my_bool_array_atom = _select_display_pcs [n_button][DISPLAY_ATOM];
        }
        break;
        case SELECT:{
            my_bool_array_atom = _select_display_pcs [n_button][SELECT_ATOM];
        }
        break;
        default:{ 
            my_bool_array_atom = NULL;
        }
        break;
    }
    if(my_bool_array_atom == NULL){
        return(NULL);
    }

  model_atom = gtk_list_store_new (
                  NUM_SELECT_DISPLAY,
                  G_TYPE_BOOLEAN,
			      G_TYPE_STRING);
   if(_pdb_str != NULL){
        gboolean my_bool;
        for(i = 0; i < _pdb_str->n_atom_type; i++){
            gtk_list_store_append (model_atom, &iter);
            my_bool = my_bool_array_atom[i];
            gtk_list_store_set (model_atom, &iter,
                          COLUMN_SELECT_DISPLAY_CHECK,
                          my_bool,
                          COLUMN_SELECT_DISPLAY_NAME,
                            _pdb_str->tab_atom_type[i],
                          -1);
        } 
   }
  return(GTK_TREE_MODEL (model_atom));
}

GtkTreeModel * 
create_residue_list_model_ds(int n_button, int type_select_display){
  gint i = 0;
  GtkListStore *model_residue;
  GtkTreeIter iter;
  gboolean * my_bool_array_residue;  
  switch(type_select_display){
        case DISPLAY:{
            my_bool_array_residue = _select_display_pcs [n_button][DISPLAY_RESIDUE];
        }
        break;
        case SELECT:{
            my_bool_array_residue = _select_display_pcs [n_button][SELECT_RESIDUE];
        }
        break;
        default:{ 
            my_bool_array_residue = NULL;
        }
        break;
    }
    if(my_bool_array_residue == NULL){
        return(NULL);
    }  
    
    model_residue = gtk_list_store_new (
                  NUM_SELECT_DISPLAY,
                  G_TYPE_BOOLEAN,
			      G_TYPE_STRING);
   if(_pdb_str != NULL){
        gboolean my_bool;
        for(i = 0; i < _pdb_str->n_residue_type; i++){
            gtk_list_store_append (model_residue, &iter);
            my_bool = my_bool_array_residue[i];
            gtk_list_store_set (model_residue, &iter,
                          COLUMN_SELECT_DISPLAY_CHECK,
                          my_bool,
                          COLUMN_SELECT_DISPLAY_NAME,
                            _pdb_str->tab_residue_type[i],
                          -1);
        }
    } 
    return(GTK_TREE_MODEL (model_residue));
}

void
add_columns_ds (GtkTreeView  *treeview_atom, GtkTreeView  *treeview_residue, 
             GtkTreeModel *model_atom, GtkTreeModel *model_residue,
             int n_button, int type_select_display, GladeXML *gxml, gboolean flag_new)
{
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;

    void (*to_connect_used) (GtkCellRendererToggle *, gchar *,gpointer);

    char type_name[10];
    to_connect_used = NULL;
    switch(type_select_display){
        case DISPLAY:{
            strcpy(type_name, "Display?");
            switch(n_button){
                case 0: { to_connect_used = display_toggled_0; 
                }
                break;
                case 1: { to_connect_used = display_toggled_1; 
                }
                break;
                case 2: { to_connect_used = display_toggled_2; 
                }
                break;
                case 3: { to_connect_used = display_toggled_3; 
                }
                break;
            }
        }
        break;
        case SELECT:{
            strcpy(type_name, "Use?");
            switch(n_button){
                case 0: { to_connect_used = select_toggled_0; 
                }
                break;
                case 1: { to_connect_used = select_toggled_1;
                }
                break;
                case 2: { to_connect_used = select_toggled_2;
                }
                break;
                case 3: { to_connect_used = select_toggled_3;
                }
                break;
            }
        }
        break;
    }
    if(to_connect_used == NULL){
        fprintf(stderr, "error, couldn't connect signal\n");
        return;
    }
     /* atom check column */   
    renderer = gtk_cell_renderer_toggle_new ();
		g_object_set (G_OBJECT (renderer), "indicator-size", TICK_SIZE, NULL);
    column = gtk_tree_view_column_new_with_attributes (type_name,
						     renderer, "active", COLUMN_SELECT_DISPLAY_CHECK,
						     NULL);
    g_signal_connect (renderer, "toggled", G_CALLBACK (to_connect_used), model_atom);    
    gtk_tree_view_append_column (treeview_atom, column);
    /* atom name column */
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("Atom name",
						     renderer,
						     "text",
						     COLUMN_SELECT_DISPLAY_NAME,
						     NULL);
    gtk_tree_view_column_set_reorderable (GTK_TREE_VIEW_COLUMN (column), TRUE);
    gtk_tree_view_column_set_sort_column_id (column, COLUMN_SELECT_DISPLAY_NAME);
    gtk_tree_view_append_column (treeview_atom, column);
    /* residue check column */
    renderer = gtk_cell_renderer_toggle_new ();
		g_object_set (G_OBJECT (renderer), "indicator-size", TICK_SIZE, NULL);
    column = gtk_tree_view_column_new_with_attributes (type_name,
						     renderer, "active", COLUMN_SELECT_DISPLAY_CHECK,
						     NULL);
    g_signal_connect (renderer, "toggled", G_CALLBACK (to_connect_used), model_residue);
    gtk_tree_view_append_column (treeview_residue, column);
    /* residue name column */
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("Residue name",
						     renderer, "text", COLUMN_SELECT_DISPLAY_NAME,
						     NULL);
    gtk_tree_view_column_set_reorderable (GTK_TREE_VIEW_COLUMN (column), TRUE);
    gtk_tree_view_column_set_sort_column_id (column, COLUMN_SELECT_DISPLAY_NAME);

    gtk_tree_view_append_column (treeview_residue, column);
}

//~ Create the models, read in previous stored structure, connect the
//~ signals etc...
void
populate_select_display(GtkWidget * window_select_display, int n_button, int type_select_display){
    GtkTreeView * treeview_atom;
    GtkTreeView * treeview_residue;
    GtkTreeModel *model_atom;
    GtkTreeModel *model_residue;
    GladeXML *gxml;
    //
    gxml = glade_get_widget_tree(window_select_display);
    if(type_select_display == DISPLAY){
        treeview_atom = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_atom_display");
        treeview_residue = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_residue_display");
    }
    else{
        treeview_atom = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_atom_select");
        treeview_residue = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_residue_select");
    }
    empty_treeview(treeview_atom, FALSE);
    empty_treeview(treeview_residue, FALSE);
    
    model_atom = create_atom_list_model_ds(n_button, type_select_display);
    model_residue = create_residue_list_model_ds(n_button, type_select_display);

    
    add_columns_ds(treeview_atom, treeview_residue, model_atom, model_residue, 
             n_button, type_select_display, gxml, TRUE);

    gtk_tree_view_set_model ((GtkTreeView *)treeview_atom, GTK_TREE_MODEL (model_atom));
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview_atom), TRUE);
    gtk_tree_view_set_model ((GtkTreeView *)treeview_residue, GTK_TREE_MODEL (model_residue));
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview_residue), TRUE);    
    
    if((model_atom == NULL)|| (model_residue == NULL)){
        return;
    }
    g_object_unref (G_OBJECT (model_atom));
    g_object_unref (G_OBJECT (model_residue));
}


int get_flag_display_select(GtkWidget * widge, int type_select_display){
    GladeXML *gxml;
    GtkLabel *label;
    const gchar * a;
    int n_button;
    gxml = glade_get_widget_tree((GtkWidget *) widge);
    if(type_select_display == SELECT){
        label = (GtkLabel *)glade_xml_get_widget (gxml, "label_flag_select");
    }
    else{
        label = (GtkLabel *)glade_xml_get_widget (gxml, "label_flag_display");
    }
    a = gtk_label_get_text (label);
    n_button = atoi(a);
    return(n_button);
}


void delete_columns(GtkTreeView * treeview){
    GList * list;
    GtkTreeViewColumn * column;
    int i;
    list = gtk_tree_view_get_columns(treeview);
    for(i = 0; i < g_list_length (list); i++){
        column = g_list_nth_data (list, i);
        gtk_tree_view_remove_column (treeview, column);
    }
    g_list_free (list);
}

// Copy the display or select filter from an experiment to an other.
// it only changes the view of the treeview, but the user has to click on
// validate to actually save it!
void
copy_display_association(GtkComboBox *combox){
   int n_button; 
   int which_copy;
   char * str_copy; 
   GtkTreeModel * mod;
   GtkTreeIter iter;
   GtkWidget * treeview_residue; 
   GtkWidget * treeview_atom;
   static GtkWidget * previous_treeview_residue = NULL; 
   static GtkWidget * previous_treeview_atom = NULL;  
   GladeXML *gxml;
   GtkTreeModel *model_atom;
   GtkTreeModel *model_residue;
   GtkScrolledWindow * scroll_residue;
   GtkScrolledWindow * scroll_atom;
   n_button = get_flag_display_select (_window_display, DISPLAY); 
    
   gxml = glade_get_widget_tree((GtkWidget * )combox);
   treeview_residue = glade_xml_get_widget (gxml, "treeview_residue_display");
   treeview_atom = glade_xml_get_widget (gxml, "treeview_atom_display");
    
   if(treeview_residue == NULL){
       treeview_residue = previous_treeview_residue;
   }
   if(treeview_atom == NULL){
       treeview_atom = previous_treeview_atom;
   }
    
   scroll_residue = (GtkScrolledWindow *) glade_xml_get_widget (gxml, "scroll_residue_display");
   scroll_atom = (GtkScrolledWindow *) glade_xml_get_widget (gxml, "scroll_atom_display"); 
   gboolean my_bool; 
   my_bool = gtk_combo_box_get_active_iter(combox, &iter);
   if (my_bool == FALSE){
       return;
   }
   mod = gtk_combo_box_get_model(GTK_COMBO_BOX(combox));
   gtk_tree_model_get(mod, &iter, COLUMN_NUMBER_TEXT, &str_copy, -1);
   if(strcmp(str_copy, "NONE") == 0){
       which_copy = n_button;
   }
   else{
    which_copy = atoi(str_copy + strlen("PCS")) - 1;
   }
   g_free(str_copy);

   model_atom = create_atom_list_model_ds (which_copy, DISPLAY);
   model_residue  = create_residue_list_model_ds (which_copy, DISPLAY);
 
    
   gtk_tree_view_set_model (GTK_TREE_VIEW(treeview_residue), GTK_TREE_MODEL (model_residue));
   gtk_widget_set_name((GtkWidget *)treeview_residue, "treeview_residue_display");
   gtk_tree_view_set_model (GTK_TREE_VIEW(treeview_atom), GTK_TREE_MODEL (model_atom));
   gtk_widget_set_name((GtkWidget *)treeview_atom, "treeview_atom_display");

   previous_treeview_residue = treeview_residue;
   previous_treeview_atom = treeview_atom;
    

   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview_atom), TRUE);
   gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_atom)),
                                 GTK_SELECTION_SINGLE);
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview_residue), TRUE);
   gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_residue)),
                                 GTK_SELECTION_SINGLE);
    
   delete_columns((GtkTreeView *)treeview_atom);
   delete_columns((GtkTreeView *)treeview_residue); 
    
   add_columns_ds ((GtkTreeView *)treeview_atom, (GtkTreeView *)treeview_residue, model_atom, model_residue, n_button, DISPLAY, gxml, FALSE);

   gtk_widget_show(treeview_atom);
   gtk_widget_show(treeview_residue);
   g_object_unref (model_atom);
   g_object_unref (model_residue);

}

void 
copy_select_association(GtkComboBox *combox){
   int n_button; 
   int which_copy;
   char * str_copy; 
   GtkTreeModel * mod;
   GtkTreeIter iter;
   GtkWidget * treeview_residue; 
   GtkWidget * treeview_atom;
   static GtkWidget * previous_treeview_residue = NULL; 
   static GtkWidget * previous_treeview_atom = NULL;  
   GladeXML *gxml;
   GtkTreeModel *model_atom;
   GtkTreeModel *model_residue;
   GtkScrolledWindow * scroll_residue;
   GtkScrolledWindow * scroll_atom;
   n_button = get_flag_display_select (_window_select, SELECT); 
    
   gxml = glade_get_widget_tree((GtkWidget * )combox);
   treeview_residue = glade_xml_get_widget (gxml, "treeview_residue_select");
   treeview_atom = glade_xml_get_widget (gxml, "treeview_atom_select");
    
   if(treeview_residue == NULL){
       treeview_residue = previous_treeview_residue;
   }
   if(treeview_atom == NULL){
       treeview_atom = previous_treeview_atom;
   }
    
   scroll_residue = (GtkScrolledWindow *) glade_xml_get_widget (gxml, "scroll_residue_select");
   scroll_atom = (GtkScrolledWindow *) glade_xml_get_widget (gxml, "scroll_atom_select"); 
   gboolean my_bool; 
   my_bool = gtk_combo_box_get_active_iter(combox, &iter);
   if (my_bool == FALSE){
       return;
   }
   mod = gtk_combo_box_get_model(GTK_COMBO_BOX(combox));
   gtk_tree_model_get(mod, &iter, COLUMN_NUMBER_TEXT, &str_copy, -1);
   if(strcmp(str_copy, "NONE") == 0){
       which_copy = n_button;
   }
   else{
    which_copy = atoi(str_copy + strlen("PCS")) - 1;
   }
   g_free(str_copy);

   model_atom = create_atom_list_model_ds (which_copy, SELECT);
   model_residue  = create_residue_list_model_ds (which_copy, SELECT);
    
   gtk_tree_view_set_model (GTK_TREE_VIEW(treeview_residue), GTK_TREE_MODEL (model_residue));
   gtk_widget_set_name((GtkWidget *)treeview_residue, "treeview_residue_select");
   gtk_tree_view_set_model (GTK_TREE_VIEW(treeview_atom), GTK_TREE_MODEL (model_atom));
   gtk_widget_set_name((GtkWidget *)treeview_atom, "treeview_atom_select");

   previous_treeview_residue = treeview_residue;
   previous_treeview_atom = treeview_atom;
    

   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview_atom), TRUE);
   gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_atom)),
                                 GTK_SELECTION_SINGLE);
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview_residue), TRUE);
   gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_residue)),
                                 GTK_SELECTION_SINGLE);
    
   delete_columns((GtkTreeView *)treeview_atom);
   delete_columns((GtkTreeView *)treeview_residue); 
    
   add_columns_ds ((GtkTreeView *)treeview_atom, (GtkTreeView *)treeview_residue, model_atom, model_residue, n_button, SELECT, gxml, FALSE);

   gtk_widget_show(treeview_atom);
   gtk_widget_show(treeview_residue);
   g_object_unref (model_atom);
   g_object_unref (model_residue);
}


void display_toggled_0 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    select_display_toggled_n (cell, path_str, data, 0, DISPLAY);
}
void display_toggled_1 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    select_display_toggled_n (cell, path_str, data, 1, DISPLAY);
}
void display_toggled_2 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    select_display_toggled_n (cell, path_str, data, 2, DISPLAY);
}
void display_toggled_3 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    select_display_toggled_n (cell, path_str, data, 3, DISPLAY);
}
void select_toggled_0 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    select_display_toggled_n (cell, path_str, data, 0, SELECT);
}
void select_toggled_1 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    select_display_toggled_n (cell, path_str, data, 1, SELECT);
}
void select_toggled_2 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    select_display_toggled_n (cell, path_str, data, 2, SELECT);
}
void select_toggled_3 (GtkCellRendererToggle *cell, gchar *path_str,gpointer data){
    select_display_toggled_n (cell, path_str, data, 3, SELECT);
}

