/*
 * tensor.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <math.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>


#include "pdb.h"
#include "methyl.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "tensor_list.h"
#include "common.h"
#include "error_analysis.h"
#include "pcsfit_metal.h"
#include "molmol_iso.h"
#include "unify_tensor.h"
#include "tensor_found.h"
#include "rdc.h"

#define B_AX_NAME STRING_DELTA_CHI_AX_UNIT
#define B_AX_START 40.0
#define B_AX_MIN 0.04
#define B_AX_MAX 400.0
#define B_AX_TYPE TYPE_NONE
#define B_RH_NAME STRING_DELTA_CHI_RH_UNIT
#define B_RH_START 4.0
#define B_RH_MIN  0.04
#define B_RH_MAX  400.0
#define B_RH_TYPE  TYPE_NONE
#define B_X_NAME "x (\303\205)"
#define B_X_START  0.0
#define B_X_MIN  -100.0
#define B_X_MAX  100.0
#define B_X_TYPE  TYPE_NONE
#define B_Y_NAME "y (\303\205)"
#define B_Y_START  0.0
#define B_Y_MIN  -100.0
#define B_Y_MAX  100.0
#define B_Y_TYPE  TYPE_NONE
#define B_Z_NAME "z (\303\205)"
#define B_Z_START  0.0
#define B_Z_MIN  -100.0
#define B_Z_MAX  100.0
#define B_Z_TYPE  TYPE_NONE
#define B_A_NAME "\316\261 (\302\260)"
#define B_A_START  0.0
#define B_A_MIN  0.0
#define B_A_MAX  360.0
#define B_A_TYPE  TYPE_NONE
#define B_B_NAME "\316\262 (\302\260)"
#define B_B_START  0.0
#define B_B_MIN  0.0
#define B_B_MAX  360.0
#define B_B_TYPE  TYPE_NONE
#define B_G_NAME "\316\263 (\302\260)"
#define B_G_START  0.0
#define B_G_MIN  0.0
#define B_G_MAX  360.0
#define B_G_TYPE  TYPE_NONE

extern GtkWidget * _window;
extern GtkWidget *_window_tensor_work;
extern GtkTreeModel * _model_tl;

enum
{
  COLUMN_NAME_BOUNDARIE,  
  COLUMN_TYPE_BOUNDARIE,
  COLUMN_START_BOUNDARIE,
  COLUMN_MIN_BOUNDARIE,
  COLUMN_MAX_BOUNDARIE,
  NUM_BOUNDARIE_COLUMNS
};

enum
{
  COLUMN_NUM_MODEL_PDB,
  COLUMN_PDB_USED,
  NUM_PDB_SELECTOR_COLUMNS
};

enum
{
	UTR,
	TF,
	M_UTR,
	M_TF
};






////Treeview_store for the 2 tensor work list
enum{
	COLUMN_TENSOR_WORK_INTERN_ID,
    COLUMN_TENSOR_WORK_AX,
    COLUMN_TENSOR_WORK_RH,
    COLUMN_TENSOR_WORK_ALPHA,
    COLUMN_TENSOR_WORK_BETA,
    COLUMN_TENSOR_WORK_GAMMA,
	NUM_ITEM_COLUMNS_TENSOR_WORK 
};

#define TYPE_NONE_STRING "None"
#define TYPE_FIXED_STRING "Fixed"
#define TYPE_BOUNDARIES_STRING "Boundaries"

GtkComboBox *_box_more_angle_unit = NULL;
GtkComboBox *_box_more_angle_orientation = NULL;
GtkComboBox *_box_more_rotation_order = NULL;
GtkComboBox *_box_more_magnitude_convention = NULL;
GtkTreeView *_treeview_more_tensor1 = NULL;
GtkTreeView *_treeview_more_tensor2 = NULL;

void display_boundaries_list (void)
{
    GtkTreeModel *model_boundaries;
    GtkTreeModel *model_type;
    GtkWidget *treeview;
    GladeXML *gxml;
    char tree_char[50];
    int i;
    gxml = glade_get_widget_tree(_window);
    for(i = 0; i < N_PCS_ARRAY; i++){
        if(_all_boundaries_pcs[i] == NULL){
           continue;
        }
        sprintf(tree_char, "treeview_boundaries_PCS%d", i+1);
        treeview = glade_xml_get_widget (gxml, tree_char);
        model_boundaries = (GtkTreeModel*) create_boundaries_model ();
        model_type = (GtkTreeModel*) create_type_boundarie_model();        
        gtk_tree_view_set_model ((GtkTreeView *)treeview, GTK_TREE_MODEL (model_boundaries));
        gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
        gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)), GTK_SELECTION_SINGLE);
        add_columns_boundaries (GTK_TREE_VIEW (treeview), model_boundaries
                                , model_type, i);
        g_object_unref (model_boundaries);
        g_object_unref (model_type);
    }
}

void boundaries_edited_0 (GtkCellRendererText *cell, const gchar *path_string,
             const gchar *new_text, gpointer data){
    boundaries_edited_n(cell, path_string,new_text, data, 0);
}
void boundaries_edited_1 (GtkCellRendererText *cell, const gchar *path_string,
             const gchar *new_text, gpointer data){
    boundaries_edited_n(cell, path_string,new_text, data, 1);
}
void boundaries_edited_2 (GtkCellRendererText *cell, const gchar *path_string,
             const gchar *new_text, gpointer data){
    boundaries_edited_n(cell, path_string,new_text, data, 2);
}
void boundaries_edited_3 (GtkCellRendererText *cell, const gchar *path_string,
             const gchar *new_text, gpointer data){
    boundaries_edited_n(cell, path_string,new_text, data, 3);
}

void
boundaries_edited_n (GtkCellRendererText *cell,
             const gchar         *path_string,
             const gchar         *new_text,
             gpointer             data, int n_button)
{
  GtkTreeModel *model = (GtkTreeModel *)data;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
  GtkTreeIter iter;
  gint column = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), "column"));
  gtk_tree_model_get_iter (model, &iter, path);

  switch (column)
    {
    case COLUMN_TYPE_BOUNDARIE:
      {
        gchar *old_text;
        gtk_tree_model_get (model, &iter, column, &old_text, -1);
        g_free (old_text);
        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                             g_strdup (new_text) , -1);
        update_boundaries(&iter, model, n_button);  
      }
      break;
    default:{
        double value;
        value = atof(new_text);       
        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                value , -1);
        update_boundaries(&iter, model, n_button); 
    }  
    break;
    }
  gtk_tree_path_free (path);
}

void update_boundaries(GtkTreeIter * iter, GtkTreeModel *model, int n_button){
  boundaries * bound;
  boundarie * boundi; 
  gchar * name;
  bound = _all_boundaries_pcs[n_button];
  boundi = NULL;  
  gtk_tree_model_get (model, iter, COLUMN_NAME_BOUNDARIE, &name , -1);   
  if(strcmp(name, B_AX_NAME) == 0){ boundi = &(bound->b_Ax);}
  if(strcmp(name, B_RH_NAME) == 0){ boundi = &(bound->b_Rh);}  
  if(strcmp(name, B_X_NAME) == 0){ boundi = &(bound->b_x);}
  if(strcmp(name, B_Y_NAME) == 0){ boundi = &(bound->b_y);}
  if(strcmp(name, B_Z_NAME) == 0){ boundi = &(bound->b_z);}
  if(strcmp(name, B_A_NAME) == 0){ boundi = &(bound->b_a);}
  if(strcmp(name, B_B_NAME) == 0){ boundi = &(bound->b_b);}
  if(strcmp(name, B_G_NAME) == 0){ boundi = &(bound->b_g);}
  g_free(name);
  if(boundi == NULL){
      return;
  }
  gchar * type;
  gtk_tree_model_get (model, iter, COLUMN_START_BOUNDARIE,&(boundi->start) , -1);
  gtk_tree_model_get (model, iter, COLUMN_MIN_BOUNDARIE,  &(boundi->min)   , -1);
  gtk_tree_model_get (model, iter, COLUMN_MAX_BOUNDARIE,  &(boundi->max)   , -1);
  gtk_tree_model_get (model, iter, COLUMN_TYPE_BOUNDARIE, &type  , -1);
  boundi->type = type_string_to_int(type);
  g_free(type);  
}



static void polish_boundaries_tensor_function (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
    double value;
    gint column_id;
    gint column_type;
    gint type_boundarie;
    gchar buf[30];
    gchar * junk_buf;
    column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (renderer), "column"));
    column_type = COLUMN_TYPE_BOUNDARIE;
    gtk_tree_model_get(model, iter, column_id, &value, -1);
    gtk_tree_model_get(model, iter, column_type, &junk_buf, -1);
    g_snprintf(buf, sizeof(buf), "%.3f", value);
    g_object_set(renderer, "text", buf, NULL);
    type_boundarie = -1;
    
    if(strcmp(junk_buf, TYPE_NONE_STRING) == 0){
        type_boundarie = TYPE_NONE;
    }
    if(strcmp(junk_buf, TYPE_FIXED_STRING) == 0){
        type_boundarie = TYPE_FIXED;
    }
    if(strcmp(junk_buf, TYPE_BOUNDARIES_STRING) == 0){
        type_boundarie = TYPE_BOUNDARIES;
    }
    
    g_object_set (renderer, "cell-background-set", FALSE, NULL);
    switch(type_boundarie){
        case TYPE_NONE: {
            if(column_id == COLUMN_MIN_BOUNDARIE){
                g_object_set (renderer, "text", "n/a", NULL);
            }
            if(column_id == COLUMN_MAX_BOUNDARIE){
                g_object_set (renderer, "text", "n/a", NULL);
            }
            break;
        }
        case TYPE_BOUNDARIES: {
        }
        break;
        case TYPE_FIXED: {
            if(column_id == COLUMN_MIN_BOUNDARIE){
                g_object_set (renderer, "text", "n/a", NULL);
            }
            if(column_id == COLUMN_MAX_BOUNDARIE){
                g_object_set (renderer, "text", "n/a", NULL);
            }
        }
        break;
        default: {
            g_object_set (renderer, "cell-background-set", FALSE, NULL);
        }
    }
}

static void polish_model_num (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
     int value;
     gint column_id;
     gchar buf[30];
    column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (renderer), "column"));
    gtk_tree_model_get(model, iter, column_id, &value, -1);
    g_snprintf(buf, sizeof(buf), "%d", value+1);
    g_object_set(renderer, "text", buf, NULL);
}

static void polish_name_tensor_function (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
     gint column_id;
     gchar* buf;
    column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (renderer), "column"));
    gtk_tree_model_get(model, iter, column_id, &buf, -1);
    
    if (TRUE){
      g_object_set(renderer, "markup", buf, NULL);
    }
    g_free(buf);
}

void
add_columns_boundaries (GtkTreeView  *treeview, 
             GtkTreeModel *model_boundaries,
             GtkTreeModel *model_type, int n_button)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  void (*to_connect_edited) (GtkCellRendererText *, const gchar *, const gchar *, gpointer);
      switch (n_button){
        case 0:{ to_connect_edited = boundaries_edited_0; }
        break;
        case 1:{ to_connect_edited = boundaries_edited_1; }
        break;
        case 2:{ to_connect_edited = boundaries_edited_2; }
        break;
        case 3:{ to_connect_edited = boundaries_edited_3; }
        break;
        default:{
            return;
        }
        break;
    }  
  /* NAME COLUMN */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set (renderer, "editable", FALSE, NULL);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_NAME_BOUNDARIE));
    
    
      column = gtk_tree_view_column_new_with_attributes ("Variables",
						     renderer,
						     "text", COLUMN_NAME_BOUNDARIE,
						     NULL);
    gtk_tree_view_append_column (treeview, column);
    
    gtk_tree_view_column_set_cell_data_func(column, renderer,
                                            polish_name_tensor_function, 
                                   NULL, NULL); //let's make nice 10^-32
    
  /* TYPE COLUMN */
  renderer = gtk_cell_renderer_combo_new ();
  g_object_set (renderer,
                "model", model_type,
                "text-column", 0,
                "has-entry", FALSE,
                "editable", TRUE,
                NULL);

  g_signal_connect (renderer, "edited",
                   G_CALLBACK (to_connect_edited), model_boundaries);

  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_TYPE_BOUNDARIE));

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "Type", renderer,
                                               "text", COLUMN_TYPE_BOUNDARIE,
                                               NULL);
    /* start column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set (renderer, "editable", TRUE, NULL);
  g_signal_connect (renderer, "edited", G_CALLBACK (to_connect_edited), model_boundaries);    
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_START_BOUNDARIE));
  column = gtk_tree_view_column_new_with_attributes ("Start",
						     renderer,
						     "text", COLUMN_START_BOUNDARIE,
						     NULL);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_boundaries_tensor_function, NULL, NULL);  
  gtk_tree_view_append_column (treeview, column);
  /* min column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set (renderer, "editable", TRUE, NULL);
  g_signal_connect (renderer, "edited", G_CALLBACK (to_connect_edited), model_boundaries);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_MIN_BOUNDARIE));
  column = gtk_tree_view_column_new_with_attributes ("Min",
						     renderer,
						     "text", COLUMN_MIN_BOUNDARIE,
						     NULL);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_boundaries_tensor_function, NULL, NULL);  
  gtk_tree_view_append_column (treeview, column); 

    /* max column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set (renderer, "editable", TRUE, NULL);
  g_signal_connect (renderer, "edited", G_CALLBACK (to_connect_edited), model_boundaries);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_MAX_BOUNDARIE));
  column = gtk_tree_view_column_new_with_attributes ("Max",
						     renderer,
						     "text", COLUMN_MAX_BOUNDARIE,
						     NULL);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_boundaries_tensor_function, NULL, NULL);  
  gtk_tree_view_append_column (treeview, column);  

    g_signal_connect (renderer, "edited", G_CALLBACK (to_connect_edited), model_boundaries);

}
const char * type_int_to_string(int type){
    switch (type){
        case TYPE_NONE: {
            return (TYPE_NONE_STRING);
        }
        break;
        case TYPE_FIXED: {
            return (TYPE_FIXED_STRING);
        }
        break;
        case TYPE_BOUNDARIES: {
            return (TYPE_BOUNDARIES_STRING);
        }
        break;
    }
    return(NULL);
}

int type_string_to_int(char * type){
    if(strcmp(type, TYPE_NONE_STRING) == 0){
        return(TYPE_NONE);
    }
    if(strcmp(type, TYPE_FIXED_STRING) == 0){
        return(TYPE_FIXED);
    }
    if(strcmp(type, TYPE_BOUNDARIES_STRING) == 0){
        return(TYPE_BOUNDARIES);
    }
    return(-2);
}

GtkTreeModel *
create_boundaries_model (void)
{
  GtkListStore *model;
  GtkTreeIter iter;
    

  model = gtk_list_store_new (NUM_BOUNDARIE_COLUMNS, G_TYPE_STRING, 
                              G_TYPE_STRING, G_TYPE_DOUBLE,
                              G_TYPE_DOUBLE, G_TYPE_DOUBLE);
    
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, COLUMN_NAME_BOUNDARIE, B_AX_NAME,
        COLUMN_TYPE_BOUNDARIE, type_int_to_string(B_AX_TYPE), COLUMN_START_BOUNDARIE, B_AX_START,
        COLUMN_MIN_BOUNDARIE,  B_AX_MIN, COLUMN_MAX_BOUNDARIE, B_AX_MAX, -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, COLUMN_NAME_BOUNDARIE, B_RH_NAME,
        COLUMN_TYPE_BOUNDARIE, type_int_to_string(B_RH_TYPE), COLUMN_START_BOUNDARIE, B_RH_START,
        COLUMN_MIN_BOUNDARIE,  B_RH_MIN, COLUMN_MAX_BOUNDARIE, B_RH_MAX, -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, COLUMN_NAME_BOUNDARIE, B_X_NAME,
        COLUMN_TYPE_BOUNDARIE, type_int_to_string(B_X_TYPE), COLUMN_START_BOUNDARIE, B_X_START,
        COLUMN_MIN_BOUNDARIE,  B_X_MIN, COLUMN_MAX_BOUNDARIE,  B_X_MAX, -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, COLUMN_NAME_BOUNDARIE, B_Y_NAME,
        COLUMN_TYPE_BOUNDARIE, type_int_to_string(B_Y_TYPE), COLUMN_START_BOUNDARIE, B_Y_START,
        COLUMN_MIN_BOUNDARIE,  B_Y_MIN, COLUMN_MAX_BOUNDARIE,  B_Y_MAX, -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, COLUMN_NAME_BOUNDARIE, B_Z_NAME,
        COLUMN_TYPE_BOUNDARIE, type_int_to_string(B_Z_TYPE), COLUMN_START_BOUNDARIE, B_Z_START,
        COLUMN_MIN_BOUNDARIE,  B_Z_MIN, COLUMN_MAX_BOUNDARIE,  B_Z_MAX, -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, COLUMN_NAME_BOUNDARIE, B_A_NAME,
        COLUMN_TYPE_BOUNDARIE, type_int_to_string(B_A_TYPE), COLUMN_START_BOUNDARIE, B_A_START,
        COLUMN_MIN_BOUNDARIE,  B_A_MIN, COLUMN_MAX_BOUNDARIE,  B_A_MAX, -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, COLUMN_NAME_BOUNDARIE, B_B_NAME,
        COLUMN_TYPE_BOUNDARIE, type_int_to_string(B_B_TYPE), COLUMN_START_BOUNDARIE, B_B_START,
        COLUMN_MIN_BOUNDARIE,  B_B_MIN, COLUMN_MAX_BOUNDARIE,  B_B_MAX, -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, COLUMN_NAME_BOUNDARIE, B_G_NAME,
        COLUMN_TYPE_BOUNDARIE, type_int_to_string(B_G_TYPE), COLUMN_START_BOUNDARIE, B_G_START,
        COLUMN_MIN_BOUNDARIE,  B_G_MIN, COLUMN_MAX_BOUNDARIE,  B_G_MAX, -1);
    
  return GTK_TREE_MODEL (model);
}

GtkTreeModel *
create_type_boundarie_model (void)
{
  GtkListStore *model;
  GtkTreeIter iter;

  model = gtk_list_store_new (1, G_TYPE_STRING);

  gtk_list_store_append (model, &iter);
  gtk_list_store_set (model, &iter, 0, TYPE_NONE_STRING, -1);
  gtk_list_store_append (model, &iter);
  gtk_list_store_set (model, &iter, 0, TYPE_FIXED_STRING, -1);
  gtk_list_store_append (model, &iter);    
  gtk_list_store_set (model, &iter, 0, TYPE_BOUNDARIES_STRING, -1);
  return GTK_TREE_MODEL (model);
}

//Called only at initialisation
void init_tensor(void){
    int i;
    for(i = 0; i < N_PCS_ARRAY; i++){
        _all_tensor_pcs[i] = NULL;
        _all_boundaries_pcs[i] = new_boundaries();
    }
    _multiple_pcs_boundaries_pcs = new_boundaries();
}

void free_tensor(tensor * tensor_to_free){
    if (tensor_to_free == NULL){
        return;
    }
    free(tensor_to_free);
    tensor_to_free = NULL;
}

tensor * new_tensor(void){
    return((tensor *) calloc(sizeof(tensor), 1));
}

boundaries * new_boundaries(void){
    boundaries * bound;
    bound = (boundaries *) calloc(sizeof(boundaries), 1);
    bound->b_Ax.start = B_AX_START ;
    bound->b_Ax.min   = B_AX_MIN   ;
    bound->b_Ax.max   = B_AX_MAX   ;
    bound->b_Ax.type  = B_AX_TYPE  ;
    bound->b_Rh.start = B_RH_START ;
    bound->b_Rh.min   = B_RH_MIN   ;
    bound->b_Rh.max   = B_RH_MAX   ;
    bound->b_Rh.type  = B_RH_TYPE  ;
    bound->b_x.start  = B_X_START  ;
    bound->b_x.min    = B_X_MIN    ;
    bound->b_x.max    = B_X_MAX    ;
    bound->b_x.type   = B_X_TYPE   ;
    bound->b_y.start  = B_Y_START  ;
    bound->b_y.min    = B_Y_MIN    ;
    bound->b_y.max    = B_Y_MAX    ;
    bound->b_y.type   = B_Y_TYPE   ;
    bound->b_z.start  = B_Z_START  ;
    bound->b_z.min    = B_Z_MIN    ;
    bound->b_z.max    = B_Z_MAX    ;
    bound->b_z.type   = B_Z_TYPE   ;
    bound->b_a.start  = B_A_START  ;
    bound->b_a.min    = B_A_MIN    ;
    bound->b_a.max    = B_A_MAX    ;
    bound->b_a.type   = B_A_TYPE   ;
    bound->b_b.start  = B_B_START  ;
    bound->b_b.min    = B_B_MIN    ;
    bound->b_b.max    = B_B_MAX    ;
    bound->b_b.type   = B_B_TYPE   ;
    bound->b_g.start  = B_G_START  ;
    bound->b_g.min    = B_G_MIN    ;
    bound->b_g.max    = B_G_MAX    ;
    bound->b_g.type   = B_G_TYPE   ;
    
    return(bound);
}

void free_boundaries(boundaries * boundaries_to_free){
        if (boundaries_to_free == NULL){
        return;
    }
    free(boundaries_to_free);
    boundaries_to_free = NULL;
}

void print_all_boundaries(void){
    int i;
    boundaries * bound;
    for(i = 0; i< N_PCS_ARRAY; i++){
        bound = _all_boundaries_pcs[i];
        fprintf(stderr, "%d\t%lf\t%lf\t%lf\n", 
                bound->b_Ax.type,
                bound->b_Ax.start,
                bound->b_Ax.min,
                bound->b_Ax.max);
                fprintf(stderr, "%d\t%lf\t%lf\t%lf\n", 
                bound->b_Rh.type,
                bound->b_Rh.start,
                bound->b_Rh.min,
                bound->b_Rh.max);
    }
}

int calculate_pcs(tensor * my_tensor, int n_button, int pcs_type_2_update, double noise, int seed
                  , gboolean use_racs, racs_data * racs_d)
{
    int i;
    int index;
    double pcs_calc;
    double pcs_exp;
    double dist;
    int * index_on_pdb = NULL;
    int * index_on_pcs = NULL;
    double noise_to_add;
    double racs;
    gsl_rng * my_rand; 
    
    if(index_on_pdb != NULL){
        free(index_on_pdb);
    }
    //index_on_pdb = (int *) calloc(my_pdb_str->n_atom, sizeof(int));
    index_on_pdb = (int *) calloc(_all_pcs[n_button]->len, sizeof(int));
    if(index_on_pdb == NULL){
        fprintf(stderr, "couldn't allocate memory to create index_on_pdb %d\n", 
                _all_pcs[n_button]->len);
        exit(-1);
    }
    
    if(index_on_pcs != NULL){
        free(index_on_pcs);
    }
    //index_on_pcs = (int *) calloc(my_pdb_str->n_atom, sizeof(int));
    index_on_pcs = (int *) calloc(_all_pcs[n_button]->len, sizeof(int));
    if(index_on_pcs == NULL){
        fprintf(stderr, "couldn't allocate memory to create index_on_pcs\n");
        exit(-1);
    }
    
    my_rand = gsl_rng_alloc (gsl_rng_ranlxs0);
    gsl_rng_set (my_rand, seed);
    double R[3][3];
    filZX_or_YZ(R, my_tensor->a/M_PI * 180.0, my_tensor->b/M_PI * 180.0, my_tensor->g/M_PI * 180.0);
    
    for(i = 0; i <_all_pcs[n_button]->len; i++){

        index = find_pcs_in_pdb(g_array_index(_all_pcs[n_button], Item_pcs, i).residue_num,
                                g_array_index(_all_pcs[n_button], Item_pcs, i).atom_name);
  
        index_on_pdb[i] = index;
        index_on_pcs[i] = i;
        
        pcs_calc = SinglePCS_new(R,
                                 my_tensor->x,
                                 my_tensor->y,
                                 my_tensor->z,
                                 my_tensor->ax,
                                 my_tensor->rh, i, n_button, index_on_pdb,
                                 index_on_pcs, 0, 0, 0, use_racs, &racs, racs_d);
        
        //THIS IS WHERE I ADD THE NOISE
        
        noise_to_add = 0.0;
        noise_to_add = gsl_ran_gaussian(my_rand, noise);
        if(noise == 0){
            noise_to_add = 0;
        }
        
        g_array_index(_all_pcs[n_button], Item_pcs, i).pcs_calc = pcs_calc;
        if(use_racs){
            if(g_array_index(_all_pcs[n_button], Item_pcs, i).is_racs){
                g_array_index(_all_pcs[n_button], Item_pcs, i).racs = racs;
            }
        }
        if(pcs_type_2_update == UPDATE_PCS_CALC){
        } 
        else {//UPDATE_PCS_EXP

            g_array_index(_all_pcs[n_button], Item_pcs, i).pcs_exp = pcs_calc + noise_to_add;
            g_array_index(_all_pcs[n_button], Item_pcs, i).useable = TRUE;
        }
        
        pcs_exp = g_array_index(_all_pcs[n_button], Item_pcs, i).pcs_exp; 
        if(pcs_exp != EXEPTION_VALUE){
            g_array_index(_all_pcs[n_button], Item_pcs, i).pcs_dev = fabs(pcs_exp - pcs_calc);
        }
        else{
            g_array_index(_all_pcs[n_button], Item_pcs, i).pcs_dev = EXEPTION_VALUE;
        }
        dist = calculate_average_distance_family(index_on_pdb, index_on_pcs, i, n_button, my_tensor->x, my_tensor->y, my_tensor->z);
        g_array_index(_all_pcs[n_button], Item_pcs, i).dist_center = dist;
    }
    gsl_rng_free (my_rand);
    
    if(index_on_pdb != NULL){
        free(index_on_pdb);
        index_on_pdb = NULL;
    }
    if(index_on_pcs != NULL){
        free(index_on_pcs);
        index_on_pcs = NULL;
    }
    return(1);
}

double distance_atom( double x1, double y1, double z1, double x2, double y2, double z2){
    double dist;
    dist = (x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
    return(sqrt(dist));
}

GtkTreeModel *
create_pdb_selector_model (int n_button)
{
  GtkListStore *model;
  GtkTreeIter iter;
  GArray * my_pdb_selector;
  if(n_button == MULTIPLE_PCS_N_BUTTON){
    my_pdb_selector = _multiple_pcs_pdb_selector_pcs;
  }
  else{  
    my_pdb_selector = _all_pdb_selector_pcs[n_button];
  }
  int i;  
  if(_pdb_str == NULL){
    return(NULL);
  }
  model = gtk_list_store_new (NUM_PDB_SELECTOR_COLUMNS, G_TYPE_INT, G_TYPE_BOOLEAN);
  for (i = 0; i < _pdb_str->n_model; i++){
        gtk_list_store_append (model, &iter);
        gtk_list_store_set (model, &iter, 
            COLUMN_NUM_MODEL_PDB, i,
            COLUMN_PDB_USED, g_array_index (my_pdb_selector, pdb_selector , i).is_used
            , -1);
    }
  return GTK_TREE_MODEL (model);
}

void delete_columns_pdb_selector(GtkTreeView * treeview){
    GList * list;
    GtkTreeViewColumn * column;
    int i;
    if(treeview == NULL){
        return;
    }
    list = gtk_tree_view_get_columns(treeview);
    for(i = 0; i < g_list_length (list); i++){
        column = g_list_nth_data (list, i);
        gtk_tree_view_remove_column (treeview, column);
    }
    g_list_free (list);
}

void update(GArray * arr, int model_num, gboolean used){
    int i;
    for(i = 0; i < arr->len; i++){
        if(g_array_index (arr, pdb_selector , i).num_model == model_num){
            g_array_index (arr, pdb_selector , i).is_used = used;
            return;
        }
    }
}

//~ what happens when someone click on a toggle of pdb selector??
//~ we update the treeview and the corresponding structure!!!
void pdb_selector_toggle (GtkCellRendererToggle *cell, gchar *path_str, gpointer data){

  int n_button;
	int * n_but;
  GtkTreeIter  iter;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
  gboolean used;
  GtkTreeModel * model;
  GtkWidget *treeview;
  GladeXML *gxml;	
  int model_num;  
  char tree_char[50];
  GArray * my_pdb_selector;
   
  gxml = glade_get_widget_tree(_window);
	
	n_but = (int*) data;
	n_button = *n_but;
  
  switch(n_button){
      case MULTIPLE_PCS_N_BUTTON:{
        if(_multiple_pcs_pdb_selector_pcs == NULL){
            return;
        }
        my_pdb_selector = _multiple_pcs_pdb_selector_pcs;
        treeview = glade_xml_get_widget (gxml, "pdb_selector_PCS_MULTI");  
        break;
      }
      default:{
        if(_all_pdb_selector_pcs[n_button] == NULL){
            return;
        }
        my_pdb_selector = _all_pdb_selector_pcs[n_button];  
        sprintf(tree_char, "pdb_selector_PCS%d", n_button+1);
        treeview = glade_xml_get_widget (gxml, tree_char);
        break;
      }
  }
  
  model = gtk_tree_view_get_model((GtkTreeView *)treeview);
  if(model == NULL){
      return;
  }
  gtk_tree_model_get_iter (model, &iter, path);
  gtk_tree_model_get (model, &iter, COLUMN_NUM_MODEL_PDB, &model_num, -1);
  gtk_tree_model_get (model, &iter, COLUMN_PDB_USED, &used, -1);
  used ^= 1;
  /* set new value */
  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_PDB_USED, used, -1);
  update(my_pdb_selector, model_num, used);
  /* clean up */
  gtk_tree_path_free (path);  
}

static void
add_columns_pdb_selector (GtkTreeView *treeview, int n_button)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  int * entier;
  entier = (int *) malloc(sizeof(int));
  delete_columns_pdb_selector(GTK_TREE_VIEW (treeview)); 
  *entier = n_button;  

  /* column for used toggles */
  renderer = gtk_cell_renderer_toggle_new ();
	g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_PDB_USED));	
  g_object_set (G_OBJECT (renderer), "indicator-size", TICK_SIZE, NULL); 
  column = gtk_tree_view_column_new_with_attributes ("Use?",
						     renderer,
						     "active", COLUMN_PDB_USED,
						     NULL);
  g_signal_connect (renderer, "toggled", G_CALLBACK (pdb_selector_toggle), (gpointer)
                      entier);  
  gtk_tree_view_append_column (treeview, column);
	
	  /* column for structure number */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_NUM_MODEL_PDB)); 
  column = gtk_tree_view_column_new_with_attributes ("Model\nnum.",
						     renderer,
						     "text", COLUMN_NUM_MODEL_PDB,
						     NULL);
    gtk_tree_view_column_set_cell_data_func(column, renderer, polish_model_num, NULL, NULL);
    gtk_tree_view_append_column (treeview, column);
    
    
}


//create a new array for PDB selector
GArray * new_array_PDB_selector(void){
    return( g_array_sized_new (FALSE, FALSE, sizeof (pdb_selector), 1));
}

//~ should be used only at initialisation
void init_PDB_selector(void){
    int i;
    for(i = 0; i < N_PCS_ARRAY; i++){
        _all_pdb_selector_pcs[i] = NULL;
        _all_pdb_selector_pcs[i] = new_array_PDB_selector();
    }
    _multiple_pcs_pdb_selector_pcs = NULL;
    _multiple_pcs_pdb_selector_pcs = new_array_PDB_selector();
}

void empty_PDB_selector(void){
    int i;
    for(i = 0; i < N_PCS_ARRAY; i++){
        if(_all_pdb_selector_pcs[i]->len  == 0){
            continue;
        }
        g_array_remove_range (_all_pdb_selector_pcs[i], 0, _all_pdb_selector_pcs[i]->len);
    }
    if(_multiple_pcs_pdb_selector_pcs->len > 0){
        g_array_remove_range (_multiple_pcs_pdb_selector_pcs, 0, _multiple_pcs_pdb_selector_pcs->len);
    }
}


//~ should be called after a new PDB is loaded (at COMMIT)
void update_PDB_selector(void){
    int i, j;
    pdb_selector foo;
    if(_pdb_str == NULL){
        return;
    }
    empty_PDB_selector();
    for(j = 0; j < N_PCS_ARRAY; j++){
        for (i = 0; i < _pdb_str->n_model; i++){
            foo.num_model = i;
            foo.is_used = TRUE;
            g_array_append_vals (_all_pdb_selector_pcs[j], &foo, 1);
        }
    }
    for (i = 0; i < _pdb_str->n_model; i++){
            foo.num_model = i;
            foo.is_used = TRUE;
            g_array_append_vals (_multiple_pcs_pdb_selector_pcs, &foo, 1);
    }
}

// attache the model to the columns of treeview
// Once the pdb has been correctly read, this is the function to call
//~ at COMMIT
void display_PDB_selector(int n_button){
    
    GtkTreeModel *model;
    GtkWidget *treeview;
    GladeXML *gxml;
    char tree_char[50];
    gxml = glade_get_widget_tree(_window);
    switch(n_button){
        case MULTIPLE_PCS_N_BUTTON:{
            if(_multiple_pcs_pdb_selector_pcs == NULL){
                return;
            }
            treeview = glade_xml_get_widget (gxml,"pdb_selector_PCS_MULTI" );
            
        break;
        }
        default:{
            if(_all_pdb_selector_pcs[n_button] == NULL){
                return;
            }
            sprintf(tree_char, "pdb_selector_PCS%d", n_button+1);
            treeview = glade_xml_get_widget (gxml, tree_char);
        }
    }    
    
      /* create tree model */
      model = create_pdb_selector_model(n_button);
      add_columns_pdb_selector(GTK_TREE_VIEW(treeview), n_button);
      /* create tree view */
      gtk_tree_view_set_model ((GtkTreeView *)treeview, GTK_TREE_MODEL (model));
      gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
      g_object_unref (G_OBJECT (model));
}

//~ called by callback when all is clicked
void all_none_clicked_pdb_selector(int n_button, gboolean T_F){
    
  GtkTreeIter  iter;
  gboolean my_bool;  
  GtkTreeModel * model;
  GtkWidget *treeview;
  GladeXML *gxml;
  int model_num;  
  char tree_char[50];
  GArray * my_pdb_selector;  
  gxml = glade_get_widget_tree(_window);
    
    switch(n_button){
      case MULTIPLE_PCS_N_BUTTON:{
        if(_multiple_pcs_pdb_selector_pcs == NULL){
            return;
        }
        my_pdb_selector = _multiple_pcs_pdb_selector_pcs;
        treeview = glade_xml_get_widget (gxml, "pdb_selector_PCS_MULTI");  
        break;
      }
      default:{
        if(_all_pdb_selector_pcs[n_button] == NULL){
            return;
        }
        my_pdb_selector = _all_pdb_selector_pcs[n_button];  
        sprintf(tree_char, "pdb_selector_PCS%d", n_button+1);
        treeview = glade_xml_get_widget (gxml, tree_char);
        break;
      }
  }
    
  model = gtk_tree_view_get_model((GtkTreeView *)treeview);
  if(model == NULL){
      return;
  }
  my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model, &iter);
  while (my_bool) {
    gtk_tree_model_get (model, &iter, COLUMN_NUM_MODEL_PDB, &model_num, -1);
    gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_PDB_USED, T_F, -1);
    update(my_pdb_selector, model_num, T_F);
    my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model, &iter);
  }
}


void print_pdb_selector(int n_button){
    int i;
    GArray * my_pdb_selector;
    switch(n_button){
      case MULTIPLE_PCS_N_BUTTON:{
        if(_multiple_pcs_pdb_selector_pcs == NULL){
            exit(-1);
        }
        my_pdb_selector = _multiple_pcs_pdb_selector_pcs;
        break;
      }
      default:{
        if(_all_pdb_selector_pcs[n_button] == NULL){
            exit(-1);
        }
        my_pdb_selector = _all_pdb_selector_pcs[n_button];  
        break;
      }
  }

    for(i = 0; i < my_pdb_selector->len; i++){
        if(g_array_index (my_pdb_selector, pdb_selector , i).is_used == TRUE){ 
            //fprintf(stderr, "%d: \t1\n", n_button);
        }
        else{
            //fprintf(stderr, "%d: \t0\n", n_button);
        }
    }
    
}

double calculate_average_distance_family(int * index_on_pdb, int * index_on_pcs, int index_global, int n_button, double xM, double yM, double zM)
{
    
    int i;
    double distance;
    double d;
    double x, y, z;
    int compteur;
    distance = 0;
    compteur = 0;
    for(i = 0; i < _all_pdb_selector_pcs[n_button]->len; i++){
        if(g_array_index (_all_pdb_selector_pcs[n_button], pdb_selector , i).is_used == FALSE){
            continue;
        }
        
        if(g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[index_global]).is_dumby == TRUE){
            
            int ind;
            double d1, d2, d3;
            ind = g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[index_global]).methyl_1;
            x = _pdb_str->tab_atom[ind].atom_coo[i].x;
            y = _pdb_str->tab_atom[ind].atom_coo[i].y;
            z = _pdb_str->tab_atom[ind].atom_coo[i].z;
            d1 = distance_atom(x, y, z, xM, yM, zM);
            
            ind = g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[index_global]).methyl_2;
            x = _pdb_str->tab_atom[ind].atom_coo[i].x;
            y = _pdb_str->tab_atom[ind].atom_coo[i].y;
            z = _pdb_str->tab_atom[ind].atom_coo[i].z;
            d2 = distance_atom(x, y, z, xM, yM, zM);
            
            ind = g_array_index(_all_pcs[n_button], Item_pcs, index_on_pcs[index_global]).methyl_3;
            x = _pdb_str->tab_atom[ind].atom_coo[i].x;
            y = _pdb_str->tab_atom[ind].atom_coo[i].y;
            z = _pdb_str->tab_atom[ind].atom_coo[i].z;
            d3 = distance_atom(x, y, z, xM, yM, zM);
            
            d = (d1 + d2 + d3)/3.0;
        }
        else{
        
            x = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].x;
            y = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].y;
            z = _pdb_str->tab_atom[index_on_pdb[index_global]].atom_coo[i].z;
            d = distance_atom(x, y, z, xM, yM, zM);
        }
        distance += d;
        compteur ++;
    }
    if (compteur == 0){
        fprintf(stderr, "Division by 0, this should have been dealt with before!!\n");
        exit(0);
    }
    return(distance/compteur);
}


void display_tensor(int gg, int SING_OR_MULT, int serial, gboolean new_node){
	double R[3][3];
	molmol_data mm_data_UTR;
    molmol_data mm_data_UTR_ER;
	molmol_data mm_data_TF;
	molmol_data mm_data_TF_ERR;
    
	mm_data_UTR.a = _all_tensor_pcs[gg]->a/M_PI*180.0;
	mm_data_UTR.b = _all_tensor_pcs[gg]->b/M_PI*180.0;
	mm_data_UTR.g = _all_tensor_pcs[gg]->g/M_PI*180.0;
	mm_data_UTR.X = _all_tensor_pcs[gg]->x;
	mm_data_UTR.Y = _all_tensor_pcs[gg]->y;
	mm_data_UTR.Z = _all_tensor_pcs[gg]->z;
	mm_data_UTR.Ax = _all_tensor_pcs[gg]->ax;
	mm_data_UTR.Rh = _all_tensor_pcs[gg]->rh;
	
	mm_data_TF.a = _all_tensor_pcs[gg]->a/M_PI*180.0;
	mm_data_TF.b = _all_tensor_pcs[gg]->b/M_PI*180.0;
	mm_data_TF.g = _all_tensor_pcs[gg]->g/M_PI*180.0;
	mm_data_TF.X = _all_tensor_pcs[gg]->x;
	mm_data_TF.Y = _all_tensor_pcs[gg]->y;
	mm_data_TF.Z = _all_tensor_pcs[gg]->z;
	mm_data_TF.Ax = _all_tensor_pcs[gg]->ax;
	mm_data_TF.Rh = _all_tensor_pcs[gg]->rh;
	
	mm_data_TF_ERR.a = _all_tensor_pcs[gg]->err_a/M_PI*180.0;
	mm_data_TF_ERR.b = _all_tensor_pcs[gg]->err_b/M_PI*180.0;
	mm_data_TF_ERR.g = _all_tensor_pcs[gg]->err_g/M_PI*180.0;
	mm_data_TF_ERR.X = _all_tensor_pcs[gg]->err_x;
	mm_data_TF_ERR.Y = _all_tensor_pcs[gg]->err_y;
	mm_data_TF_ERR.Z = _all_tensor_pcs[gg]->err_z;
	mm_data_TF_ERR.Ax = _all_tensor_pcs[gg]->err_ax;
	mm_data_TF_ERR.Rh = _all_tensor_pcs[gg]->err_rh;
    
    mm_data_UTR_ER.a = _all_tensor_pcs[gg]->err_a/M_PI*180.0;
	mm_data_UTR_ER.b = _all_tensor_pcs[gg]->err_b/M_PI*180.0;
	mm_data_UTR_ER.g = _all_tensor_pcs[gg]->err_g/M_PI*180.0;
	mm_data_UTR_ER.X = _all_tensor_pcs[gg]->err_x;
	mm_data_UTR_ER.Y = _all_tensor_pcs[gg]->err_y;
	mm_data_UTR_ER.Z = _all_tensor_pcs[gg]->err_z;
	mm_data_UTR_ER.Ax = _all_tensor_pcs[gg]->err_ax;
	mm_data_UTR_ER.Rh = _all_tensor_pcs[gg]->err_rh;
	
    
	fprintf(stdout, "+----------------------------------------------------------------+\n");
    fprintf(stdout, "|                Tensor found                                    |\n");
    fprintf(stdout, "+----------------------------------------------------------------+\n\n");	
    filZX_or_YZ(R, mm_data_TF.a, mm_data_TF.b, mm_data_TF.g);
	print_tensor(&mm_data_TF); 
	if(SING_OR_MULT == MULTIPLE){
		display_matrix(R, gg, M_TF);
	}
    if(SING_OR_MULT == SINGLE){
        display_matrix(R, gg, TF);
	}
	
	unique_tensor(&mm_data_UTR);//, &mm_data_UTR_ER);
	filZX_or_YZ(R, mm_data_UTR.a, mm_data_UTR.b, mm_data_UTR.g);
	fprintf(stdout, "\n");
	fprintf(stdout, "+----------------------------------------------------------------+\n");
  fprintf(stdout, "|                Tensor found in UTR                             |\n");
  fprintf(stdout, "+----------------------------------------------------------------+\n\n");
	print_tensor(&mm_data_UTR); 
	if(SING_OR_MULT == MULTIPLE){
		display_matrix(R, gg, M_UTR);
	}
    if(SING_OR_MULT == SINGLE){
        display_matrix(R, gg, UTR);
	}

	append_tensor(&mm_data_TF, &mm_data_TF_ERR, &mm_data_UTR, &mm_data_UTR_ER, 
                  gg, SING_OR_MULT, serial, new_node);
}

int display_matrix(double R[3][3], int n_button, int type_matrix){
	
	char label[50];
	char type[4];
	char ext[3];
	char txt[10];
	int l, c;
	GtkLabel * lab;
	GladeXML *gxml;	
	
	gxml = glade_get_widget_tree((GtkWidget*) _window);
	
	switch(type_matrix){
		case UTR: {
			sprintf(type, "UTR");
			ext[0] =  '\0';
		}
		break;
		case TF: {
			sprintf(type, "TF");
			ext[0] = '\0';
		}
		break;
		case M_UTR: {
			sprintf(type, "UTR");
			sprintf(ext, "M");
		}
		break;
		case M_TF: {
			sprintf(type, "TF");
			sprintf(ext, "M");
		}
		break;
		default:
		{
			return(-1);
		}
		break;
	}

	
	for(l = 0; l < 3; l++){
		for(c = 0; c < 3; c++){
			
			sprintf(label, "label_%s_%d%d_PCS%s%d", type, l, c, ext,n_button+1);
			lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
			sprintf(txt, "% .3lf ", R[l][c]);
    		gtk_label_set_text (lab, txt);
		}
	}
	return(1);
}


void display_matrix_init(void){
	char label[50];
	char type[5];
	char txt[10];
	int l, c, g;
	GtkLabel * lab;
	GladeXML *gxml;	
	
	gxml = glade_get_widget_tree((GtkWidget*) _window);
	
	sprintf(type, "UTR");
	for(g = 0; g < N_PCS_ARRAY; g++){
		for(l = 0; l < 3; l++){
			for(c = 0; c < 3; c++){		
				sprintf(label, "label_%s_%d%d_PCS%d", type, l, c, g+1);
				lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
				sprintf(txt, "?");
    			gtk_label_set_text (lab, txt);
			}	
		}
	}
	
	for(g = 0; g < N_PCS_ARRAY; g++){
		for(l = 0; l < 3; l++){
			for(c = 0; c < 3; c++){		
				sprintf(label, "label_%s_%d%d_PCSM%d", type, l, c, g+1);
				lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
				sprintf(txt, "?");
    			gtk_label_set_text (lab, txt);
			}	
		}
	}
	
	
	
	sprintf(type, "TF");
	for(g = 0; g < N_PCS_ARRAY; g++){
		for(l = 0; l < 3; l++){
			for(c = 0; c < 3; c++){
				sprintf(label, "label_%s_%d%d_PCS%d", type, l, c, g+1);
				lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
				sprintf(txt, "?");
    			gtk_label_set_text (lab, txt);
			}
		}
	}
	
	for(g = 0; g < N_PCS_ARRAY; g++){
		for(l = 0; l < 3; l++){
			for(c = 0; c < 3; c++){		
				sprintf(label, "label_%s_%d%d_PCSM%d", type, l, c, g+1);
				lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
				sprintf(txt, "?");
    			gtk_label_set_text (lab, txt);
			}	
		}
	}
	
	
}


void display_matrix_multi_init(void){
	char label[50];
	char type[5];
	char txt[10];
	char txt2[50];
	int l, c, g;
	GtkLabel * lab;
	GladeXML *gxml;	
	
	gxml = glade_get_widget_tree((GtkWidget*) _window);
	
	sprintf(type, "UTR");
	
	for(g = 0; g < N_PCS_ARRAY; g++){
		for(l = 0; l < 3; l++){
			for(c = 0; c < 3; c++){		
				sprintf(label, "label_%s_%d%d_PCSM%d", type, l, c, g+1);
				lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
				sprintf(txt, "?");
    			gtk_label_set_text (lab, txt);
				
				lab = get_correct_label(g, MULTIPLE);
				sprintf(txt2, "<b>Tensor PCS%d (id ?)</b>", g+1);
				gtk_label_set_text (lab, txt2);
                gtk_label_set_markup(lab, txt2);
			}	
		}
	}
	
	sprintf(type, "TF");
	
	for(g = 0; g < N_PCS_ARRAY; g++){
		for(l = 0; l < 3; l++){
			for(c = 0; c < 3; c++){		
				sprintf(label, "label_%s_%d%d_PCSM%d", type, l, c, g+1);
				lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
				sprintf(txt, "?");
    			gtk_label_set_text (lab, txt);
			}	
		}
	}
	
	
}

void set_visible_column(GtkTreeView * treeview, char * column_name, gboolean visible){
		GtkTreeViewColumn* column;
		GList * list_column;
		gboolean found;
		const gchar * title;
		int i;
		
		list_column = gtk_tree_view_get_columns (treeview);
		found = FALSE;
		for(i = 0; i < g_list_length(list_column); i++){
        column = (GtkTreeViewColumn *) g_list_nth_data (list_column, i);
        title = gtk_tree_view_column_get_title(column);
        if(strcmp(title, column_name)==0){
					found = TRUE;
					break;
				}
    }
		if(found == FALSE){
				fprintf(stderr, "Column not found, can't make it visible / unvisible\n");
		}
		else{
				gtk_tree_view_column_set_visible (column, visible);
		}
}

void more_tensor(int n_button, GtkButton * button){
		GtkTreeView * treeview;
		GladeXML *gxml;
		
    if(_pdb_str == NULL){
        //box_error( "You should first load a pdb file.\n", NULL);
        //return;
    }
    
    box_error( "This function is not yet working properly.\n\
Please, wait for the next version of Numbat.", NULL);
        return;

    
    gxml = glade_get_widget_tree(_window_tensor_work);
		treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview5");
		uncheck_all((GtkTreeModel *) _model_tl, COLUMN_USED_TL);
		uncheck_all((GtkTreeModel *) _model_tl, COLUMN_USED_TL2);
		set_visible_column(treeview, TITLE_COLUMN_USED_TL2, TRUE);
		
		update_tensor_work();
    gtk_widget_show(_window_tensor_work);
}

void ok_more_tensor(void){
	GtkTreeView * treeview;
	GladeXML *gxml;
		
  gxml = glade_get_widget_tree(_window_tensor_work);
	treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview5");
	
	set_visible_column(treeview, TITLE_COLUMN_USED_TL2, FALSE);
	gtk_widget_hide(_window_tensor_work);
}

void cancel_more_tensor(void){
		ok_more_tensor();
}

GtkTreeModel * create_tensor_work_model(void){
	GtkListStore *model;
    
  model = gtk_list_store_new (NUM_ITEM_COLUMNS_TENSOR_WORK, 
                              G_TYPE_INT, //intern id
                              G_TYPE_DOUBLE, //Ax
                              G_TYPE_DOUBLE, //Rh
                              G_TYPE_DOUBLE, //alpha
                              G_TYPE_DOUBLE, //beta
                              G_TYPE_DOUBLE //gamma
                              );

  return GTK_TREE_MODEL (model);
}

void init_more_tensor(void){
    GladeXML *gxml;
    gxml = glade_get_widget_tree(_window_tensor_work);
    GtkTreeView * treeview;
		GtkTreeModel * model;
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview5");
    add_columns_tl(treeview, TENSOR_ONLY_DUO);
    gtk_tree_view_set_model ((GtkTreeView *)treeview, GTK_TREE_MODEL (_model_tl));
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
	  set_multiple_selection(GTK_TREE_VIEW (treeview));
	  uncheck_all((GtkTreeModel *) _model_tl, COLUMN_USED_TL);
		GtkTreeModel * model_tensor_work1;
		GtkTreeModel * model_tensor_work2;
		
		model = create_angle_unit_model();
		_box_more_angle_unit = (GtkComboBox *)glade_xml_get_widget (gxml, "box_more_angle_unit");
		gtk_combo_box_set_model(_box_more_angle_unit, model);
		gtk_combo_box_set_active(_box_more_angle_unit, 0);
		g_object_unref (model);
		
		model = create_angle_orientation_model();
		_box_more_angle_orientation = (GtkComboBox *)glade_xml_get_widget (gxml, "box_more_angle_orientation");
		gtk_combo_box_set_model(_box_more_angle_orientation, model);
		gtk_combo_box_set_active(_box_more_angle_orientation, 0);
		g_object_unref (model);
		
		model = create_rotation_order_model();
		_box_more_rotation_order = (GtkComboBox *)glade_xml_get_widget (gxml, "box_more_rotation_order");
		gtk_combo_box_set_model(_box_more_rotation_order, model);
		gtk_combo_box_set_active(_box_more_rotation_order, 0);
		g_object_unref (model);
		
		model = create_magnitude_convention_model();
		_box_more_magnitude_convention = (GtkComboBox *)glade_xml_get_widget (gxml, "box_more_magnitude_convention");
		gtk_combo_box_set_model(_box_more_magnitude_convention, model);
		gtk_combo_box_set_active(_box_more_magnitude_convention, 0);
		g_object_unref (model);
		
		_treeview_more_tensor1 = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_more_tensor1");
		_treeview_more_tensor2 = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_more_tensor2");
		add_columns_tensor_work (_treeview_more_tensor1);
		add_columns_tensor_work (_treeview_more_tensor2);
		
		model_tensor_work1 = create_tensor_work_model();
		gtk_tree_view_set_model (GTK_TREE_VIEW (_treeview_more_tensor1), model_tensor_work1);
		model_tensor_work2 = create_tensor_work_model();
		gtk_tree_view_set_model (GTK_TREE_VIEW (_treeview_more_tensor2), model_tensor_work2);
		
		reinit_matrix_TW(1);
		reinit_matrix_TW(2);
		reinit_xyz_coordinate_TW(1);
		reinit_xyz_coordinate_TW(2);
        reinit_anisotropic_TW(1);
        reinit_anisotropic_TW(2);

}


void add_manual_more_tensor(void){
    GladeXML *gxml;
    double x, y, z, Ax, Rh, a, b, g;
    
    gxml = glade_get_widget_tree(_window_tensor_work);

    x = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_x3"));
    y = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_y3"));
    z = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_z3"));
    a = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_alpha3"));
    b = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_beta3"));
    g = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_gamma3"));
    Ax = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Ax3"));
    Rh = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Rh3"));
    add_to_tensor_listing(Ax, Rh, x, y, z, a, b, g, MANUAL_TENSOR, -1,
						   0, 0, 0, 0, 0, 0, 0, 0);
}


void copy_more_tensor(void){
  GtkTreeView *treeview;
  GladeXML *gxml;  
	
  gxml = glade_get_widget_tree(_window_tensor_work);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview5"); 
  copy_tensor_from_list(gxml, treeview, 3);
}

void remove_more_tensor(void){
  GtkTreeView *treeview;
  GladeXML *gxml;  
    
  gxml = glade_get_widget_tree(_window_tensor_work);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview5");
  remove_tensor_from_list(gxml, treeview, 3);
	update_tensor_work();
}


GtkTreeModel * create_angle_unit_model(void){
  GtkListStore *model;
  GtkTreeIter iter;
  model = gtk_list_store_new (NUM_COLUMNS_ANGLE_UNIT, 
							  G_TYPE_STRING,
							  G_TYPE_INT
							  );
  gtk_list_store_append (model, &iter);
  gtk_list_store_set (model, &iter,
						  COLUMN_ANGLE_UNIT_STR, DEGREE_CODE_STR,
						  COLUMN_ANGLE_UNIT_INT, DEGREE_CODE_INT,				
                          -1);
	gtk_list_store_append (model, &iter);
	gtk_list_store_set (model, &iter,
						  COLUMN_ANGLE_UNIT_STR, RADIAN_CODE_STR,
						  COLUMN_ANGLE_UNIT_INT, RADIAN_CODE_INT,				
                       		-1);
  return GTK_TREE_MODEL (model);
}

GtkTreeModel * create_angle_orientation_model(void){
  GtkListStore *model;
  GtkTreeIter iter;
  model = gtk_list_store_new (NUM_COLUMNS_ANGLE_ORIENTATION, 
							  G_TYPE_STRING,
							  G_TYPE_INT
							  );
	gtk_list_store_append (model, &iter);
	gtk_list_store_set (model, &iter,
						  COLUMN_ANGLE_ORIENTATION_STR, RIGHT_HANDED_CODE_STR,
						  COLUMN_ANGLE_ORIENTATION_INT, RIGHT_HANDED_CODE_INT,				
                       		-1);
  gtk_list_store_append (model, &iter);
  gtk_list_store_set (model, &iter,
						  COLUMN_ANGLE_ORIENTATION_STR, LEFT_HANDED_CODE_STR,
						  COLUMN_ANGLE_ORIENTATION_INT, LEFT_HANDED_CODE_INT,				
                          -1);

  return GTK_TREE_MODEL (model);
}

GtkTreeModel * create_rotation_order_model(void){
  GtkListStore *model;
  GtkTreeIter iter;
  model = gtk_list_store_new (NUM_COLUMNS_ROTATION_ORDER, 
							  G_TYPE_STRING,
							  G_TYPE_INT
							  );
  gtk_list_store_append (model, &iter);
  gtk_list_store_set (model, &iter,
						  COLUMN_ROTATION_ORDER_STR, ZYZ_CODE_STR,
						  COLUMN_ROTATION_ORDER_INT, ZYZ_CODE_INT,				
                          -1);
	gtk_list_store_append (model, &iter);
	gtk_list_store_set (model, &iter,
						  COLUMN_ROTATION_ORDER_STR, ZXZ_CODE_STR,
						  COLUMN_ROTATION_ORDER_INT, ZXZ_CODE_INT,				
                       		-1);
  return GTK_TREE_MODEL (model);
}

GtkTreeModel * create_magnitude_convention_model(void){
  GtkListStore *model;
  GtkTreeIter iter;
  model = gtk_list_store_new (NUM_COLUMNS_MAGNITUDE_CONVENTION, 
							  G_TYPE_STRING,
							  G_TYPE_INT
							  );
  gtk_list_store_append (model, &iter);
  gtk_list_store_set (model, &iter,
						  COLUMN_MAGNITUDE_CONVENTION_STR, Z_Y_X_CODE_STR,
						  COLUMN_MAGNITUDE_CONVENTION_INT, Z_Y_X_CODE_INT,				
                          -1);
	gtk_list_store_append (model, &iter);
	gtk_list_store_set (model, &iter,
						  COLUMN_MAGNITUDE_CONVENTION_STR, NONE_CODE_STR,
						  COLUMN_MAGNITUDE_CONVENTION_INT, NONE_CODE_INT,				
                       		-1);
  return GTK_TREE_MODEL (model);
}

void force_UTR(void){
		gtk_combo_box_set_active(_box_more_angle_unit, 0);
		gtk_combo_box_set_active(_box_more_angle_orientation, 0);
		gtk_combo_box_set_active(_box_more_rotation_order, 0);
		gtk_combo_box_set_active(_box_more_magnitude_convention, 0);
		update_tensor_work();
}

void change_polish_radian_degree(int rad_or_deg, GtkTreeView * treeview){
    GtkTreeViewColumn * tree_column;
    GList * list_renderer;
    GtkCellRenderer *renderer; 

    if(_treeview_more_tensor1 == NULL){
        return;
    }
    //ALPHA
    tree_column =  gtk_tree_view_get_column (treeview, COLUMN_TENSOR_WORK_ALPHA - 1);
     
    list_renderer = gtk_tree_view_column_get_cell_renderers (tree_column);
    if(g_list_length(list_renderer)==0){
        g_list_free(list_renderer);
        return;
    }
    renderer = (GtkCellRenderer *) g_list_nth_data(list_renderer, 0);
    
    if(rad_or_deg == RADIAN_CODE_INT){
    gtk_tree_view_column_set_cell_data_func(tree_column, renderer, polish_float_cell_data_function_tl_radian, NULL, NULL);
    }
    else{
        gtk_tree_view_column_set_cell_data_func(tree_column, renderer, polish_float_cell_data_function_tl, NULL, NULL);
    }
    //BETA
        tree_column =  gtk_tree_view_get_column (treeview, COLUMN_TENSOR_WORK_BETA - 1);
     
    list_renderer = gtk_tree_view_column_get_cell_renderers (tree_column);
    if(g_list_length(list_renderer)==0){
        g_list_free(list_renderer);
        return;
    }
    renderer = (GtkCellRenderer *) g_list_nth_data(list_renderer, 0);
    
    if(rad_or_deg == RADIAN_CODE_INT){
    gtk_tree_view_column_set_cell_data_func(tree_column, renderer, polish_float_cell_data_function_tl_radian, NULL, NULL);
    }
    else{
        gtk_tree_view_column_set_cell_data_func(tree_column, renderer, polish_float_cell_data_function_tl, NULL, NULL);
    }
    //GAMMA
        tree_column =  gtk_tree_view_get_column (treeview, COLUMN_TENSOR_WORK_GAMMA - 1);
     
    list_renderer = gtk_tree_view_column_get_cell_renderers (tree_column);
    if(g_list_length(list_renderer)==0){
        g_list_free(list_renderer);
        return;
    }
    renderer = (GtkCellRenderer *) g_list_nth_data(list_renderer, 0);
    
    if(rad_or_deg == RADIAN_CODE_INT){
    gtk_tree_view_column_set_cell_data_func(tree_column, renderer, polish_float_cell_data_function_tl_radian, NULL, NULL);
    }
    else{
        gtk_tree_view_column_set_cell_data_func(tree_column, renderer, polish_float_cell_data_function_tl, NULL, NULL);
    }
}

void update_tensor_work(void){
		gint angle_unit_int;
		gint angle_orientation_int;
		gint rotation_order_int;
		gint magnitude_convention_int;
		GtkTreeModel*  model;
		GtkTreeIter iter;
		gboolean my_bool;
		gboolean is_toggle;
		double x1, y1, z1, Ax1, Rh1, a1, b1, g1;
		double x2, y2, z2, Ax2, Rh2, a2, b2, g2;
        
		
		if((_box_more_angle_unit == NULL)||(_box_more_angle_orientation == NULL)||
			 (_box_more_magnitude_convention == NULL)||(_box_more_rotation_order == NULL)){
				return;
		}
		
		model = gtk_combo_box_get_model  (_box_more_angle_unit);
		if(model == NULL){ return; }
		gtk_combo_box_get_active_iter (_box_more_angle_unit, &iter);
  	gtk_tree_model_get (model, &iter, COLUMN_ANGLE_UNIT_INT, &angle_unit_int, -1);
		
		model = gtk_combo_box_get_model  (_box_more_angle_orientation);
		if(model == NULL){ return; }
		gtk_combo_box_get_active_iter (_box_more_angle_orientation, &iter);
  	gtk_tree_model_get (model, &iter, COLUMN_ANGLE_ORIENTATION_INT, &angle_orientation_int, -1);
		
		model = gtk_combo_box_get_model  (_box_more_rotation_order);
		if(model == NULL){ return; }
		gtk_combo_box_get_active_iter (_box_more_rotation_order, &iter);
  	gtk_tree_model_get (model, &iter, COLUMN_ROTATION_ORDER_INT, &rotation_order_int, -1);
		
		model = gtk_combo_box_get_model  (_box_more_magnitude_convention);
		if(model == NULL){ return; }
		gtk_combo_box_get_active_iter (_box_more_magnitude_convention, &iter);
  	gtk_tree_model_get (model, &iter, COLUMN_MAGNITUDE_CONVENTION_INT, &magnitude_convention_int, -1);
		
		fprintf(stderr, "%d %d %d %d\n", angle_unit_int, angle_orientation_int,
						rotation_order_int, magnitude_convention_int);
		
		//////////
		
		if(_model_tl == NULL){
				return;
		}
		
		is_toggle = FALSE;
		my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)_model_tl, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (_model_tl, &iter, COLUMN_USED_TL, &is_toggle, -1); 
        if (is_toggle == TRUE){
            break;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)_model_tl, &iter);
    }
    
    if(is_toggle == FALSE){
				reinit_matrix_TW(1);
				reinit_xyz_coordinate_TW(1);
                reinit_list_TW(1);
                reinit_anisotropic_TW(1);
        //unvalidate_tensor_work1();
    }
		else{
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_X_TL, &x1, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_Y_TL, &y1, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_Z_TL, &z1, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_AX_TL, &Ax1, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_RH_TL, &Rh1, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_ALPHA_TL, &a1, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_BETA_TL, &b1, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_GAMMA_TL, &g1, -1);
			//fprintf(stderr, "TW1 %lf %lf %lf %lf %lf %lf %lf %lf\n",
				//			x1, y1, z1, Ax1, Rh1, a1, b1, g1);
				set_up_xyz_coordinate_TW(1, x1, y1, z1);
				add_to_TW(1, 0, Ax1,  Rh1,  a1,  b1, g1,
                          rotation_order_int, angle_orientation_int, magnitude_convention_int);
		}
		
		
		is_toggle = FALSE;
		my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)_model_tl, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (_model_tl, &iter, COLUMN_USED_TL2, &is_toggle, -1); 
        if (is_toggle == TRUE){
            break;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)_model_tl, &iter);
    }
    
    if(is_toggle == FALSE){
				reinit_matrix_TW(2);
				reinit_xyz_coordinate_TW(2);
                reinit_list_TW(2);
                reinit_anisotropic_TW(2);
        //unvalidate_tensor_work2();
    }
		else{
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_X_TL, &x2, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_Y_TL, &y2, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_Z_TL, &z2, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_AX_TL, &Ax2, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_RH_TL, &Rh2, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_ALPHA_TL, &a2, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_BETA_TL, &b2, -1);
    	gtk_tree_model_get(_model_tl, &iter, COLUMN_GAMMA_TL, &g2, -1);
			//fprintf(stderr, "TW2 %lf %lf %lf %lf %lf %lf %lf %lf\n",
				//			x2, y2, z2, Ax2, Rh2, a2, b2, g2);
				
			set_up_xyz_coordinate_TW(2, x2, y2, z2);
            
			add_to_TW(2, 0, Ax2,  Rh2,  a2,  b2, g2,
                          rotation_order_int, angle_orientation_int, magnitude_convention_int);
		}
    
    change_polish_radian_degree(angle_unit_int, _treeview_more_tensor1);
    change_polish_radian_degree(angle_unit_int, _treeview_more_tensor2);
}


// add columns to the treeview.
void
add_columns_tensor_work (GtkTreeView  *treeview)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;

  // Ax 
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_TENSOR_WORK_AX));   
  column = gtk_tree_view_column_new_with_attributes (STRING_DELTA_CHI_AX_ALTERNATIF,
						     renderer,
						     "text",
						     COLUMN_TENSOR_WORK_AX,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_TENSOR_WORK_AX);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function_tl, NULL, NULL); 
  //change_column_properties_tl(column);
  
  gtk_tree_view_append_column (treeview, column);
    gtk_tree_view_column_set_visible (column, FALSE);
    
  // Rh
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_TENSOR_WORK_RH));   
  column = gtk_tree_view_column_new_with_attributes (STRING_DELTA_CHI_RH_ALTERNATIF,
						     renderer,
						     "text",
						     COLUMN_TENSOR_WORK_RH,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_TENSOR_WORK_RH);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function_tl, NULL, NULL);  
  //change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
  gtk_tree_view_column_set_visible (column, FALSE);

  // alpha
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_TENSOR_WORK_ALPHA));   
  column = gtk_tree_view_column_new_with_attributes (STRING_ALPHA,
						     renderer,
						     "text",
						     COLUMN_TENSOR_WORK_ALPHA,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_TENSOR_WORK_ALPHA);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function_tl, NULL, NULL);  
  //change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
    
  // beta 
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_TENSOR_WORK_BETA));   
  column = gtk_tree_view_column_new_with_attributes (STRING_BETA,
						     renderer,
						     "text",
						     COLUMN_TENSOR_WORK_BETA,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_TENSOR_WORK_BETA);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function_tl, NULL, NULL);  
  //change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
    
  // gamma 
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_TENSOR_WORK_GAMMA));   
  column = gtk_tree_view_column_new_with_attributes (STRING_GAMMA,
						     renderer,
						     "text",
						     COLUMN_TENSOR_WORK_GAMMA,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_TENSOR_WORK_GAMMA);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function_tl, NULL, NULL);  
  //change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
}

void reinit_list_TW(int number_one_or_two){
    GladeXML * gxml;
	gxml = glade_get_widget_tree(_window_tensor_work);
	GtkTreeModel * model;
	if((number_one_or_two != 1)&&(number_one_or_two != 2)){
		return;
	}
    
    if(_treeview_more_tensor1 == NULL){
        return;
    }
    if(_treeview_more_tensor2 == NULL){
        return;
    }
    
    if(number_one_or_two == 1){
        model = gtk_tree_view_get_model(_treeview_more_tensor1);
        empty_tree_model_general((GtkTreeModel *) model);
    }
    
    if(number_one_or_two == 2){
        model = gtk_tree_view_get_model(_treeview_more_tensor2);
        empty_tree_model_general((GtkTreeModel *) model);
    }
}

void reinit_matrix_TW(int number_one_or_two){
		GladeXML * gxml;
		int l, c;
		GtkLabel * lab;
		gxml = glade_get_widget_tree(_window_tensor_work);
		char label[50];
		if((number_one_or_two != 1)&&(number_one_or_two != 2)){
				return;
		}
		for(l = 0; l < 3; l++){
			for(c = 0; c < 3; c++){
				sprintf(label, "label_%d%d_TW%d", l, c, number_one_or_two);
				lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
    		gtk_label_set_text (lab, "?");
			}	
		}
}

void reinit_anisotropic_TW(int number_one_or_two){
		GladeXML * gxml;
		GtkLabel * lab;
		gxml = glade_get_widget_tree(_window_tensor_work);
		char label[50];
		if((number_one_or_two != 1)&&(number_one_or_two != 2)){
				return;
		}
    
        sprintf(label, "label_Ax_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
        gtk_label_set_text (lab, "?");
		
		sprintf(label, "label_Rh_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
        gtk_label_set_text (lab, "?");
		
		sprintf(label, "label_Xx_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
        gtk_label_set_text (lab, "?");
		
		sprintf(label, "label_Xy_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
        gtk_label_set_text (lab, "?");
		
		sprintf(label, "label_Xz_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
        gtk_label_set_text (lab, "?");
}

void reinit_xyz_coordinate_TW(int number_one_or_two){
		GladeXML * gxml;
		GtkLabel * lab;
		gxml = glade_get_widget_tree(_window_tensor_work);
		char label[50];
		if((number_one_or_two != 1)&&(number_one_or_two != 2)){
				return;
		}
		
		sprintf(label, "label_x_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
    gtk_label_set_text (lab, "?");
		
		sprintf(label, "label_y_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
    gtk_label_set_text (lab, "?");
		
		sprintf(label, "label_z_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
    gtk_label_set_text (lab, "?");
		
}

void set_up_anisotropy_parameters_TW( int number_one_or_two, double Ax, double Rh){
   		GladeXML * gxml;
		GtkLabel * lab;
        double Xx, Xy, Xz;
		gxml = glade_get_widget_tree(_window_tensor_work);
		char label[50];
        char txt[100];
		if((number_one_or_two != 1)&&(number_one_or_two != 2)){
            return;
		}
    
        sprintf(label, "label_Ax_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
        sprintf(txt, "% .3lf ", Ax);
        gtk_label_set_text (lab, txt);
    
        sprintf(label, "label_Rh_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
        sprintf(txt, "% .3lf ", Rh);
        gtk_label_set_text (lab, txt);
    
    Xx = -Ax/3.0 + Rh/2.0;
    Xy = -Ax/3.0 - Rh/2.0;
    Xz = 2.0/3.0*Ax;
    
        sprintf(label, "label_Xx_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
        sprintf(txt, "% .3lf ", Xx);
        gtk_label_set_text (lab, txt);
    
        sprintf(label, "label_Xy_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
        sprintf(txt, "% .3lf ", Xy);
        gtk_label_set_text (lab, txt);
    
        sprintf(label, "label_Xz_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
        sprintf(txt, "% .3lf ", Xz);
        gtk_label_set_text (lab, txt);
}

void set_up_xyz_coordinate_TW(int number_one_or_two, double x, double y, double z){
		GladeXML * gxml;
		GtkLabel * lab;
		gxml = glade_get_widget_tree(_window_tensor_work);
		char label[50];
		char txt[100];
		if((number_one_or_two != 1)&&(number_one_or_two != 2)){
				return;
		}
		
		sprintf(label, "label_x_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
		sprintf(txt, "% .3lf ", x);
        gtk_label_set_text (lab, txt);
		
		sprintf(label, "label_y_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
		sprintf(txt, "% .3lf ", y);
    gtk_label_set_text (lab, txt);
		
		sprintf(label, "label_z_TW%d", number_one_or_two);
		lab = (GtkLabel *) glade_xml_get_widget (gxml, label);
		sprintf(txt, "% .3lf ", z);
    gtk_label_set_text (lab, txt);
		
}

void empty_tree_model_general(GtkTreeModel * model){
    GtkTreeIter iter;
    GtkTreePath *path;
    gboolean test_bool;
    gint i;
    
    if(model == NULL){
        return;
    }
    test_bool = gtk_tree_model_get_iter_first(model, &iter);

    while(test_bool!= FALSE){
        path = gtk_tree_model_get_path (model, &iter);
        i = gtk_tree_path_get_indices (path)[0];
        gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
        //g_array_remove_index (_articles_CH3, i);
        gtk_tree_path_free (path);
        test_bool =  gtk_tree_model_get_iter_first(model, &iter);
    }
}

void add_to_TW(int number_one_or_two, int intern_id, double Ax, double Rh, 
							double alpha, double beta, double gamma,
               int rotation_order_int, int angle_orientation_int, int magnitude_convention_int)
{
		GladeXML * gxml;
		GtkTreeView *treeview;
		GtkTreeModel * model;
		GtkTreeIter iter;
        GtkTreePath * path;
        molmol_data mm_data_origin;
        molmol_data mm_data_8[8];
		int i;
    
		if((number_one_or_two != 1)&&(number_one_or_two != 2)){
				return;
		}
		gxml = glade_get_widget_tree(_window_tensor_work);
		
		switch(number_one_or_two){
				case 1:{
						treeview = _treeview_more_tensor1;
				}
				break;
				case 2:{
						treeview = _treeview_more_tensor2;
				}
				break;
				default:{
						return;
				}
		}
		if(treeview == NULL){
				return;
		}
    model = (GtkTreeModel*) gtk_tree_view_get_model  (treeview);
		if(model == NULL){
				return;
		}		
		empty_tree_model_general(model);
    
    
    mm_data_origin.a = alpha;
    mm_data_origin.b = beta;
    mm_data_origin.g = gamma;
    mm_data_origin.Ax = Ax;
    mm_data_origin.Rh = Rh;
    
    calculate_8_tensors_from_convention( &mm_data_origin, mm_data_8, 
            rotation_order_int,  angle_orientation_int,  magnitude_convention_int);
    set_up_anisotropy_parameters_TW(number_one_or_two, mm_data_8[0].Ax, mm_data_8[0].Rh);
    for(i = 0; i < 8; i++){
        gtk_list_store_prepend ((GtkListStore * ) model, &iter);
        gtk_list_store_set ((GtkListStore * )model, &iter,
                          COLUMN_TENSOR_WORK_INTERN_ID, intern_id,
  												COLUMN_TENSOR_WORK_AX, mm_data_8[i].Ax,
  												COLUMN_TENSOR_WORK_RH, mm_data_8[i].Rh,
  												COLUMN_TENSOR_WORK_ALPHA, mm_data_8[i].a,
  												COLUMN_TENSOR_WORK_BETA, mm_data_8[i].b,
  												COLUMN_TENSOR_WORK_GAMMA, mm_data_8[i].g, 
                          -1);
    }
    path = gtk_tree_model_get_path ((GtkTreeModel *) model, &iter);
    gtk_tree_view_set_cursor (treeview, path, NULL, FALSE);
    gtk_tree_path_free (path);
}
