/*
 * tensor.h
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#define MULTIPLE_PCS_N_BUTTON -3

enum
{
  UPDATE_PCS_CALC,
  UPDATE_PCS_EXP  
};

enum
{
    TYPE_NONE,
    TYPE_FIXED,
    TYPE_BOUNDARIES
};


typedef struct
{
    double ax;
    double rh;
    double x;
    double y;
    double z;
    double a;
    double b;
    double g;
    double err_ax;
    double err_rh;
    double err_x;
    double err_y;
    double err_z;
    double err_a;
    double err_b;
    double err_g;
} 
tensor;

tensor *_all_tensor_pcs[N_PCS_ARRAY];

typedef struct
{
    double start;
    double min;
    double max;
    int type;
}
boundarie;

typedef struct
{
    boundarie b_Ax;
    boundarie b_Rh;
    boundarie b_x;
    boundarie b_y;
    boundarie b_z;
    boundarie b_a;
    boundarie b_b;
    boundarie b_g;
}
boundaries;

typedef struct 
{
    int num_model;
    gboolean is_used;
} pdb_selector;

GArray * _all_pdb_selector_pcs[N_PCS_ARRAY];

GArray * _multiple_pcs_pdb_selector_pcs;

boundaries * _all_boundaries_pcs[N_PCS_ARRAY];

boundaries * _multiple_pcs_boundaries_pcs;


////FOR ANGLE UNIT COMBOBOX
enum{
	COLUMN_ANGLE_UNIT_STR,
	COLUMN_ANGLE_UNIT_INT,
	NUM_COLUMNS_ANGLE_UNIT
};
enum{
		DEGREE_CODE_INT,
		RADIAN_CODE_INT
};
#define DEGREE_CODE_STR "Degree"
#define RADIAN_CODE_STR "Radian"

////FOR ANGLE ORIENTATION COMBOBOX
enum{
	COLUMN_ANGLE_ORIENTATION_STR,
	COLUMN_ANGLE_ORIENTATION_INT,
	NUM_COLUMNS_ANGLE_ORIENTATION
};
enum{
		RIGHT_HANDED_CODE_INT,
		LEFT_HANDED_CODE_INT
};
#define LEFT_HANDED_CODE_STR "Left handed"
#define RIGHT_HANDED_CODE_STR "Right handed"

////FOR ROTATION ORDER COMBOBOX
enum{
	COLUMN_ROTATION_ORDER_STR,
	COLUMN_ROTATION_ORDER_INT,
	NUM_COLUMNS_ROTATION_ORDER
};
enum{
		ZYZ_CODE_INT,
		ZXZ_CODE_INT
};
#define ZYZ_CODE_STR "ZYZ"
#define ZXZ_CODE_STR "ZXZ"

////FOR MAGNITUDE CONVENTION COMBOBOX
enum{
	COLUMN_MAGNITUDE_CONVENTION_STR,
	COLUMN_MAGNITUDE_CONVENTION_INT,
	NUM_COLUMNS_MAGNITUDE_CONVENTION
};
enum{
		Z_Y_X_CODE_INT,
		NONE_CODE_INT
};
#define Z_Y_X_CODE_STR STRING_MAGNITUDE_CONVENTION_dXz_dXy_dXx
#define NONE_CODE_STR "NONE"

GtkTreeModel *
create_boundaries_model (void);
void free_tensor(tensor *);
tensor * new_tensor(void);
void init_tensor(void);
GtkTreeModel *
create_type_boundarie_model (void);
void all_none_clicked_pdb_selector(int n_button, gboolean T_F);
void display_boundaries_list(void);
void update_PDB_selector(void);
void display_PDB_selector(int n_button);
void
add_columns_boundaries (GtkTreeView  *treeview, 
             GtkTreeModel *items_model,
             GtkTreeModel *numbers_model, int n_button);

boundaries * new_boundaries(void);
void init_PDB_selector(void);
void
boundaries_edited_n (GtkCellRendererText *cell,
             const gchar         *path_string,
             const gchar         *new_text,
             gpointer             data, int n_button);
void update_boundaries(GtkTreeIter * iter, GtkTreeModel *model, int n_button);
int type_string_to_int(char * type);

int calculate_pcs(tensor * my_tensor, int n_button, int pcs_type_2_update, double noise, int seed
                  , gboolean use_racs, racs_data * racs_d);


double distance_atom( double x1, double y1, double z1, double x2, double y2, double z2);

void print_pdb_selector(int n_button);


double calculate_average_distance_family(int * index_on_pdb, int * index_on_pcs, int index_global, int n_button, double xM, double yM, double zM);
int display_matrix(double R[3][3], int n_button, int type_matrix);
void display_tensor(int gg, int SING_OR_MULT, int serial, gboolean new_node);
void display_matrix_init(void);

void ok_more_tensor(void);
void cancel_more_tensor(void);
void more_tensor(int n_button, GtkButton * button);
void remove_more_tensor(void);
void add_manual_more_tensor(void);
void copy_more_tensor(void);
void init_more_tensor(void);
void display_matrix_multi_init(void);
GtkTreeModel * create_angle_unit_model(void);
GtkTreeModel * create_angle_orientation_model(void);
GtkTreeModel * create_rotation_order_model(void);
GtkTreeModel * create_magnitude_convention_model(void);
void force_UTR(void);
void
add_columns_tensor_work (GtkTreeView  *treeview);
void update_tensor_work(void);
void reinit_xyz_coordinate_TW(int number_one_or_two);
void reinit_matrix_TW(int number_one_or_two);
void set_up_xyz_coordinate_TW(int number_one_or_two, double x, double y, double z);
void add_to_TW(int number_one_or_two, int intern_id, double Ax, double Rh, 
							double alpha, double beta, double gamma,
               int rotation_order_int, int angle_orientation_int, int magnitude_convention_int);
void empty_tree_model_general(GtkTreeModel * model);
void reinit_list_TW(int number_one_or_two);
void reinit_anisotropic_TW(int number_one_or_two);
