/*
 *tensor_found.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>


#include "pdb.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "error_analysis.h"
#include "pcsfit_metal.h"
#include "tensor_list.h"
#include "molmol_iso.h"
#include "tensor_found.h"
#include "common.h"




extern GtkWidget * _window;
GtkTreeView * _treeview_PCS1;
GtkTreeView * _treeview_PCS2;
GtkTreeView * _treeview_PCS3;
GtkTreeView * _treeview_PCS4;
GtkTreeView * _treeview_PCSM;

GtkTreeModel * _model_PCS1;
GtkTreeModel * _model_PCS2;
GtkTreeModel * _model_PCS3;
GtkTreeModel * _model_PCS4;
GtkTreeModel * _model_PCSM;

GtkLabel * label_tensor_1;
GtkLabel * label_tensor_2;
GtkLabel * label_tensor_3;
GtkLabel * label_tensor_4;
GtkLabel * label_tensor_M1;
GtkLabel * label_tensor_M2;
GtkLabel * label_tensor_M3;
GtkLabel * label_tensor_M4;

void init_TF(void){
	GladeXML *gxml;	
	gxml = glade_get_widget_tree(_window);
	_treeview_PCS1 = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_tensor_found_PCS1");
	_treeview_PCS2 = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_tensor_found_PCS2");
	_treeview_PCS3 = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_tensor_found_PCS3");
	_treeview_PCS4 = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_tensor_found_PCS4");
	_treeview_PCSM = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview_tensor_found_PCSM");
	_model_PCS1 = create_model_tensor_found();
	_model_PCS2 = create_model_tensor_found();
	_model_PCS3 = create_model_tensor_found();
	_model_PCS4 = create_model_tensor_found();
	_model_PCSM = create_model_tensor_found();
	add_columns_TF(_treeview_PCS1);
	add_columns_TF(_treeview_PCS2);
	add_columns_TF(_treeview_PCS3);
	add_columns_TF(_treeview_PCS4);
	add_columns_TF(_treeview_PCSM);
	gtk_tree_view_set_model (_treeview_PCS1, _model_PCS1);
	gtk_tree_view_set_model (_treeview_PCS2, _model_PCS2);
	gtk_tree_view_set_model (_treeview_PCS3, _model_PCS3);
	gtk_tree_view_set_model (_treeview_PCS4, _model_PCS4);
	gtk_tree_view_set_model (_treeview_PCSM, _model_PCSM);
	gtk_tree_view_set_rules_hint (_treeview_PCS1, TRUE);
	gtk_tree_view_set_rules_hint (_treeview_PCS2, TRUE);
	gtk_tree_view_set_rules_hint (_treeview_PCS3, TRUE);
	gtk_tree_view_set_rules_hint (_treeview_PCS4, TRUE);
	gtk_tree_view_set_rules_hint (_treeview_PCSM, TRUE);
	label_tensor_1 = (GtkLabel *) glade_xml_get_widget (gxml, "label_tensor_1");
	label_tensor_2 = (GtkLabel *) glade_xml_get_widget (gxml, "label_tensor_2");
	label_tensor_3 = (GtkLabel *) glade_xml_get_widget (gxml, "label_tensor_3");
	label_tensor_4 = (GtkLabel *) glade_xml_get_widget (gxml, "label_tensor_4");
	label_tensor_M1 = (GtkLabel *) glade_xml_get_widget (gxml, "label_tensor_M1");
	label_tensor_M2 = (GtkLabel *) glade_xml_get_widget (gxml, "label_tensor_M2");
	label_tensor_M3 = (GtkLabel *) glade_xml_get_widget (gxml, "label_tensor_M3");
	label_tensor_M4 = (GtkLabel *) glade_xml_get_widget (gxml, "label_tensor_M4");
    gtk_label_set_use_markup (label_tensor_M1, TRUE);
}

GtkTreeModel *
create_model_tensor_found (void)
{
  GtkTreeStore *model;

  /* create tree store */
  model = gtk_tree_store_new (NUM_COLUMNS_TF,
			      G_TYPE_STRING, //Name of parameter
			      G_TYPE_DOUBLE, //found
				  G_TYPE_DOUBLE, //error
				  G_TYPE_DOUBLE, //UTR
                  G_TYPE_DOUBLE, //UTR_ERR
				  G_TYPE_BOOLEAN //visible
			      );	
  return GTK_TREE_MODEL (model);
}

void append_tensor(molmol_data * mm_data_TF, molmol_data * mm_data_ERR,
				   molmol_data * mm_data_UTR, molmol_data * mm_data_UTR_ERR,
                   int n_button, int SING_OR_MULT,
				   int serial, gboolean new_node)

{
	GtkTreeStore * model;
 	GtkTreeIter iter;
  	GtkTreeIter child_iter;
	GtkTreeIter * iter_2_append;
	GtkLabel * label;
	GtkTreePath * path;
	GtkTreeView * treeview;
	char name_t[50];
	static char name_node[150];
	static GtkTreeIter parent_iter;
	
	model  = (GtkTreeStore * ) get_correct_model(n_button, SING_OR_MULT);
	treeview = get_correct_treeview(n_button, SING_OR_MULT);
	label  = (GtkLabel * ) get_correct_label(n_button, SING_OR_MULT);
	if(model == NULL){
		fprintf(stderr, "pbm with switch for model choice\n");
		return;
	}
	if(label == NULL){
		fprintf(stderr, "pbm with switch for label choice\n");
		return;
	}
	if(SING_OR_MULT == MULTIPLE){
		sprintf(name_t,"<b>Tensor PCS%d (id: %d)</b>", n_button + 1, serial);
		
	}
	else{
		sprintf(name_t,"<b>Tensor id: %d</b>", serial);
	}
	gtk_label_set_text(label, name_t);
	gtk_label_set_markup(label,name_t);
    
	
	if(SING_OR_MULT == MULTIPLE){
		if(new_node == TRUE){
			sprintf(name_node, "Tensor id: %d", serial);
			gtk_tree_store_prepend (model, &parent_iter, NULL);
			gtk_tree_store_set (model, &parent_iter,
						COLUMN_NAME_TF, name_node,
  						COLUMN_TF, 0.0,
						COLUMN_ER, 0.0,
  						COLUMN_UTR, 0.0,
              COLUMN_UTR_ER, 0.0,
  						VISIBLE_COLUMN, FALSE,
						-1);
			iter_2_append = &parent_iter;
		}
		else{
			char txt[30];
			sprintf(txt, "\nTensor id: %d", serial);
			strcat(name_node, txt);
			iter_2_append = &parent_iter;
			gtk_tree_store_set (model, &parent_iter,
						COLUMN_NAME_TF, name_node, -1);
		}
	}
	else{
		iter_2_append = NULL;
	}
	if(SING_OR_MULT == MULTIPLE){
		sprintf(name_t,"Tensor id: %d", serial);
		gtk_tree_store_append (model, &iter, iter_2_append);
		gtk_tree_store_set (model, &iter,
						COLUMN_NAME_TF, name_t,
  						COLUMN_TF, 0.0,
						COLUMN_ER, 0.0,
  						COLUMN_UTR, 0.0,
              COLUMN_UTR_ER, 0.0,
  						VISIBLE_COLUMN, FALSE,
						-1);
	}
	else{
		sprintf(name_t,"Tensor id: %d", serial);
		gtk_tree_store_prepend (model, &iter, iter_2_append);
		gtk_tree_store_set (model, &iter,
						COLUMN_NAME_TF, name_t,
  						COLUMN_TF, 0.0,
							COLUMN_ER, 0.0,
  						COLUMN_UTR, 0.0,
              COLUMN_UTR_ER, 0.0,
  						VISIBLE_COLUMN, FALSE,
						-1);
	}

	
	gtk_tree_store_append (model, &child_iter, &iter);
	gtk_tree_store_set (model, &child_iter,
			      COLUMN_NAME_TF, STRING_DELTA_CHI_AX,
			      COLUMN_TF, mm_data_TF->Ax,
				  	COLUMN_ER, mm_data_ERR->Ax,
			      COLUMN_UTR, mm_data_UTR->Ax,
            COLUMN_UTR_ER, 999.0,//0.102,//0.0,//mm_data_UTR_ERR->Ax,      
			      VISIBLE_COLUMN, TRUE,
			      -1);
	gtk_tree_store_append (model, &child_iter, &iter);
	gtk_tree_store_set (model, &child_iter,
			      COLUMN_NAME_TF, STRING_DELTA_CHI_RH,
			      COLUMN_TF, mm_data_TF->Rh,
				  	COLUMN_ER, mm_data_ERR->Rh,
			      COLUMN_UTR, mm_data_UTR->Rh,
            COLUMN_UTR_ER, 999.0,//0.142,//0.0,//mm_data_UTR_ERR->Rh,
			      VISIBLE_COLUMN, TRUE,
			      -1);
	gtk_tree_store_append (model, &child_iter, &iter);
	gtk_tree_store_set (model, &child_iter,
			      COLUMN_NAME_TF, "x",
			      COLUMN_TF, mm_data_TF->X,
				  COLUMN_ER, mm_data_ERR->X,
			      COLUMN_UTR, mm_data_UTR->X,
                  COLUMN_UTR_ER, mm_data_ERR->X,
			      VISIBLE_COLUMN, TRUE,
			      -1);
	gtk_tree_store_append (model, &child_iter, &iter);
	gtk_tree_store_set (model, &child_iter,
			      COLUMN_NAME_TF, "y",
			      COLUMN_TF, mm_data_TF->Y,
				  COLUMN_ER, mm_data_ERR->Y,
			      COLUMN_UTR, mm_data_UTR->Y,
                  COLUMN_UTR_ER, mm_data_ERR->Y,
			      VISIBLE_COLUMN, TRUE,
			      -1);
	gtk_tree_store_append (model, &child_iter, &iter);
	gtk_tree_store_set (model, &child_iter,
			      COLUMN_NAME_TF, "z",
			      COLUMN_TF, mm_data_TF->Z,
				  COLUMN_ER, mm_data_ERR->Z,
			      COLUMN_UTR, mm_data_UTR->Z,
                  COLUMN_UTR_ER, mm_data_ERR->Z,
			      VISIBLE_COLUMN, TRUE,
			      -1);
	gtk_tree_store_append (model, &child_iter, &iter);
	gtk_tree_store_set (model, &child_iter,
			      COLUMN_NAME_TF, STRING_ALPHA,
			      COLUMN_TF, mm_data_TF->a,
				  	COLUMN_ER, mm_data_ERR->a,
			      COLUMN_UTR, mm_data_UTR->a,
             COLUMN_UTR_ER, 999.0,//mm_data_UTR_ERR->a,
			      VISIBLE_COLUMN, TRUE,
			      -1);
	gtk_tree_store_append (model, &child_iter, &iter);
	gtk_tree_store_set (model, &child_iter,
			      COLUMN_NAME_TF, STRING_BETA,
			      COLUMN_TF, mm_data_TF->b,
				  	COLUMN_ER, mm_data_ERR->b,
			      COLUMN_UTR, mm_data_UTR->b,
            COLUMN_UTR_ER, 999.0,//mm_data_UTR_ERR->b,
			      VISIBLE_COLUMN, TRUE,
			      -1);
	gtk_tree_store_append (model, &child_iter, &iter);
	gtk_tree_store_set (model, &child_iter,
			      COLUMN_NAME_TF, STRING_GAMMA,
			      COLUMN_TF, mm_data_TF->g,
				  	COLUMN_ER, mm_data_ERR->g,
			      COLUMN_UTR, mm_data_UTR->g,
            COLUMN_UTR_ER, 999.0,//mm_data_UTR_ERR->g,
			      VISIBLE_COLUMN, TRUE,
			      -1);
	
	if(SING_OR_MULT == MULTIPLE){
		path = gtk_tree_model_get_path ((GtkTreeModel *) model, &parent_iter);
	}
	else{
  		path = gtk_tree_model_get_path ((GtkTreeModel *) model, &iter);
	}
  gtk_tree_view_expand_row ( treeview, path, TRUE);
  gtk_tree_view_set_cursor (treeview, path, NULL, FALSE);
  gtk_tree_path_free (path);
}

GtkTreeModel * get_correct_model(int n_button, int SING_OR_MULT){
	if(SING_OR_MULT == MULTIPLE){
		return(_model_PCSM);
	}
	else{
		switch(n_button){
			case 0:{
				return(_model_PCS1);
				break;
			}
			case 1:{
				return(_model_PCS2);
				break;
			}
			case 2:{
				return(_model_PCS3);
				break;
			}
			case 3:{
				return(_model_PCS4);
				break;
			}
			default:
				return(NULL);
		}
	}
}


GtkTreeView * get_correct_treeview(int n_button, int SING_OR_MULT){
	if(SING_OR_MULT == MULTIPLE){
		return(_treeview_PCSM);
	}
	else{
		switch(n_button){
			case 0:{
				return(_treeview_PCS1);
				break;
			}
			case 1:{
				return(_treeview_PCS2);
				break;
			}
			case 2:{
				return(_treeview_PCS3);
				break;
			}
			case 3:{
				return(_treeview_PCS4);
				break;
			}
			default:
				return(NULL);
		}
	}
}


GtkLabel * get_correct_label(int n_button, int SING_OR_MULT){

		switch(n_button){
			case 0:{
				if(SING_OR_MULT == MULTIPLE){
					return(label_tensor_M1);
				}
				return(label_tensor_1);
				break;
			}
			case 1:{
				if(SING_OR_MULT == MULTIPLE){
					return(label_tensor_M2);
				}
				return(label_tensor_2);
				break;
			}
			case 2:{
				if(SING_OR_MULT == MULTIPLE){
					return(label_tensor_M3);
				}
				return(label_tensor_3);
				break;
			}
			case 3:{
				if(SING_OR_MULT == MULTIPLE){
					return(label_tensor_M4);
				}
				return(label_tensor_4);
				break;
			}
			default:
				return(NULL);
		}
}

static void polish_tensor_name (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
     gint column_id;
     gchar* buf;
    column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (renderer), "column"));
    gtk_tree_model_get(model, iter, column_id, &buf, -1);
  
    if (TRUE){
      g_object_set(renderer, "markup", buf, NULL);
    }
    g_free(buf);
}

void
add_columns_TF (GtkTreeView *treeview)
{
  gint col_offset;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  /* column variable name */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set (renderer, "xalign", 0.0, NULL);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_NAME_TF));
    
  col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
							    -1, "Variables",
							    renderer, "text",							
							    COLUMN_NAME_TF,
							    NULL);
  column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
  gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);
  gtk_tree_view_column_set_cell_data_func(column, renderer,            polish_tensor_name, NULL, NULL);
	

  /* column found */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set (renderer, "xalign", 0.0, NULL);

  col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
							    -1, "Found",
							    renderer, "text",
							    COLUMN_TF,
								"visible",
							    VISIBLE_COLUMN,
							    NULL);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_TF));  
  column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function_tl, NULL, NULL);	
  gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);

	  /* column error */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set (renderer, "xalign", 0.0, NULL);

  col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
							    -1, "Error",
							    renderer, "text",
							    COLUMN_ER,
								"visible", VISIBLE_COLUMN,	
							    NULL);
  column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_ER));
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function_error, NULL, NULL);
  gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);
	
	  /* column utr */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set (renderer, "xalign", 0.0, NULL);

  col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
							    -1, "UTR",
							    renderer, "text",
							    COLUMN_UTR,
								"visible",
							    VISIBLE_COLUMN,
							    NULL);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_UTR));
  column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function_tl, NULL, NULL);
  gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);
    
    	  /* column utr error */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set (renderer, "xalign", 0.0, NULL);

  col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
							    -1, "Error",
							    renderer, "text",
							    COLUMN_UTR_ER,
								"visible",
							    VISIBLE_COLUMN,
							    NULL);
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_UTR_ER));
  column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
  gtk_tree_view_column_set_cell_data_func(column, renderer, polish_float_cell_data_function_error, NULL, NULL);
  gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);
	
}

void polish_float_cell_data_function_error (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
    double value;
    gint column_id;
    gchar buf[40];
    column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (renderer), "column"));
    gtk_tree_model_get(model, iter, column_id, &value, -1);
    g_snprintf(buf, sizeof(buf), "\302\261%.3f", value);
    g_object_set(renderer, "text", buf, NULL);
}
