/*
 * tensor_list.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <gsl/gsl_multifit_nlin.h> //M_PI



#include "pdb.h"
#include "common.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "tensor_list.h"
#include "error_analysis.h"

#define _STR_TYPE_MULTI "calcMult"
#define _STR_EXP_EXP "PCS"


GtkTreeModel * _model_tl;
extern GtkWidget * _window_tensor_list;

void polish_float_cell_data_function_tl (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
    double value;
    gint column_id;
    gchar buf[20];
    double fac;
    fac = 1.0;
    column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (renderer), "column"));
    gtk_tree_model_get(model, iter, column_id, &value, -1);
    g_snprintf(buf, sizeof(buf), "%.3f", value * fac);
    g_object_set(renderer, "text", buf, NULL);
}

void polish_float_cell_data_function_tl_radian (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
    double value;
    gint column_id;
    gchar buf[20];
    double fac;
    fac = M_PI/180.0;
    column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (renderer), "column"));
    gtk_tree_model_get(model, iter, column_id, &value, -1);
    g_snprintf(buf, sizeof(buf), "%.3f", value * fac);
    g_object_set(renderer, "text", buf, NULL);
}

void  
change_column_properties_tl(GtkTreeViewColumn *column){
  gtk_tree_view_column_set_reorderable (GTK_TREE_VIEW_COLUMN (column),
				   TRUE);
  gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN (column),
				   TRUE);
  gtk_tree_view_column_set_sort_indicator (GTK_TREE_VIEW_COLUMN (column),
				   FALSE);
}

GtkTreeModel *
create_tensor_listing_model (void){
  GtkListStore *model;
  GtkTreeIter iter;
    
  model = gtk_list_store_new (NUM_ITEM_COLUMNS_TL, 
                              G_TYPE_INT,
                              G_TYPE_DOUBLE, //alpha
                              G_TYPE_DOUBLE, //beta
                              G_TYPE_DOUBLE, //gamma
                              G_TYPE_DOUBLE, //Ax
                              G_TYPE_DOUBLE, //Rh
                              G_TYPE_DOUBLE, //x
                              G_TYPE_DOUBLE, //y
                              G_TYPE_DOUBLE, //z
                              G_TYPE_STRING, //type
                              G_TYPE_STRING, //exp
                              G_TYPE_BOOLEAN, // USED
                              G_TYPE_BOOLEAN, // USED2 FOR COMPARISON
                              G_TYPE_DOUBLE, //alpha_err
                              G_TYPE_DOUBLE, //beta_err
                              G_TYPE_DOUBLE, //gamma_err
                              G_TYPE_DOUBLE, //Ax_err
                              G_TYPE_DOUBLE, //Rh_err
                              G_TYPE_DOUBLE, //x_err
                              G_TYPE_DOUBLE, //y_err
                              G_TYPE_DOUBLE //z_err
                              );
	
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter,
                            COLUMN_SERIAL_TL, 0,
                            COLUMN_ALPHA_TL, 0.0,
                            COLUMN_BETA_TL, 0.0,
                            COLUMN_GAMMA_TL, 0.0,
                            COLUMN_AX_TL, 40.0,
                            COLUMN_RH_TL, 4.0,
                            COLUMN_X_TL, 0.0,
                            COLUMN_Y_TL, 0.0,
                            COLUMN_Z_TL, 0.0,
                            COLUMN_TYPE_TL, "default",
                            COLUMN_EXP_TL, "none",
                            COLUMN_USED_TL, TRUE,
                            COLUMN_USED_TL2, FALSE,
                            COLUMN_ALPHA_ERR_TL, 0.0,
                            COLUMN_BETA_ERR_TL, 0.0,
                            COLUMN_GAMMA_ERR_TL, 0.0,
                            COLUMN_AX_ERR_TL, 0.0,
                            COLUMN_RH_ERR_TL, 0.0,
                            COLUMN_X_ERR_TL, 0.0,
                            COLUMN_Y_ERR_TL, 0.0,
                            COLUMN_Z_ERR_TL, 0.0,
						
                          -1);
  return GTK_TREE_MODEL (model);
}

void uncheck_all(GtkTreeModel * model, int column_number){
	GtkTreeIter iter;
	gboolean my_bool;
	my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model, &iter);
    while (my_bool)
    {
        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_number, FALSE, -1); 
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model, &iter);
    }
}

gboolean is_tensor_multi(char *type_exp){
	if (strcmp(type_exp, _STR_TYPE_MULTI) == 0){
		return(TRUE);
	}
	return(FALSE);
}

int which_pcs_experiment(char * type_exp){
	int result;
	if(strlen(type_exp) <= strlen(_STR_EXP_EXP)){
		fprintf(stderr, "Problem with length\n");
		return(-1);
	}
	result = atoi(type_exp + strlen(_STR_EXP_EXP));
	result = result -1;
	return(result);
}

static void polish_toggle_color_data_function (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
    int serial;
	char * type_exp;
	gboolean is_multi;
	
  gtk_tree_model_get (GTK_TREE_MODEL (model), iter, 
								COLUMN_SERIAL_TL, &serial,
								COLUMN_TYPE_TL, &type_exp,
								-1);
	is_multi = is_tensor_multi(type_exp);
	
	if(is_multi == FALSE){
    	if(is_tensor_single_valid(serial) == TRUE){
       	 	g_object_set (renderer, "cell-background", "#00D434", NULL);
        	g_object_set (renderer, "cell-background-set", TRUE, NULL);
		}
		else{
        	g_object_set (renderer, "cell-background", "#E92000", NULL);
       		g_object_set (renderer, "cell-background-set", TRUE, NULL);
		}
	}
	else {
		int * tab;
		tab = NULL;
		if(is_tensor_multi_valid(serial, &tab) == TRUE){
       	 	g_object_set (renderer, "cell-background", "#00A2FF", NULL);
        	g_object_set (renderer, "cell-background-set", TRUE, NULL);
		}
		else{
        	g_object_set (renderer, "cell-background", "#E92000", NULL);
       		g_object_set (renderer, "cell-background-set", TRUE, NULL);
		}
	}

}

void toggle_tl2(GtkCellRendererToggle *cell, gchar *path_str, gpointer data){
		GtkListStore *model;
    GtkTreeIter iter;
		
	  model = (GtkListStore *) _model_tl;
		
    if(model == NULL){
        return;
    }
    //first we uncheck everyone
    uncheck_all((GtkTreeModel *) model, COLUMN_USED_TL2);
		fprintf(stderr, "uncheck all\n");
		//second we check the correct one
  	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);

 	  gtk_tree_model_get_iter ((GtkTreeModel*)model, &iter, path);
		gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_USED_TL2, TRUE, -1);
		update_tensor_work();
		/* clean up */
  	gtk_tree_path_free (path);
}

void toggle_tl (GtkCellRendererToggle *cell, gchar *path_str, gpointer data){
    GtkListStore *model;
    GtkTreeIter iter;
    gboolean BOOL_ONLY_TENSOR_VALID;
    gboolean BOOL_MULTI;
    gboolean BOOL_DUO;
		
    switch(GPOINTER_TO_INT(data)){
		case TENSOR_ANY: {
						BOOL_ONLY_TENSOR_VALID = FALSE;
            BOOL_MULTI = FALSE;
						BOOL_DUO = FALSE;
				fprintf(stderr, "TENSOR_ANY\n");
        }
		break;
		case TENSOR_ONLY_VALID:{
            BOOL_ONLY_TENSOR_VALID = TRUE;
            BOOL_MULTI = FALSE;
						BOOL_DUO = FALSE;
				fprintf(stderr, "TENSOR_ONLY_VALID\n");
        }
		break;
        case TENSOR_ANY_MULTI:{
            BOOL_MULTI = TRUE;
            BOOL_ONLY_TENSOR_VALID = FALSE;
						BOOL_DUO = FALSE;
						fprintf(stderr, "TENSOR_ANY_MULTI\n");
        }
    break;
				case TENSOR_ONLY_DUO:{
						BOOL_MULTI = FALSE;
						BOOL_ONLY_TENSOR_VALID = FALSE;
						BOOL_DUO = TRUE;
						fprintf(stderr, "TENSOR_ONLY_DUO\n");
				}
		break;
		default:{
						BOOL_ONLY_TENSOR_VALID = FALSE;
            BOOL_MULTI = FALSE;
						BOOL_DUO = FALSE;
				fprintf(stderr, "TENSOR_defaults\n");
        }
		break;
	}

    model = (GtkListStore *) _model_tl;
    if(model == NULL){
        return;
    }
    //first we uncheck everyone
    if(BOOL_MULTI == FALSE){
        uncheck_all((GtkTreeModel *) model, COLUMN_USED_TL);
				fprintf(stderr, "uncheck all\n");
    }

  //second we check the correct one
  GtkTreePath *path = gtk_tree_path_new_from_string (path_str);

  gtk_tree_model_get_iter ((GtkTreeModel*)model, &iter, path);
  /* set new value */
	if(BOOL_ONLY_TENSOR_VALID == FALSE){
				if(BOOL_MULTI == TRUE){
					gboolean local_bool;
					gtk_tree_model_get (GTK_TREE_MODEL(model), &iter, COLUMN_USED_TL, &local_bool, -1);
					local_bool ^= 1;
					gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_USED_TL, local_bool, -1);
				}
				else{//BOOL_MULTI == FALSE
						if(BOOL_DUO == TRUE){
								gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_USED_TL, TRUE, -1);
								update_tensor_work();
						}
						else{//BOOL_DUO == FALSE
								gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_USED_TL, TRUE, -1);
						}
						
				}
	}
	else{ //BOOL_ONLY_TENSOR_VALID == TRUE
		int serial;
		int k;
		char * type_exp;
		char * exp_exp;
			
		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 
							COLUMN_SERIAL_TL, &serial,
							COLUMN_EXP_TL, &exp_exp,
							COLUMN_TYPE_TL, &type_exp,
							-1);
		if(is_tensor_multi(type_exp) == FALSE){
			if(is_tensor_single_valid(serial) == TRUE){
				int tab[N_PCS_ARRAY];
				int my_n_button;
				for(k = 0; k < N_PCS_ARRAY; k++){
					tab[k] = -1;
				}
				my_n_button = which_pcs_experiment(exp_exp);
				gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_USED_TL, TRUE, -1);
				tab[my_n_button] = serial;
				change_view_frame(tab);
			}
		}
		else{
			int * tab;
			if(is_tensor_multi_valid(serial, &tab) == TRUE){
				int i;
				gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_USED_TL, TRUE, -1);
				change_view_frame(tab);
				for(i = 0; i < N_PCS_ARRAY; i++){
					if(tab[i] != -1){
						GtkTreeIter other_iter;
						gboolean my_bool;
						int other_serial;
						my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model,
                                                        &other_iter);
						while(my_bool){
							gtk_tree_model_get ((GtkTreeModel *)model, &other_iter, COLUMN_SERIAL_TL, &other_serial, -1);
							if(tab[i] == other_serial){
								gtk_list_store_set (GTK_LIST_STORE (model), &other_iter, COLUMN_USED_TL, TRUE, -1);
							}
							my_bool = gtk_tree_model_iter_next((GtkTreeModel *)model,
                                                        &other_iter);
						}
					}
				}
			}
		}
	}

  /* clean up */
  gtk_tree_path_free (path);
}

// add columns to the treeview.
void
add_columns_tl (GtkTreeView  *treeview, int TENSOR_SELECTION_TYPE)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
    
  	
   // column for used toggles 
  renderer = gtk_cell_renderer_toggle_new ();
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_USED_TL));
	g_object_set (G_OBJECT (renderer), "indicator-size", TICK_SIZE, NULL);
  column = gtk_tree_view_column_new_with_attributes (TITLE_COLUMN_USED_TL1,
						     renderer,
						     "active", COLUMN_USED_TL,
						     NULL);
  switch(TENSOR_SELECTION_TYPE){
	  case TENSOR_ANY:{
  		 g_signal_connect (renderer, "toggled", G_CALLBACK (toggle_tl), GINT_TO_POINTER(TENSOR_ANY));
		  break;
	  }
	  case TENSOR_ONLY_VALID:{
		 g_signal_connect (renderer, "toggled", G_CALLBACK (toggle_tl), GINT_TO_POINTER(TENSOR_ONLY_VALID));
		 gtk_tree_view_column_set_cell_data_func(column, renderer, 
											polish_toggle_color_data_function, 
											NULL, NULL);
		  break;
	  }
      case TENSOR_ANY_MULTI: {
          g_signal_connect (renderer, "toggled", G_CALLBACK (toggle_tl), GINT_TO_POINTER(TENSOR_ANY_MULTI));
      }
			break;
			case TENSOR_ONLY_DUO: {
					g_signal_connect (renderer, "toggled", G_CALLBACK (toggle_tl), GINT_TO_POINTER(TENSOR_ONLY_DUO));
			}
			break;
  }

  gtk_tree_view_append_column (treeview, column);
 	
		// column for used toggles 2
  renderer = gtk_cell_renderer_toggle_new ();
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_USED_TL2));
	g_object_set (G_OBJECT (renderer), "indicator-size", TICK_SIZE, NULL);
  column = gtk_tree_view_column_new_with_attributes (TITLE_COLUMN_USED_TL2,
						     renderer,
						     "active", COLUMN_USED_TL2,
						     NULL);
  g_signal_connect (renderer, "toggled", G_CALLBACK (toggle_tl2), NULL);
	gtk_tree_view_append_column (treeview, column);
	gtk_tree_view_column_set_visible (column, FALSE);
  
	// serial_num
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_SERIAL_TL));   
  column = gtk_tree_view_column_new_with_attributes ("id",
						     renderer,
						     "text",
						     COLUMN_SERIAL_TL,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_SERIAL_TL);
  gtk_tree_view_append_column (treeview, column);

  // Ax 
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_AX_TL));   
  column = gtk_tree_view_column_new_with_attributes (STRING_DELTA_CHI_AX_ALTERNATIF,
						     renderer,
						     "text",
						     COLUMN_AX_TL,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_AX_TL);
  gtk_tree_view_column_set_cell_data_func(column, renderer,     polish_float_cell_data_function_tl, NULL, NULL); 
  change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
    
    
    // Rh
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_RH_TL));   
  column = gtk_tree_view_column_new_with_attributes (STRING_DELTA_CHI_RH_ALTERNATIF,
						     renderer,
						     "text",
						     COLUMN_RH_TL,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_RH_TL);
        gtk_tree_view_column_set_cell_data_func(column, renderer,     polish_float_cell_data_function_tl, NULL, NULL);  
      change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
    
    // x
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_X_TL));   
  column = gtk_tree_view_column_new_with_attributes ("x",
						     renderer,
						     "text",
						     COLUMN_X_TL,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_X_TL);
        gtk_tree_view_column_set_cell_data_func(column, renderer,     polish_float_cell_data_function_tl, NULL, NULL);  
      change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
    
    // y 
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_Y_TL));   
  column = gtk_tree_view_column_new_with_attributes ("y",
						     renderer,
						     "text",
						     COLUMN_Y_TL,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_Y_TL);
        gtk_tree_view_column_set_cell_data_func(column, renderer,     polish_float_cell_data_function_tl, NULL, NULL);  
      change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);

    // z 
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_Z_TL));   
  column = gtk_tree_view_column_new_with_attributes ("z",
						     renderer,
						     "text",
						     COLUMN_Z_TL,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_Z_TL);
        gtk_tree_view_column_set_cell_data_func(column, renderer,     polish_float_cell_data_function_tl, NULL, NULL);  
      change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
    
    // alpha
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_ALPHA_TL));   
  column = gtk_tree_view_column_new_with_attributes (STRING_ALPHA,
						     renderer,
						     "text",
						     COLUMN_ALPHA_TL,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_ALPHA_TL);
        gtk_tree_view_column_set_cell_data_func(column, renderer,     polish_float_cell_data_function_tl, NULL, NULL);  
      change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
    
    // beta 
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_BETA_TL));   
  column = gtk_tree_view_column_new_with_attributes (STRING_BETA,
						     renderer,
						     "text",
						     COLUMN_BETA_TL,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_BETA_TL);
        gtk_tree_view_column_set_cell_data_func(column, renderer,     polish_float_cell_data_function_tl, NULL, NULL);  
      change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
    
    // gamma 
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_GAMMA_TL));   
  column = gtk_tree_view_column_new_with_attributes (STRING_GAMMA,
						     renderer,
						     "text",
						     COLUMN_GAMMA_TL,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_GAMMA_TL);
        gtk_tree_view_column_set_cell_data_func(column, renderer,     polish_float_cell_data_function_tl, NULL, NULL);  
      change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
    
    // type 
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_TYPE_TL));   
  column = gtk_tree_view_column_new_with_attributes ("Type",
						     renderer,
						     "text",
						     COLUMN_TYPE_TL,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_TYPE_TL);
      change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
    
    // exp 
  renderer = gtk_cell_renderer_text_new (); 
  g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (COLUMN_EXP_TL));   
  column = gtk_tree_view_column_new_with_attributes ("Exp.",
						     renderer,
						     "text",
						     COLUMN_EXP_TL,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_EXP_TL);
      change_column_properties_tl(column);
  gtk_tree_view_append_column (treeview, column);
   
}


void init_tensor_listing(void){
    GladeXML *gxml;
    gxml = glade_get_widget_tree(_window_tensor_list);
    GtkTreeView * treeview;
    treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview3");
    GtkTreeModel * model_tl = create_tensor_listing_model();
    _model_tl = model_tl;
    add_columns_tl(treeview, TENSOR_ANY);
    set_multiple_selection(GTK_TREE_VIEW (treeview));
    gtk_tree_view_set_model ((GtkTreeView *)treeview, GTK_TREE_MODEL (model_tl));
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
    gtk_widget_hide(_window_tensor_list);
}


int add_to_tensor_listing(double Ax, double Rh, double x, double y, double z
                           , double alpha, double beta, double gamma, int type, int experiment,
						   double Ax_err, double Rh_err, double x_err, double y_err, double z_err
                           , double alpha_err, double beta_err, double gamma_err)
{
    static int serial = 0; //use for default.
    serial ++;
    char type_char[10];
    char exp_char[10];
    
    if(type == CALCULATED_TENSOR){
        sprintf(type_char, "calc");
    }
    if(type == CALCULATED_TENSOR_MULTI){
        sprintf(type_char, _STR_TYPE_MULTI);
    }
    if(type == MANUAL_TENSOR){
        sprintf(type_char, "manual");
    }
    
    if(experiment != -1){
        sprintf(exp_char, "%s%d", _STR_EXP_EXP, experiment);
    }
    else{
        sprintf(exp_char, "none");
    }
    GtkTreeIter iter;
    gtk_list_store_prepend ((GtkListStore * ) _model_tl, &iter);

    gtk_list_store_set ((GtkListStore * )_model_tl, &iter,
                            COLUMN_SERIAL_TL, serial,
                            COLUMN_ALPHA_TL, alpha,
                            COLUMN_BETA_TL, beta,
                            COLUMN_GAMMA_TL, gamma,
                            COLUMN_AX_TL, Ax, 
                            COLUMN_RH_TL, Rh,
                            COLUMN_X_TL, x,
                            COLUMN_Y_TL, y,
                            COLUMN_Z_TL, z,
                            COLUMN_TYPE_TL, type_char,
                            COLUMN_EXP_TL, exp_char,
                            COLUMN_USED_TL, FALSE,
														COLUMN_USED_TL2, FALSE,
                          -1);
	return(serial);
}

void add_manual_tl(void){
    GladeXML *gxml;
    GtkWidget* label_flag;
    int exp;
    char flag_char;
    double x, y, z, Ax, Rh, a, b, g;
    
    gxml = glade_get_widget_tree(_window_tensor_list);
    label_flag = glade_xml_get_widget (gxml, "flag_tl");
    const gchar * my_char;
    my_char = gtk_label_get_text ((GtkLabel *)label_flag);
    flag_char = my_char[0];
    switch(flag_char){
        case '1':{ exp = 1; break;}
        case '2':{ exp = 2; break;}
        case '3':{ exp = 3; break;}
        case '4':{ exp = 4; break;}
        default: {exp = -1;}
    }

    x = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_x0"));
    y = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_y0"));
    z = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_z0"));
    a = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_alpha0"));
    b = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_beta0"));
    g = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_gamma0"));
    Ax = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Ax0"));
    Rh = gtk_spin_button_get_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_Rh0"));
    add_to_tensor_listing(Ax, Rh, x, y, z, a, b, g, MANUAL_TENSOR, exp,
						  0, 0, 0, 0, 0, 0, 0, 0);
}

void start_tl(int n_button){
    GladeXML *gxml;
    GtkWidget* label_flag;
    if(_pdb_str == NULL){
        box_error( "You should first load a pdb file.\n", NULL);
        return;
    }
	uncheck_all((GtkTreeModel *) _model_tl, COLUMN_USED_TL);
	
    gxml = glade_get_widget_tree(_window_tensor_list);
    label_flag = glade_xml_get_widget (gxml, "flag_tl");
    gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_noise"), 0.0);
    gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, "spin_seed"), 1);
    char fl[2];
    sprintf(fl, "%d", n_button + 1);
    gtk_label_set_text ((GtkLabel *)label_flag, fl);
    gtk_widget_show(_window_tensor_list);
}

void cancel_tl(void){
    gtk_widget_hide(_window_tensor_list);
}

int read_tensor_only_from_tl(GtkTreeModel * model, tensor * my_tensor){
	GtkTreeIter iter;
    gboolean is_toggle;
	gboolean my_bool;
	int serial;
	
	is_toggle = FALSE;
    my_bool = gtk_tree_model_get_iter_first ((GtkTreeModel *)model, &iter);
    while (my_bool)
    {
        gtk_tree_model_get (model, &iter, COLUMN_USED_TL, &is_toggle, -1); 
        if (is_toggle == TRUE){
            break;
        }
        my_bool = gtk_tree_model_iter_next ((GtkTreeModel *)model, &iter);
    }
    
    if(is_toggle == FALSE){
        box_error("Exactly one tensor should be selected for calculation.", NULL);
        return (-1);
    }
    gtk_tree_model_get(model, &iter, COLUMN_X_TL, &(my_tensor->x), -1);
    gtk_tree_model_get(model, &iter, COLUMN_Y_TL, &(my_tensor->y), -1);
    gtk_tree_model_get(model, &iter, COLUMN_Z_TL, &(my_tensor->z), -1);
    gtk_tree_model_get(model, &iter, COLUMN_AX_TL, &(my_tensor->ax), -1);
    gtk_tree_model_get(model, &iter, COLUMN_RH_TL, &(my_tensor->rh), -1);
    gtk_tree_model_get(model, &iter, COLUMN_ALPHA_TL, &(my_tensor->a), -1);
    gtk_tree_model_get(model, &iter, COLUMN_BETA_TL, &(my_tensor->b), -1);
    gtk_tree_model_get(model, &iter, COLUMN_GAMMA_TL, &(my_tensor->g), -1);
	gtk_tree_model_get(model, &iter, COLUMN_SERIAL_TL, &serial, -1);
	my_tensor->a = my_tensor->a/180*M_PI;
	my_tensor->b = my_tensor->b/180*M_PI;
	my_tensor->g = my_tensor->g/180*M_PI;
	return(serial);
}

void ok_tl(void){
    double noise;
    int seed;
    GladeXML *gxml;
    int n_button;
    GtkWidget* label_flag;
    char flag_char;
    tensor my_tensor;
    GtkTreeModel *model;
    racs_data  racs_d;
    get_racs_parameters(&racs_d);
    gxml = glade_get_widget_tree(_window_tensor_list);

    label_flag = glade_xml_get_widget (gxml, "flag_tl");
    const gchar * my_char;
    my_char = gtk_label_get_text ((GtkLabel *)label_flag);
    flag_char = my_char[0];
    switch(flag_char){
        case '1':{ n_button = 0; break;}
        case '2':{ n_button = 1; break;}
        case '3':{ n_button = 2; break;}
        case '4':{ n_button = 3; break;}
        default: { n_button = 0;}
    }
    
    model = (GtkTreeModel *) _model_tl;
    if(model == NULL){
        return;
    }
	int resu = read_tensor_only_from_tl(_model_tl, &my_tensor);
	if(resu <0){
		return;
	}
	
	gtk_widget_hide(_window_tensor_list);
    noise = gtk_spin_button_get_value ((GtkSpinButton *)glade_xml_get_widget (gxml, "spin_noise"));
    seed = gtk_spin_button_get_value ((GtkSpinButton *)glade_xml_get_widget (gxml, "spin_seed"));
    
    calculate_pcs(&my_tensor, n_button, UPDATE_PCS_EXP, noise, seed, is_use_racs_generate()
                  , &racs_d);
    display_pcs_list(n_button, resu);
	clear_tensor_valid(n_button);
}

void copy_tl(void){
	  GladeXML *gxml; 
	  GtkTreeView *treeview;

  gxml = glade_get_widget_tree(_window_tensor_list);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview3"); 
  copy_tensor_from_list(gxml, treeview, 0);
	
}

void copy_tensor_from_list(GladeXML* gxml, GtkTreeView * treeview, int flag){
	GtkTreeIter iter;
  	GtkTreeModel *model;
  	GtkTreeSelection *selection; 
	char my_spin[50];
	int i;
  	double x, y, z, Ax, Rh, a, b, g;        
	GList * list;
	
	selection = gtk_tree_view_get_selection (treeview);
  	model = gtk_tree_view_get_model (treeview);
  	list = gtk_tree_selection_get_selected_rows(selection, &model);
	
	if(g_list_length(list) == 0)
		return;
	{
	  i = 0;

      GtkTreePath *path;
	  path = g_list_nth_data(list, i);
	  gtk_tree_model_get_iter (model, &iter, path);
		
      gtk_tree_model_get(model, &iter, COLUMN_X_TL, &x, -1);
      gtk_tree_model_get(model, &iter, COLUMN_Y_TL, &y, -1);
      gtk_tree_model_get(model, &iter, COLUMN_Z_TL, &z, -1);
      gtk_tree_model_get(model, &iter, COLUMN_AX_TL, &Ax, -1);
      gtk_tree_model_get(model, &iter, COLUMN_RH_TL, &Rh, -1);
      gtk_tree_model_get(model, &iter, COLUMN_ALPHA_TL, &a, -1);
      gtk_tree_model_get(model, &iter, COLUMN_BETA_TL, &b, -1);
      gtk_tree_model_get(model, &iter, COLUMN_GAMMA_TL, &g, -1);
		
	  sprintf(my_spin, "%s%d", "spin_x", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), x);
	  sprintf(my_spin, "%s%d", "spin_y", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), y);
	  sprintf(my_spin, "%s%d", "spin_z", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), z);
	  sprintf(my_spin, "%s%d", "spin_alpha", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), a);
	  sprintf(my_spin, "%s%d", "spin_beta", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), b);
	  sprintf(my_spin, "%s%d", "spin_gamma", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), g);
	  sprintf(my_spin, "%s%d", "spin_Ax", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), Ax);
	  sprintf(my_spin, "%s%d", "spin_Rh", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), Rh);
      gtk_tree_path_free (path);
	}
	g_list_free (list);
}

void remove_tl(void){
  GtkTreeView *treeview;
  GladeXML *gxml;  
	
  gxml = glade_get_widget_tree(_window_tensor_list);    
  treeview = (GtkTreeView *) glade_xml_get_widget (gxml, "treeview3");   
  remove_tensor_from_list(gxml, treeview, 0);
}

void remove_tensor_from_list(GladeXML* gxml, GtkTreeView * treeview, int flag){
  GtkTreeIter iter;
  GtkTreeModel *model;
  GtkTreeSelection *selection; 
  GList * list;
  char my_spin[50];
  int i;
  double x, y, z, Ax, Rh, a, b, g;  
	
  selection = gtk_tree_view_get_selection (treeview);
  model = gtk_tree_view_get_model (treeview);
  list = gtk_tree_selection_get_selected_rows(selection, &model);
	
	for(i = g_list_length(list) - 1; i >= 0; i--){

      GtkTreePath *path;
	  path = g_list_nth_data(list, i);
	  gtk_tree_model_get_iter (model, &iter, path);
      gtk_tree_model_get(model, &iter, COLUMN_X_TL, &x, -1);
      gtk_tree_model_get(model, &iter, COLUMN_Y_TL, &y, -1);
      gtk_tree_model_get(model, &iter, COLUMN_Z_TL, &z, -1);
      gtk_tree_model_get(model, &iter, COLUMN_AX_TL, &Ax, -1);
      gtk_tree_model_get(model, &iter, COLUMN_RH_TL, &Rh, -1);
      gtk_tree_model_get(model, &iter, COLUMN_ALPHA_TL, &a, -1);
      gtk_tree_model_get(model, &iter, COLUMN_BETA_TL, &b, -1);
      gtk_tree_model_get(model, &iter, COLUMN_GAMMA_TL, &g, -1);
      sprintf(my_spin, "%s%d", "spin_x", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), x);
	  sprintf(my_spin, "%s%d", "spin_y", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), y);
	  sprintf(my_spin, "%s%d", "spin_z", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), z);
	  sprintf(my_spin, "%s%d", "spin_alpha", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), a);
	  sprintf(my_spin, "%s%d", "spin_beta", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), b);
	  sprintf(my_spin, "%s%d", "spin_gamma", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), g);
	  sprintf(my_spin, "%s%d", "spin_Ax", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), Ax);
	  sprintf(my_spin, "%s%d", "spin_Rh", flag);
      gtk_spin_button_set_value ((GtkSpinButton *) glade_xml_get_widget (gxml, my_spin), Rh);
      gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
      gtk_tree_path_free (path);
    }
	g_list_free (list);
}
