/*
 * unify_tensor.c
 * Copyright (C) Christophe Schmitz and Thomas Huber 2008 <schmitz@maths.uq.edu.au> <numbat.pcs@gmail.com>
 *
 *
 *     This file is part of Numbat.
 * 
 *     Numbat is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     Numbat is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with Numbat.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "pdb.h"
#include "common.h"
#include "racs.h"
#include "pcs.h"
#include "tensor.h"
#include "tensor_list.h"
#include "molmol_iso.h"
#include "pymol_iso.h"
#include "unify_tensor.h"


#define XX 0
#define YY 1
#define ZZ 2
#define YZX 0
#define ZXY 1


typedef struct vec {
  float x;
  float y;
  float z;
} Vec;

//double R_ZXZ[3][3], R_ZYZ[3][3];

double R_X_90[3][3] = {{1, 0, 0},    {0, 0, 1},     {0, -1, 0}};
double R_Y_90[3][3] = {{0, 0, -1},   {0, 1, 0},     {1, 0, 0}};
double R_Z_90[3][3] = {{0, 1, 0},    {-1, 0, 0},    {0, 0, 1}};
double R_no[3][3]   = {{1, 0, 0},    {0, 1, 0},     {0, 0, 1}};
//double R_temp1[3][3];
//double R_temp2[3][3];

double correct(double c, double s);
int mult_mat(double R1[3][3], double R2[3][3], double R3[3][3]);

int mult_mat(double R1[3][3], double R2[3][3], double R3[3][3]){
  
  int i, j;
  
  for(i = 0; i < 3; i++){
    for(j = 0; j < 3; j++){
      R3[i][j] = R1[i][0] * R2[0][j] + R1[i][1] * R2[1][j] + R1[i][2] * R2[2][j];
    }
  }
return(1);
}



double correct(double c, double s){
  //c in [0;M_PI]
  //s in [-M_PI/2;M_PI/2]  

  if( c <= M_PI/2.0 ){   
    if( s < 0 ){
      s+=2*M_PI;
    }
    return(s);
  }
  else { //c>M_PI/2.0
    if( s > 0 ){
      return(c);
    }
    else{
      return(-c + 2 * M_PI);
    }
  }
}

int euler_from_Z_X_Z(double R[3][3], double Ex[3]){ 
  	Vec X, Y, Z;
  	double c, s;
  	double temp[4];
  	int i, j;
	int state1, state2;
	double rot_1[3][3];
	double rot_2[3][3];
	double E1[3], E2[3];
	double compare1, compare2;
  
 	X.x = R[0][0];
 	X.y = R[0][1];
 	X.z = R[0][2];
  
  	Y.x = R[1][0];
 	Y.y = R[1][1];
  	Y.z = R[1][2];
  
 	Z.x = R[2][0];
 	Z.y = R[2][1];
 	Z.z = R[2][2];
	
  	E1[1] = acos( Z.z );
  	temp[0] = Z.x / sin(E1[1]);
  	temp[1] = -1.0 * Z.y / sin(E1[1]);
  	temp[2] = X.z/sin(E1[1]);
  	temp[3] = Y.z/sin(E1[1]);
	
	state1 = 0;
  	for(i = 0; i < 4; i++){
		if((temp[i]<=1)&&(temp[i] >=-1)){ ; }
  		else{ state1 = -1; }
  	}
	if(state1 == 0){
  		s = asin( temp[0] ); //a31
  		c = acos( temp[1] ); //a32
  		E1[0] = correct(c, s);
  		s = asin( temp[2] ); //a13
  		c = acos( temp[3] ); //a23
  		E1[2] = correct(c, s);
	}
	else{ //beta equal 0
		E1[2] = 0;
		c = acos(R[0][0]);
		s = asin(R[0][1]);
		E1[0] = correct(c, s);
		fprintf(stderr, "WARNING beta equal to 0, gamma is set to 0 (first try)\n");
	}
	
  	E2[1] = acos( Z.z );
	E2[1] = -E2[1] + 2 * M_PI;
  	temp[0] = Z.x / sin(E2[1]);
  	temp[1] = -1.0 * Z.y / sin(E2[1]);
  	temp[2] = X.z/sin(E2[1]);
  	temp[3] = Y.z/sin(E2[1]);
	
	state2 = 0;
  	for(i = 0; i < 4; i++){
		if((temp[i]<=1)&&(temp[i] >=-1)){ ; }
  		else{ state2 = -1; }
  	}
	if(state2 == 0){
  		s = asin( temp[0] ); //a31
  		c = acos( temp[1] ); //a32
  		E2[0] = correct(c, s);
  		s = asin( temp[2] ); //a13
  		c = acos( temp[3] ); //a23
  		E2[2] = correct(c, s);
	}
	else{
		E2[2] = 0;
		c = acos(R[0][0]);
		s = asin(R[0][1]);
		E2[0] = correct(c, s);
		fprintf(stderr, "WARNING beta equal to 0, gamma is set to 0 (second try)\n");
	}

	filZX_or_YZ(rot_1, E1[0]/M_PI*180.0, E1[1]/M_PI*180.0, E1[2]/M_PI*180.0);
	filZX_or_YZ(rot_2, E2[0]/M_PI*180.0, E2[1]/M_PI*180.0, E2[2]/M_PI*180.0);
	
	compare1 = 0;
	compare2 = 0;
	for(i = 0; i < 3; i++){
		for(j = 0; j < 3; j++){
			compare1 = (R[i][j] - rot_1[i][j])* (R[i][j] - rot_1[i][j]);
			compare2 = (R[i][j] - rot_2[i][j])* (R[i][j] - rot_2[i][j]);
		}
	}

	double aaa, bbb, ggg;
	aaa = E1[0]/M_PI*180;
	bbb = E1[1]/M_PI*180;
	ggg = E1[2]/M_PI*180;
	between_0_180(&aaa, &bbb, &ggg);
	fprintf(stderr, "%10.3lf %10.lf %10.3lf %lf\n", E1[0]/M_PI*180, E1[1]/M_PI*180, E1[2]/M_PI*180, compare1); 
	fprintf(stderr, "%10.3lf %10.lf %10.3lf %lf\n", aaa, bbb, ggg, compare1); 
	
	aaa = E2[0]/M_PI*180;
	bbb = E2[1]/M_PI*180;
	ggg = E2[2]/M_PI*180;
	between_0_180(&aaa, &bbb, &ggg);
	fprintf(stderr, "%10.3lf %10.lf %10.3lf %lf\n", E2[0]/M_PI*180, E2[1]/M_PI*180, E2[2]/M_PI*180, compare2);
	fprintf(stderr, "%10.3lf %10.lf %10.3lf %lf\n", aaa, bbb, ggg, compare1); 
	if(compare1 + 0.000001 < compare2){
		Ex[0] = E2[0];
		Ex[1] = E2[1];
		Ex[2] = E2[2];
		fprintf(stderr, "second one is taken, SHOULDN'T HAPPEN BUT OK\n");
		//in this case, there is a problem with a ub [0 2PI], b in [0, PI], g in [0, 2PI]
		//fprintf(stderr, "EULER FROM ZXZ END ///////////////////////\n");
		return(state2);
	}
	else{
		Ex[0] = E1[0];
		Ex[1] = E1[1];
		Ex[2] = E1[2];
		//fprintf(stderr,"first one is taken OK\n");
		//fprintf(stderr, "EULER FROM ZXZ END ///////////////////////\n");
		return(state1);
	}
}


int euler_from_Z_Y_Z(double R[3][3], double Ex[3]){ 
  	Vec X, Y, Z;
  	double c, s;
  	double temp[4];
  	int i, j;
	int state1, state2;
	double rot_1[3][3];
	double rot_2[3][3];
	double E1[3], E2[3];
	double compare1, compare2;
  
 	X.x = R[0][0];
 	X.y = R[0][1];
 	X.z = R[0][2];
  
  	Y.x = R[1][0];
 	Y.y = R[1][1];
  	Y.z = R[1][2];
  
 	Z.x = R[2][0];
 	Z.y = R[2][1];
 	Z.z = R[2][2];
	
  	E1[1] = acos( Z.z ); //OKAY
  	temp[0] = Z.x / sin(E1[1]);
  	temp[1] = Z.y / sin(E1[1]);
  	temp[2] = -1.0* X.z/sin(E1[1]);
  	temp[3] = Y.z/sin(E1[1]);
	
	state1 = 0;
  	for(i = 0; i < 4; i++){
		if((temp[i]<=1)&&(temp[i] >=-1)){ ; }
  		else{ state1 = -1; }
  	}
	if(state1 == 0){
  		s = asin( temp[1] ); //a31
  		c = acos( temp[0] ); //a32
  		E1[0] = correct(c, s);
  		s = asin( temp[3] ); //a13
  		c = acos( temp[2] ); //a23
  		E1[2] = correct(c, s);
	}
	else{
		E1[2] = 0;
		c = acos(R[0][0]);
		s = asin(R[0][1]);
		E1[0] = correct(c, s);
		fprintf(stderr, "WARNING beta equal to 0, gamma is set to 0 (first try)\n");
	}
	
  	E2[1] = acos( Z.z );
	E2[1] = -E2[1] + 2 * M_PI;
  	temp[0] = Z.x / sin(E2[1]);
  	temp[1] = Z.y / sin(E2[1]);
  	temp[2] = -1.0* X.z/sin(E2[1]);
  	temp[3] = Y.z/sin(E2[1]);
	
	state2 = 0;
  	for(i = 0; i < 4; i++){
		if((temp[i]<=1)&&(temp[i] >=-1)){ ; }
  		else{ state2 = -1; }
  	}
	if(state2 == 0){
  		s = asin( temp[1] ); //a31
  		c = acos( temp[0] ); //a32
  		E2[0] = correct(c, s);
  		s = asin( temp[3] ); //a13
  		c = acos( temp[2] ); //a23
  		E2[2] = correct(c, s);
	}
	else{
		E2[2] = 0;
		c = acos(R[0][0]);
		s = asin(R[0][1]);
		E2[0] = correct(c, s);
		fprintf(stderr, "WARNING beta equal to 0, gamma is set to 0 (second try)\n");
	}
	
	filZX_or_YZ(rot_1, E1[0]/M_PI*180.0, E1[1]/M_PI*180.0, E1[2]/M_PI*180.0);
	filZX_or_YZ(rot_2, E2[0]/M_PI*180.0, E2[1]/M_PI*180.0, E2[2]/M_PI*180.0);
	
	compare1 = 0;
	compare2 = 0;
	for(i = 0; i < 3; i++){
		for(j = 0; j < 3; j++){
			compare1 = (R[i][j] - rot_1[i][j])* (R[i][j] - rot_1[i][j]);
			compare2 = (R[i][j] - rot_2[i][j])* (R[i][j] - rot_2[i][j]);
		}
	}

	if(compare1 + 0.000001 < compare2){
		Ex[0] = E2[0];
		Ex[1] = E2[1];
		Ex[2] = E2[2];
		fprintf(stderr, "second one is taken, SHOULDN'T HAPPEN BUT OK\n");
		//in this case, there is a problem with a ub [0 2PI], b in [0, PI], g in [0, 2PI]
		return(state2);
	}
	else{
		Ex[0] = E1[0];
		Ex[1] = E1[1];
		Ex[2] = E1[2];
		//fprintf(stderr,"first one is taken OK\n");
		return(state1);
	}
}

int euler_from_ZX_or_YZ(double R[3][3], double Ex[3]){
    if(EULER_CONVENTION == ZXZ_CONVENTION){
        return(euler_from_Z_X_Z(R, Ex));
    }
    if(EULER_CONVENTION == ZYZ_CONVENTION){
        return(euler_from_Z_Y_Z(R, Ex));
    }
    fprintf(stderr, "WARNING, euler convention not defined, see common.h\n");
}

void angle_positif(double * pu){
	double u;
	u = *pu;
	
	while(u > 0){
			u = u - 360.0;
	} 
		
	while(u < 0){
		u = u + 360.0;
	}
	
	// to avoid the -0.0000
	if(u == 0){
		u = 0;
	}
	//mot sure how it works with optimizations...
	
	*pu = u;
}


void between_0_180(double * pa, double * pb, double * pg){
	double a, b, g;
	a = *pa;
	b = *pb;
	g = *pg;
	
	angle_positif(&a);
	angle_positif(&b);
	angle_positif(&g);			  
	
	if((0<=a)&&(a<180.0)){ //a in 0, 180
		if((0<=b)&&(b<180.0)){ //b in 0, 180
			if((0<=g)&&(g<180.0)){ //g in 0, 180
				;
			}
			else{ //g in 180, 360
				g = g + 180.0;
			}
		}
		else{ //b in 180, 360
			if((0<=g)&&(g<180.0)){ //g in 0, 180
				b = b + 180.0;
				g = - g + 180.0;
			}
			else{ //g in 180, 360
				b = b + 180.0;
				g = -g;
			}
		}

	}
	else{ //a in 180, 360
		if((0<=b)&&(b<180.0)){ //b in 0, 180
			if((0<=g)&&(g<180.0)){ //g in 0, 180
				a = a + 180.0;
				b = -b + 180.0;
				g = -g + 180.0;
			}
			else{ //g in 180, 360
				a = a + 180.0;
				b = -b + 180.0;
				g = -g;
			}
		}
		else{ //b in 180, 360
			if((0<=g)&&(g<180.0)){ //g in 0, 180
				a = a + 180.0;
				b = -b;
				g = g;
			}
			else{ //g in 180, 360
				a = a + 180.0;
				b = -b;
				g = g + 180.0;
			}
		}
	}
	
	angle_positif(&a);
	angle_positif(&b);
	angle_positif(&g);
	*pa = a;
	*pb = b;
	*pg = g;
}



void print_all_64(double a, double b, double g, molmol_data* mm_data){
	//piece of code to check equivalent set of euler angles
	//with respect to ISOSURFACES USED FOR DEV ONLY
	
	char name[50];
	
	double A[4];
	double B[4];
	double G[4];
	int i, j, k;
	
	A[0] = a;
	B[0] = b;
	G[0] = g;
	
	A[1] = a + 180.0;
	B[1] = b + 180.0;
	G[1] = g + 180.0;
	
	A[2] = -a + 180.0;
	B[2] = -b + 180.0;
	G[2] = -g + 180.0;
	
	A[3] = -a;
	B[3] = -b;
	G[3] = -g;
	
	
	for(i = 0; i < 4; i++){
		for(j = 0; j < 4; j++){
			for(k = 0; k < 4; k++){
				sprintf(name, "./f_%d%d%d.pml", i, j, k);
				mm_data->a = A[i];
				mm_data->b = B[j];
				mm_data->g = G[k];
				if((i == 0)&&(j == 0)&&(k == 0)){
					write_for_pymol(mm_data , name, 0);
				}
				else{
					write_for_pymol(mm_data , name, 1);	
				}
			}
		}
	}
}

void swap_double( double * a, double *b){
    double temp;
    temp = *a;
    *a = *b;
    *b = temp;
}

//angle must be in: 
void unique_tensor(molmol_data * mm_data){
	double Dx, Dy, Dz, Ax, Rh;
	double aDx, aDy, aDz;
	double a, b, g;
	double R[3][3];
	double R_t[3][3];
	double E[3];

	a = mm_data->a;
	b = mm_data->b;
	g = mm_data->g;
	Ax = mm_data->Ax;
	Rh = mm_data->Rh;
	
	angle_positif(&a);
	angle_positif(&b);
	angle_positif(&g);
	
	filZX_or_YZ(R, a, b, g);
    
	Dx = -Ax/3.0 + Rh/2.0;
	Dy = -Ax/3.0 - Rh/2.0;
	Dz = 2.0/3.0*Ax;

	aDx = fabs(Dx);
	aDy = fabs(Dy);
	aDz = fabs(Dz);
	
	if((aDz >= aDy)&&(aDy >= aDx)){ //case 1
		//Nothing to do
		fprintf(stderr, "case1\n");
		if(b == 0){
			a = a + g;
			g = 0;
		}
		goto save_t;
	}
	
	
	if((aDz >= aDx)&&(aDx >= aDy)){ //case 2
		g = g + 90.0;
		if(b == 0){
			a = a + g;
			g = 0;
		}
    swap_double(&Dy, &Dx);
		goto save_t;
	}
	
	if((aDy >= aDz)&&(aDz >= aDx)){ //case 3
    swap_double(&Dy, &Dz);
		mult_mat(R_X_90, R, R_t);
		euler_from_ZX_or_YZ(R_t, E);
		a = E[0]/M_PI*180.0;
		b = E[1]/M_PI*180.0;
		g = E[2]/M_PI*180.0; 
		fprintf(stderr, "case3\n");
		goto save_t;
	}
	
	if((aDy >= aDx)&&(aDx >= aDz)){ //case 4
		g = g + 90.0;
		mm_data->g = g;    
		filZX_or_YZ(R, a, b, g);  
    swap_double(&Dy, &Dx);
    swap_double(&Dz, &Dx);
		mult_mat( R_Y_90, R, R_t);
		euler_from_ZX_or_YZ(R_t, E);
		a = E[0]/M_PI*180.0;
		b = E[1]/M_PI*180.0;
		g = E[2]/M_PI*180.0;
		fprintf(stderr, "case4\n");
		goto save_t;
	}
	if((aDx >= aDz)&&(aDz >= aDy)){ //case 5
		g = g + 90.0;
		mm_data->g = g;
		filZX_or_YZ(R, a, b, g);
    swap_double(&Dy, &Dx); 
    swap_double(&Dy, &Dz);
		mult_mat( R_X_90, R, R_t);
		euler_from_ZX_or_YZ(R_t, E);
		a = E[0]/M_PI*180.0;
		b = E[1]/M_PI*180.0;
		g = E[2]/M_PI*180.0;
		fprintf(stderr, "case5\n");
		goto save_t;
	}
	if((aDx >= aDy)&&(aDy >= aDz)){ //case 6
    swap_double(&Dz, &Dx);
		mult_mat( R_Y_90, R, R_t);
		euler_from_ZX_or_YZ(R_t, E);
		a = E[0]/M_PI*180.0;
		b = E[1]/M_PI*180.0;
		g = E[2]/M_PI*180.0;
		fprintf(stderr, "case6\n");
		goto save_t;
	}
	
	save_t:
	Ax = Dz - (Dx + Dy)/2.0;
	Rh = Dx - Dy;
	between_0_180(&a, &b, &g);
	mm_data->Ax = Ax;
	mm_data->Rh = Rh;
	mm_data->a = a;
	mm_data->b = b;
	mm_data->g = g;
}

void print_matrix(double R[3][3]){
  
  printf("\t[ % .3f % .3f % .3f ]\n", R[0][0], R[0][1], R[0][2]);
  printf("\t[ % .3f % .3f % .3f ]\n", R[1][0], R[1][1], R[1][2]);
  printf("\t[ % .3f % .3f % .3f ]\n", R[2][0], R[2][1], R[2][2]);

}

void calculate_8_tensors_from_convention(
    molmol_data * mm_data_origine, molmol_data mm_data_8[8], 
    int rotation_order_int, int angle_orientation_int, int magnitude_convention_int)
{   
    //double Dx, Dy, Dz;
    double Ax, Rh;
	//double aDx, aDy, aDz;
	double a, b, g;
    int i;
	//double R[3][3];
	//double R_t[3][3];
	//double E[3];
    
    if(magnitude_convention_int == NONE_CODE_INT){
        fprintf(stderr, "Nothing to do for magnitude\n");
    }
    else{//Need to translate to get Xz > Xy >Xx
        fprintf(stderr, "Translating to get Xz > Xy >Xx\n");
        fprintf(stderr, "Ax, Rh, a, b, g might be updated\n");
        unique_tensor(mm_data_origine);
    }
    
    a = mm_data_origine->a;
	b = mm_data_origine->b;
	g = mm_data_origine->g;
	Ax = mm_data_origine->Ax;
	Rh = mm_data_origine->Rh;
    
    //At this stage, Ax and Rh is good.
    for( i = 0; i < 8; i++){
        mm_data_8[i].Ax = Ax;
        mm_data_8[i].Rh = Rh;
    }
    
    if((angle_orientation_int == RIGHT_HANDED_CODE_INT)&&
       (
       ((rotation_order_int == ZYZ_CODE_INT) && (EULER_CONVENTION == ZYZ_CONVENTION))
        ||
       ((rotation_order_int == ZXZ_CODE_INT) && (EULER_CONVENTION == ZXZ_CONVENTION))
        )){
        fprintf(stderr, "Nothing to do for angle\n");  
    }
    else{//Need to translate alpha beta gamma
        fprintf(stderr, "Translating alpha beta gamma\n");
    }
    
    //At this stage, we have alpha beta and gamma correct,
    //but we need the 8 equivalent alpha beta gamma.
       
    for( i = 0; i < 8; i++){
        mm_data_8[i].a = a;
        mm_data_8[i].b = b;
        mm_data_8[i].g = g;
    }
}
